﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using Com.Alipay;

namespace AlipayStatement
{
    public class AlipayStatementDownloader
    {
        /// <summary>
        /// return true when download is succeeded.
        /// return false, when there is an error returned from server, check errorCode for details
        /// exception, check the inner message in this exception.
        /// </summary>
        /// <param name="date"></param>
        /// <param name="results"></param>
        /// <param name="errorCode"></param>
        /// <returns></returns>
        public bool Download(DateTime date, List<AlipayStatementQuery> results, out string errorCode)
        {
            string begDate = date.ToString("yyyy-MM-dd ");
            string endDate = begDate + "23:59:59";
            begDate += "00:00:00";

            bool hasModePage = true;
            int page = 1;
            errorCode = "";
            while (hasModePage)
            {
                if (!Download(begDate, endDate, page++, results, out hasModePage, out errorCode))
                {
                    return false;
                }
            }
            return true;
        }

        bool CheckTrueFalseNode(XmlDocument xml, string name, string trueValue, string falseValue)
        {
            XmlNode node = xml.SelectSingleNode(name);
            
            if (node == null)
                throw new Exception(string.Format("Node {0} doesn't exist.", name));

            if (node.InnerText == trueValue)
                return true;

            if (node.InnerText == falseValue)
                return false;

            throw new Exception(string.Format("value {0} is not expected for node {1}", node.InnerText, name));
        }

        
        bool Download(string begDate, string endDate, int page, List<AlipayStatementQuery> results, out bool hasMorePage, out string errorCode)
        {
            hasMorePage = false;
            errorCode = "";

            SortedDictionary<string, string> sParaTemp = new SortedDictionary<string, string>();
            sParaTemp.Add("partner", Config.Partner);
            sParaTemp.Add("_input_charset", Config.Input_charset.ToLower());
            sParaTemp.Add("service", "account.page.query");
            sParaTemp.Add("page_no", page.ToString());

            sParaTemp.Add("gmt_start_time", begDate);
            sParaTemp.Add("gmt_end_time", endDate);
            string sHtmlText = Submit.BuildRequest(sParaTemp);
            XmlDocument xml = new XmlDocument();
            xml.LoadXml(sHtmlText);
            try
            {
                bool isSuccessResult = CheckTrueFalseNode(xml, "alipay/is_success", "T", "F");

                if (!isSuccessResult)  // "F"
                {
                    XmlNode errorNode = xml.SelectSingleNode("alipay/error");
                    if (errorNode == null)
                    {
                        throw new Exception("Call failed (alipay/is_success = \"F\"). alipay/error doesn't exist.\r\n");
                    }
                    if (string.IsNullOrEmpty(errorNode.InnerText))
                    {
                        throw new Exception("Call failed (alipay/is_success = \"F\"). alipay/error doesn't have value.");
                    }
                    errorCode = errorNode.InnerText;
                    return false;
                }
                else   // "T"
                {
                    bool hasNextPageResult = CheckTrueFalseNode(xml, "alipay/response/account_page_query_result/has_next_page", "T", "F");
                    if (hasNextPageResult) // "T"
                    {
                        hasNextPageResult = true;
                    }
                    else // "F"
                    {
                        hasNextPageResult = false;
                    }

                    XmlNodeList nodes = xml.SelectNodes("alipay/response/account_page_query_result/account_log_list/AccountQueryAccountLogVO");

                    foreach (XmlNode node in nodes)
                    {
                        results.Add(AlipayStatementQueryBuilder.FromXmlNode(node));
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message + sHtmlText);
            }
        }
    }
}
