﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using CCSPayments;
using CCSPrototypeCommon;
namespace CCSPayments.Model
{

    public abstract class WxPaymentData : PaymentData
    {
        public static Logger logger
        {
            get
            {
                return new Logger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name);
            }

        }
        protected WxConfiguration m_paymentConfig;
        public WxPaymentData(WxConfiguration wxConfiguration)
        {
            m_paymentConfig = wxConfiguration;
        }
        public override string MakeSign()
        {
            //转url格式
            string str = ToUrl();
            //在string后加入API KEY
            str += "&key=" + m_paymentConfig.PaymentSecretKey;
            //MD5加密
            var md5 = MD5.Create();
            var bs = md5.ComputeHash(Encoding.UTF8.GetBytes(str));
            var sb = new StringBuilder();
            foreach (byte b in bs)
            {
                sb.Append(b.ToString("x2"));
            }
            //所有字符转为大写
            return sb.ToString().ToUpper();
        }

        public override bool CheckSign()
        {
            //如果没有设置签名，则跳过检测
            if (!IsSet("sign"))
            {
                logger.Error(this.GetType().ToString(), "WxPayData签名存在但不合法!");
                throw new PaymentException("WxPayData签名存在但不合法!");
            }
            //如果设置了签名但是签名为空，则抛异常
            else if (GetValue("sign") == null || GetValue("sign").ToString() == "")
            {
                logger.Error(this.GetType().ToString(), "WxPayData签名存在但不合法!");
                throw new PaymentException("WxPayData签名存在但不合法!");
            }

            //获取接收到的签名
            string return_sign = GetValue("sign").ToString();

            //在本地计算新的签名
            string cal_sign = MakeSign();

            if (cal_sign == return_sign)
            {
                return true;
            }

            logger.Error(this.GetType().ToString(), "WxPayData签名验证错误!");
            throw new PaymentException("WxPayData签名验证错误!");
        }

        public override string ToXml()
        {
            //数据为空时不能转化为xml格式
            if (0 == m_values.Count)
            {
                logger.Error(this.GetType().ToString(), "PaymentData数据为空!");
                throw new PaymentException("PaymentData数据为空!");
            }

            string xml = "<xml>";
            foreach (KeyValuePair<string, object> pair in m_values)
            {
                //字段值不能为null，会影响后续流程
                if (pair.Value == null)
                {
                    logger.Error(this.GetType().ToString(), "PaymentData内部含有值为null的字段!");
                    throw new PaymentException("PaymentData内部含有值为null的字段!");
                }

                if (pair.Value is int)
                {
                    xml += "<" + pair.Key + ">" + pair.Value + "</" + pair.Key + ">";
                }
                else if (pair.Value is string)
                {
                    xml += "<" + pair.Key + ">" + "<![CDATA[" + pair.Value + "]]></" + pair.Key + ">";
                }
                else//除了string和int类型不能含有其他数据类型
                {
                    logger.Error(this.GetType().ToString(), "PaymentData字段数据类型错误!");
                    throw new PaymentException("PaymentData字段数据类型错误!");
                }
            }
            xml += "</xml>";
            return xml;
        }

        public override SortedDictionary<string, object> FromXml(string xml)
        {
            if (string.IsNullOrEmpty(xml))
            {
                logger.Error(this.GetType().ToString(), "将空的xml串转换为PaymentData不合法!");
                throw new PaymentException("将空的xml串转换为PaymentData不合法!");
            }

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.LoadXml(xml);
            XmlNode xmlNode = xmlDoc.FirstChild;//获取到根节点<xml>
            XmlNodeList nodes = xmlNode.ChildNodes;
            foreach (XmlNode xn in nodes)
            {
                XmlElement xe = (XmlElement)xn;
                m_values[xe.Name] = xe.InnerText;//获取xml的键值对到WxPayData内部的数据中
            }

            //2015-06-29 错误是没有签名
            if (m_values["return_code"] != "SUCCESS")
            {
                return m_values;
            }

            CheckSign();//验证签名,不通过会抛异常

            return m_values;
        }

        public override string ToUrl()
        {
            string buff = "";
            foreach (KeyValuePair<string, object> pair in m_values)
            {
                if (pair.Value == null)
                {
                    logger.Error(this.GetType().ToString(), "PaymentData内部含有值为null的字段!");
                    throw new PaymentException("PaymentData内部含有值为null的字段!");
                }

                if (pair.Key != "sign" && pair.Value.ToString() != "")
                {
                    buff += pair.Key + "=" + pair.Value + "&";
                }
            }
            buff = buff.Trim('&');
            return buff;
        }

    }
}
