﻿using System;
using System.Linq;
using System.Linq.Expressions;
using System.Collections.Generic;
using Microsoft.WindowsAzure.Storage.Queue;
using Microsoft.WindowsAzure.Storage;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;

namespace CCSPrototypeCommon.Query
{
    public enum QueryType
    {
        Undefined = -1,
        MeterProfileQuery = 0,
        RawDataQuery = 1,
        BillQuery = 2,
        PaymentQuery = 3,
        VerifiedBillQuery = 4,
        SuccessfulWeChatPayment = 5,
        MisMatchWeChatPayment = 6,
        CCSOnlyWeChatPayment = 7,
        WeChatOnlyPayment = 8,
        UnknownSubVerifiedBillQuery = 9,
    }

    public static class SubVerifiedBillQuery
    {
        public const string SuccessfulWeChatPayment = "Successful WeChat Payment";
        public const string MisMatchWeChatPayment = "Mismatch WeChat Payment";
        public const string CcsOnlyWeChatPayment = "CCS only WeChat Payment";
        public const string WeChatOnlyPayment = "WeChat only WeChat Payment";

        private static Dictionary<string, QueryType> SubQueryMap = new Dictionary<string, QueryType>
        {
            {SuccessfulWeChatPayment, QueryType.SuccessfulWeChatPayment},
            {MisMatchWeChatPayment, QueryType.MisMatchWeChatPayment},
            {CcsOnlyWeChatPayment, QueryType.CCSOnlyWeChatPayment},
            {WeChatOnlyPayment, QueryType.WeChatOnlyPayment},
        };

        public static QueryType GetQueryType(string queryStr)
        {
            if (string.IsNullOrEmpty("queryStr"))
            {
                throw new ArgumentNullException(queryStr);
            }
            QueryType outType;
            if (!SubQueryMap.TryGetValue(queryStr, out outType))
            {
                return QueryType.UnknownSubVerifiedBillQuery;
            }

            return outType;
        }

    }

}
