﻿namespace CCSPrototypeCommon
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Threading.Tasks;

    public class Commons
    {
        class DistinctPaymentsComparer : IEqualityComparer<CCS_Payment_V1>
        {

            public bool Equals(CCS_Payment_V1 x, CCS_Payment_V1 y)
            {
                return x.PaymentID == y.PaymentID;
            }

            public int GetHashCode(CCS_Payment_V1 obj)
            {
                return obj.PaymentID.GetHashCode();
            }
        }

        public static List<String> getAliPayNotifSuccessStatusList()
        {
            List<String> aliPayNotifSuccessStatusList = new List<String>();
            aliPayNotifSuccessStatusList.Add(Constants.AliPayNotifPaymentStatusSuccess);
            return aliPayNotifSuccessStatusList;
        }

        public static List<String> getWeChatNotifSuccessStatusList()
        {
            List<String> weChatNotifSuccessStatusList = new List<String>();
            weChatNotifSuccessStatusList.Add(Constants.WeChatNotifPaymentStatusSuccess);
            return weChatNotifSuccessStatusList;
        }

        public static bool isPaymentSuccessfulBasedOnNotifResult(CCS_Payment_AliPayNotif_V1 p)
        {
            List<String> aliPayNotifSuccessStatusList = getAliPayNotifSuccessStatusList();
            List<String> weChatNotifSuccessStatusList = getWeChatNotifSuccessStatusList();
            return aliPayNotifSuccessStatusList.Contains(p.PaymentTradeStatus) || weChatNotifSuccessStatusList.Contains(p.WeChatResultCode);
        }

        public static List<CCS_Payment_V1> getCCSVendorPaymentsForUserID(BillContext billContext, 
            PaymentContext paymentContext, List<CCS_Staff_V1> staffs, string userInputID,
            DateTime? QueryStartDate = null, DateTime? QueryEndDate = null)
        {
            if (billContext == null
                 || billContext.Database == null
                 || !billContext.Database.Exists()
                 || billContext.BillTable.Count() < 1
                 || staffs == null
                 || staffs.Count < 1
                 || paymentContext == null
                 || paymentContext.Database == null
                 || !paymentContext.Database.Exists()
                 || paymentContext.PaymentTable.Count() < 1)
            {
                return new List<CCS_Payment_V1>();
            }

            List<String> aliPayNotifSuccessStatusList = getAliPayNotifSuccessStatusList();
            List<String> weChatNotifSuccessStatusList = getWeChatNotifSuccessStatusList();

            DateTime start = QueryStartDate ?? DateTime.Now.AddDays(-1);
            DateTime end = QueryEndDate ?? DateTime.Now;
            end = end.AddDays(1);

            List<Guid?> QueryStaffGuids = new List<Guid?>();
            foreach (var s in staffs)
            {
                QueryStaffGuids.Add(s.StaffID);
            }
            
            IQueryable<CCS_Payment_V1> payments = null;

            if (string.IsNullOrWhiteSpace(userInputID))
            {
                if (QueryStartDate == null && QueryStartDate == null)
                {
                    payments = from p in paymentContext.PaymentAliPayNotif
                               where p.PaymentInfo.Payment.StaffID != null
                               && QueryStaffGuids.Contains(p.PaymentInfo.Payment.StaffID)
                               && (aliPayNotifSuccessStatusList.Contains(p.PaymentTradeStatus) || weChatNotifSuccessStatusList.Contains(p.WeChatResultCode))
                               select p.PaymentInfo.Payment;
                }
                else
                {
                    if (QueryEndDate == null)
                    {
                        payments = from p in paymentContext.PaymentAliPayNotif
                                   where p.PaymentInfo.Payment.StaffID != null
                                   && QueryStaffGuids.Contains(p.PaymentInfo.Payment.StaffID)
                                   && (aliPayNotifSuccessStatusList.Contains(p.PaymentTradeStatus) || weChatNotifSuccessStatusList.Contains(p.WeChatResultCode))
                                   && p.PaymentInfo.Payment.PaySubmitTime > start
                                   select p.PaymentInfo.Payment;
                    }
                    else if (QueryStartDate == null)
                    {
                        payments = from p in paymentContext.PaymentAliPayNotif
                                   where p.PaymentInfo.Payment.StaffID != null
                                   && QueryStaffGuids.Contains(p.PaymentInfo.Payment.StaffID)
                                   && (aliPayNotifSuccessStatusList.Contains(p.PaymentTradeStatus) || weChatNotifSuccessStatusList.Contains(p.WeChatResultCode))
                                   && p.PaymentInfo.Payment.PaySubmitTime < end
                                   select p.PaymentInfo.Payment;
                    }
                    else
                    {
                        payments = from p in paymentContext.PaymentAliPayNotif
                                   where p.PaymentInfo.Payment.StaffID != null
                                   && QueryStaffGuids.Contains(p.PaymentInfo.Payment.StaffID)
                                   && (aliPayNotifSuccessStatusList.Contains(p.PaymentTradeStatus) || weChatNotifSuccessStatusList.Contains(p.WeChatResultCode))
                                   && p.PaymentInfo.Payment.PaySubmitTime > start
                                   && p.PaymentInfo.Payment.PaySubmitTime < end
                                   select p.PaymentInfo.Payment;
                    }
                }
            }
            else
            {
                IEnumerable<CCS_User_V1> users = BillCommon.getCCSUserListWithInputID(billContext, userInputID);
                IEnumerable<Guid> QueryGuids = users.Select(x => x.UserID);

                if (QueryStartDate == null && QueryStartDate == null)
                {
                    payments = from p in paymentContext.PaymentAliPayNotif
                               where p.PaymentInfo.Payment.StaffID != null
                               && QueryStaffGuids.Contains(p.PaymentInfo.Payment.StaffID)
                               && QueryGuids.Contains(p.PaymentInfo.Payment.PaymentBill.UserID)
                               && (aliPayNotifSuccessStatusList.Contains(p.PaymentTradeStatus) || weChatNotifSuccessStatusList.Contains(p.WeChatResultCode))
                               select p.PaymentInfo.Payment;
                }
                else
                {
                    if (QueryEndDate == null)
                    {
                        payments = from p in paymentContext.PaymentAliPayNotif
                                   where p.PaymentInfo.Payment.StaffID != null
                                   && QueryStaffGuids.Contains(p.PaymentInfo.Payment.StaffID)
                                   && QueryGuids.Contains(p.PaymentInfo.Payment.PaymentBill.UserID)
                                   && (aliPayNotifSuccessStatusList.Contains(p.PaymentTradeStatus) || weChatNotifSuccessStatusList.Contains(p.WeChatResultCode))
                                   && p.PaymentInfo.Payment.PaySubmitTime > start
                                   select p.PaymentInfo.Payment;
                    }
                    else if (QueryStartDate == null)
                    {
                        payments = from p in paymentContext.PaymentAliPayNotif
                                   where p.PaymentInfo.Payment.StaffID != null
                                   && QueryStaffGuids.Contains(p.PaymentInfo.Payment.StaffID)
                                   && QueryGuids.Contains(p.PaymentInfo.Payment.PaymentBill.UserID)
                                   && (aliPayNotifSuccessStatusList.Contains(p.PaymentTradeStatus) || weChatNotifSuccessStatusList.Contains(p.WeChatResultCode))
                                   && p.PaymentInfo.Payment.PaySubmitTime < end
                                   select p.PaymentInfo.Payment;
                    }
                    else
                    {
                        payments = from p in paymentContext.PaymentAliPayNotif
                                   where p.PaymentInfo.Payment.StaffID != null
                                   && QueryStaffGuids.Contains(p.PaymentInfo.Payment.StaffID)
                                   && QueryGuids.Contains(p.PaymentInfo.Payment.PaymentBill.UserID)
                                   && (aliPayNotifSuccessStatusList.Contains(p.PaymentTradeStatus) || weChatNotifSuccessStatusList.Contains(p.WeChatResultCode))
                                   && p.PaymentInfo.Payment.PaySubmitTime > start
                                   && p.PaymentInfo.Payment.PaySubmitTime < end
                                   select p.PaymentInfo.Payment;
                    }
                }
            }


            return payments.ToList().Distinct(new DistinctPaymentsComparer()).ToList();
        }
    }
}