﻿#region

using System;
using System.Diagnostics;

#endregion

namespace CCSPrototypeCommon
{
    /// <summary>
    ///     Trace based logger
    /// </summary>
    public class Logger
    {
        private string name;
        public string Name {get{return name;}}
        public Logger(String name)
        {
            this.name = name;
        }

        public void logException(Exception e)
        {
            if (e != null && e.GetType().ToString() == "System.Threading.ThreadAbortException")
            {
                return;
            }
            String message = "Exception - " + (e != null ? String.Format("Type {0}, Message {1}, StackTrace {2}", e.GetType(), e.Message, e.StackTrace) : "Undefined");

            if (e != null)
            {
                e = e.InnerException;
                message += " InnerException - " + (e != null ? String.Format("Type {0}, Message {1}, StackTrace {2}", e.GetType(), e.Message, e.StackTrace) : "Undefined");
            }

            var logsContext = (LogsContext)LogsContextFactory.GetInstance().GetDBEFContext();

            if (logsContext == null
               || logsContext.Database == null
               || !logsContext.Database.Exists())
            {
                return;
            }

            logsContext.SaveChanges();
            CCS_Logs_V1 logItem = new CCS_Logs_V1();
            logItem.LogsID = Guid.NewGuid();
            logItem.Type = LogLevel.ERROR;
            logItem.ClassName = this.Name;
            logItem.Message = message;
            logItem.ModifiedDate = DateTime.UtcNow;
            logsContext.LogsTable.Add(logItem);
            logsContext.SaveChangesAsync();

        }

        private void addLog(String message, LogLevel level)
        {
            //Trace.TraceInformation(message, args);
            //Console.WriteLine(message, args);
            var logsContext = (LogsContext)LogsContextFactory.GetInstance().GetDBEFContext();

            if (logsContext == null
               || logsContext.Database == null
               || !logsContext.Database.Exists())
            {
                return;
            }

            logsContext.SaveChanges();
            CCS_Logs_V1 logItem = new CCS_Logs_V1();
            logItem.LogsID = Guid.NewGuid();
            logItem.Type = level;
            logItem.ClassName = this.name;
            logItem.Message = message;
            logItem.ModifiedDate = DateTime.UtcNow;
            logsContext.LogsTable.Add(logItem);
            logsContext.SaveChangesAsync();
        }

        public void Debug(string message, params object[] args)
        {
            addLog(message, LogLevel.DEBUG);
        }

        public void Info(string message, params object[] args)
        {
            addLog(message, LogLevel.INFO);
        }

        public void Warning(string message, params object[] args)
        {
            addLog(message, LogLevel.WARNING);
        }
        public void Error(string message, params object[] args)
        {
            addLog(message, LogLevel.ERROR);
        }

        public void Critical(string message, params object[] args)
        {
            addLog(message, LogLevel.CRITICAL);
        }

    }
}