﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using PushSharp;
using PushSharp.Apple;
using System.Security.Cryptography.X509Certificates;

using PushSharp.Core;
using PushSharp.Windows;

namespace CCSPrototypeCommon
{
    public class PushNotificationsManager
    {
        private enum PushNotificationType
        {
            USERSTAFFFAVORITES  = 1
        };
        private Logger logger = new Logger("PushNotificationsManager"); 
        private static PushNotificationsManager pushNotifManagerInstance;
        ApnsConfiguration apnsconfig = null;
        private PushNotificationsManager() {

            string thumbprint = Constants.PushNotifThumbprint; 
            var store = new X509Store(StoreName.My, StoreLocation.LocalMachine);
            store.Open(OpenFlags.ReadOnly);
            var certificate = store.Certificates.Find(
                X509FindType.FindByThumbprint, thumbprint, validOnly: false)
                .Cast<X509Certificate2>().SingleOrDefault();
            logger.Info("Certificate " + certificate.Thumbprint);

            apnsconfig = new ApnsConfiguration(ApnsConfiguration.ApnsServerEnvironment.Production, certificate);
          

        }

       public static PushNotificationsManager PushNotifManagerInstance
       {
          get 
          {
              if (pushNotifManagerInstance == null)
             {
                 pushNotifManagerInstance = new PushNotificationsManager();
             }
              return pushNotifManagerInstance;
          }
       }

       public String MakePushMessage(String msg)
       {

           String pushMsg = "{\"aps\":{\"alert\":\"" + msg + "\", \"sound\":\"default\", \"badge\":7}, \"pushnotificationtype\":1}"; //"{ \"aps\" : { \"alert\" : {\"message\" : \"" + msg + "\", \"controller\" : \"usermsg\"} } }";
           return pushMsg;
       }

       public void SendPushNotificationToUsers(IQueryable<CCS_User_V1> users, String message)
       {
           try
           {
               if (users == null)
               {

                   logger.Info("Undefined users list.");
                   return;

               }
               String msg = MakePushMessage(message);
               // Create a new broker
               var apnsBroker = new ApnsServiceBroker(apnsconfig);

               // Wire up events
               apnsBroker.OnNotificationFailed += (notification, aggregateEx) =>
               {

                   aggregateEx.Handle(ex =>
                   {

                       // See what kind of exception it was to further diagnose
                       if (ex is ApnsNotificationException)
                       {
                           var notificationException = (ApnsNotificationException)ex;

                           // Deal with the failed notification
                           var apnsNotification = notificationException.Notification;
                           var statusCode = notificationException.ErrorStatusCode;

                           logger.Info("Apple Notification Failed: ID={" + apnsNotification.Identifier + "}, Code={" + statusCode + "}");

                       }
                       else
                       {
                           // Inner exception might hold more useful information like an ApnsConnectionException           
                           logger.Info("Apple Notification Failed for some unknown reason : {" + ex.InnerException + "}");
                       }

                       // Mark it as handled
                       return true;
                   });
               };

               apnsBroker.OnNotificationSucceeded += (notification) =>
               {
                   logger.Info("Apple Notification Sent!");
               };

               // Start the broker
               apnsBroker.Start();

               logger.Info(string.Format("Send to {0} users", users.Count()));
               logger.Info(string.Format("Certificate thumbprint {0} {1}", apnsconfig.Certificate.Thumbprint, apnsconfig.ServerEnvironment));
                foreach (var user in users.ToList())
                {
                    //var user = users.First();
                    //CCS_User_V1 user = userInfo.UserTable;
                    string deviceToken = user.iOSDeviceToken;

                    if (!String.IsNullOrEmpty(deviceToken))
                    {
                        logger.Info(string.Format("Send notification to {0} {1}", deviceToken, msg));
                        apnsBroker.QueueNotification(new ApnsNotification()
                        {
                            DeviceToken = deviceToken,
                            //Payload = Newtonsoft.Json.Linq.JObject.Parse("{\"aps\":{\"alert\":\"8\", \"sound\":\"default\", \"badge\":7}}")
                            Payload = Newtonsoft.Json.Linq.JObject.Parse(msg)
                        });

                    }

                }
                apnsBroker.Stop();
           }
           catch (Exception e)
           {
               logger.logException(e);
           }
       }

       public void SendPushNotificationToUserStaffFavorites(IQueryable<CCS_UserStaffFavorites_V1> usersStaffFavorites, String message)
       {
           try
           {
               if (usersStaffFavorites == null)
               {

                   logger.Info("Underfined usersStaffFavorites list");
                   return;
               }
               String msg = MakePushMessage(message);
               var push = new ApnsServiceBroker(apnsconfig);
               //Wire up the events for all the services that the broker registers
               // Wire up events
                push.OnNotificationFailed += (notification, aggregateEx) => {

                    aggregateEx.Handle (ex => {

                        // See what kind of exception it was to further diagnose
                        if (ex is ApnsNotificationException) {
                            var notificationException = (ApnsNotificationException)ex;

                            // Deal with the failed notification
                            var apnsNotification = notificationException.Notification;
                            var statusCode = notificationException.ErrorStatusCode;

                            logger.Info ("Apple Notification Failed: ID="+apnsNotification.Identifier + "Code=" + statusCode);

                        } else {
                            // Inner exception might hold more useful information like an ApnsConnectionException           
                            logger.Info ("Apple Notification Failed for some unknown reason :" + ex.InnerException);
                        }

                        // Mark it as handled
                        return true;
                    });
                };

                push.OnNotificationSucceeded += (notification) => {
                    logger.Info ("Apple Notification Sent!");
                };

               push.Start();
               logger.Info(string.Format("Send to {0} users", usersStaffFavorites.Count()));
               foreach (var userStaffFavorite in usersStaffFavorites.ToList())
                {
                    //var user = users.First();
                    //CCS_User_V1 user = userInfo.UserTable;
                    string deviceToken = userStaffFavorite.UserTable.iOSDeviceToken;

                    if (!String.IsNullOrEmpty(deviceToken))
                    {
                        logger.Info(string.Format("Send notification to {0}", deviceToken));
                        push.QueueNotification(new ApnsNotification()
                        {
                            DeviceToken = deviceToken,
                            Payload = Newtonsoft.Json.Linq.JObject.Parse(msg)
                        });

                    }

                }

               

               push.Stop();
           }
           catch (Exception e)
           {
               logger.logException(e);
           }

       }


      

    }
}
