﻿USE [cloudcomspace]
GO

/***CCS_User_V1***/
CREATE TABLE [dbo].[CCS_User_V1] (
    [UserID]       UNIQUEIDENTIFIER DEFAULT (newsequentialid()) NOT NULL,
    [FirstName]    NVARCHAR (255)   NULL,
    [LastName]     NVARCHAR (255)   NULL,
    [Cellphone]    NVARCHAR (255)   NULL,
    [SecondPhone]  NVARCHAR (255)   NULL,
    [Email]        NVARCHAR (255)   NULL,
    [Address]      NVARCHAR (255)   NULL,
    [IDNumber]     NVARCHAR (255)   NULL,
    [iOSDeviceToken]      NVARCHAR (255)   NULL,
    [ModifiedDate] DATETIME         NOT NULL,
    CONSTRAINT [PK_dbo.CCS_User_V1] PRIMARY KEY CLUSTERED ([UserID] ASC)
);

/***CCS_UserInfo_V1***/
CREATE TABLE [dbo].[CCS_UserInfo_V1] (
    [UserInfoID] BIGINT           IDENTITY (1, 1) NOT NULL,
    [UserID]     UNIQUEIDENTIFIER NOT NULL,
    [Name]       NVARCHAR (255)   NULL,
    [Value]      NVARCHAR (255)   NULL,
    [Status]     TINYINT          NOT NULL,
    CONSTRAINT [PK_dbo.CCS_UserInfo_V1] PRIMARY KEY CLUSTERED ([UserInfoID] ASC),
    CONSTRAINT [FK_dbo.CCS_UserInfo_V1_dbo.CCS_User_V1_UserID] FOREIGN KEY ([UserID]) REFERENCES [dbo].[CCS_User_V1] ([UserID]) ON DELETE CASCADE
);


GO
CREATE NONCLUSTERED INDEX [IX_UserID]
    ON [dbo].[CCS_UserInfo_V1]([UserID] ASC);


/***CCS_UserPasswords_V1***/
CREATE TABLE [dbo].[CCS_UserPasswords_V1] (
    [UserPasswordID]             UNIQUEIDENTIFIER DEFAULT (newsequentialid()) NOT NULL,
    [MobilePhone]                NVARCHAR (MAX)   NULL,
    [VerificationCode]           NVARCHAR (MAX)   NULL,
    [Password]                   NVARCHAR (MAX)   NULL,
    [VerificationCodeExpiration] DATETIME         NULL
);


GO
CREATE NONCLUSTERED INDEX [IX_UserPasswordID]
    ON [dbo].[CCS_UserPasswords_V1]([UserPasswordID] ASC);


GO
ALTER TABLE [dbo].[CCS_UserPasswords_V1]
    ADD CONSTRAINT [PK_dbo.UserPassword] PRIMARY KEY CLUSTERED ([UserPasswordID] ASC);


/***CCS_Bill_V1***/
CREATE TABLE [dbo].[CCS_Bill_V1] (
    [BillID]        UNIQUEIDENTIFIER DEFAULT (newsequentialid()) NOT NULL,
    [AmountDue]     FLOAT (53)       NOT NULL,
    [AccountNumber] NVARCHAR (255)   NULL,
    [UserNumber]    NVARCHAR (255)   NULL,
    [BillPeriod]    DATE             NOT NULL,
    [BillType]      NVARCHAR (255)   NULL,
    [CustomerID]    UNIQUEIDENTIFIER NOT NULL,
    [UserID]        UNIQUEIDENTIFIER NOT NULL,
    [ModifiedDate]  DATETIME         NOT NULL,
    CONSTRAINT [PK_dbo.CCS_Bill_V1] PRIMARY KEY CLUSTERED ([BillID] ASC),
	CONSTRAINT [FK_dbo.CCS_Bill_V1_dbo.CCS_Customer_V1_CustomerID] FOREIGN KEY ([CustomerID]) REFERENCES [dbo].[CCS_Customer_V1] ([CustomerID]),
    CONSTRAINT [FK_dbo.CCS_Bill_V1_dbo.CCS_User_V1_UserID] FOREIGN KEY ([UserID]) REFERENCES [dbo].[CCS_User_V1] ([UserID]) ON DELETE CASCADE
);


GO
CREATE NONCLUSTERED INDEX [IX_UserID]
    ON [dbo].[CCS_Bill_V1]([UserID] ASC);

/***CCS_BillInfo_V1***/
CREATE TABLE [dbo].[CCS_BillInfo_V1] (
    [BillInfoID] BIGINT           IDENTITY (1, 1) NOT NULL,
    [BillID]     UNIQUEIDENTIFIER NOT NULL,
    [Name]       NVARCHAR (255)   NULL,
    [Value]      NVARCHAR (255)   NULL,
    [Status]     TINYINT          NOT NULL,
    CONSTRAINT [PK_dbo.CCS_BillInfo_V1] PRIMARY KEY CLUSTERED ([BillInfoID] ASC),
    CONSTRAINT [FK_dbo.CCS_BillInfo_V1_dbo.CCS_Bill_V1_BillID] FOREIGN KEY ([BillID]) REFERENCES [dbo].[CCS_Bill_V1] ([BillID]) ON DELETE CASCADE
);

GO
CREATE NONCLUSTERED INDEX [IX_BillID]
    ON [dbo].[CCS_BillInfo_V1]([BillID] ASC);

	
/***CCS_BillTemplate_V1***/
CREATE TABLE [dbo].[CCS_BillTemplate_V1] (
    [BillTemplateID] BIGINT           IDENTITY (1, 1) NOT NULL,
    [CustomerID] UNIQUEIDENTIFIER NOT NULL,
    [BillType]   NVARCHAR (MAX)   NULL,
    [Column]     NVARCHAR (MAX)   NULL,
    [ColumnType] NVARCHAR (MAX)   NULL,
);

GO
CREATE NONCLUSTERED INDEX [IX_BillTemplateID]
    ON [dbo].[CCS_BillTemplate_V1]([BillTemplateID] ASC);

	
/***CCS_RawData_V1***/
CREATE TABLE [dbo].[CCS_RawData_V1] (
	[RawDataID]		UNIQUEIDENTIFIER DEFAULT (newsequentialid()) NOT NULL,
	[MeterDataID]            UNIQUEIDENTIFIER NOT NULL,
	[MeasureType]		NVARCHAR (MAX)   NOT NULL,
    [RateSchedule]		NVARCHAR (MAX)   NULL,
    [ReadType]			NVARCHAR (MAX)   NULL,
	[Read]				FLOAT (53)   NOT NULL,
	[Consumption]		FLOAT (53)   NOT NULL,
	[Status]			NVARCHAR (MAX)   NULL,
    [UpdateTimeStamp]	DateTime	NULL,
	CONSTRAINT [PK_dbo.CCS_RawData_V1] PRIMARY KEY CLUSTERED ([RawDataID] ASC),
	CONSTRAINT [FK_dbo.CCS_RawData_V1_dbo.CCS_MeterData_V1_MeterDataID] FOREIGN KEY ([MeterDataID]) REFERENCES [dbo].[CCS_MeterData_V1] ([MeterDataID]) ON DELETE CASCADE
);
	
/***CCS_Customer_V1***/
CREATE TABLE [dbo].[CCS_Customer_V1] (
    [CustomerID]      UNIQUEIDENTIFIER DEFAULT (newsequentialid()) NOT NULL,
    [Name]            NVARCHAR (255)   NOT NULL,
	[DomainName]      NVARCHAR (255)   NOT NULL,
    [Region]          INT              NOT NULL,
    [State]           TINYINT          NOT NULL,
    [LastUpdatedDate] DATETIME         NOT NULL,
	[CompanyCode]     NVARCHAR (15)    NOT NULL,
    CONSTRAINT [PK_dbo.CCS_Customer_V1] PRIMARY KEY CLUSTERED ([CustomerID] ASC)
);

/***CCS_CustomerInfo_V1***/
CREATE TABLE [dbo].[CCS_CustomerInfo_V1] (
    [CustomerInfoID] BIGINT           IDENTITY (1, 1) NOT NULL,
    [CustomerID]     UNIQUEIDENTIFIER NOT NULL,
    [Name]           NVARCHAR (255)   NULL,
    [Value]          NVARCHAR (255)   NULL,
    [Status]         TINYINT          NOT NULL,
    CONSTRAINT [PK_dbo.CCS_CustomerInfo_V1] PRIMARY KEY CLUSTERED ([CustomerInfoID] ASC),
    CONSTRAINT [FK_dbo.CCS_CustomerInfo_V1_dbo.CCS_Customer_V1_CustomerID] FOREIGN KEY ([CustomerID]) REFERENCES [dbo].[CCS_Customer_V1] ([CustomerID]) ON DELETE CASCADE
);


GO
CREATE NONCLUSTERED INDEX [IX_CustomerID]
    ON [dbo].[CCS_CustomerInfo_V1]([CustomerID] ASC);

/***CCS_Staff_V1***/
CREATE TABLE [dbo].[CCS_Staff_V1] (
    [StaffID]         UNIQUEIDENTIFIER  DEFAULT (newsequentialid()) NOT NULL,
    [CustomerID]      UNIQUEIDENTIFIER  NOT NULL,
    [Name]            NVARCHAR (255)    NOT NULL,
    [Cellphone]       NVARCHAR (255)    NOT NULL,
    [Role]            TINYINT           NULL,
    [SNyunzi]         NVARCHAR (255)    NULL,
    [ADAccount]       NVARCHAR (255)    NULL,
    [ADDomain]        NVARCHAR (255)    NULL,
    [latitude]        FLOAT (53)        NOT NULL,
    [longitude]       FLOAT (53)        NOT NULL,
    [geography]       [sys].[geography] NULL,
    [Address]         NVARCHAR (256)    NULL,
    [State]           TINYINT           NOT NULL,
	[RedirectToUri]   NVARCHAR (255)    NULL,
    [LastUpdatedDate] DATETIME          NOT NULL,
	[ExpirationDate]  DATETIME          NOT NULL,
    CONSTRAINT [PK_dbo.CCS_Staff_V1] PRIMARY KEY CLUSTERED ([StaffID] ASC),
    CONSTRAINT [FK_dbo.CCS_Staff_V1_dbo.CCS_Customer_V1_CustomerID] FOREIGN KEY ([CustomerID]) REFERENCES [dbo].[CCS_Customer_V1] ([CustomerID]) ON DELETE CASCADE
);


GO
CREATE NONCLUSTERED INDEX [IX_CustomerID]
    ON [dbo].[CCS_Staff_V1]([CustomerID] ASC);

/***CCS_Payment_V1***/
CREATE TABLE [dbo].[CCS_Payment_V1] (
    [PaymentID]         UNIQUEIDENTIFIER NOT NULL,
    [PaymentType]       NVARCHAR (MAX)   NULL,
    [PaymentMethod]     INT              NULL,
	[PayerRole]			TINYINT              NULL,
    [PaymentCellNumber] NVARCHAR (MAX)   NULL,
    [PaySubmitTime]     DATETIME         NOT NULL,
    [Amount]            FLOAT (53)       NOT NULL,
    [BillID]            UNIQUEIDENTIFIER NOT NULL,
	[StaffID]            UNIQUEIDENTIFIER  NULL,
    CONSTRAINT [PK_dbo.CCS_Payment_V1] PRIMARY KEY CLUSTERED ([PaymentID] ASC),
    CONSTRAINT [FK_dbo.CCS_Payment_V1_dbo.CCS_Bill_V1_BillID] FOREIGN KEY ([BillID]) REFERENCES [dbo].[CCS_Bill_V1] ([BillID]) ON DELETE CASCADE,
	CONSTRAINT [FK_dbo.CCS_Payment_V1_dbo.CCS_Staff_V1_StaffID] FOREIGN KEY ([StaffID]) REFERENCES [dbo].[CCS_Staff_V1] ([StaffID]) ON DELETE CASCADE
);


GO
CREATE NONCLUSTERED INDEX [IX_BillID]
    ON [dbo].[CCS_Payment_V1]([BillID] ASC);
CREATE NONCLUSTERED INDEX [IX_StaffID]
    ON [dbo].[CCS_Payment_V1]([StaffID] ASC);

/***CCS_PaymentInfo_V1***/
CREATE TABLE [dbo].[CCS_PaymentInfo_V1] (
    [PaymentInfoID]     UNIQUEIDENTIFIER DEFAULT (newsequentialid()) NOT NULL,
    [PaymentID]         UNIQUEIDENTIFIER NOT NULL,
    [PaymentOutTradeNo] NVARCHAR (MAX)   NULL,
    [result]            NVARCHAR (MAX)   NULL,
    CONSTRAINT [PK_dbo.CCS_PaymentInfo_V1] PRIMARY KEY CLUSTERED ([PaymentInfoID] ASC),
    CONSTRAINT [FK_dbo.CCS_PaymentInfo_V1_dbo.CCS_Payment_V1_PaymentID] FOREIGN KEY ([PaymentID]) REFERENCES [dbo].[CCS_Payment_V1] ([PaymentID]) ON DELETE CASCADE
);


GO
CREATE NONCLUSTERED INDEX [IX_PaymentID]
    ON [dbo].[CCS_PaymentInfo_V1]([PaymentID] ASC);



/***CCS_MeterData_V1***/
CREATE TABLE [dbo].[CCS_MeterData_V1] (
    [MeterDataID]  UNIQUEIDENTIFIER DEFAULT (newsequentialid()) NOT NULL,
    [MeterType]    NVARCHAR (MAX) NULL,
    [UserName]    NVARCHAR (MAX) NULL,
    [AccountNumber]  NVARCHAR (MAX) NOT NULL,
    [UserNumber]  NVARCHAR (MAX) NULL,
    [MeterSN]  NVARCHAR (MAX) NULL,
    [MeterID]  NVARCHAR (MAX) NOT NULL,
    [MeterNumber]  NVARCHAR (MAX) NULL,
	[Manufacturer]		NVARCHAR (MAX)   NULL,
    [Address]  NVARCHAR (MAX) NULL,
    [ApartmentNumber]  NVARCHAR (MAX) NULL,
    [UserType]    NVARCHAR (MAX) NULL,
    [Office]    NVARCHAR (MAX) NULL,
    [PhoneNumber]    NVARCHAR (MAX) NULL,
    [Memo1]    NVARCHAR (MAX) NULL,
    [Memo2]    NVARCHAR (MAX) NULL,
    [CutoffDate] DATETIME,
    [CollectType]    NVARCHAR (MAX) NULL,
    [Factor]			FLOAT (53),
    [TerminalNumber]    NVARCHAR (MAX) NULL,
    [TerminalAddress]   NVARCHAR (MAX) NULL,
    [TerminalEquipmentID]   NVARCHAR (MAX) NULL,
    [LocationCode]   NVARCHAR (MAX) NULL,
    [Latitude]   FLOAT (53) NULL,
    [Longitude]  FLOAT (53) NULL,
    [geography]       [sys].[geography] NULL,
    [CreationDate] DATETIME NULL,
    [ModifiedDate] DATETIME NULL,
	[CustomerID] UNIQUEIDENTIFIER NULL,
    CONSTRAINT [PK_dbo.CCS_MeterData_V1] PRIMARY KEY CLUSTERED ([MeterDataID] ASC)
);

/***CCS_MeterData_Sub_V1***/
CREATE TABLE [dbo].[CCS_MeterData_Sub_V1] (
    [MeterDataSubID]    UNIQUEIDENTIFIER DEFAULT (newsequentialid()) NOT NULL,
    [MeterDataID] UNIQUEIDENTIFIER NOT NULL,
    [PictureBlobUri] NVARCHAR (MAX)   NULL,
    CONSTRAINT [PK_dbo.CCS_MeterData_Sub_V1] PRIMARY KEY CLUSTERED ([MeterDataSubID] ASC),
    CONSTRAINT [FK_dbo.CCS_MeterData_Sub_V1_dbo.CCS_MeterData_V1_MeterDataID] FOREIGN KEY ([MeterDataID]) REFERENCES [dbo].[CCS_MeterData_V1] ([MeterDataID]) ON DELETE CASCADE
);


/***CCS_PhoneData_V1***/
CREATE TABLE [dbo].[CCS_PhoneData_V1] (

	[PhoneID]		UNIQUEIDENTIFIER DEFAULT (newsequentialid()) NOT NULL,
    [PhoneUUID] NVARCHAR (20)    NOT NULL,
    [PhoneAccessCode] NVARCHAR (MAX)   NULL,
    [PhoneInfo] NVARCHAR (MAX)   NULL,
    [Memo] NVARCHAR (MAX)   NULL,
    [DisabledDate] DATETIME      NULL,
    [ModifiedDate] DATETIME      NULL,
	[CustomerID]      UNIQUEIDENTIFIER  NULL,
    CONSTRAINT [PK_dbo.CCS_PhoneData_V1] PRIMARY KEY CLUSTERED ([PhoneID] ASC)
);



/***CCS_PhoneMeterRef_V1***/
CREATE TABLE [dbo].[CCS_PhoneMeterRef_V1] (
    [PhoneMeterRefID]    UNIQUEIDENTIFIER DEFAULT (newsequentialid()) NOT NULL,
    [MeterDataID]      UNIQUEIDENTIFIER  NOT NULL,
    [PhoneID]      UNIQUEIDENTIFIER  NOT NULL,
    [ModifiedDate] DATETIME         NOT NULL,
    CONSTRAINT [PK_dbo.CCS_PhoneMeterRef_V1] PRIMARY KEY CLUSTERED 

([PhoneMeterRefID] ASC),

    CONSTRAINT [FK_dbo.CCS_PhoneMeterRef_V1_dbo.CCS_PhoneData_V1_PhoneID] 

FOREIGN KEY ([PhoneID]) REFERENCES [dbo].[CCS_PhoneData_V1] ([PhoneID]) ON DELETE 

CASCADE,
    CONSTRAINT [FK_dbo.CCS_PhoneMeterRef_V1_dbo.CCS_MeterData_V1_MeterDataID] 

FOREIGN KEY ([MeterDataID]) REFERENCES [dbo].[CCS_MeterData_V1] ([MeterDataID]) ON 

DELETE CASCADE
);

GO
CREATE NONCLUSTERED INDEX [IX_PhoneID]
    ON [dbo].[CCS_PhoneMeterRef_V1]([PhoneID] ASC);
CREATE NONCLUSTERED INDEX [IX_MeterDataID]
    ON [dbo].[CCS_PhoneMeterRef_V1]([MeterDataID] ASC);


/***CCS_Payment_AliPayNotif_V1***/
CREATE TABLE [dbo].[CCS_Payment_AliPayNotif_V1] (
    [PaymentNotifID]     UNIQUEIDENTIFIER DEFAULT (newsequentialid()) NOT NULL,
    [PaymentInfoID]         UNIQUEIDENTIFIER NOT NULL,
    [PaymentTradeNo] NVARCHAR (MAX)   NULL,
    [PaymentBuyerEmail]            NVARCHAR (MAX)   NULL,
    [PaymentSubject]            NVARCHAR (MAX)   NULL,
    [PaymentDiscount]       FLOAT (53)        NULL,
    [PaymentType]       INT        NULL,
    [PaymentNotifyType]            NVARCHAR (MAX)   NULL,
    [PaymentQuantity]       INT         NULL,
    [PaymentCreateTime]     DATETIME         NULL,
    [PaymentSellerID]            NVARCHAR (MAX)   NULL,
    [PaymentNotifyTime]     DATETIME         NULL,
    [PaymentBody]            NVARCHAR (MAX)   NULL,
    [PaymentTradeStatus]            NVARCHAR (MAX)   NULL,
    [PaymentIsTotalFeeAdjust]            NVARCHAR (MAX)   NULL,
    [PaymentTotalFee]       FLOAT (53)        NULL,
    [PaymentGMTPayment]     DATETIME         NULL,
    [PaymentSellerEmail]            NVARCHAR (MAX)   NULL,
    [PaymentPrice]       FLOAT (53)        NULL,
    [PaymentNotifyID]     NVARCHAR (MAX)   NULL,
    [PaymentUseCoupon]      NVARCHAR (MAX)   NULL,
    [PaymentSignType]     NVARCHAR (MAX)   NULL,
    [PaymentSign]      NVARCHAR (MAX)   NULL,
    [WeChatAppId]      NVARCHAR (MAX)   NULL,
    [WeChatBankType]      NVARCHAR (MAX)   NULL,
    [WeChatCashFee]      FLOAT (53)   NULL,
    [WeChatFeeType]      NVARCHAR (MAX)   NULL,
    [WeChatIsSubscribe]      NVARCHAR (MAX)   NULL,
    [WeChatMchId]      NVARCHAR (MAX)   NULL,
    [WeChatNonce]      NVARCHAR (MAX)   NULL,
    [WeChatOpenId]      NVARCHAR (MAX)   NULL,
    [WeChatResultCode]      NVARCHAR (MAX)   NULL,
    [WeChatReturnCode]      NVARCHAR (MAX)   NULL,
    [WeChatSign]      NVARCHAR (MAX)   NULL,
    [WeChatTimeEnd]      NVARCHAR (MAX)   NULL,
    [WeChatTotalFee]      FLOAT (53)   NULL,
    [WeChatTradeType]      NVARCHAR (MAX)   NULL,
    [WeChatTransactionId]      NVARCHAR (MAX)   NULL,
	[CurrentTimestamp]		DATETIME	DEFAULT getdate() NOT NULL,					

    CONSTRAINT [PK_dbo.CCS_Payment_AliPayNotif_V1] PRIMARY KEY CLUSTERED ([PaymentNotifID] ASC),
    CONSTRAINT [FK_dbo.CCS_Payment_AliPayNotif_V1_dbo.CCS_Payment_AliPayNotif_V1_PaymentInfoID] FOREIGN KEY ([PaymentInfoID]) REFERENCES [dbo].[CCS_PaymentInfo_V1] ([PaymentInfoID]) ON DELETE CASCADE
);

GO
CREATE NONCLUSTERED INDEX [IX_PaymentInfoID]
    ON [dbo].[CCS_Payment_AliPayNotif_V1]([PaymentInfoID] ASC);


/***CCS_UserStaffFavorites_V1***/
CREATE TABLE [dbo].[CCS_UserStaffFavorites_V1] (
    [UserStaffFavoritesID]         UNIQUEIDENTIFIER  DEFAULT (newsequentialid()) NOT NULL,
    [UserID]      UNIQUEIDENTIFIER  NOT NULL,
    [StaffID]      UNIQUEIDENTIFIER  NOT NULL,
    [FavoriteStatus]            INT    NULL,
    [ModifiedDate] DATETIME         NOT NULL,
    CONSTRAINT [PK_dbo.CCS_UserStaffFavorites_V1] PRIMARY KEY CLUSTERED ([UserStaffFavoritesID] ASC),
    CONSTRAINT [FK_dbo.CCS_UserStaffFavorites_V1_dbo.CCS_Staff_V1_StaffID] FOREIGN KEY ([StaffID]) REFERENCES [dbo].[CCS_Staff_V1] ([StaffID]) ON DELETE CASCADE,
	CONSTRAINT [FK_dbo.CCS_UserStaffFavorites_V1_dbo.CCS_User_V1_UserID] FOREIGN KEY ([UserID]) REFERENCES [dbo].[CCS_User_V1] ([UserID]) ON DELETE CASCADE
);


GO

/***CCS_Payment_Statement_Wechat_V1***/
CREATE TABLE [dbo].[CCS_Payment_Statement_Wechat_V1](
	[StatementID] UNIQUEIDENTIFIER DEFAULT (newsequentialid()) NOT NULL,
	[TradeTime] [nvarchar](max) NULL,
	[WxAppId] [nvarchar](max) NULL,
	[MerchantId] [nvarchar](max) NULL,
	[SubMerchantId] [nvarchar](max) NULL,
	[DeviceId] [nvarchar](max) NULL,
	[WxOrderId] [nvarchar](max) NULL,
	[MerchantOrderId] [nvarchar](max) NULL,
	[UserId] [nvarchar](max) NULL,
	[TransactionType] [nvarchar](max) NULL,
	[TransactionStatus] [nvarchar](max) NULL,
	[PayerBank] [nvarchar](max) NULL,
	[CurrencyType] [nvarchar](max) NULL,
	[TotalPaymentAmount] [float] NOT NULL,
	[TotalCouponPaymentAmount] [float] NOT NULL,
	[ProductName] [nvarchar](max) NULL,
	[ProductDescription] [nvarchar](max) NULL,
	[ProcessingFee] [float] NOT NULL,
	[ProcessingFeeRate] [nvarchar](max) NULL,
	[WxRefundId] [nvarchar](max) NULL,
	[MerchantRefundId] [nvarchar](max) NULL,
	[RefundAmount] [float] NOT NULL,
	[RefundCouponAmount] [float] NOT NULL,
	[RefundType] [nvarchar](max) NULL,
	[RefundStatus] [nvarchar](max) NULL
);

GO

INSERT INTO [dbo].[CCS_Customer_V1] ([CustomerID], [Name], [DomainName], [Region], [State], [LastUpdatedDate]) 
VALUES (N'e1581298-5894-44ee-8fe3-7017d09b206b', N'DefaultWaterCompanyCustomerID', N'infoxus.com', 1, 1, N'2008-11-11 13:23:44')

GO


CREATE VIEW Deduplicated_Payment_AliPay_Notif_V1 AS

select * from (select *, row_number() over(partition by [PaymentInfoID] order by [WeChatTimeEnd] desc) as row from  [CCS_Payment_AliPayNotif_V1] where [WeChatResultCode] = 'SUCCESS' and [WeChatReturnCode] = 'SUCCESS') a where row = 1

GO

CREATE VIEW Payment_Wechat_Statement_Comparison_SUCCESS_V1 AS

SELECT A.[PaymentOutTradeNo], B.*, C.*
  FROM [dbo].[CCS_PaymentInfo_V1] A
  INNER JOIN [dbo].[Deduplicated_Payment_AliPay_Notif_V1] B
  On A.[PaymentInfoId] = B.[PaymentInfoId]
  INNER JOIN [CCS_Payment_Statement_Wechat_V1] C ON
	 A.[PaymentOutTradeNo] = C.[MerchantOrderId] 
	and C.[TotalPaymentAmount] = B.[WeChatTotalFee]
  Where B.[WeChatReturnCode] = 'SUCCESS' 
	and B.[WeChatResultCode] = 'SUCCESS'

GO


CREATE VIEW Payment_Wechat_Statement_Comparison_Fail_Transaction_Match_Amount_Not_Match_V1 AS

SELECT A.[PaymentOutTradeNo], B.*, C.*
  FROM [dbo].[CCS_PaymentInfo_V1] A
  INNER JOIN [dbo].[Deduplicated_Payment_AliPay_Notif_V1] B
  On A.[PaymentInfoId] = B.[PaymentInfoId]
  INNER JOIN [CCS_Payment_Statement_Wechat_V1] C ON
	 A.[PaymentOutTradeNo] = C.[MerchantOrderId] 
	and C.[TotalPaymentAmount] != B.[WeChatTotalFee]
  Where B.[WeChatReturnCode] = 'SUCCESS' 
	and B.[WeChatResultCode] = 'SUCCESS'
GO



CREATE VIEW Payment_Wechat_Statement_Comparison_Fail_Transaction_On_CCS_Only_V1 AS

SELECT A.[PaymentOutTradeNo], B.*, C.*
  FROM [dbo].[CCS_PaymentInfo_V1] A
  INNER JOIN [dbo].[Deduplicated_Payment_AliPay_Notif_V1] B
  
  On A.[PaymentInfoId] = B.[PaymentInfoId]
  LEFT JOIN [CCS_Payment_Statement_Wechat_V1] C ON
	 A.[PaymentOutTradeNo] = C.[MerchantOrderId] 
  Where B.[WeChatReturnCode] = 'SUCCESS' 
	and B.[WeChatResultCode] = 'SUCCESS'
	and C.[MerchantOrderId] is null
GO

CREATE VIEW Payment_Wechat_Statement_Comparison_Fail_Transaction_On_Wechat_Only_V1 AS

SELECT A.[PaymentOutTradeNo], B.*, C.*
  FROM [dbo].[CCS_PaymentInfo_V1] A
  INNER JOIN [dbo].[Deduplicated_Payment_AliPay_Notif_V1] B
  
  On A.[PaymentInfoId] = B.[PaymentInfoId]
  RIGHT JOIN [CCS_Payment_Statement_Wechat_V1] C ON
	 A.[PaymentOutTradeNo] = C.[MerchantOrderId] 
  Where B.[WeChatReturnCode] = 'SUCCESS' 
	and B.[WeChatResultCode] = 'SUCCESS'
	and A.[PaymentOutTradeNo] is null
GO

CREATE TABLE [dbo].[CCS_StaffInfo_V1] (
    [StaffInfoID] BIGINT           IDENTITY (1, 1) NOT NULL,
    [StaffID]     UNIQUEIDENTIFIER NOT NULL,
    [Name]        NVARCHAR (255)   NULL,
    [Content]       NVARCHAR (255)   NULL,
    [Status]      TINYINT          NOT NULL
);


GO
CREATE NONCLUSTERED INDEX [IX_StaffID]
    ON [dbo].[CCS_StaffInfo_V1]([StaffID] ASC);


GO
ALTER TABLE [dbo].[CCS_StaffInfo_V1]
    ADD CONSTRAINT [PK_dbo.CCS_StaffInfo_V1] PRIMARY KEY CLUSTERED ([StaffInfoID] ASC);


GO
ALTER TABLE [dbo].[CCS_StaffInfo_V1]
    ADD CONSTRAINT [FK_dbo.CCS_StaffInfo_V1_dbo.CCS_Staff_V1_StaffID] FOREIGN KEY ([StaffID]) REFERENCES [dbo].[CCS_Staff_V1] ([StaffID]) ON DELETE CASCADE;




/****** Object: Table [dbo].[CCS_UploadedPayment_V1] Script Date: 4/4/2016 11:01:48 PM ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE TABLE [dbo].[CCS_UploadedPayment_V1] (
    [UploadedPaymentID] UNIQUEIDENTIFIER DEFAULT (newsequentialid()) NOT NULL,
    [Amount]            FLOAT (53)       NOT NULL,
    [UserID]            UNIQUEIDENTIFIER NULL,
	[CustomerID]		UNIQUEIDENTIFIER NULL,
    [PaySubmitTime]     DATETIME         NULL,
    [ModifiedDate]      DATETIME         NULL,
	CONSTRAINT [PK_dbo.CCS_UploadedPayment_V1] PRIMARY KEY CLUSTERED ([UploadedPaymentID] ASC),
    CONSTRAINT [FK_dbo.CCS_UploadedPayment_V1_dbo.CCS_User_V1_UserID] FOREIGN KEY ([UserID]) REFERENCES [dbo].[CCS_User_V1] ([UserID]) ON DELETE CASCADE,
	CONSTRAINT [FK_dbo.CCS_UploadedPayment_V1_dbo.CCS_Customer_V1_CustomerID] FOREIGN KEY ([CustomerID]) REFERENCES [dbo].[CCS_Customer_V1] ([CustomerID]) ON DELETE CASCADE
);

CREATE TABLE [dbo].[CCS_MonthlyAmountDueSummary_V1] (
    [Id]           BIGINT           IDENTITY (1, 1) NOT NULL,
    [Balance]      FLOAT (53)       NOT NULL,
    [AmountDue]    FLOAT (53)       NOT NULL,
    [CustomerID]   UNIQUEIDENTIFIER NOT NULL,
    [BillPeriod]   DATE             NOT NULL,
    [ModifiedDate] DATETIME         NOT NULL,
    PRIMARY KEY CLUSTERED ([Id] ASC)
);

CREATE PROCEDURE [dbo].[spMonthlyAmountDue_V1]
AS BEGIN 

    declare @latestDate Date
    declare @lastDate Date
    declare @beginDate Date
    declare @beginMonth Date
    declare @lastDateOfMonth Date
    declare @index Int


    set @latestDate = (select MAX(SnapshotDate) from ccs_daily_bill_with_payment_v1)
    set @lastDate = (select MAX(BillPeriod) from CCS_MonthlyAmountDueSummary_V1)

    if (@latestDate is not null)
    begin
        if (@lastDate is null) set @beginDate = (select MIN(SnapshotDate) from ccs_daily_bill_with_payment_v1)
        else set @beginDate = (select MIN(SnapshotDate) from ccs_daily_bill_with_payment_v1 where SnapshotDate > @lastDate)
        
        if (@beginDate is not null)
        begin
            set @beginMonth = EOMONTH (@beginDate)
            set @lastDateOfMonth = (select MAX(SnapshotDate) from ccs_daily_bill_with_payment_v1 where SnapshotDate <= @beginMonth)
            while (DATEDIFF(month, @lastDateOfMonth, @latestDate) > 0 or DATEDIFF(month, @lastDate, @latestDate) > 0)
            begin
                insert into CCS_MonthlyAmountDueSummary_V1 (Balance, AmountDue, CustomerID, BillPeriod, ModifiedDate)
                select SUM(Balance) as Balance, SUM(AmountDue) as AmountDue, CustomerID, MAX(SnapshotDate) as BillPeriod, GETUTCDATE() as ModifiedDate
                from ccs_daily_bill_with_payment_v1
                where SnapshotDate = @lastDateOfMonth
                group by CustomerID
                
                set @beginMonth = EOMONTH (@beginMonth, 1)
                set @lastDateOfMonth = (select MAX(SnapshotDate) from ccs_daily_bill_with_payment_v1 where SnapshotDate <= @beginMonth)
                set @lastDate = (select MAX(BillPeriod) from CCS_MonthlyAmountDueSummary_V1)
            end
            if (DATEDIFF(month, @latestDate, @lastDate) = 0)
            begin
                update CCS_MonthlyAmountDueSummary_V1
                set Balance = d.Balance, AmountDue = d.AmountDue, BillPeriod = d.BillPeriod, ModifiedDate = d.ModifiedDate
                from (select SUM(Balance) as Balance, SUM(AmountDue) as AmountDue, MAX(SnapshotDate) as BillPeriod, GETUTCDATE() as ModifiedDate 
                    from ccs_daily_bill_with_payment_v1
                    where SnapshotDate = @latestDate
                    group by CustomerID) as d
                where CCS_MonthlyAmountDueSummary_V1.BillPeriod = @lastDate
            end
        end
    end
END



/***CCS_FailedActions_V1***/
CREATE TABLE [dbo].[CCS_FailedActions_V1] (
    [ID]       UNIQUEIDENTIFIER DEFAULT (newsequentialid()) NOT NULL,
    [Parameters]   NVARCHAR (MAX)		 NULL,
    [Message]   NVARCHAR (MAX)		 NULL,
	[ActionType]            INT    NULL,
    [ModifiedDate] DATETIME         NOT NULL,
    CONSTRAINT [PK_dbo.CCS_FailedActions_V1] PRIMARY KEY CLUSTERED ([ID] ASC),
    
);

GO




/********* create table [CCS_SubscriptionPayment_V1] ************/
CREATE TABLE [dbo].[CCS_SubscriptionPayment_V1](
	[PaymentID] [uniqueidentifier] NOT NULL,
	[PaymentCellNumber] [nvarchar](max) NULL,
	[PaySubmitTime] [datetime] NOT NULL,
	[ExtendMonth] [int] NULL,
	[Amount] [float] NOT NULL,
	[StaffID] [uniqueidentifier] NULL,
	[PayerRole] [tinyint] NULL,
 CONSTRAINT [PK_dbo.CCS_SubscriptionPayment_V1] PRIMARY KEY CLUSTERED 
(
	[PaymentID] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON)
)

GO

ALTER TABLE [dbo].[CCS_SubscriptionPayment_V1]  WITH CHECK ADD  CONSTRAINT [FK_dbo.CCS_SubscriptionPayment_V1_dbo.CCS_Staff_V1_StaffID] FOREIGN KEY([StaffID])
REFERENCES [dbo].[CCS_Staff_V1] ([StaffID])
ON DELETE CASCADE
GO

ALTER TABLE [dbo].[CCS_SubscriptionPayment_V1] CHECK CONSTRAINT [FK_dbo.CCS_SubscriptionPayment_V1_dbo.CCS_Staff_V1_StaffID]
GO


/************ create table [CCS_SubscriptionPaymentInfo_V1] **********/
CREATE TABLE [dbo].[CCS_SubscriptionPaymentInfo_V1](
	[PaymentInfoID] [uniqueidentifier] NOT NULL DEFAULT (newsequentialid()),
	[PaymentID] [uniqueidentifier] NOT NULL,
	[PaymentOutTradeNo] [nvarchar](max) NULL,
	[result] [nvarchar](max) NULL,
 CONSTRAINT [PK_dbo.CCS_SubscriptionPaymentInfo_V1] PRIMARY KEY CLUSTERED 
(
	[PaymentInfoID] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON)
)

GO

ALTER TABLE [dbo].[CCS_SubscriptionPaymentInfo_V1]  WITH CHECK ADD  CONSTRAINT [FK_dbo.CCS_SubscriptionPaymentInfo_V1_dbo.CCS_SubscriptionPayment_V1_PaymentID] FOREIGN KEY([PaymentID])
REFERENCES [dbo].[CCS_SubscriptionPayment_V1] ([PaymentID])
ON DELETE CASCADE
GO

ALTER TABLE [dbo].[CCS_SubscriptionPaymentInfo_V1] CHECK CONSTRAINT [FK_dbo.CCS_SubscriptionPaymentInfo_V1_dbo.CCS_SubscriptionPayment_V1_PaymentID]
GO


