﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

using CCSAdmin;
using CCSPrototypeCommon;
using CCSPrototypeCommon.Query;
using HomeWeb.Models;
using System.Data;
using Microsoft.WindowsAzure.Storage.Blob;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.ServiceRuntime;
using System.Globalization;

namespace HomeWeb.Controllers
{
    public class BaseController : Controller
    {

        protected CloudBlobContainer _vendorProfilePictureBlobContainer;

        public BaseController()
        {
            CloudStorageAccount storageAccount = CloudStorageAccount.Parse(ConfigurationMgr.GetSetting("StorageConnectionString"));
            CloudBlobClient blobClient = storageAccount.CreateCloudBlobClient();

            var blobAccess = new BlobContainerPermissions();
            blobAccess.PublicAccess = BlobContainerPublicAccessType.Blob;

            _vendorProfilePictureBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.VendorProfilePictureBlobContainerName);
            //_vendorProfilePictureBlobContainer.CreateIfNotExists();
            _vendorProfilePictureBlobContainer.SetPermissions(blobAccess);


        }


        public static List<CCS_Customer_V1> GetAllCurrentCustomers(HttpSessionStateBase Session, System.Security.Principal.IPrincipal User)
        {

            if (User != null)
            {
                string userId = User.Identity.Name;

                using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                {
                    var record = vendorEFContext.StaffTable.Where(entry => entry.ADAccount == userId 
                        && entry.CustomerTable.State == CustomerAccountState.Active);
                    if (record != null && record.Count() > 0)
                    {
                        return record.Select(r => r.CustomerTable).ToList();
                    }
                }
            }

            return new List<CCS_Customer_V1>();
        }

        public static string GetCurrentCustomerID(HttpSessionStateBase Session, System.Security.Principal.IPrincipal User)
        {
            var ob = Session[SessionConstants.CustomerID];
            if (ob != null)
                return ob.ToString();

            string customerID = string.Empty;
            string customerName = string.Empty;
            string companyLogo = string.Empty;
            if (User != null)
            {
                string userId = User.Identity.Name;

                using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                {
                    var record = vendorEFContext.StaffTable.Where(entry => entry.ADAccount == userId 
                        && entry.State == CustomerAccountState.Active
                        && entry.CustomerTable.State == CustomerAccountState.Active).OrderByDescending(entry => entry.LastUpdatedDate).FirstOrDefault();
                    if (record != null)
                    {
                        customerID = record.CustomerID.ToString();
                    }
                }
            }

            if (!String.IsNullOrEmpty(customerID))
            {
                Session[SessionConstants.CustomerID] = customerID;

                if (!String.IsNullOrEmpty(customerName))
                {
                    Session[SessionConstants.CustomerName] = customerName;
                }

                if (!String.IsNullOrEmpty(companyLogo))
                {
                    Session[SessionConstants.CompanyLogo] = companyLogo;
                }
            }
            return customerID;
        }

        // return the corresponding customer ID (aka Company ID) from a logged user
        protected string GetCurrentCustomerGuidString(Guid? DefaultGuid = null)
        {
            string customerID = GetCurrentCustomerID(Session, User);
            if (string.IsNullOrEmpty(customerID))
            {
                if (DefaultGuid == null)
                {
                    throw new Exception(ErrorConstants.NoAssociatedCompany);
                }
                else
                {
                    return (DefaultGuid ?? Guid.Empty).ToString();
                }
            }
            else
            {
                return customerID;
            }
        }

        protected Guid GetCurrentCustomerGuid(Guid? DefaultGuid = null)
        {
            string customerID = GetCurrentCustomerGuidString(DefaultGuid);
            Guid CustomerGuid;
            if (!Guid.TryParse(customerID, out CustomerGuid))
            {
                if (DefaultGuid == null)
                {
                    throw new Exception(ErrorConstants.NoAssociatedCompany);
                }
                else
                {
                    return DefaultGuid ?? Guid.Empty;
                }
            }
            else
            {
                return CustomerGuid;
            }
        }

        protected CCS_Staff_V1 GetCurrentStaff(string CustomerID = null)
        {
            Guid CustomerGuid = Guid.Empty;
            CCS_Staff_V1 staff = null;
            
            try
            {
                var adAccount = User.Identity.Name;


                using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                {
                    if (!string.IsNullOrWhiteSpace(CustomerID) && Guid.TryParse(CustomerID, out CustomerGuid))
                    {
                        staff = vendorEFContext.StaffTable.Where(entry => entry.ADAccount == adAccount
                            && entry.CustomerID == CustomerGuid 
                            && entry.State == CustomerAccountState.Active).FirstOrDefault();

                    }
                    else
                    {
                        staff = vendorEFContext.StaffTable.Where(entry => entry.ADAccount == adAccount
                            && entry.State == CustomerAccountState.Active).OrderByDescending(entry => entry.LastUpdatedDate).FirstOrDefault();
                    }

                    if (staff != null)
                    {
                        if (vendorEFContext.Entry(staff).Reference(s => s.CustomerTable).IsLoaded == false)
                        {
                            vendorEFContext.Entry(staff).Reference(s => s.CustomerTable).Load();
                        }
                        return staff;
                    }
                }
            }
            catch (DataException)
            {
                return null;
            }
            return null;
        }

        protected List<CCS_Staff_V1> GetCurrentStaffs()
        {
            try
            {
                var adAccount = User.Identity.Name;
                using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                {
                    var staffs = vendorEFContext.StaffTable.Where(entry => entry.ADAccount == adAccount
                        && entry.State == CustomerAccountState.Active);
                    if (staffs != null && staffs.Count() > 0)
                    {

                        var result = staffs.ToList();
                        foreach (var staff in result)
                        {
                            if (vendorEFContext.Entry(staff).Reference(s => s.CustomerTable).IsLoaded == false)
                            {
                                vendorEFContext.Entry(staff).Reference(s => s.CustomerTable).Load();
                            }
                        }

                        return result;
                    }
                }
            }
            catch (DataException)
            {
                ;
            }
            return new List<CCS_Staff_V1>();
        }

        protected CCS_Staff_V1 GetStaffByGuid(Guid guid)
        {
            try
            {
                using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                {
                    var record = vendorEFContext.StaffTable.Where(entry => entry.StaffID == guid).FirstOrDefault();
                    if (record != null)
                    {
                        if (vendorEFContext.Entry(record).Reference(s => s.CustomerTable).IsLoaded == false)
                        {
                            vendorEFContext.Entry(record).Reference(s => s.CustomerTable).Load();
                        }
                        return record;
                    }
                }
            }
            catch (DataException)
            {
                return null;
            }
            return null;
        }
        protected List<CCS_StaffInfo_V1> GetStaffInfosByGuid(Guid guid)
        {
            List<CCS_StaffInfo_V1> list = new List<CCS_StaffInfo_V1>();
            try
            {
                using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                {
                    var record = vendorEFContext.StaffInfoTable.Where(entry => entry.StaffID == guid);
                    if (record != null)
                    {
                        list = record.ToList();
                    }
                }
            }
            catch (DataException)
            {
                ;
            }
            return list;
        }
        protected bool IsVendorExpired(string CustomerID, out Guid StaffID)
        {
            StaffID = Guid.Empty;

            if(string.IsNullOrWhiteSpace(CustomerID))
            {
                CustomerID = Session[CCSPrototypeCommon.SessionConstants.CustomerID] as string;
            }

            var currentUser = GetCurrentStaff(CustomerID);
            if (currentUser == null)
            {
                throw new Exception("当前账户无权限进行此项操作，请使用其他账户登录。");
            }

            StaffID = currentUser.StaffID;

            if (currentUser.Role != OperationLevel.Vendor)
            {
                return false;
            }
            if (currentUser.ExpirationDate < DateTime.UtcNow)
            {
                return true;
            }
            return false;
        }

        protected string GetProfilePictureUri(string staffID)
        {
            string blobPrefix = staffID + '/';
            var blobList = _vendorProfilePictureBlobContainer.ListBlobs(blobPrefix, true);
            var dateTimeFormat = "yyyyMMddHHmmss";
            var uri = "";
            DateTime latestDate = new DateTime();
            foreach (var blob in blobList)
            {
                DateTime uploadDate;
                if (!DateTime.TryParseExact(blob.Parent.Uri.MakeRelativeUri(blob.Uri).ToString(),
                    dateTimeFormat, CultureInfo.InvariantCulture, DateTimeStyles.None, out uploadDate))
                {
                    if (string.IsNullOrWhiteSpace(uri))
                    {
                        uri = blob.Uri.AbsoluteUri;
                    }
                }
                else if (latestDate < uploadDate)
                {
                    latestDate = uploadDate;
                    uri = blob.Uri.AbsoluteUri;
                }
            }
            return uri;
        }

        public static GeoRegion GetRegion(string region)
        {
            if (string.IsNullOrEmpty(region))
            {
                return GeoRegion.Beijing;
            }
            switch (region.ToLower())
            {
                case "北京":
                case "beijing":
                    return GeoRegion.Beijing;
                case "上海":
                case "shanghai":
                    return GeoRegion.Shanghai;
                case "天津":
                case "tianjin":
                    return GeoRegion.Tianjin;
                case "重庆":
                case "chongqing":
                    return GeoRegion.Chongqing;
                case "湖北":
                case "hubei":
                    return GeoRegion.Hubei;
                default:
                    return GeoRegion.Beijing;
            }
        }

        protected Guid GetUserGuidFromSession()
        {
            Guid? UserGuid = Session[SessionConstants.UserLoginGuid] as Guid?;
            if(UserGuid != null)
            {
                return (UserGuid ?? Guid.Empty);
            }

            string mobile = Session[SessionConstants.UserLoginMobileNumber] as string;

            if (UserPasswordContext.GetInstance().isUserAuthenticated(Session, Request)
                && !string.IsNullOrWhiteSpace(mobile))
            {
                BillContext DbContext = (BillContext)BillContextFactory.GetInstance().GetDBEFContext();

                if (DbContext != null)
                {
                    CCS_User_V1 user = BillCommon.getCCSUserListWithInputID(DbContext, mobile).FirstOrDefault();
                    if (user != null)
                    {
                        Session[SessionConstants.UserLoginGuid] = user.UserID;
                        return user.UserID;
                    }
                }

            }


            return Guid.Empty;
        }
    }
}