﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Data.Entity.Spatial;

using Newtonsoft.Json;
using CCSAdmin;
using CCSPrototypeCommon;
using HomeWeb.Models;
using System.Data.Entity.Validation;
using Microsoft.WindowsAzure.Storage.Blob;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.Storage.RetryPolicies;
using System.Threading.Tasks;

namespace HomeWeb.Controllers
{
    public class MapController : BaseController
    {
        public static Logger logger
        {
            get
            {
                return new Logger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name);
            }

        }

        private ActionsContext _actionsDB = (ActionsContext)ActionsContextFactory.GetInstance().GetDBEFContext();

        public const int MaxCountOfFavouriteVendors = 2000;

        //private static BillContext _billDB = (BillContext)BillContextFactory.GetInstance().GetDBEFContext();

        // GET: Map
        public ActionResult Index()
        {
            return RedirectToAction("ShowMap");
        }

        //YUZI delimiters
        public static char[] YunziInputDelims = new char[] { ',', '|', ';' };

        //public ActionResult ShowMapMerchantVendor(double? CenterLatitude, double? CenterLongitude, string YunziIDs,
        //    int? StartIndex, int? Count, string ShowType, string Market, string UserId, bool? NavigateOnLoad)
        //{
        //    return ShowMap(CenterLatitude, CenterLongitude, YunziIDs,
        //        StartIndex, Count, ShowType, Market, "merchantvendor", UserId, NavigateOnLoad ?? false);
        //}

        public ActionResult ShowMapUserFavorites(double? CenterLatitude, double? CenterLongitude, string YunziIDs, string Guids,
            int? StartIndex, int? Count, string ShowType, string Market, string MapStyle,
            bool? FullScreen, bool? NavigateOnLoad, bool? FetchMoreDataAfterCenterChange)
        {
            if(UserPasswordContext.GetInstance().isUserAuthenticated(Session, Request))
            {
                Guid? UserGuid = Session[SessionConstants.UserLoginGuid] as Guid?;
                if (UserGuid != null && UserGuid != Guid.Empty)
                {

                    int fetchCount = BillCommon.GetFetchCount(Count);
                    SetViewBagCommonValues(CenterLatitude, CenterLongitude, YunziIDs, Guids, StartIndex, ShowType, Market, null,
                        FullScreen ?? false , NavigateOnLoad, fetchCount, FetchMoreDataAfterCenterChange);

                    //List<VendorModel> list = FetchFavouriteVendorList(MobilePhone, StartIndex, Count);


                    return View();
                }
            }
            return RedirectToAction("Index", "UserModelHome");
        }

        public ActionResult ShowUserFavoritesMessages()
        {
            if (UserPasswordContext.GetInstance().isUserAuthenticated(Session, Request))
            {
                Guid? UserGuid = Session[SessionConstants.UserLoginGuid] as Guid?;
                if (UserGuid != null && UserGuid != Guid.Empty)
                {

                    
                    return View();
                }
            }
            return RedirectToAction("Index", "UserModelHome");
        }

        public ActionResult ShowMapMerchantVendor(double? CenterLatitude, double? CenterLongitude, string YunziIDs, string Guids,
            int? StartIndex, int? Count, string ShowType, string Market, string MapStyle, string UserId,
            bool? FullScreen, bool? NavigateOnLoad, bool? FetchMoreDataAfterCenterChange)
        {
            if (!string.IsNullOrEmpty(MapStyle))
            {
                if (MapStyle.ToLower().Contains("merchantfieldstaff"))
                {
                    return RedirectToAction("ShowMapMerchantFieldStaff",
                        new
                        {
                            CenterLatitude = CenterLatitude,
                            CenterLongitude = CenterLongitude,
                            YunziIDs = YunziIDs,
                            Guids = Guids,
                            StartIndex = StartIndex,
                            Count = Count,
                            ShowType = ShowType,
                            Market = Market,
                            MapStyle = MapStyle,
                            UserId = UserId,
                            FullScreen = FullScreen,
                            NavigateOnLoad = NavigateOnLoad,
                            FetchMoreDataAfterCenterChange = FetchMoreDataAfterCenterChange
                        });
                }
                else if (MapStyle.ToLower().Contains("merchantmeter"))
                {
                    return RedirectToAction("ShowMapMerchantMeter",
                        new
                        {
                            CenterLatitude = CenterLatitude,
                            CenterLongitude = CenterLongitude,
                            YunziIDs = YunziIDs,
                            Guids = Guids,
                            StartIndex = StartIndex,
                            Count = Count,
                            ShowType = ShowType,
                            Market = Market,
                            MapStyle = MapStyle,
                            UserId = UserId,
                            FullScreen = FullScreen,
                            NavigateOnLoad = NavigateOnLoad,
                            FetchMoreDataAfterCenterChange = FetchMoreDataAfterCenterChange
                        });
                }
            }

            int FetchCount = BillCommon.GetFetchCount(Count);
            SetViewBagCommonValues(CenterLatitude, CenterLongitude, YunziIDs, Guids, StartIndex, ShowType,
                Market, UserId, FullScreen ?? false, NavigateOnLoad, FetchCount, FetchMoreDataAfterCenterChange);
            ViewBag.MapStyle = string.IsNullOrEmpty(MapStyle) ? "merchantvendor" : MapStyle;

            //List<VendorModel> StaffList = GetStaffList(CenterLatitude, CenterLongitude, YunziIDs, Guids, StartIndex ?? 0, FetchCount);
            FillInAllVendorGeo();

            return View();
        }


        public ActionResult ShowMapMerchantFieldStaff(double? CenterLatitude, double? CenterLongitude, string YunziIDs, string Guids,
            int? StartIndex, int? Count, string ShowType, string Market, string MapStyle, string UserId,
            bool? FullScreen, bool? NavigateOnLoad, bool? FetchMoreDataAfterCenterChange)
        {
   
            if (!string.IsNullOrEmpty(MapStyle))
            {
                if (MapStyle.ToLower().Contains("merchantmeter"))
                {
                    return RedirectToAction("ShowMapMerchantMeter",
                        new
                        {
                            CenterLatitude = CenterLatitude,
                            CenterLongitude = CenterLongitude,
                            YunziIDs = YunziIDs,
                            Guids = Guids,
                            StartIndex = StartIndex,
                            Count = Count,
                            ShowType = ShowType,
                            Market = Market,
                            MapStyle = MapStyle,
                            UserId = UserId,
                            FullScreen = FullScreen,
                            NavigateOnLoad = NavigateOnLoad,
                            FetchMoreDataAfterCenterChange = FetchMoreDataAfterCenterChange
                        });
                }
                else if (MapStyle.ToLower().Contains("merchantvendor"))
                {
                    return RedirectToAction("ShowMapMerchantVendor",
                        new
                        {
                            CenterLatitude = CenterLatitude,
                            CenterLongitude = CenterLongitude,
                            YunziIDs = YunziIDs,
                            Guids = Guids,
                            StartIndex = StartIndex,
                            Count = Count,
                            ShowType = ShowType,
                            Market = Market,
                            MapStyle = MapStyle,
                            UserId = UserId,
                            FullScreen = FullScreen,
                            NavigateOnLoad = NavigateOnLoad,
                            FetchMoreDataAfterCenterChange = FetchMoreDataAfterCenterChange
                        });
                }

            }


            int FetchCount = BillCommon.GetFetchCount(Count);

            // TO-DO select only Field Staffs only
            List<VendorModel> StaffList = GetStaffList(CenterLatitude, CenterLongitude, YunziIDs, Guids, StartIndex ?? 0, FetchCount);

            SetViewBagCommonValues(CenterLatitude, CenterLongitude, YunziIDs, Guids, StartIndex, ShowType,
                Market, UserId, FullScreen ?? false, NavigateOnLoad, FetchCount, FetchMoreDataAfterCenterChange);
            ViewBag.MapStyle = string.IsNullOrEmpty(MapStyle) ? "merchantfieldstaff" : MapStyle;


            return View("ShowMapMerchantVendor", StaffList);

        }

        private List<VendorModel> GetStaffList(double? CenterLatitude, double? CenterLongitude, string YunziIDs, string Guids, int StartIndex, int FetchCount)
        {
            List<VendorModel> StaffList = null;

            if (string.IsNullOrEmpty(YunziIDs) && string.IsNullOrEmpty(Guids))
            {
                // No center, nowhere to fetch data from
                if (CenterLatitude == null || CenterLongitude == null)
                {
                    StaffList = new List<VendorModel>();
                }
                // Has center, and Yunzi ID is not provided
                else
                {
                    StaffList = FetchNearestLocations((double)CenterLatitude, (double)CenterLongitude, FetchCount);
                }
            }
            else
            {
                int YunziCount;
                StaffList = FetchYunziLocations(YunziIDs, StartIndex, FetchCount, out YunziCount);


                int NextStartIndex, NextFetchCount;
                RecalculateIndexAndCount(StaffList, YunziCount, StartIndex, FetchCount, out NextStartIndex, out NextFetchCount);

                int GuidCountPassedIn;
                StaffList.AddRange(FetchGuidLocations(Guids, NextStartIndex, NextFetchCount, out GuidCountPassedIn));
            }
            return StaffList;
        }

        private void RecalculateIndexAndCount(List<VendorModel> StaffList, int YunziCount, int StartIndex, int FetchCount,
            out int NextStartIndex, out int NextFetchCount)
        {
            NextStartIndex = 0;
            NextFetchCount = 0;

            if (StaffList == null || StaffList.Count < 1)
            {
                NextStartIndex = StartIndex - YunziCount;
                NextFetchCount = FetchCount; 
            }
            else if (StaffList.Count >= FetchCount)
            {
                NextStartIndex = 0;
                NextFetchCount = 0;
            }
            else
            {
                NextStartIndex = 0;
                NextFetchCount = FetchCount - StaffList.Count;             
            }

        }

        private void SetViewBagCommonValues(double? CenterLatitude, double? CenterLongitude, string YunziIDs, string Guids,
            int? StartIndex, string ShowType, string Market, string UserId, bool? FullScreen, bool? NavigateOnLoad, int FetchCount,
            bool? FetchMoreDataAfterCenterChangeFlag)
        {
            ViewBag.ShowType = ShowType;
            ViewBag.YunziIDs = YunziIDs;
            ViewBag.Guids = Guids;
            ViewBag.StartIndex = StartIndex;

            ViewBag.FullScreen = FullScreen;
            ViewBag.NavigateOnLoad = NavigateOnLoad ?? true;
            ViewBag.BingMapCred = ConfigurationMgr.GetSetting("BingMapCredentialKey"); ;
            ViewBag.FetchCount = FetchCount;
            
            if(string.IsNullOrEmpty(UserId))
            {
                Guid UserGuid = GetUserGuidFromSession();
                ViewBag.UserId = (UserGuid == Guid.Empty) ? String.Empty : UserGuid.ToString();
            }
            else
            {
                ViewBag.UserId = UserId;
            }

            if (CenterLatitude == null || CenterLatitude == 0)
            {
                ViewBag.CenterLatitude = null;
            }
            else
            {
                ViewBag.CenterLatitude = CenterLatitude;
            }

            if (CenterLongitude == null || CenterLongitude == 0)
            {
                ViewBag.CenterLongitude = null;
            }
            else
            {
                ViewBag.CenterLongitude = CenterLongitude;
            }

            ViewBag.Market = GetMarketBasedOnCoordinates(Market, CenterLatitude, CenterLongitude);
            ViewBag.FetchMoreDataAfterCenterChange = FetchMoreDataAfterCenterChangeFlag;
        }



        public ActionResult ShowMapWhenShake(double? CenterLatitude, double? CenterLongitude, string YunziIDs, string Guids,
            int? StartIndex, int? Count, string ShowType, string Market, string MapStyle,
            bool? FullScreen, bool? NavigateOnLoad, bool? FetchMoreDataAfterCenterChange)
        {
            int fetchCount = BillCommon.GetFetchCount(Count);
            SetViewBagCommonValues(CenterLatitude, CenterLongitude, YunziIDs, Guids, StartIndex, ShowType, Market, null, 
                FullScreen ?? false, NavigateOnLoad, fetchCount, FetchMoreDataAfterCenterChange);

            //List<VendorModel> StaffList = GetStaffList(CenterLatitude, CenterLongitude, YunziIDs, StartIndex, fetchCount);

            return View();


        }
        public ActionResult ReadyForShake(bool? ShowLoginBtn, bool? ShowBackBtn)
        {
            ViewBag.ShowLoginBtn = ShowLoginBtn ?? true;
            ViewBag.ShowBackBtn = ShowBackBtn ?? false;
            return View();
        }

        public ActionResult ShowMapMobile(double? CenterLatitude, double? CenterLongitude, string YunziIDs, string ShowType, string UserId)
        {
            Utility.LatLngModel latLngBaidu = new Utility.LatLngModel(CenterLatitude, CenterLongitude);
            Utility.LatLngModel latLng = Utility.Bd2Gcj(CenterLatitude, CenterLongitude);
            CenterLatitude = latLng.lat;
            CenterLongitude = latLng.lng;

            int FetchCount = BillCommon.GetFetchCount(null);
          
            //List<VendorModel> StaffList = GetStaffList(CenterLatitude, CenterLongitude, YunziIDs, Guids, StartIndex ?? 0, FetchCount);

            string targetMarket = GetMarketBasedOnCoordinates(null, CenterLatitude, CenterLongitude);

            SetViewBagCommonValues(CenterLatitude, CenterLongitude, YunziIDs, null, null, ShowType,
                null, UserId,  true, null, FetchCount, null);

            if ("zh-cn".Equals(targetMarket, StringComparison.OrdinalIgnoreCase))
            {
                //this is the only place to set Baidu coordinates
                //no need to do it in ShowMap since it won't be called from the mobile side
                SetViewBagCoordinates(latLngBaidu.lat, latLngBaidu.lng);
                return View("ShowMap_Baidu");
            }
            else
            {
                return View("ShowMap");
            }
        }

        // GET: VendorLocations
        // Will use YunziID to fetch the Vendors assigned with Yunzi.
        // If Guid is also provided, will append the Vendors associated with StaffTable.StaffID in Guids
        public ActionResult ShowMap(double? CenterLatitude, double? CenterLongitude, string YunziIDs, string Guids,
            int? StartIndex, int? Count, string ShowType, string Market, string MapStyle, string UserId, 
            bool? FullScreen, bool? NavigateOnLoad, bool? FetchMoreDataAfterCenterChange)
            {
            if (!string.IsNullOrEmpty(MapStyle))
            {
                if (MapStyle.ToLower().Contains("merchantfieldstaff"))
                {
                    return RedirectToAction("ShowMapMerchantFieldStaff",
                        new
                        {
                            CenterLatitude = CenterLatitude,
                            CenterLongitude = CenterLongitude,
                            YunziIDs = YunziIDs,
                            Guids = Guids,
                            StartIndex = StartIndex,
                            Count = Count,
                            ShowType = ShowType,
                            Market = Market,
                            MapStyle = MapStyle,
                            UserId = UserId,
                            FullScreen = FullScreen,
                            NavigateOnLoad = NavigateOnLoad,
                            FetchMoreDataAfterCenterChange = FetchMoreDataAfterCenterChange
                        });
                }
                else if (MapStyle.ToLower().Contains("merchantmeter"))
                {
                    return RedirectToAction("ShowMapMerchantMeter",
                        new
                        {
                            CenterLatitude = CenterLatitude,
                            CenterLongitude = CenterLongitude,
                            YunziIDs = YunziIDs,
                            Guids = Guids,
                            StartIndex = StartIndex,
                            Count = Count,
                            ShowType = ShowType,
                            Market = Market,
                            MapStyle = MapStyle,
                            UserId = UserId,
                            FullScreen = FullScreen,
                            NavigateOnLoad = NavigateOnLoad,
                            FetchMoreDataAfterCenterChange = FetchMoreDataAfterCenterChange
                        });
                }
                else if (MapStyle.ToLower().Contains("merchant"))
                {
                    return RedirectToAction("ShowMapMerchantVendor",
                        new
                {
                            CenterLatitude = CenterLatitude,
                            CenterLongitude = CenterLongitude,
                            YunziIDs = YunziIDs,
                            Guids = Guids,
                            StartIndex = StartIndex,
                            Count = Count,
                            ShowType = ShowType,
                            Market = Market,
                            MapStyle = MapStyle,
                            UserId = UserId,
                            FullScreen = FullScreen,
                            NavigateOnLoad = NavigateOnLoad,
                            FetchMoreDataAfterCenterChange = FetchMoreDataAfterCenterChange
                        });
                }
            }

            int FetchCount = BillCommon.GetFetchCount(Count);
            //List<VendorModel> StaffList = GetStaffList(CenterLatitude, CenterLongitude, YunziIDs, Guids, StartIndex ?? 0, FetchCount);

            string targetMarket = GetMarketBasedOnCoordinates(Market, CenterLatitude, CenterLongitude);

            SetViewBagCommonValues(CenterLatitude, CenterLongitude, YunziIDs, Guids, StartIndex, ShowType,
                Market, UserId, FullScreen ?? true, NavigateOnLoad, FetchCount, FetchMoreDataAfterCenterChange);

            if ("zh-cn".Equals(targetMarket, StringComparison.OrdinalIgnoreCase))
            {
                return View("ShowMap_Baidu");
            }
            else
            {
                return View();
            }
        }

        public void SetViewBagCoordinates(double? CenterLatitude, double? CenterLongitude)
        {
            if (CenterLatitude == null || CenterLatitude == 0)
            {
                ViewBag.CenterLatitude = null;
            }
            else
            {
                ViewBag.CenterLatitude = CenterLatitude;
            }

            if (CenterLongitude == null || CenterLongitude == 0)
            {
                ViewBag.CenterLongitude = null;
            }
            else
            {
                ViewBag.CenterLongitude = CenterLongitude;
            }

        }
        
        [HttpPost]
        public JsonResult AddVendorToFavorites(string userid, string staffid)
        {
            int actionStatusCode;
            String actionStatusMsg;
            Guid UserGuid = Guid.Empty;
            Guid StaffGuid = Guid.Empty;

            try
            {
                using (var vendorEFContext = (BillContext)BillContextFactory.GetInstance().GetDBEFContext())
                {

                    if (string.IsNullOrEmpty(userid))
                    {
                        UserGuid = GetUserGuidFromSession();
                    }
                    else if (Guid.TryParse(userid, out UserGuid))
                    {
                        ;
                    }
                    else
                    {
                        UserGuid = Guid.Empty;
                    }
                    
                    if (Guid.Empty == UserGuid)
                    {
                        return new JsonResult() { Data = new { statuscode = 0, statusmsg = "用户当前登录已过期或无效，请重新登录。" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                    }

                    if (!Guid.TryParse(staffid, out StaffGuid)
                        || Guid.Empty == StaffGuid)
                    {
                        return new JsonResult() { Data = new { statuscode = 0, statusmsg = "所选择目标无效，请刷新页面重试。" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                    }

                    var usersInfo = vendorEFContext.UserStaffFavoritesTable.Where(o => o.UserID == UserGuid && o.StaffID == StaffGuid && o.ModifiedDate == null).ToList();
                    if (usersInfo.Count() > 1)
                    {
                        return new JsonResult() { Data = new { statuscode = 0, statusmsg = "More than one record with the specified (user id, staff id) is found." }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                    }

                    foreach (var userInfo in usersInfo)
                    {
                        userInfo.ModifiedDate = DateTime.UtcNow;
                    }
                    CCS_UserStaffFavorites_V1 usersFavorites = new CCS_UserStaffFavorites_V1();
                    usersFavorites.UserID = UserGuid;
                    usersFavorites.StaffID = StaffGuid;
                    usersFavorites.ModifiedDate = null;
                    usersFavorites.FavoriteStatus = FavoriteStatus.Favorite;

                    vendorEFContext.UserStaffFavoritesTable.Add(usersFavorites);
                    vendorEFContext.SaveChanges();

                    actionStatusCode = 1;
                    actionStatusMsg = "添加收藏成功。";
                }
                
            }
            catch (Exception e)
            {
                if(e.InnerException != null){
                    actionStatusCode = 0;
                    actionStatusMsg =  "InnerException: " + e.InnerException.Message;
                }
                else
                {
                    actionStatusCode = 0;
                    actionStatusMsg =  "Exception: " + e.Message;
                }
            }

            CCS_FailedActions_V1 action = new CCS_FailedActions_V1();
            action.ActionType = ACTIONSTYPE.ADDTOFAVORITES;
            action.Message = actionStatusMsg;
            action.Parameters = string.Format(Constants.actionTypeToParametersFormatMap[ACTIONSTYPE.ADDTOFAVORITES], userid, staffid);
            action.ModifiedDate = DateTime.UtcNow;

            _actionsDB.FailedActionsTable.Add(action);
            _actionsDB.SaveChanges();

            return new JsonResult() { Data = new { statuscodeF = actionStatusCode, statusmsg = actionStatusMsg }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
            
        }

        [HttpPost]
        public JsonResult RemoveVendorFromFavorites(string userid, string staffid)
        {
            int actionStatusCode;
            String actionStatusMsg;
            Guid UserGuid = Guid.Empty;
            Guid StaffGuid = Guid.Empty;

            try
            {
                using (var vendorEFContext = (BillContext)BillContextFactory.GetInstance().GetDBEFContext())
                {

                    if (string.IsNullOrEmpty(userid))
                    {
                        UserGuid = GetUserGuidFromSession();
                    }
                    else if (Guid.TryParse(userid, out UserGuid))
                    {
                        ;
                    }
                    else
                    {
                        UserGuid = Guid.Empty;
                    }

                    if (Guid.Empty == UserGuid)
                    {
                        return new JsonResult() { Data = new { statuscode = 0, statusmsg = "用户当前登录已过期或无效，请重新登录。" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                    }

                    if (!Guid.TryParse(staffid, out StaffGuid)
                        || Guid.Empty == StaffGuid)
                    {
                        return new JsonResult() { Data = new { statuscode = 0, statusmsg = "所选择目标无效，请刷新页面重试。" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                    }

                    var usersInfo = vendorEFContext.UserStaffFavoritesTable.Where(o => o.UserID == UserGuid && o.StaffID == StaffGuid && o.ModifiedDate == null).ToList();
                    if (usersInfo.Count() > 1)
                    {
                        return new JsonResult() { Data = new { statuscode = 0, statusmsg = "More than one record with the specified (user id, staff id) is found." }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                    }

                    foreach (var userInfo in usersInfo)
                    {
                        userInfo.ModifiedDate = DateTime.UtcNow;
                    }
                    CCS_UserStaffFavorites_V1 usersFavorites = new CCS_UserStaffFavorites_V1();
                    usersFavorites.UserID = UserGuid;
                    usersFavorites.StaffID = StaffGuid;
                    usersFavorites.ModifiedDate = null;
                    usersFavorites.FavoriteStatus = FavoriteStatus.Neutral;

                    vendorEFContext.UserStaffFavoritesTable.Add(usersFavorites);
                    vendorEFContext.SaveChanges();

                    actionStatusCode = 1;
                    actionStatusMsg = "添加收藏成功。";
                }

            }
            catch (Exception e)
            {
                if (e.InnerException != null)
                {
                    actionStatusCode = 0;
                    actionStatusMsg = "InnerException: " + e.InnerException.Message;
                }
                else
                {
                    actionStatusCode = 0;
                    actionStatusMsg = "Exception: " + e.Message;
                }
            }

            CCS_FailedActions_V1 action = new CCS_FailedActions_V1();
            action.ActionType = ACTIONSTYPE.REMOVEFROMFAVORITES;
            action.Message = actionStatusMsg;
            action.Parameters = string.Format(Constants.actionTypeToParametersFormatMap[ACTIONSTYPE.REMOVEFROMFAVORITES], userid, staffid);
            action.ModifiedDate = DateTime.UtcNow;

            _actionsDB.FailedActionsTable.Add(action);
            _actionsDB.SaveChanges();

            return new JsonResult() { Data = new { statuscode = actionStatusCode, statusmsg = actionStatusMsg }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };

        }

        [Authorize(Roles = "Vendor, Gatherer")]
        [HttpPost]
        public JsonResult SetVendorLocationWithCoordinates(double latitude, double longitude)
        {
            int statusCode = -1;
            string statusMsg = "NA";
            try
            {
                using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                {
                    List<CCS_Staff_V1> currentStaffs = GetCurrentStaffs();

                    foreach (CCS_Staff_V1 staff in currentStaffs)
                    {
                        staff.latitude = latitude;
                        staff.longitude = longitude;
                        staff.geography = DbGeography.FromText(string.Format("POINT({1} {0})", latitude, longitude));

                        vendorEFContext.StaffTable.Attach(staff);
                        vendorEFContext.Entry(staff).Property(x => x.latitude).IsModified = true;
                        vendorEFContext.Entry(staff).Property(x => x.longitude).IsModified = true;
                        vendorEFContext.Entry(staff).Property(x => x.geography).IsModified = true;
                    }


                    if (currentStaffs == null || currentStaffs.Count < 1)
                    {
                        statusCode = 200;
                        statusMsg = String.Format("No data found for this staff {0}. ", User.Identity.Name);

                    }
                    else
                    {
                        vendorEFContext.SaveChanges();
                        statusCode = 200;
                        statusMsg = String.Format("Location was set successfully for staff {0}. Location ({1}, {2}). ", User.Identity.Name, latitude, longitude);

                    }
                }


            }
            catch(Exception ex)
            {
                statusCode = -1;
                statusMsg = "Failed to set the location." + ex.Message;
            }
            return new JsonResult() { Data = new { statuscode = statusCode, statusmsg = statusMsg}, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
        }

        private string GetMarketBasedOnCoordinates(string Market, double? CenterLatitude, double? CenterLongitude)
        {
            if (string.IsNullOrEmpty(Market))
            {
                if (CenterLongitude == null || CenterLongitude >= 0)
                {
                    return "zh-cn";
                }
                else
                {
                    return "en-us";
                }
            }
            else if (Market.Equals("en-us", StringComparison.OrdinalIgnoreCase))
            {
                return "en-us";
            }

            return "zh-cn";

        }


        public JsonResult GetYunziLocations(double? latitude, double? longitude, string YunziIDs, int? StartIndex, int? Count)
        {
            int YunziCountPassedIn;
            var vendors = FetchYunziLocations(YunziIDs, StartIndex, Count, out YunziCountPassedIn);
            string targetMarket = GetMarketBasedOnCoordinates(null, latitude, longitude);
            if ("zh-cn".Equals(targetMarket, StringComparison.OrdinalIgnoreCase))
            {
                foreach (var vendor in vendors)
                    {
                        if (vendor != null && vendor.StaffTable != null)
                        {
                            Utility.LatLngModel latLng = Utility.Gcj2Bd(vendor.StaffTable.latitude, vendor.StaffTable.longitude);
                            double lat = (latLng.lat != null && latLng.lat.HasValue) ? latLng.lat.Value : 0;
                            double lng = (latLng.lng != null && latLng.lng.HasValue) ? latLng.lng.Value : 0;
                            vendor.StaffTable.latitude = lat;
                            vendor.StaffTable.longitude = lng;
                            //vendor.StaffTable.geography = DbGeography.FromText(string.Format("POINT({1} {0})", lat, lng));

                        }
                      
                    }
                
            }
            return Json(vendors, JsonRequestBehavior.AllowGet);
        }

        public JsonResult GetLocationsUserFavorites(int? StartIndex, int? Count)
        {
            string MobilePhone = Session[SessionConstants.UserLoginMobileNumber] as string;
            List<VendorModel> list = FetchFavouriteVendorList(MobilePhone, StartIndex, Count);
            return Json(list, JsonRequestBehavior.AllowGet);
        }
        public JsonResult GetUserFavoritesMessages(int? StartIndex, int? Count)
        {
            List<CloudBlockBlob> listOfMessageBlobs = new List<CloudBlockBlob>();
            List<MerchantMsgModel> listOfMessageContent = new List<MerchantMsgModel>();
            try
            {


                int Index = StartIndex ?? 0;
                if (Index < 0 || Index > MaxCountOfFavouriteVendors)
                {
                    Index = 0;
                }
                int FetchCount = BillCommon.GetFetchCount(Count);


                string MobilePhone = Session[SessionConstants.UserLoginMobileNumber] as string;
                List<VendorModel> list = FetchFavouriteVendorList(MobilePhone, StartIndex, Count, true);
                List<String> customerIdsFromVendords = new List<String>();
                foreach (var v in list)
                {
                    customerIdsFromVendords.Add(v.StaffTable.CustomerID.ToString().ToLower());
                    logger.Info(v.StaffTable.CustomerID.ToString());
                }
                logger.Info(string.Format("Fetched {0} vendors.", customerIdsFromVendords.Count));
                CloudStorageAccount storageAccount = CloudStorageAccount.Parse(ConfigurationMgr.GetSetting("StorageConnectionString"));
                CloudBlobClient blobClient = storageAccount.CreateCloudBlobClient();
                blobClient.DefaultRequestOptions.RetryPolicy = new LinearRetry(TimeSpan.FromSeconds(3), 3);

                // Init Ads text container
                CloudBlobContainer _adsBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.AdsBlobContainerName);
               
                listOfMessageBlobs = _adsBlobContainer.ListBlobs(useFlatBlobListing: true).OfType<CloudBlockBlob>().OrderByDescending(b => b.Properties.LastModified).Where(b => customerIdsFromVendords.Contains(b.Name.Split('/')[0])).Skip(Index).Take(FetchCount).ToList();
                logger.Info(string.Format("Fetched {0} blobs {1} messages", listOfMessageBlobs.Count, listOfMessageContent.Count));
                foreach(CloudBlockBlob cbb in listOfMessageBlobs)
                {
                    string blobContent = cbb.DownloadText();
                    MerchantMsgModel msgModel = JsonConvert.DeserializeObject<MerchantMsgModel>(blobContent);
                    listOfMessageContent.Add(msgModel);
                    cbb.FetchAttributes();
                    logger.Info("Blob" + cbb.Name + " " + cbb.Properties.LastModified);
                }
                
            }
            catch(Exception e)
            {
                logger.Info(e.Message);
            }
            return Json(listOfMessageContent, JsonRequestBehavior.AllowGet);
        }

        private List<VendorModel> FetchFavouriteVendorList(string MobilePhone, int? StartIndex, int? Count, bool FetchAll = false)
        {
            List<VendorModel> resultList = new List<VendorModel>();


            int Index = StartIndex ?? 0;
            if (Index < 0 || Index > MaxCountOfFavouriteVendors)
            {
                Index = 0;
            }
            int FetchCount = BillCommon.GetFetchCount(Count);

            if(string.IsNullOrEmpty(MobilePhone))
            {
                return resultList;
            }
            //log error here
            //throw new Exception("mobile phone can't be empty");

            var usersWithThisMobile = BillCommon.getCCSUserListWithInputID((BillContext)BillContextFactory.GetInstance().GetDBEFContext(), MobilePhone);
            if(usersWithThisMobile.Count() == 0)
            {
                //throw new Exception(string.Format("No user was found with mobile number {0}.", mobilePhone));
                return resultList;
            }

            /*if (usersWithThisMobile.Count() > 1)
            {
                throw new Exception(string.Format("More than one user was found with mobile number {0}.", mobilePhone));
            }*/

            CCS_User_V1 user = usersWithThisMobile.First();
            BillContext _billContextDB = (BillContext)BillContextFactory.GetInstance().GetDBEFContext();
            VendorContext _vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext();
            List<CCS_UserStaffFavorites_V1> userStaffFavorites;
            logger.Info(string.Format("Three {0} {1}", user.UserID, FetchAll));
            if(FetchAll)
            {
                userStaffFavorites = _billContextDB.UserStaffFavoritesTable.Where(x => x.UserID == user.UserID && x.ModifiedDate == null && x.FavoriteStatus == FavoriteStatus.Favorite).ToList();
            }
            else
            {
                userStaffFavorites = _billContextDB.UserStaffFavoritesTable.Where(x => x.UserID == user.UserID && x.ModifiedDate == null && x.FavoriteStatus == FavoriteStatus.Favorite).OrderByDescending(x => x.StaffID).Skip(Index).Take(FetchCount).ToList();
            }

            List<Guid> staffIds = new List<Guid>();
            foreach(var userStaffFavorite in userStaffFavorites)
            {
                staffIds.Add(userStaffFavorite.StaffID);
            }

            var staffs = (from b in _vendorEFContext.StaffTable.AsNoTracking().Where(s => (s.State == CustomerAccountState.Active) && staffIds.Contains(s.StaffID)) select b);
                 
            foreach (var staff in staffs)
            {
                if(staff != null)
                {
                    staff.geography = null;
                    var vendorModel = new VendorModel();
                    vendorModel.StaffTable = staff;
                    vendorModel.ProfilePictureUri = GetProfilePictureUri(staff.StaffID.ToString());
                    resultList.Add(vendorModel);
                }
                        
            }

            return  resultList;
        }

        public readonly static string[] EmptyYunziString = new string[] { "-1", "0" };
        private List<VendorModel> FetchYunziLocations(string YunziIDs, int? StartIndex, int? Count, out int YunziCountPassedIn)
        {

            YunziCountPassedIn = 0;

            if (string.IsNullOrEmpty(YunziIDs)
                || EmptyYunziString.Contains(YunziIDs))
            {
                return new List<VendorModel>();
            }
            IEnumerable<string> YunziIDArray = YunziIDs.Split(YunziInputDelims, StringSplitOptions.RemoveEmptyEntries);

            YunziCountPassedIn = YunziIDArray.Count();

            int FetchCount = Count ?? 10;

            if (FetchCount < 0)
            {
                return new List<VendorModel>();
            }
            else if (FetchCount > 50)
            {
                FetchCount = 50;
            }

            int FetchStartIndex = StartIndex ?? 0;
            IEnumerable<string> QueryArray = null;

            if (FetchStartIndex < 0 || FetchStartIndex >= YunziIDArray.Count())
            {
                return new List<VendorModel>();
            }
            else
            {
                QueryArray = YunziIDArray.Skip(FetchStartIndex).Take(FetchCount);
            }
            List<VendorModel> resultList = new List<VendorModel>();
            using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
            {
                vendorEFContext.Configuration.LazyLoadingEnabled = false;
                IQueryable<CCS_Staff_V1> staffs = null;


                if (User != null && User.Identity.IsAuthenticated && User.IsInRole(OperationLevel.Staff.ToString()))
                {
                    staffs = (from s in vendorEFContext.StaffTable.AsNoTracking()
                              where s.State == CustomerAccountState.Active
                                  && QueryArray.Contains(s.SNyunzi)
                                  && (s.Role == OperationLevel.Vendor ||
                                      s.Role == OperationLevel.Gatherer ||
                                      s.Role == OperationLevel.MeterReader)
                                  && s.geography != null
                                  && s.geography.Latitude != 0
                                  && s.geography.Longitude != 0
                              select s);
                }
                else if (User != null && User.Identity.IsAuthenticated && User.IsInRole(OperationLevel.Admin.ToString()))
                {
                    staffs = (from s in vendorEFContext.StaffTable.AsNoTracking()
                              where s.State == CustomerAccountState.Active
                                  && QueryArray.Contains(s.SNyunzi)
                                  && s.Role == OperationLevel.Staff
                                  && s.geography != null
                                  && s.geography.Latitude != 0
                                  && s.geography.Longitude != 0
                              select s);
                }
                else
                {
                    staffs = (from s in vendorEFContext.StaffTable.AsNoTracking()
                              where s.State == CustomerAccountState.Active
                                  && QueryArray.Contains(s.SNyunzi)
                                  && s.Role == OperationLevel.Vendor
                                  && s.geography != null
                                  && s.geography.Latitude != 0
                                  && s.geography.Longitude != 0
                              select s);
                }
                if(staffs != null)
                {
                    List<CCS_Staff_V1> temp = staffs.ToList();
                    staffs = null;

                    foreach (var staff in temp)
                    {
                        if(staff != null)
                        {
                            if (staff.CustomerTable == null && staff.CustomerID != Guid.Empty)
                            {
                                vendorEFContext.Entry(staff).Reference(x => x.CustomerTable).Load();
                            }

                            staff.geography = null;
                            var vendorModel = new VendorModel();
                            vendorModel.StaffTable = staff;
                            vendorModel.ProfilePictureUri = GetProfilePictureUri(staff.StaffID.ToString());

                            resultList.Add(vendorModel);
                        }
                        
                    }
                }
               
            }
            return CreateReturnList(resultList, QueryArray, new GetCompareKeyFromVendorModel(x => x.StaffTable.SNyunzi));
        }


        private List<VendorModel> FetchGuidLocations(string Guids, int? StartIndex, int? Count, out int GuidCountPassedIn)
        {

            GuidCountPassedIn = 0;

            if (string.IsNullOrEmpty(Guids)
                || EmptyYunziString.Contains(Guids))
            {
                return new List<VendorModel>();
            }
            IEnumerable<string> GuidArray = Guids.Split(YunziInputDelims, StringSplitOptions.RemoveEmptyEntries);

            GuidCountPassedIn = GuidArray.Count();

            int FetchCount = BillCommon.GetFetchCount(Count);
            
            if (FetchCount < 0)
            {
                return new List<VendorModel>();
            }

            int FetchStartIndex = StartIndex ?? 0;

            List<Guid> QueryArray = null;

            if (FetchStartIndex < 0 || FetchStartIndex >= GuidArray.Count())
            {
                return new List<VendorModel>();
            }
            else
            {
                var a = GuidArray.Skip(FetchStartIndex).Take(FetchCount);
                QueryArray = new List<Guid>();
                Guid TempGuid;
                foreach (string s in a)
                {
                    QueryArray.Add(Guid.TryParse(s, out TempGuid)?TempGuid:TempGuid);
                }
            }
            List<VendorModel> resultList = new List<VendorModel>();
            using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
            {
                vendorEFContext.Configuration.LazyLoadingEnabled = false;
                IQueryable<CCS_Staff_V1> staffs = null;


                if (User != null && User.Identity.IsAuthenticated && User.IsInRole(OperationLevel.Staff.ToString()))
                {
                    staffs = (from s in vendorEFContext.StaffTable.AsNoTracking()
                              where s.State == CustomerAccountState.Active
                                  && QueryArray.Contains(s.StaffID)
                                  && (s.Role == OperationLevel.Vendor ||
                                      s.Role == OperationLevel.Gatherer ||
                                      s.Role == OperationLevel.MeterReader)
                                  && s.geography != null
                                  && s.geography.Latitude != 0
                                  && s.geography.Longitude != 0
                              select s);
                }
                else if (User != null && User.Identity.IsAuthenticated && User.IsInRole(OperationLevel.Admin.ToString()))
                {
                    staffs = (from s in vendorEFContext.StaffTable.AsNoTracking()
                              where s.State == CustomerAccountState.Active
                                  && QueryArray.Contains(s.StaffID)
                                  && s.Role == OperationLevel.Staff
                                  && s.geography != null
                                  && s.geography.Latitude != 0
                                  && s.geography.Longitude != 0
                              select s);
                }
                else
                {
                    staffs = (from s in vendorEFContext.StaffTable.AsNoTracking()
                              where s.State == CustomerAccountState.Active
                                  && QueryArray.Contains(s.StaffID)
                                  && s.Role == OperationLevel.Vendor
                                  && s.geography != null
                                  && s.geography.Latitude != 0
                                  && s.geography.Longitude != 0
                              select s);
                }

                if (staffs != null)
                {
                    foreach (var staff in staffs)
                    {
                        if (staff != null)
                        {
                            staff.geography = null;
                            var vendorModel = new VendorModel();
                            vendorModel.StaffTable = staff;
                            vendorModel.ProfilePictureUri = GetProfilePictureUri(staff.StaffID.ToString());
                            resultList.Add(vendorModel);
                        }

                    }
                }

            }
            return CreateReturnList(resultList, GuidArray, new GetCompareKeyFromVendorModel(x=> x.StaffTable.StaffID.ToString()));
        }

        delegate string GetCompareKeyFromVendorModel(VendorModel target);
        private List<VendorModel> CreateReturnList(List<VendorModel> SortTarget, IEnumerable<string> KeyArray, GetCompareKeyFromVendorModel KeyDelegate)
        {
            var ResultList = new List<VendorModel>();
            if (SortTarget == null
                || KeyArray == null || KeyArray.Count() < 1)
            {
                return ResultList;
            }
            var dic = new Dictionary<string, VendorModel>();

            foreach (VendorModel staff in SortTarget)
            {
                if (dic.ContainsKey(KeyDelegate(staff)))//staff.StaffTable.SNyunzi
                {
                    continue;
                }
                else
                {
                    staff.StaffTable.geography = null;
                    dic.Add(staff.StaffTable.SNyunzi, staff);
                }
            }

            foreach (string key in KeyArray)
            {
                if (dic.ContainsKey(key))
                {
                    ResultList.Add(dic[key]);
                }
                else
                {
                    ResultList.Add(new VendorModel());
                }
            }
            return ResultList;
        }

        public JsonResult GetLocations(double latitude, double longitude, int? Count)
        {
            var vendors = FetchNearestLocations(latitude, longitude, BillCommon.GetFetchCount(Count));
            string targetMarket = GetMarketBasedOnCoordinates(null, latitude, longitude);
            if ("zh-cn".Equals(targetMarket, StringComparison.OrdinalIgnoreCase))
            {
                foreach (var vendor in vendors)
                {
                    if (vendor != null && vendor.StaffTable != null)
                    {
                        Utility.LatLngModel latLng = Utility.Gcj2Bd(vendor.StaffTable.latitude, vendor.StaffTable.longitude);
                        double lat = (latLng.lat != null && latLng.lat.HasValue) ? latLng.lat.Value : 0;
                        double lng = (latLng.lng != null && latLng.lng.HasValue) ? latLng.lng.Value : 0;
                        vendor.StaffTable.latitude = lat;
                        vendor.StaffTable.longitude = lng;
                        //vendor.StaffTable.geography = DbGeography.FromText(string.Format("POINT({1} {0})", lat, lng));

                    }

                }

            }

            return Json(vendors, JsonRequestBehavior.AllowGet);
        }


        public JsonResult GetMoreLocations(double latitude, double longitude, double? beginLatitude, double? beginLongitude, int? Count)
        {
            var vendors = FetchNearestLocations(latitude, longitude, beginLatitude, beginLongitude, BillCommon.GetFetchCount(Count));
            string targetMarket = GetMarketBasedOnCoordinates(null, latitude, longitude);
            if ("zh-cn".Equals(targetMarket, StringComparison.OrdinalIgnoreCase))
            {
                foreach (var vendor in vendors)
                {
                    if (vendor != null && vendor.StaffTable != null)
                    {
                        Utility.LatLngModel latLng = Utility.Gcj2Bd(vendor.StaffTable.latitude, vendor.StaffTable.longitude);
                        double lat = (latLng.lat != null && latLng.lat.HasValue) ? latLng.lat.Value : 0;
                        double lng = (latLng.lng != null && latLng.lng.HasValue) ? latLng.lng.Value : 0;
                        vendor.StaffTable.latitude = lat;
                        vendor.StaffTable.longitude = lng;
                        //vendor.StaffTable.geography = DbGeography.FromText(string.Format("POINT({1} {0})", lat, lng));

                    }

                }

            }

            return Json(vendors, JsonRequestBehavior.AllowGet);
        }

        //public JsonResult GetPreviousLocations(double latitude, double longitude, double? beginLatitude, double? beginLongitude, int? Count)
        //{
        //    var vendors = FetchFarestLocations(latitude, longitude, beginLatitude, beginLongitude, GetFetchCount(Count));
        //    return Json(vendors, JsonRequestBehavior.AllowGet);
        //}

        private List<VendorModel> FetchNearestLocations(double centerLatitude, double centerLongitude, int Count)
        {
            var defaultLoc = DbGeography.FromText(string.Format("POINT({1} {0})", centerLatitude, centerLongitude));
            List<VendorModel> returnList = new List<VendorModel>();
            using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
            {
                vendorEFContext.Configuration.LazyLoadingEnabled = false;
                IQueryable<CCS_Staff_V1> staffs = null;

                if (User != null && User.Identity.IsAuthenticated && User.IsInRole(OperationLevel.Staff.ToString()))
                {
                    var curCustomerID = new Guid(GetCurrentCustomerGuidString());

                    staffs= (from s in vendorEFContext.StaffTable
                        where s.CustomerID == curCustomerID 
                            && s.State == CustomerAccountState.Active
                            && (s.Role == OperationLevel.Vendor ||
                                s.Role == OperationLevel.Gatherer ||
                                s.Role == OperationLevel.MeterReader)
                            && s.geography != null
                            && s.geography.Latitude != 0
                            && s.geography.Longitude != 0
                        orderby s.geography.Distance(defaultLoc)
                        select s).Take(Count);
                }
                else if (User != null && User.Identity.IsAuthenticated && User.IsInRole(OperationLevel.Admin.ToString()))
                {
                    var curCustomerID = new Guid(GetCurrentCustomerGuidString());
                    staffs = (from s in vendorEFContext.StaffTable
                        where s.CustomerID == curCustomerID
                            && s.State == CustomerAccountState.Active
                            && s.Role == OperationLevel.Staff
                            && s.geography != null
                            && s.geography.Latitude != 0
                            && s.geography.Longitude != 0
                        orderby s.geography.Distance(defaultLoc)
                        select s).Take(Count);
                }
                else
                {
                    staffs = (from s in vendorEFContext.StaffTable
                        where s.State == CustomerAccountState.Active 
                            && s.Role == OperationLevel.Vendor
                            && s.geography != null
                            && s.geography.Latitude != 0
                            && s.geography.Longitude != 0
                        orderby s.geography.Distance(defaultLoc)
                        select s).Take(Count);
                }

                List<CCS_Staff_V1> temp = staffs.ToList();
                staffs = null;

                foreach (CCS_Staff_V1 staff in temp)
                {
                    if (staff.CustomerTable == null && staff.CustomerID != Guid.Empty)
                    {
                        vendorEFContext.Entry(staff).Reference(x => x.CustomerTable).Load();
                    }

                    staff.geography = null;
                    var vendorModel = new VendorModel();
                    vendorModel.StaffTable = staff;
                    vendorModel.ProfilePictureUri = GetProfilePictureUri(staff.StaffID.ToString());

                    returnList.Add(vendorModel);
                }
            }
            return returnList;
        }

        private List<VendorModel> FetchNearestLocations(double centerLatitude, double centerLongitude,
            double? beginLatitude, double? beginLongitude, int Count = 10)
        {

            if (beginLatitude == null || beginLongitude == null)
            {
                return FetchNearestLocations(centerLatitude, centerLongitude, Count);
            }
            var defaultLoc = DbGeography.FromText(string.Format("POINT({1} {0})", centerLatitude, centerLongitude));
            var beginLoc = DbGeography.FromText(string.Format("POINT({1} {0})", beginLatitude, beginLongitude));
            double? distance = beginLoc.Distance(defaultLoc);
            List<VendorModel> returnList = new List<VendorModel>();

            using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
            {
                vendorEFContext.Configuration.LazyLoadingEnabled = false;
                IQueryable<CCS_Staff_V1> staffs = null;
                if (User!=null && User.Identity.IsAuthenticated && User.IsInRole(OperationLevel.Staff.ToString()))
                {
                    var curCustomerID = new Guid(GetCurrentCustomerGuidString());
                    staffs = (from s in vendorEFContext.StaffTable
                              where s.CustomerID == curCustomerID
                                  && s.State == CustomerAccountState.Active
                                  && (s.Role == OperationLevel.Vendor ||
                                      s.Role == OperationLevel.Gatherer ||
                                      s.Role == OperationLevel.MeterReader)
                                  && s.geography != null
                                  && s.geography.Latitude != 0
                                  && s.geography.Longitude != 0
                                  && s.geography.Distance(defaultLoc) > distance
                              orderby s.geography.Distance(defaultLoc)
                              select s).Take(Count);
                }
                else if (User != null && User.Identity.IsAuthenticated && User.IsInRole(OperationLevel.Admin.ToString()))
                {
                    var curCustomerID = new Guid(GetCurrentCustomerGuidString());
                    staffs = (from s in vendorEFContext.StaffTable
                              where s.CustomerID == curCustomerID
                                  && s.State == CustomerAccountState.Active
                                  && s.Role == OperationLevel.Staff
                                  && s.geography != null
                                  && s.geography.Latitude != 0
                                  && s.geography.Longitude != 0
                                  && s.geography.Distance(defaultLoc) > distance
                              orderby s.geography.Distance(defaultLoc)
                              select s).Take(Count);
                }
                else
                {

                    staffs = (from s in vendorEFContext.StaffTable
                              where s.State == CustomerAccountState.Active
                                  && s.Role == OperationLevel.Vendor
                                  && s.geography != null
                                  && s.geography.Latitude != 0
                                  && s.geography.Longitude != 0
                                  && s.geography.Distance(defaultLoc) > distance
                              orderby s.geography.Distance(defaultLoc)
                              select s).Take(Count);
                }

                List<CCS_Staff_V1> temp = staffs.ToList();
                staffs = null;

                foreach (var staff in temp)
                {
                    if (staff.CustomerTable == null && staff.CustomerID != Guid.Empty)
                    {
                        vendorEFContext.Entry(staff).Reference(x => x.CustomerTable).Load();
                    }

                    staff.geography = null;
                    var vendorModel = new VendorModel();
                    vendorModel.StaffTable = staff;
                    vendorModel.ProfilePictureUri = GetProfilePictureUri(staff.StaffID.ToString());

                    returnList.Add(vendorModel);
                }
            }
            return returnList;
        }
        #region Deleted
        //private List<VendorModel> FetchFarestLocations(double centerLatitude, double centerLongitude,
        //    double? endLatitude, double? endLongitude, int Count)
        //{

        //    if (endLatitude == null || endLongitude == null)
        //    {
        //        return FetchNearestLocations(centerLatitude, centerLongitude, Count);
        //    }
        //    var defaultLoc = DbGeography.FromText(string.Format("POINT({1} {0})", centerLatitude, centerLongitude));
        //    var beginLoc = DbGeography.FromText(string.Format("POINT({1} {0})", endLatitude, endLongitude));

        //    double? distance = beginLoc.Distance(defaultLoc);

        //    var returnList = new List<VendorModel>();
        //    using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
        //    {
        //        vendorEFContext.Configuration.LazyLoadingEnabled = false;
        //        IQueryable<CCS_Staff_V1> staffs = null;


        //        if (User != null && User.Identity.IsAuthenticated && User.IsInRole(OperationLevel.Staff.ToString()))
        //        {
        //             var curCustomerID = new Guid(GetCurrentCustomerIDString());
        //             staffs = (from s in vendorEFContext.StaffTable.AsNoTracking()
        //                       where s.CustomerID == curCustomerID
        //                           && s.State == CustomerAccountState.Active
        //                           && (s.Role == OperationLevel.Vendor ||
        //                               s.Role == OperationLevel.Gatherer ||
        //                               s.Role == OperationLevel.MeterReader)
        //                           && s.geography != null
        //                           && s.geography != EmptyDbGeography
        //                           && s.geography.Distance(defaultLoc) < distance
        //                       orderby s.geography.Distance(defaultLoc) descending
        //                       select s).Take(Count);
        //        }
        //        else if (User != null && User.Identity.IsAuthenticated && User.IsInRole(OperationLevel.Admin.ToString()))
        //        {
        //            var curCustomerID = new Guid(GetCurrentCustomerIDString());
        //            staffs = (from s in vendorEFContext.StaffTable.AsNoTracking()
        //                      where s.CustomerID == curCustomerID
        //                          && s.State == CustomerAccountState.Active
        //                          && s.Role == OperationLevel.Staff
        //                          && s.geography != null
        //                          && s.geography != EmptyDbGeography
        //                          && s.geography.Distance(defaultLoc) < distance
        //                      orderby s.geography.Distance(defaultLoc) descending
        //                      select s).Take(Count);
        //        }
        //        else
        //        {
        //            staffs = (from s in vendorEFContext.StaffTable.AsNoTracking()
        //                      where s.State == CustomerAccountState.Active
        //                          && s.Role == OperationLevel.Vendor
        //                          && s.geography != null
        //                          && s.geography != EmptyDbGeography
        //                          && s.geography.Distance(defaultLoc) < distance
        //                      orderby s.geography.Distance(defaultLoc) descending
        //                      select s).Take(Count);
        //        }
        //        foreach (var staff in staffs)
        //        {
        //            staff.geography = null;
        //            var vendorModel = new VendorModel();
        //            vendorModel.StaffTable = staff;
        //            vendorModel.ProfilePictureUri = GetProfilePictureUri(staff.StaffID.ToString());
        //            returnList.Add(vendorModel);
        //        }
        //        returnList.Reverse();
        //    }
        //    return returnList;
        //}
        #endregion

        public ActionResult ShowMapMerchantMeter(double? CenterLatitude, double? CenterLongitude, string YunziIDs, string Guids,
            int? StartIndex, int? Count, string ShowType, string Market, string MapStyle, string UserId,
            bool? FullScreen, bool? NavigateOnLoad, bool? FetchMoreDataAfterCenterChange)
        {
            int fetchCount = BillCommon.GetFetchCount(Count);
            SetViewBagCommonValues(CenterLatitude, CenterLongitude, YunziIDs, Guids, StartIndex, ShowType,
                Market, UserId, FullScreen ?? false, NavigateOnLoad, fetchCount, FetchMoreDataAfterCenterChange);
            ViewBag.MapStyle = string.IsNullOrEmpty(MapStyle) ? "merchantmeter" : MapStyle;

            FillInAllMeterGeo();

            List<CCS_MeterData_V1> MeterList = null;
            //if (CenterLatitude == null && CenterLongitude == null)
            {
                MeterList = new List<CCS_MeterData_V1>();
            }
            //else
            //{
            //    MeterList = FetchNearestMeters((double)CenterLatitude, (double)CenterLongitude, fetchCount);
            //}

            return View();
        }

        private void FillInAllVendorGeo()
        {
            try
            {
                //var curCustomerID = new Guid(GetCurrentCustomerGuidString());
                using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                {
                    //vendorEFContext.Configuration.LazyLoadingEnabled = false;
                    var vendors = (from v in vendorEFContext.StaffTable
                                  where v.geography == null && v.latitude != 0 && v.longitude != 0
                                  select v);

                    foreach (var v in vendors)
                    {
                        v.geography = DbGeography.FromText(string.Format("POINT({1} {0})", v.latitude, v.longitude));
                        vendorEFContext.StaffTable.Attach(v);
                        vendorEFContext.Entry(v).Property(s=>s.geography).IsModified = true;
                    }

                    if (vendors.Count() > 0)
                    {
                        vendorEFContext.SaveChanges();
                    }
                }
            }
            catch (DbEntityValidationException e)
            {
                String s = "empty";
                foreach (var eve in e.EntityValidationErrors)
                {
                    s += String.Format("Entity of type \"{0}\" in state \"{1}\" has the following validation errors:",
                        eve.Entry.Entity.GetType().Name, eve.Entry.State);
                    foreach (var ve in eve.ValidationErrors)
                    {
                        s += String.Format("- Property: \"{0}\", Error: \"{1}\"",
                            ve.PropertyName, ve.ErrorMessage);
                    }
                }
                throw new Exception(s);
            }
        }
        private void FillInAllMeterGeo()
        {
            try
            {
                var curCustomerID = new Guid(GetCurrentCustomerGuidString());
                using (var TempDBContext = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext())
                {
                    //TempDBContext.Configuration.LazyLoadingEnabled = false;
                    var meters = (from b in TempDBContext.MeterDataTable
                                  where b.Latitude != null && b.Longitude != null && b.CustomerID == curCustomerID
                                  && (b.geography == null || b.geography.Latitude != b.Latitude || b.geography.Longitude != b.Longitude)
                                  select b);

                    foreach (var m in meters)
                    {
                        m.geography = DbGeography.FromText(string.Format("POINT({1} {0})", m.Latitude, m.Longitude));
                        TempDBContext.Entry(m).State = System.Data.Entity.EntityState.Modified;
                        TempDBContext.MeterDataTable.Attach(m);
                        TempDBContext.Entry(m).Property(s => s.geography).IsModified = true;
                    }

                    if (meters.Count() > 0)
                    {
                        TempDBContext.SaveChanges();
                    }
                }
            }
            catch (DbEntityValidationException e)
            {
                String s = "empty";
                foreach (var eve in e.EntityValidationErrors)
                {
                    s += String.Format("Entity of type \"{0}\" in state \"{1}\" has the following validation errors:",
                        eve.Entry.Entity.GetType().Name, eve.Entry.State);
                    foreach (var ve in eve.ValidationErrors)
                    {
                        s+= String.Format("- Property: \"{0}\", Error: \"{1}\"",
                            ve.PropertyName, ve.ErrorMessage);
                    }
                }
                throw new Exception(s);
            }
            
        }

        private List<CCS_MeterData_V1> FetchNearestMeters(double centerLatitude, double centerLongitude, int Count)
        {
            var defaultLoc = DbGeography.FromText(string.Format("POINT({1} {0})", centerLatitude, centerLongitude));

            List<CCS_MeterData_V1> returnList = new List<CCS_MeterData_V1>();

            Guid curCustomerID = GetCurrentCustomerGuid(Guid.Empty);

            if (curCustomerID == Guid.Empty)
            {
                return returnList;
            }

            using (var TempDBContext = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext())
            {
                TempDBContext.Configuration.LazyLoadingEnabled = false;
                var meters = (from b in TempDBContext.MeterDataTable
                              where b.Latitude != null && b.Longitude != null && b.CustomerID == curCustomerID
                              orderby defaultLoc.Distance(b.geography)
                              select b).Take(Count);

                foreach (var m in meters)
                {
                    m.geography = null;
                }

                returnList = meters.ToList();

                //if (returnList.Count == 0)
                //{
                //    var emptymeters = (from b in TempDBContext.MeterDataTable
                //                  where b.Latitude == null && b.Longitude == null 
                //                  && b.geography == null && b.CustomerID == curCustomerID
                //                  select b);
                //    Random r = new Random(DateTime.Now.Second);
                //    foreach(var meter in emptymeters)
                //    {
                //        meter.Latitude = r.NextDouble() * 10 * (r.NextDouble() > 0.5? 1: -1) + 47;
                //        meter.Longitude = r.NextDouble() * 10 * (r.NextDouble() > 0.5 ? 1 : -1) - 122;
                //        meter.geography = DbGeography.FromText(string.Format("POINT({1} {0})", meter.Latitude, meter.Longitude));
                //        meter.Memo2 = "Test geography info";
                //        TempDBContext.Entry(meter).State = System.Data.Entity.EntityState.Modified;
                //    }
                //    TempDBContext.SaveChanges();

                //    returnList = emptymeters.ToList();
                //}
            }

            return returnList;
        }

        public JsonResult GetMeters(double latitude, double longitude, int? Count)
        {
            var vendors = FetchNearestMeters(latitude, longitude, BillCommon.GetFetchCount(Count));
            return Json(vendors, JsonRequestBehavior.AllowGet);
        }


        public JsonResult GetMoreMeters(double latitude, double longitude, double? beginLatitude, double? beginLongitude, int? Count)
        {
            var vendors = FetchNearestMeters(latitude, longitude, beginLatitude, beginLongitude, BillCommon.GetFetchCount(Count));
            return Json(vendors, JsonRequestBehavior.AllowGet);
        }

        //public JsonResult GetPreviousMeters(double latitude, double longitude, double? beginLatitude, double? beginLongitude, int? Count)
        //{
        //    var vendors = FetchFarestMeters(latitude, longitude, beginLatitude, beginLongitude, GetFetchCount(Count));
        //    return Json(vendors, JsonRequestBehavior.AllowGet);
        //}


        private List<CCS_MeterData_V1> FetchNearestMeters(double centerLatitude, double centerLongitude,
            double? beginLatitude, double? beginLongitude, int Count = 10)
        {

            if (beginLatitude == null || beginLongitude == null)
            {
                return FetchNearestMeters(centerLatitude, centerLongitude, Count);
            }
            var defaultLoc = DbGeography.FromText(string.Format("POINT({1} {0})", centerLatitude, centerLongitude));
            var beginLoc = DbGeography.FromText(string.Format("POINT({1} {0})", beginLatitude, beginLongitude));

            double? distance = beginLoc.Distance(defaultLoc);

            List<CCS_MeterData_V1> returnList = new List<CCS_MeterData_V1>();
            var curCustomerID = new Guid(GetCurrentCustomerGuidString());
            using (var TempDBContext = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext())
            {
                TempDBContext.Configuration.LazyLoadingEnabled = false;
                var meters = (from b in TempDBContext.MeterDataTable
                              where b.Latitude != null && b.Longitude != null && b.CustomerID == curCustomerID
                                && defaultLoc.Distance(b.geography) > distance
                              orderby defaultLoc.Distance(b.geography)
                              select b).Take(Count);

                foreach (var m in meters)
                {
                    m.geography = null;
                }

                returnList = meters.ToList();
            }

            return returnList;
        }

        //private List<CCS_MeterData_V1> FetchFarestMeters(double centerLatitude, double centerLongitude,
        //    double? endLatitude, double? endLongitude, int Count)
        //{

        //    if (endLatitude == null || endLongitude == null)
        //    {
        //        return FetchNearestMeters(centerLatitude, centerLongitude, Count);
        //    }
        //    var defaultLoc = DbGeography.FromText(string.Format("POINT({1} {0})", centerLatitude, centerLongitude));
        //    var beginLoc = DbGeography.FromText(string.Format("POINT({1} {0})", endLatitude, endLongitude));

        //    double? distance = beginLoc.Distance(defaultLoc);

        //    List<CCS_MeterData_V1> returnList = new List<CCS_MeterData_V1>();
        //    var curCustomerID = new Guid(GetCurrentCustomerIDString());
        //    using (var TempDBContext = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext())
        //    {
        //        TempDBContext.Configuration.LazyLoadingEnabled = false;
        //        var meters = (from b in TempDBContext.MeterDataTable

        //                      where b.Latitude != null && b.Longitude != null && b.CustomerID == curCustomerID
        //                        && defaultLoc.Distance(b.geography) < distance
        //                      orderby defaultLoc.Distance(b.geography) descending
        //                      select b).Take(Count);

        //        foreach (var m in meters)
        //        {
        //            m.geography = null;
        //        }

        //        returnList = meters.ToList();
        //        returnList.Reverse();

        //    }

        //    return returnList;
        //}


        public JsonResult GetsMeterPhotos(string MeterID)
        {
            var vendors = FetchMeterSubInfos(MeterID);
            return Json(vendors, JsonRequestBehavior.AllowGet);
        }

        private List<CCS_MeterData_Sub_V1> FetchMeterSubInfos(string MeterID)
        {
            Guid QueryGuid = new Guid();
            if (string.IsNullOrEmpty(MeterID)
                && !Guid.TryParse(MeterID,out QueryGuid))
            {
                return new List<CCS_MeterData_Sub_V1>();
            }

            List<CCS_MeterData_Sub_V1> returnList = new List<CCS_MeterData_Sub_V1>();
            using (var TempDBContext = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext())
            {
                TempDBContext.Configuration.LazyLoadingEnabled = false;
                var meters = from b in TempDBContext.MeterDataSubTable
                             where b.MeterDataID == QueryGuid
                             select b;

                returnList = meters.ToList();
            }

            return returnList;
        }

    }
}
