﻿using CCSAdmin;
using CCSPrototypeCommon;
using HomeWeb.Models;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;
using Microsoft.WindowsAzure.Storage.Queue;
using Microsoft.WindowsAzure.Storage.RetryPolicies;
using Microsoft.WindowsAzure.Storage.Table;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity.Infrastructure;
using System.Data.Entity.Spatial;
using System.Data.Entity.Validation;
using System.Data.SqlClient;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Threading.Tasks;
using System.Web;
using System.Web.Mvc;
using System.Reflection;
using System.ComponentModel.Design;
using System.ComponentModel;
namespace HomeWeb.Controllers
{
    // Merchant - Staff (Water Company)
    public partial class MerchantController : QueryableController
    {
        public static Logger logger
        {
            get
            {
                return new Logger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name);
            }

        }

        private CloudBlobContainer _adsBlobContainer;
        private CloudBlobContainer _rawDataBlobContainer;
        private CloudBlobContainer _adsMediaBlobContainer;
        private CloudBlobContainer _adsBlobArchiveContainer;

        private CloudBlobContainer _billBlobContainer;
        private CloudBlobContainer _uploadedPaymentsBlobContainer;

        private CloudQueue _billQueue;
        private CloudQueue _uploadedPaymentsQueue;

        private CloudQueue _rawDataQueue;
        private CloudQueue filesQueue;
        private CloudBlobContainer filesBlobContainer;
        private CloudBlobContainer paymentsBlobContainer;
        private static Dictionary<string, Uri> BlobHashMap = new Dictionary<string, Uri>();
        private CloudBlobContainer _meterProfileBlobContainer;
        private CloudTable _csvTemplateTable;


        private CloudTable _billTemplateTable;

        const string QueryDownloadBlobName = "DownloadQueryResult";

        private BillContext _billDB = (BillContext)BillContextFactory.GetInstance().GetDBEFContext();
        private ActionsContext _actionsDB = (ActionsContext)ActionsContextFactory.GetInstance().GetDBEFContext();

        public MerchantController()
            : base()
        {
            InitializeCloudStorage();
        }

        public static CCS_Customer_V1 GetCustomerInfo(Guid customerID)
        {
            using (var customerEFContext = (CustomerContext)CustomerContextFactory.GetInstance().GetDBEFContext())
            {
                customerEFContext.Configuration.LazyLoadingEnabled = false;
                var record = customerEFContext.CustomerTable.AsNoTracking().Where(entry => entry.CustomerID == customerID).FirstOrDefault();
                if (record != null)
                {
                    return record;
                }
            }
            throw new Exception(ErrorConstants.NoAssociatedCompany);
        }


        protected override void InitializeCloudStorage()
        {

            // Open storage account
            CloudStorageAccount storageAccount = CloudStorageAccount.Parse(ConfigurationMgr.GetSetting("StorageConnectionString"));
            CloudBlobClient blobClient = storageAccount.CreateCloudBlobClient();
            blobClient.DefaultRequestOptions.RetryPolicy = new LinearRetry(TimeSpan.FromSeconds(3), 3);

            // Init Ads text container
            _adsBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.AdsBlobContainerName);
            //_adsBlobContainer.CreateIfNotExists();
            _adsBlobArchiveContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.AdsBlobArchiveContainerName);
            //_adsBlobArchiveContainer.CreateIfNotExists();

            _meterProfileBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.MeterProfileStorageName);
            //_meterProfileBlobContainer.CreateIfNotExists();

            // Init Ads multimedia container
            _adsMediaBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.AdsMultimediaBlobContainerName);
           // _adsMediaBlobContainer.CreateIfNotExists();
            var blobAccess = new BlobContainerPermissions();
            blobAccess.PublicAccess = BlobContainerPublicAccessType.Blob;
            _adsMediaBlobContainer.SetPermissions(blobAccess);

            filesBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.BillStorageName);

            //filesBlobContainer.CreateIfNotExists();

            paymentsBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.PaymentStorageName);

            //paymentsBlobContainer.CreateIfNotExists();

            // Get queue context object
            CloudQueueClient queueClient = storageAccount.CreateCloudQueueClient();
            queueClient.DefaultRequestOptions.RetryPolicy = new LinearRetry(TimeSpan.FromSeconds(3), 3);
            filesQueue = queueClient.GetQueueReference(CCSPrototypeCommon.Constants.BillStorageName);

           // filesQueue.CreateIfNotExists();


            blobClient.DefaultRequestOptions.RetryPolicy = new LinearRetry(TimeSpan.FromSeconds(3), 3);
            _billBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.BillStorageName);
            //_billBlobContainer.CreateIfNotExists();
            _uploadedPaymentsBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.UploadedPaymentsStorageName);
           // _uploadedPaymentsBlobContainer.CreateIfNotExists();
            _rawDataBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.RawDataStorageName);
            //_rawDataBlobContainer.CreateIfNotExists();
            queryBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.QueryStorageName);
           // queryBlobContainer.CreateIfNotExists();

            _billQueue = queueClient.GetQueueReference(CCSPrototypeCommon.Constants.BillStorageName);
           // _billQueue.CreateIfNotExists();
            _uploadedPaymentsQueue = queueClient.GetQueueReference(CCSPrototypeCommon.Constants.UploadedPaymentsStorageName);
           // _uploadedPaymentsQueue.CreateIfNotExists();
            queryInitiateQueue = queueClient.GetQueueReference(CCSPrototypeCommon.Constants.QueryInitStorageName);
            //queryInitiateQueue.CreateIfNotExists();
            queryDownloadQueue = queueClient.GetQueueReference(CCSPrototypeCommon.Constants.QueryDownloadStorageName);
           // queryDownloadQueue.CreateIfNotExists();

            _rawDataQueue = queueClient.GetQueueReference(CCSPrototypeCommon.Constants.RawDataStorageName);
            //_rawDataQueue.CreateIfNotExists();

            CloudTableClient tableClient = storageAccount.CreateCloudTableClient();
            tableClient.DefaultRequestOptions.RetryPolicy = new LinearRetry(TimeSpan.FromSeconds(3), 3);
            _billTemplateTable = tableClient.GetTableReference(CCSPrototypeCommon.Constants.BillCsvTemplateTableName);
           // _billTemplateTable.CreateIfNotExists();

            _csvTemplateTable = tableClient.GetTableReference(CCSPrototypeCommon.Constants.CsvTemplateTableName);
            //_csvTemplateTable.CreateIfNotExists();

        }

        // Show default page
        public ActionResult Index()
        {
            /*
            String msg = "A message from a vendor!!!";
            String phoneNumber = "13670947028";
            var users = _billDB.UserTable.Where(o => o.Cellphone == phoneNumber && o.iOSDeviceToken != null);
            PushNotificationsManager.PushNotifManagerInstance.SendPushNotificationToUsers(users, msg);*/
            return View();
        }

        public ActionResult GetDailyTotalBalance()
        {
            DateTime date = DateTime.Now.Date;
            var firstDayToFetchFrom = new DateTime(date.Year, date.Month, 1);
            List<vwCCS_Daily_Total_Balance_V1> dailyTotalBalanceList = new List<vwCCS_Daily_Total_Balance_V1>();
            try
            {
                String currentCustomerID = GetCurrentCustomerGuidString();
                using (var _viewsCtx = (ViewsContext)ViewsContextFactory.GetInstance().GetDBEFContext())
                {
                    dailyTotalBalanceList = (from u in _viewsCtx.vwDailyTotalBalance
                                             where u.customerID.ToString() == currentCustomerID && u.snapshotdate >= firstDayToFetchFrom
                                             select u).ToList();
                }
            }
            catch (Exception e)
            {
                ViewBag.Msg = e.Message;
                return View("GenericErrorMsgMrct");
            }

            return View("DailyTotalBalance", dailyTotalBalanceList);
        }

        public ActionResult GetTotalPaymentPerRole()
        {
            DateTime date = DateTime.Now.Date;
            var firstDayToFetchFrom = new DateTime(date.Year, date.Month, 1);
            List<CCS_Daily_Payment_V1> dailyPaymentPerRoleList = new List<CCS_Daily_Payment_V1>();
            try
            {
                String currentCustomerID = GetCurrentCustomerGuidString();
                using (var _viewsCtx = (ViewsContext)ViewsContextFactory.GetInstance().GetDBEFContext())
                {
                    dailyPaymentPerRoleList = (from u in _viewsCtx.dailyPaymentPerRole
                                               where u.CustomerID.ToString() == currentCustomerID && u.SnapshotDate >= firstDayToFetchFrom && u.PayerRole != null
                                               select u).ToList();
                }
            }
            catch (Exception e)
            {
                ViewBag.Msg = e.Message;
                return View("GenericErrorMsgMrct");
            }

            return View("DailyPaymentPerRole", dailyPaymentPerRoleList);
        }

        public ActionResult GetTotalBalancePerUser()
        {
            DateTime date = DateTime.Now.Date;
            var firstDayToFetchFrom = new DateTime(date.Year, date.Month, 1);
            List<vwCCS_Total_Balance_With_UserInfo_V1.GroupPerCellPhone> dailyBalancePerUserList = new List<vwCCS_Total_Balance_With_UserInfo_V1.GroupPerCellPhone>();
            try
            {
                String currentCustomerID = GetCurrentCustomerGuidString();
                using (var _viewsCtx = (ViewsContext)ViewsContextFactory.GetInstance().GetDBEFContext())
                {
                    var dailyBalancePerUserTemp = (from u in _viewsCtx.vwTotalBalanceWithUserInfo
                                                   where u.CustomerID.ToString() == currentCustomerID && u.SnapshotDate >= firstDayToFetchFrom
                                                   group u by new { u.CellPhone, u.SnapshotDate } into g
                                                   select new
                                                   {
                                                       CellPhone = g.Key.CellPhone,
                                                       SnapshotDate = g.Key.SnapshotDate,
                                                       AmountDue = g.Sum(u => u.AmountDue),
                                                       Payment = g.Sum(u => u.Payment),
                                                       Balance = g.Sum(u => u.Balance),

                                                   }).OrderByDescending(u => u.SnapshotDate).ToList();

                    foreach (var item in dailyBalancePerUserTemp)
                    {
                        dailyBalancePerUserList.Add(new vwCCS_Total_Balance_With_UserInfo_V1.GroupPerCellPhone(item.SnapshotDate, item.CellPhone, item.AmountDue, item.Payment, item.Balance));
                    }
                    return View("DailyBalancePerUser", dailyBalancePerUserList);

                }
            }
            catch (Exception e)
            {
                ViewBag.Msg = e.Message;
                return View("GenericErrorMsgMrct");
            }


        }


        public List<CCSPrototypeCommon.vwCCS_Total_Balance_With_UserInfo_V1.GroupPerCellPhone> GetTotalBalanceFoeSingleUser(string CellPhone)
        {
            DateTime date = DateTime.Now.Date;
            var firstDayToFetchFrom = new DateTime(date.Year, date.Month, 1);
            List<vwCCS_Total_Balance_With_UserInfo_V1.GroupPerCellPhone> dailyBalancePerUserList = new List<vwCCS_Total_Balance_With_UserInfo_V1.GroupPerCellPhone>();
            try
            {
                String currentCustomerID = GetCurrentCustomerGuidString();
                using (var _viewsCtx = (ViewsContext)ViewsContextFactory.GetInstance().GetDBEFContext())
                {
                    var dailyBalancePerUserTemp = (from u in _viewsCtx.vwTotalBalanceWithUserInfo
                                                   where u.CustomerID.ToString() == currentCustomerID && u.SnapshotDate >= firstDayToFetchFrom && u.CellPhone.Equals(CellPhone)
                                                   group u by new { u.CellPhone, u.SnapshotDate } into g
                                                   select new
                                                   {
                                                       CellPhone = g.Key.CellPhone,
                                                       SnapshotDate = g.Key.SnapshotDate,
                                                       AmountDue = g.Sum(u => u.AmountDue),
                                                       Payment = g.Sum(u => u.Payment),
                                                       Balance = g.Sum(u => u.Balance),

                                                   }).OrderByDescending(u => u.SnapshotDate).ToList();

                    foreach (var item in dailyBalancePerUserTemp)
                    {
                        dailyBalancePerUserList.Add(new vwCCS_Total_Balance_With_UserInfo_V1.GroupPerCellPhone(item.SnapshotDate, item.CellPhone, item.AmountDue, item.Payment, item.Balance));
                    }
                    return dailyBalancePerUserList;

                }
            }
            catch (Exception e)
            {
                return null;
            }
        }

        [Authorize(Roles = "Staff")]
        public ActionResult BlacklistUsers()
        {
            var UserBalanceList = new List<UserBalanceModel>();
            try
            {
                using (var sqlConn = SQLConnFactory.GetInstance().GetSqlConnection())
                {
                    sqlConn.Open();
                    var queryDefaultBlackListTemplate = @"
declare @CustomerID nvarchar(max)
set @CustomerID = '{0}'
select * from [fnQueryDefaultBlackList] (@CustomerID)";

                    var queryDefaultBlackList = string.Format(queryDefaultBlackListTemplate,
                                            GetCurrentCustomerGuidString());
                    var sqlCmd = new SqlCommand(queryDefaultBlackList, sqlConn);
                    var reader = sqlCmd.ExecuteReader();
                    while (reader.Read())
                    {
                        var record = new UserBalanceModel
                        {
                            CellPhone = reader.GetString(0),
                            BillDue = reader.GetDouble(1),
                        };
                        UserBalanceList.Add(record);
                    }
                }
            }
            catch (Exception e)
            {
                ViewBag.Msg = e.Message;
                return View("GenericErrorMsgMrct");
            }

            return View(UserBalanceList);

        }

        [HttpPost]
        public ActionResult FilterPhoneNumberPayments(string filterValue)
        {
            List<CCSPrototypeCommon.vwCCS_Total_Balance_With_UserInfo_V1.GroupPerCellPhone> totalPayments = GetTotalBalanceFoeSingleUser(filterValue);
            return PartialView("~/Views/Merchant/DailyBalancePerUser_PrtView.cshtml", totalPayments);


        }

        public ActionResult VendorStartPage()
        {
            return View();
        }

        [HttpPost]
        public ActionResult VendorStartPage([Bind(Include = "CompanyCode")]CompanyCodeModel model)
        {

            return RedirectToAction("RegisterVendor", model);

            //using (var customerEFContext = (CustomerContext)CustomerContextFactory.GetInstance().GetDBEFContext())
            //{
            //    var record = customerEFContext.CustomerTable.AsNoTracking().Where(entry => entry.CompanyCode == model.CompanyCode).FirstOrDefault();
            //    if (record != null)
            //    {
            //        ViewBag.CompanyName = record.Name;
            //        return RedirectToAction("RegisterVendor", model);
            //    }
            //}
            //ViewBag.Error = "无效公司代码";

            //return RedirectToAction("GenericErrorMsgMrct");
        }

        public JsonResult VerifyCompanyCode([Bind(Include = "CompanyCode")]CompanyCodeModel model)
        {
            try
            {
                if (string.IsNullOrEmpty(model.CompanyCode))
                {
                    return Json(false);
                }

                using (var customerEFContext = (CustomerContext)CustomerContextFactory.GetInstance().GetDBEFContext())
                {

                    var record = customerEFContext.CustomerTable.AsNoTracking().Where(entry => entry.CompanyCode == model.CompanyCode).FirstOrDefault();

                    return Json(record != null);
                }
            }
            catch (DataException e)
            {
                logger.Error(string.Format("Failed to verify CompanyCode because exception was thrown {0}", e.Message));
                return Json(false);
            }

            catch (Exception e)
            {
                logger.Error(string.Format("Failed to verify CompanyCode because unknown exception was thrown {0}", e.Message));
                return Json(false);
            }

        }

        [HttpGet]
        [Authorize(Roles = "Vendor")]
        public ActionResult VendorExpiredPage(bool? AlreadyExpired, string VendorID)
        {
            ViewBag.AlreadyExpired = AlreadyExpired ?? false;
            CCS_Staff_V1 Vendor = null;
            Guid VendorGuid = Guid.Empty;
            if(string.IsNullOrEmpty(VendorID))
            {
                Vendor = GetCurrentStaff();
            }
            else if (Guid.TryParse(VendorID, out VendorGuid)
                && VendorGuid != Guid.Empty)
            {
                Vendor = GetStaffByGuid(VendorGuid);
            }

            if (Vendor == null)
            {
                ViewBag.Error = "无法找到该账户，请重试";
                return View("GenericErrorMsgMrct");
            }

            ViewBag.CompanyName = Vendor.CustomerTable.Name;
            ViewBag.CompanyID = Vendor.CustomerID.ToString();

            Session[CCSPrototypeCommon.SessionConstants.CustomerName] = Vendor.CustomerTable.Name;
            Session[CCSPrototypeCommon.SessionConstants.CustomerID] = Vendor.CustomerID.ToString();

            return View(Vendor);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        [Authorize(Roles = "Vendor")]
        public ActionResult VendorExpiredPage([Bind(Include = "StaffID, ExpirationDate, ADAccount")]CCS_Staff_V1 model,
            string ExtendMonth)
        {
            int monthCount = 0;
            if (string.IsNullOrWhiteSpace(ExtendMonth) || !int.TryParse(ExtendMonth, out monthCount)
                || monthCount < 1 || monthCount > 60)
            {
                ViewBag.Error = "月份数应从1到60。输入为: " + ExtendMonth;
                return View(model);
            }

            if (model.StaffID == Guid.Empty)
            {
                ViewBag.Error = "未指定续费账户，请重试";
                return View("GenericErrorMsgMrct");
            }
            //var adAccount = User.Identity.Name;


            // TBD need a new page for successfully extending vendor account.

            ViewBag.Error = null;

            CCS_SubscriptionPayment_V1 subscription = new CCS_SubscriptionPayment_V1()
            {
                PaymentID = new Guid(),
                Amount = 100 + 10 * monthCount,
                StaffID = model.StaffID,
                SubscriberStaff = null,
                ExtendMonth = monthCount
            };
            return RedirectToAction("PaySubscription", "Payments", subscription);
            //return View("ExtendVendorComplete",staff);
        }


        [HttpPost]
        [ValidateAntiForgeryToken]
        [Authorize(Roles = "Vendor")]
        public async Task<ActionResult> VendorExpiredPageTestPay([Bind(Include = "StaffID, ExpirationDate, ADAccount")]CCS_Staff_V1 model,
            string ExtendMonth)
        {
            int monthCount = 0;
            if (string.IsNullOrWhiteSpace(ExtendMonth) || !int.TryParse(ExtendMonth, out monthCount)
                || monthCount < 1 || monthCount > 60)
            {
                ViewBag.Error = "月份数应从1到60。输入为: " + ExtendMonth;
                return View(model);
            }

            if (model.StaffID == Guid.Empty)
            {
                ViewBag.Error = "未指定续费账户，请重试";
                return View("GenericErrorMsgMrct");
            }
            //var adAccount = User.Identity.Name;


            // TBD need a new page for successfully extending vendor account.

            ViewBag.Error = null;
            CCS_Staff_V1 staff = null;
            using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
            {
                staff = vendorEFContext.StaffTable.Where(entry => entry.StaffID == model.StaffID).FirstOrDefault();

                if (staff == null)
                {
                    ViewBag.Error = "未找到续费账户，请重试";
                    return View("GenericErrorMsgMrct");
                }
                else
                {
                    staff.ExpirationDate = staff.ExpirationDate.AddMonths(monthCount);
                    vendorEFContext.Configuration.ValidateOnSaveEnabled = false;
                    vendorEFContext.Entry(staff).Property(c => c.State).IsModified = true;
                    await vendorEFContext.SaveChangesAsync();
                }
            }
            return View("ExtendVendorComplete",staff);
        }

        [HttpGet]
        public ActionResult RegisterVendor([Bind(Include = "CompanyCode")]CompanyCodeModel model, 
            string CompanyIdStr, string AddToStaffId)
        {



            if (model != null && ModelState.IsValid
                && !string.IsNullOrWhiteSpace(model.CompanyCode))
            {

                using (var customerEFContext = (CustomerContext)CustomerContextFactory.GetInstance().GetDBEFContext())
                {
                    var record = customerEFContext.CustomerTable.AsNoTracking().Where(entry => entry.CompanyCode == model.CompanyCode 
                        && entry.State == CustomerAccountState.Active).FirstOrDefault();
                    if (record == null)
                    {
                        ViewBag.Error = "无效公司代码。";
                        return View("GenericErrorMsgMrct");
                    }

                    ViewBag.CompanyID = record.CustomerID.ToString();
                    ViewBag.CompanyName = record.Name;


                    Session[CCSPrototypeCommon.SessionConstants.CustomerName] = record.Name;
                    Session[CCSPrototypeCommon.SessionConstants.CustomerID] = record.CustomerID.ToString();
                }

            }
            else  
            {
                Guid TempCompanyGuid;
                Guid.TryParse(CompanyIdStr, out TempCompanyGuid);
            }


            CustomerModel cmodel = new CustomerModel();
            Guid ExistingStaffGuid = Guid.Empty;
            Guid.TryParse(AddToStaffId, out ExistingStaffGuid);
            string ADAcount = string.Empty;

            if (ExistingStaffGuid != Guid.Empty)
            {
                using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                {
                    var ExistingStaff = vendorEFContext.StaffTable.AsNoTracking().Where(entry => entry.StaffID == ExistingStaffGuid).FirstOrDefault();
                    if (ExistingStaff == null)
                    {
                        ViewBag.Error = "当前登录用户无效,请重新登录后添加授权。";
                        return View("GenericErrorMsgMrct");
                    }
                    else
                    {
                        ADAcount = ExistingStaff.ADAccount;
                        var duplication = vendorEFContext.StaffTable.AsNoTracking().Where(entry => entry.ADAccount == ADAcount
                            && entry.State == CustomerAccountState.Active && entry.CustomerTable.CompanyCode == model.CompanyCode).FirstOrDefault();

                        if (duplication == null)
                        {
                            cmodel.StaffTable.ADAccount = ExistingStaff.ADAccount;
                            cmodel.StaffTable.Address = ExistingStaff.Address;
                            cmodel.StaffTable.Cellphone = ExistingStaff.Cellphone;
                            cmodel.StaffTable.geography = ExistingStaff.geography;
                            cmodel.StaffTable.latitude = ExistingStaff.latitude;
                            cmodel.StaffTable.longitude = ExistingStaff.longitude;
                            cmodel.StaffTable.Name = ExistingStaff.Name;
                            cmodel.StaffTable.RedirectToUri = ExistingStaff.RedirectToUri;

                            ViewBag.AddOnly = true;
                        }
                        else
                        {
                            ViewBag.Error = "当前授权已经存在，无需重复申请。";
                            return View("GenericErrorMsgMrct");
                        }
                    }

                }
            }

            List<string> roleList = RoleNameAllowedToCreatedByCurrent();
            ViewBag.RoleList = new SelectList(roleList);

            return View(cmodel);
        }


        protected List<string> RoleNameAllowedToCreatedByCurrent()
        {
            List<string> roleList = new List<string>();

            if (User.IsInRole(OperationLevel.Admin.ToString()))
            {
                roleList.Add(OperationLevel.Staff.GetDisplayName());
            }
            else if (User.IsInRole(OperationLevel.SysAdmin.ToString()))
            {
                roleList.Add(OperationLevel.Admin.GetDisplayName());
            }
            else if (User.IsInRole(OperationLevel.Staff.ToString()))
            {
                roleList.Add(OperationLevel.Vendor.GetDisplayName());
                roleList.Add(OperationLevel.Gatherer.GetDisplayName());
                roleList.Add(OperationLevel.MeterReader.GetDisplayName());
            }
            else
            {
                roleList.Add(OperationLevel.Vendor.GetDisplayName());
            }

            return roleList;
        }
        [Authorize(Roles = "Staff, Admin")]
        public ActionResult ManageVendor()
        {
            var curCustomerID = new Guid(GetCurrentCustomerGuidString());
            var vendorMap = new Dictionary<Guid, int>();
            var vendorList = new List<VendorModel>();
            try
            {
                using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                {
                    var recordQuery = (from staff in vendorEFContext.StaffTable.AsNoTracking().Where(s => (s.CustomerID == curCustomerID && s.State == CustomerAccountState.Active))
                                       join staffInfo in vendorEFContext.StaffInfoTable.AsNoTracking().Where(si => si.Status == DBItemState.Valid).DefaultIfEmpty()
                                       on staff.StaffID equals staffInfo.StaffID
                                       into ssi
                                       from item in ssi.DefaultIfEmpty()
                                       select new
                                       {
                                           Staff = staff,
                                           StaffInfo = item
                                       });
                    if (User.IsInRole(OperationLevel.Staff.ToString()))
                    {
                        recordQuery = recordQuery.Where(r => (r.Staff.Role == OperationLevel.Vendor ||
                            r.Staff.Role == OperationLevel.Gatherer ||
                            r.Staff.Role == OperationLevel.MeterReader));
                    }
                    else if (User.IsInRole(OperationLevel.Admin.ToString()))
                    {
                        recordQuery = recordQuery.Where(r => r.Staff.Role == OperationLevel.Staff);
                    }
                    var recordList = recordQuery.ToList();
                    foreach (var record in recordList)
                    {
                        if (vendorMap.ContainsKey(record.Staff.StaffID))
                        {
                            if (record.StaffInfo != null)
                            {
                                vendorList[vendorMap[record.Staff.StaffID]].StaffInfoList.Add(record.StaffInfo);
                            }
                        }
                        else
                        {
                            vendorMap.Add(record.Staff.StaffID, vendorList.Count);
                            var vendor = new VendorModel()
                            {
                                StaffTable = record.Staff
                            };
                            if (record.StaffInfo != null)
                            {
                                vendor.StaffInfoList.Add(record.StaffInfo);
                            }
                            vendorList.Add(vendor);
                        }
                    }
                }
            }
            catch (DataException)
            {
                throw;
            }
            return View(vendorList);
        }

        [Authorize(Roles = "Staff, Admin, SysAdmin")]
        public async Task<ActionResult> DisableVendor(Guid? staffID)
        {
            string nextAction = "ManageVendor";
            string Cellphone = string.Empty;

            if (User.IsInRole(OperationLevel.SysAdmin.ToString()))
            {
                nextAction = "SetCompanyAdmin";
            }
            
            if (staffID == null)
            {
                return RedirectToAction(nextAction);
            }

            try
            {
                using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                {
                    //var vendor = new CCS_Staff_V1()
                    //{
                    //    StaffID = (Guid)staffID,
                    //    State = CustomerAccountState.Disable
                    //};
                    var staff = vendorEFContext.StaffTable.Find((Guid)staffID);

                    if (staff != null)
                    {
                        Cellphone = staff.Cellphone;
                        staff.State = CustomerAccountState.Disable;
                        staff.LastUpdatedDate = DateTime.UtcNow;

                        vendorEFContext.StaffTable.Attach(staff);
                        vendorEFContext.Configuration.ValidateOnSaveEnabled = false;
                        vendorEFContext.Entry(staff).Property(c => c.State).IsModified = true;
                        vendorEFContext.Entry(staff).Property(c => c.LastUpdatedDate).IsModified = true;

                        await vendorEFContext.SaveChangesAsync();
                    }
                }

                if (!string.IsNullOrEmpty(Cellphone))
                {
                    await CcsAdmin.Instance.DisableUserAsync(Cellphone);
                }
            }
            catch (DbEntityValidationException e)
            {
                ViewBag.Msg = string.Format("停用次账户失败: {1}。数据验证失败，具体原因: {0}", e.Message, staffID);
                
                logger.Error(ViewBag.Msg);

                logger.Error("DbEntityValidationException: " + e.Message);
                foreach (var eev in e.EntityValidationErrors)
                {
                    foreach (var error in eev.ValidationErrors)
                    {
                        logger.Error("Property:{0}, Error: {1} ", error.PropertyName, error.ErrorMessage);
                    }
                }
                return View("GenericErrorMsgMrct");
            }
            catch (DbUpdateException e)
            {
                ViewBag.Msg = string.Format("停用次账户失败: {1}。具体原因: {0}", e.Message, staffID);
                logger.Error(ViewBag.Msg);
                return View("GenericErrorMsgMrct");
            }
            catch (RetryLimitExceededException e)
            {
                ViewBag.Msg = string.Format("停用次账户失败: {1}。具体原因: {0}", e.Message, staffID);
                logger.Error(ViewBag.Msg);
                return View("GenericErrorMsgMrct");
            }
            catch (Exception e)
            {
                ViewBag.Msg = string.Format("停用次账户失败: {1}。具体原因: {0}", e.Message, staffID);
                logger.Error(ViewBag.Msg);
                return View("GenericErrorMsgMrct");

            }
            return RedirectToAction(nextAction);
        }

        [Authorize(Roles = "Staff, Admin, SysAdmin")]
        public ActionResult EditVendorInfo(Guid? staffID)
        {
            string nextAction = "ManageVendor";
            if (User.IsInRole(OperationLevel.SysAdmin.ToString()))
            {
                nextAction = "SetCompanyAdmin";
            }
            if (staffID == null)
            {
                return RedirectToAction(nextAction);
            }

            VendorModel vendor = new VendorModel();

            try
            {
                using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                {
                    var record = (from staff in vendorEFContext.StaffTable.AsNoTracking().Where(s => (s.StaffID == staffID && s.State == CustomerAccountState.Active))
                                  join staffInfo in vendorEFContext.StaffInfoTable.AsNoTracking().Where(si => si.Status == DBItemState.Valid).DefaultIfEmpty()
                                  on staff.StaffID equals staffInfo.StaffID
                                  into ssi
                                  from item in ssi.DefaultIfEmpty()
                                  select new
                                  {
                                      Staff = staff,
                                      StaffInfo = item
                                  }).ToList();
                    if(record!= null && record.Count > 0)
                    {
                        vendor.StaffTable = record[0].Staff;
                    };
                    foreach (var info in record)
                    {
                        if (info.StaffInfo != null)
                        {
                            vendor.StaffInfoList.Add(info.StaffInfo);
                        }
                    }
                    vendor.ProfilePictureUri = GetProfilePictureUri(staffID.ToString());
                    //VendorModel.Current = vendor;
                }
            }
            catch (DataException)
            {
                throw;
            }
            return View(vendor);
        }

        [HttpPost]
        //[Authorize(Roles = "Staff, Admin, SysAdmin")]
        public ActionResult ViewVendorInfo(Guid? StaffGUID)
        {
            if (StaffGUID == null)
            {
                return PartialView("_ViewVendorInfo");
            }
            CCS_Staff_V1 StaffRecord = null;
            using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
            {
                vendorEFContext.Configuration.LazyLoadingEnabled = false;
                StaffRecord = vendorEFContext.StaffTable.Find(StaffGUID);

                if (vendorEFContext.Entry(StaffRecord).Collection(s => s.StaffInfos).IsLoaded == false)
                {
                    vendorEFContext.Entry(StaffRecord).Collection(s => s.StaffInfos).Load();
                }


                if (StaffRecord.StaffInfos == null)
                {
                    StaffRecord.StaffInfos = new List<CCS_StaffInfo_V1>();
                }
                else if (StaffRecord.StaffInfos.Count > 0)
                {
                    int count = StaffRecord.StaffInfos.Count;
                    List<CCS_StaffInfo_V1> DisabledInfo = new List<CCS_StaffInfo_V1>();
                    foreach (CCS_StaffInfo_V1 staffInfo in StaffRecord.StaffInfos)
                    {
                        if (staffInfo.Status == DBItemState.Invalid)
                        {
                            DisabledInfo.Add(staffInfo);
                        }
                    }

                    foreach (CCS_StaffInfo_V1 staffInfo in DisabledInfo)
                    {
                        StaffRecord.StaffInfos.Remove(staffInfo);
                    }
                }
            }

            return PartialView("_ViewVendorInfo", StaffRecord);
        }

        [HttpPost]
        [Authorize(Roles = "Staff, Admin, SysAdmin")]
        public async Task<ActionResult> EditVendorInfo(FormCollection collection,
            [Bind(Include = "StaffTable")]VendorModel vendor,
            string[] Name, string[] Content)
        {
            ViewBag.LoginRole = OperationLevel.Unkown;

            string nextAction = "ManageVendor";
            if (User.IsInRole(OperationLevel.SysAdmin.ToString()))
            {
                nextAction = "SetCompanyAdmin";
            }

            if (vendor.StaffTable == null)
            {
                ViewBag.Msg = "编辑账户信息失败。未指定账户GUID。";
                logger.Error(ViewBag.Msg);
                return View("GenericErrorMsgMrct");
            }

            try
            {
                if (ModelState.IsValid)
                {
                    using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                    {
                        var OriginalStaffTable = vendorEFContext.StaffTable.Find(vendor.StaffTable.StaffID);

                        if (OriginalStaffTable == null)
                        {
                            ViewBag.Msg = string.Format("编辑账户信息失败。无法通过Guid找到该账户信息: {0}", vendor.StaffTable.StaffID);
                            logger.Error("Unable to find Staff record with StaffID =  " + vendor.StaffTable.StaffID);
                            return View("GenericErrorMsgMrct");

                        }
                        string key = string.Empty;
                        foreach (PropertyInfo p in (typeof(CCS_Staff_V1)).GetProperties())
                        {
                            if ((p.PropertyType.IsValueType && p.PropertyType.Name != "Guid")
                                || p.PropertyType.Name == "String")
                            {
                                key = "StaffTable." + p.Name;

                                if (ModelState.ContainsKey(key))
                                {
                                    p.SetValue(OriginalStaffTable, p.GetValue(vendor.StaffTable));
                                }
                            }

                        }
                        OriginalStaffTable.LastUpdatedDate = DateTime.UtcNow;
                        OriginalStaffTable.geography = DbGeography.FromText(string.Format("POINT({1} {0})", OriginalStaffTable.latitude, OriginalStaffTable.longitude));

                        vendorEFContext.StaffTable.Attach(OriginalStaffTable);
                        vendorEFContext.Entry(OriginalStaffTable).State = System.Data.Entity.EntityState.Modified;

                        //TBD. Need switch to Dictionary for better preformance
                        List<CCS_StaffInfo_V1> OriginalStaffInfoList = new List<CCS_StaffInfo_V1>();
                        if (OriginalStaffTable.StaffInfos != null)
                        {
                            OriginalStaffInfoList.AddRange(OriginalStaffTable.StaffInfos);
                        }

                        // TBD. Need rewrite based on Dictionary to solve overwriting and improve performance
                        //if (collection.AllKeys.Contains("item.Name") &&
                        //    collection.AllKeys.Contains("item.Value"))

                        {
                            //string[] keyList = collection["item.Name"].Split(',');
                            //string[] valueList = collection["item.Value"].Split(',');
                            int KeyLength = Name == null ? 0 : Name.Length;
                            int ValueLength = Content == null ? 0 : Content.Length;

                            for (int i = 0; i < KeyLength; i++)
                            {
                                if (string.IsNullOrWhiteSpace(Name[i]))
                                {
                                    continue;
                                }

                                if (i < OriginalStaffInfoList.Count)
                                {
                                    var staffInfo = OriginalStaffInfoList[i];
                                    staffInfo.Name = Name[i];
                                    staffInfo.Content = Content == null ? string.Empty : (i < Content.Length ? Content[i] : string.Empty);

                                    vendorEFContext.StaffInfoTable.Attach(staffInfo);
                                    vendorEFContext.Entry(staffInfo).State = System.Data.Entity.EntityState.Modified;
                                }
                                else
                                {
                                    var staffInfo = new CCS_StaffInfo_V1()
                                    {
                                        StaffID = OriginalStaffTable.StaffID,
                                        Name = Name[i],
                                        Content = Content == null ? string.Empty : (i < Content.Length ? Content[i] : string.Empty)
                                    };
                                    vendorEFContext.StaffInfoTable.Attach(staffInfo);
                                    vendorEFContext.Entry(staffInfo).State = System.Data.Entity.EntityState.Added;
                                }
                            }

                            for (int i = KeyLength; i < OriginalStaffInfoList.Count; i++)
                            {
                                var staffInfo = OriginalStaffInfoList[i];
                                vendorEFContext.StaffInfoTable.Attach(staffInfo);
                                vendorEFContext.Entry(staffInfo).State = System.Data.Entity.EntityState.Deleted;
                            }
                        }

                        await vendorEFContext.SaveChangesAsync();
                    }
                }
                else
                {
                    System.Text.StringBuilder sb = new System.Text.StringBuilder("编辑账户信息失败。数据格式错误: ");

                    foreach (var pair in ModelState)
                    {
                        if(pair.Value.Errors.Count > 0)
                        {
                            sb.Append(pair.Key);
                            sb.Append(";");
                        }
                    }

                    ViewBag.Msg = sb.ToString();
                    logger.Error(ViewBag.Msg);

                    return View("GenericErrorMsgMrct");
                }
            }
            catch (DbEntityValidationException e)
            {
                ViewBag.Msg = string.Format("编辑账户信息失败。数据验证失败，具体原因: {0}", e.Message);
                logger.Error(ViewBag.Msg);

                logger.Error("DbEntityValidationException: " + e.Message);
                foreach (var eev in e.EntityValidationErrors)
                {
                    foreach (var error in eev.ValidationErrors)
                    {
                        logger.Error("Property:{0}, Error: {1} ", error.PropertyName, error.ErrorMessage);
                    }
                }
                return View("GenericErrorMsgMrct");
            }
            catch (DbUpdateException e)
            {
                ViewBag.Msg = string.Format("更新账户信息失败。具体原因: {0}", e.Message);
                logger.Error(ViewBag.Msg);
                return View("GenericErrorMsgMrct");
            }
            catch (Exception e)
            {
                ViewBag.Msg = string.Format("编辑账户信息失败。具体原因: {0}", e.Message);
                logger.Error(ViewBag.Msg);
                return View("GenericErrorMsgMrct");
            }

            return RedirectToAction(nextAction);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public async Task<ActionResult> RegisterVendor(string RoleType, string CompanyIdStr, 
            string[] Name, string[] Content,
            [Bind(Include = "StaffTable, Password, ComparePassword")]CustomerModel customer)
        {
            // for vendor register
            try
            {
                logger.Info(string.Format("Trying to register a user"));

                if (customer != null && customer.StaffTable != null)
                {
                    logger.Info(string.Format("Trying to register with staff table as follows: cell {0}, name {1}", customer.StaffTable.Cellphone, customer.StaffTable.Name));
                }
                else
                {
                    string msg = "Either customer or customer staff table is NULL. The method cannot proceed with the next steps";
                    logger.Error(msg);
                    throw new Exception(msg);
                }

                Guid curCustomerID;
                if (string.IsNullOrEmpty(CompanyIdStr) || !Guid.TryParse(CompanyIdStr, out curCustomerID))
                {
                    if (User.IsInRole(OperationLevel.SysAdmin.ToString()))
                    {
                        logger.Info(string.Format("Going back to SetCompanyAdmin page. Unable to register new Admin account under current System Admin, because of invalidate Company GUID: {0}", CompanyIdStr));
                        return RedirectToAction("SetCompanyAdmin");
                    }
                    curCustomerID = new Guid(GetCurrentCustomerGuidString());

                    logger.Info(string.Format("Current customer id is {0}", curCustomerID));

                }
                var company = GetCustomerInfo(curCustomerID);
                if (ModelState.IsValid)
                {

                    bool AppendOnly = false;
                    if (!string.IsNullOrWhiteSpace(customer.StaffTable.ADAccount)
                        && customer.StaffTable.ADAccount == User.Identity.Name)
                    {
                        AppendOnly = true;
                    }
                    // Fill default values
                    customer.StaffTable.LastUpdatedDate = DateTime.UtcNow;
                    customer.StaffTable.ExpirationDate = DateTime.UtcNow;

                    // To do: Convert to phonetic symbol
                    customer.StaffTable.CustomerID = curCustomerID;

                    using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                    {
                        if (string.IsNullOrWhiteSpace(customer.StaffTable.Name) ||
                            ((string.IsNullOrWhiteSpace(customer.Password) || string.IsNullOrWhiteSpace(customer.ComparePassword)) && !AppendOnly))
                        {
                            // to do return webpage an error
                            return View(customer);
                        }
                        customer.StaffTable.geography = DbGeography.FromText(string.Format("POINT({1} {0})", customer.StaffTable.latitude, customer.StaffTable.longitude));
                        vendorEFContext.StaffTable.Add(customer.StaffTable);

                        if (Name != null && Name.Length > 0)
                        {
                            for (int index = 0; index < Name.Length; ++index)
                            {
                                if (string.IsNullOrWhiteSpace(Name[index]))
                                {
                                    continue;
                                }
                                var staffInfo = new CCS_StaffInfo_V1()
                                {
                                    Name = Name[index],
                                    Content = Content == null ? string.Empty: (index < Content.Length ? Content[index] : string.Empty)
                                };
                                vendorEFContext.StaffInfoTable.Add(staffInfo);
                            }
                        }

                        //
                        // ;
                        OperationLevel TargetRole = ConvertRoleType(RoleType);
                        if (RoleNameAllowedToCreatedByCurrent().Contains(TargetRole.GetDisplayName()))
                        {
                            customer.StaffTable.Role = TargetRole;
                        }
                        else
                        {
                            customer.StaffTable.Role = OperationLevel.Vendor;
                        }

                        logger.Info(string.Format("Creating a user in AD with domain {0} and username {1}", company.DomainName, customer.StaffTable.Cellphone.ToString()));
                        customer.StaffTable.ADDomain = company.DomainName;

                        if (AppendOnly)
                        {
                            logger.Info(string.Format("User {0} has been granted permission to Company {1}", customer.StaffTable.ADAccount, company.Name));
                        }
                        else
                        {
                            customer.StaffTable.ADAccount =
                                await CcsAdmin.Instance.CreateUserAsync(customer.StaffTable.Cellphone.ToString(),
                                customer.Password, null, customer.StaffTable.Role.ToString(), company.DomainName);
                            logger.Info(string.Format("User {0} has been added successfully", customer.StaffTable.ADAccount));

                        }

                        vendorEFContext.SaveChanges();
                        logger.Info("Changes are successfully saved");
                        return View("RegisterResult", new MerchantRegisterUserModel { VendorAccountName = customer.StaffTable.ADAccount,
                            AppendOnly = AppendOnly, Company = company });
                    }
                }
            }
            catch (DbEntityValidationException e)
            {
                ViewBag.Msg = string.Format("注册账户失败。数据验证失败，具体原因: {0}", e.Message);
                logger.Error(ViewBag.Msg);

                logger.Error("DbEntityValidationException: " + e.Message);
                foreach (var eev in e.EntityValidationErrors)
                {
                    foreach (var error in eev.ValidationErrors)
                    {
                        logger.Error("Property:{0}, Error: {1} ", error.PropertyName, error.ErrorMessage);
                    }
                }
                return View("GenericErrorMsgMrct");
            }
            catch (DbUpdateException e)
            {
                logger.Error(string.Format("DbUpdateException {0}", e.Message));

                throw;
            }
            catch (RetryLimitExceededException e)
            {
                logger.Error(string.Format("RetryLimitExceededException {0}", e.Message));
                //Log the error (uncomment dex variable name and add a line here to write a log.)
                ModelState.AddModelError("", "Unable to save changes. Try again, and if the problem persists, see your system administrator.");
            }
            catch (Exception e)
            {
                logger.Error(string.Format("Unknown exception {0}", e.Message));
                throw;
            }
            List<string> roleList = RoleNameAllowedToCreatedByCurrent();
            ViewBag.RoleList = new SelectList(roleList);

            return View(customer);
        }

        protected static Dictionary<string, OperationLevel> CnName2RoleType = null;
        protected static OperationLevel ConvertRoleType(string RoleType, OperationLevel DefaultRoleType = OperationLevel.Vendor)
        {

            if(CnName2RoleType == null)
            {
                CnName2RoleType = new Dictionary<string, OperationLevel>();

                foreach (OperationLevel r in Enum.GetValues(typeof(OperationLevel)))
                {
                    CnName2RoleType[r.GetDisplayName().ToLower()] = r;
                }
            }

            if (string.IsNullOrWhiteSpace(RoleType))
            {
                return DefaultRoleType;
            }
            else if(CnName2RoleType.ContainsKey(RoleType.ToLower()))
            {
                return CnName2RoleType[RoleType];
            }
            else
            {
                return DefaultRoleType;
            }
            //if (string.IsNullOrEmpty(RoleType))
            //{
            //    customer.StaffTable.Role = OperationLevel.Vendor;
            //}
            //else if (string.Compare(RoleType, OperationLevel.Staff.GetDisplayName(), StringComparison.OrdinalIgnoreCase) == 0)
            //{
            //    customer.StaffTable.Role = OperationLevel.Staff;
            //}
            //else if (string.Compare(RoleType, OperationLevel.Admin.GetDisplayName(), StringComparison.OrdinalIgnoreCase) == 0)
            //{
            //    customer.StaffTable.Role = OperationLevel.Admin;
            //}
            //else if (string.Compare(RoleType, OperationLevel.Gatherer.GetDisplayName(), StringComparison.OrdinalIgnoreCase) == 0)
            //{
            //    customer.StaffTable.Role = OperationLevel.Gatherer;
            //}
            //else if (string.Compare(RoleType, OperationLevel.MeterReader.GetDisplayName(), StringComparison.OrdinalIgnoreCase) == 0)
            //{
            //    customer.StaffTable.Role = OperationLevel.MeterReader;
            //}
            //else
            //{
            //    customer.StaffTable.Role = OperationLevel.Vendor;
            //}
        }

        [Authorize(Roles = "SysAdmin")]
        public ActionResult RegisterCompany()
        {
            var selectList = Enum.GetValues(
                typeof(CCSPrototypeCommon.GeoRegion)).Cast<CCSPrototypeCommon.GeoRegion>().Where(
                e => e != CCSPrototypeCommon.GeoRegion.Unkown).Select(
                e => new SelectListItem
                {
                    Value = ((int)e).ToString(),
                    Text = e.GetDisplayName()
                });
            ViewBag.RegionList = selectList;
            return View();
        }

        [HttpPost]
        [Authorize(Roles = "SysAdmin")]
        public ActionResult RegisterCompany(FormCollection collection,
            [Bind(Include = "CustomerTable")]CompanyModel company)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    // Fill default values
                    company.CustomerTable.LastUpdatedDate = DateTime.UtcNow;

                    using (var customerEFContext = (CustomerContext)CustomerContextFactory.GetInstance().GetDBEFContext())
                    {
                        if (string.IsNullOrWhiteSpace(company.CustomerTable.Name) ||
                            string.IsNullOrWhiteSpace(company.CustomerTable.DomainName) ||
                            string.IsNullOrWhiteSpace(company.CustomerTable.CompanyCode))
                        {
                            // to do return webpage an error
                            return RedirectToAction("RegisterCompany");
                        }
                        customerEFContext.CustomerTable.Add(company.CustomerTable);

                        if (collection.AllKeys.Contains("item.Name") &&
                            collection.AllKeys.Contains("item.Value"))
                        {
                            string[] keyList = collection["item.Name"].Split(',');
                            string[] valueList = collection["item.Value"].Split(',');
                            for (int index = 0; index < keyList.Length; ++index)
                            {
                                if (string.IsNullOrWhiteSpace(keyList[index]))
                                {
                                    continue;
                                }
                                var customerInfo = new CCS_CustomerInfo_V1()
                                {
                                    Name = keyList[index],
                                    Value = valueList[index]
                                };
                                customerEFContext.CustomerInfoTable.Add(customerInfo);
                            }
                        }
                        customerEFContext.SaveChanges();
                        return RedirectToAction("ManageCompany");
                    }
                }
            }
            catch (DbEntityValidationException e)
            {
                ViewBag.Msg = string.Format("注册公司信息失败。数据验证失败，具体原因: {0}", e.Message);
                logger.Error(ViewBag.Msg);

                logger.Error("DbEntityValidationException: " + e.Message);
                foreach (var eev in e.EntityValidationErrors)
                {
                    foreach (var error in eev.ValidationErrors)
                    {
                        logger.Error("Property:{0}, Error: {1} ", error.PropertyName, error.ErrorMessage);
                    }
                }
                return View("GenericErrorMsgMrct");
            }
            catch (DbUpdateException)
            {
                throw;
            }
            catch (RetryLimitExceededException /* dex */)
            {
                //Log the error (uncomment dex variable name and add a line here to write a log.)
                ModelState.AddModelError("", "Unable to save changes. Try again, and if the problem persists, see your system administrator.");
            }
            var selectList = Enum.GetValues(
                typeof(CCSPrototypeCommon.GeoRegion)).Cast<CCSPrototypeCommon.GeoRegion>().Where(
                e => e != CCSPrototypeCommon.GeoRegion.Unkown).Select(
                e => new SelectListItem
                {
                    Value = ((int)e).ToString(),
                    Text = e.GetDisplayName()
                });
            ViewBag.RegionList = selectList;
            return View();
        }

        [Authorize(Roles = "SysAdmin")]
        public ActionResult ManageCompany()
        {
            var companyInfoMap = new Dictionary<Guid, int>();
            var companyList = new List<CompanyModel>();
            try
            {
                using (var customerEFContext = (CustomerContext)CustomerContextFactory.GetInstance().GetDBEFContext())
                {
                    var recordList = (from company in customerEFContext.CustomerTable.AsNoTracking().Where(c => (c.State == CustomerAccountState.Active))
                                      join companyInfo in customerEFContext.CustomerInfoTable.AsNoTracking().Where(ci => ci.Status == DBItemState.Valid).DefaultIfEmpty()
                                      on company.CustomerID equals companyInfo.CustomerID
                                      into ssi
                                      from item in ssi.DefaultIfEmpty()
                                      select new
                                      {
                                          Company = company,
                                          CompanyInfo = item
                                      }).ToList();
                    foreach (var record in recordList)
                    {
                        if (companyInfoMap.ContainsKey(record.Company.CustomerID))
                        {
                            if (record.CompanyInfo != null)
                            {
                                companyList[companyInfoMap[record.Company.CustomerID]].CustomerInfoList.Add(record.CompanyInfo);
                            }
                        }
                        else
                        {
                            companyInfoMap.Add(record.Company.CustomerID, companyList.Count);
                            var company = new CompanyModel()
                            {
                                CustomerTable = record.Company
                            };
                            if (record.CompanyInfo != null)
                            {
                                company.CustomerInfoList.Add(record.CompanyInfo);
                            }
                            companyList.Add(company);
                        }
                    }
                }
            }
            catch (DataException)
            {
                throw;
            }
            return View(companyList);
        }

        [Authorize(Roles = "SysAdmin")]
        public ActionResult EditCompanyInfo(Guid? companyID)
        {
            if (companyID == null)
            {
                return RedirectToAction("ManageCompany");
            }
            try
            {
                using (var customerEFContext = (CustomerContext)CustomerContextFactory.GetInstance().GetDBEFContext())
                {
                    var record = (from company in customerEFContext.CustomerTable.AsNoTracking().Where(c => (c.CustomerID == companyID && c.State == CustomerAccountState.Active))
                                  join companyInfo in customerEFContext.CustomerInfoTable.AsNoTracking().Where(ci => ci.Status == DBItemState.Valid).DefaultIfEmpty()
                                  on company.CustomerID equals companyInfo.CustomerID
                                  into ssi
                                  from item in ssi.DefaultIfEmpty()
                                  select new
                                  {
                                      Company = company,
                                      CompanyInfo = item
                                  }).ToList();
                    var companay = new CompanyModel()
                    {
                        CustomerTable = record[0].Company
                    };
                    foreach (var info in record)
                    {
                        if (info.CompanyInfo != null)
                        {
                            companay.CustomerInfoList.Add(info.CompanyInfo);
                        }
                    }
                    CompanyModel.Current = companay;
                }
            }
            catch (DataException)
            {
                throw;
            }
            var selectList = Enum.GetValues(
                typeof(CCSPrototypeCommon.GeoRegion)).Cast<CCSPrototypeCommon.GeoRegion>().Where(
                e => e != CCSPrototypeCommon.GeoRegion.Unkown).Select(
                e => new SelectListItem
                {
                    Value = ((int)e).ToString(),
                    Text = e.GetDisplayName()
                });
            ViewBag.RegionList = selectList;

            return View(CompanyModel.Current);
        }


        [HttpPost]
        [Authorize(Roles = "SysAdmin")]
        public async Task<ActionResult> EditCompanyInfo(FormCollection collection,
            [Bind(Include = "CustomerTable")]CustomerModel company)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    using (var customerEFContext = (CustomerContext)CustomerContextFactory.GetInstance().GetDBEFContext())
                    {
                        var unmodifiedCompany = CompanyModel.Current;
                        company.CustomerTable.LastUpdatedDate = DateTime.UtcNow;
                        company.CustomerTable.CustomerID = unmodifiedCompany.CustomerTable.CustomerID;
                        company.CustomerTable.State = unmodifiedCompany.CustomerTable.State;
                        customerEFContext.Entry(company.CustomerTable).State = System.Data.Entity.EntityState.Modified;
                        if (collection.AllKeys.Contains("item.Name") &&
                            collection.AllKeys.Contains("item.Value"))
                        {
                            string[] keyList = collection["item.Name"].Split(',');
                            string[] valueList = collection["item.Value"].Split(',');
                            var newLength = keyList.Length;
                            var oldLength = unmodifiedCompany.CustomerInfoList.Count;
                            for (int index = 0; index < ((newLength <= oldLength) ? newLength : oldLength); ++index)
                            {
                                if (string.IsNullOrWhiteSpace(keyList[index]))
                                {
                                    continue;
                                }
                                var customerInfo = new CCS_CustomerInfo_V1()
                                {
                                    CustomerInfoID = unmodifiedCompany.CustomerInfoList[index].CustomerInfoID,
                                    CustomerID = unmodifiedCompany.CustomerTable.CustomerID,
                                    Name = keyList[index],
                                    Value = valueList[index],
                                    Status = unmodifiedCompany.CustomerInfoList[index].Status
                                };
                                customerEFContext.Entry(customerInfo).State = System.Data.Entity.EntityState.Modified;
                            }
                            for (int index = oldLength; index < newLength; ++index)
                            {
                                if (string.IsNullOrWhiteSpace(keyList[index]))
                                {
                                    continue;
                                }
                                var customerInfo = new CCS_CustomerInfo_V1()
                                {
                                    CustomerID = unmodifiedCompany.CustomerTable.CustomerID,
                                    Name = keyList[index],
                                    Value = valueList[index]
                                };
                                customerEFContext.Entry(customerInfo).State = System.Data.Entity.EntityState.Added;
                            }
                            for (int index = newLength; index < oldLength; ++index)
                            {
                                var staffInfo = new CCS_StaffInfo_V1()
                                {
                                    StaffInfoID = unmodifiedCompany.CustomerInfoList[index].CustomerInfoID
                                };
                                customerEFContext.Entry(staffInfo).State = System.Data.Entity.EntityState.Deleted;
                            }
                        }
                        await customerEFContext.SaveChangesAsync();
                    }
                }
            }
            catch (DbEntityValidationException e)
            {
                ViewBag.Msg = string.Format("编辑公司信息失败。数据验证失败，具体原因: {0}", e.Message);
                logger.Error(ViewBag.Msg);

                logger.Error("DbEntityValidationException: " + e.Message);
                foreach (var eev in e.EntityValidationErrors)
                {
                    foreach (var error in eev.ValidationErrors)
                    {
                        logger.Error("Property:{0}, Error: {1} ", error.PropertyName, error.ErrorMessage);
                    }
                }
                return View("GenericErrorMsgMrct");
            }
            catch (DbUpdateException)
            {
                throw;
            }
            return RedirectToAction("ManageCompany");
        }


        [Authorize(Roles = "SysAdmin")]
        public ActionResult SetCompanyAdmin()
        {
            var companyList = new List<CompanyModel>();
            try
            {
                using (var customerEFContext = (CustomerContext)CustomerContextFactory.GetInstance().GetDBEFContext())
                {
                    var recordList = (from company in customerEFContext.CustomerTable.AsNoTracking().Where(c => (c.State == CustomerAccountState.Active))
                                      select new
                                      {
                                          Company = company,
                                      }).ToList();
                    foreach (var record in recordList)
                    {
                        var company = new CompanyModel()
                        {
                            CustomerTable = record.Company
                        };
                        companyList.Add(company);
                    }
                }
            }
            catch (DataException)
            {
                throw;
            }
            return View(companyList);
        }

        [Authorize(Roles = "SysAdmin")]
        public async Task<ActionResult> EditMeterAppNotice()
        {

            String blobName = Utils.Constants.MeterAppNoticeBlobName;
            if (string.IsNullOrEmpty(blobName))
            {
                return View();
            }
            MerchantMsgModel adsMsgModel = null;
            try
            {
                var msgBlob = _adsBlobArchiveContainer.GetBlockBlobReference(blobName);
                string blobContent = await msgBlob.DownloadTextAsync();
                adsMsgModel = await Task.Factory.StartNew(() => JsonConvert.DeserializeObject<MerchantMsgModel>(blobContent));
            }
            catch (Exception)
            {
                //To do: 
            }
            return (adsMsgModel == null) ? View() : View(adsMsgModel);
        }

        [HttpPost]
        [ValidateInput(false)]
        [Authorize(Roles = "SysAdmin")]
        public async Task<ActionResult> EditMeterAppNotice(
            [Bind(Include = "Title, Author, Digest, Content, BlobName")]MerchantMsgModel adsMsgModel, string Button)
        {
            if (ModelState.IsValid && adsMsgModel != null)
            {
                try
                {
                    var blobcontent = await Task.Factory.StartNew(() => JsonConvert.SerializeObject(adsMsgModel));

                    var blobName = Utils.Constants.MeterAppNoticeBlobName;
                    if (string.IsNullOrEmpty(adsMsgModel.BlobName))
                    {
                        adsMsgModel.BlobName = blobName;
                    }
                    var content = JsonConvert.SerializeObject(adsMsgModel);

                    var fileBlob = _adsBlobArchiveContainer.GetBlockBlobReference(adsMsgModel.BlobName);
                    await fileBlob.UploadTextAsync(content);

                }
                catch (Exception)
                {
                    //To do: 
                    throw;
                }
            }
            else
            {
                throw new Exception("Model is invalid");
            }
            return View();
        }


        [HttpPost]
        [Authorize(Roles = "SysAdmin")]
        public ActionResult OnSelectCompanyChanged(string CompanyIdStr)
        {
            Guid companyID;
            if (string.IsNullOrEmpty(CompanyIdStr) || !Guid.TryParse(CompanyIdStr, out companyID))
            {
                return RedirectToAction("SetCompanyAdmin");
            }
            var vendorMap = new Dictionary<Guid, int>();
            var vendorList = new List<VendorModel>();
            try
            {
                using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                {
                    var recordQuery = (from staff in vendorEFContext.StaffTable.AsNoTracking().Where(s => (s.CustomerID == companyID && s.State == CustomerAccountState.Active))
                                       join staffInfo in vendorEFContext.StaffInfoTable.AsNoTracking().Where(si => si.Status == DBItemState.Valid).DefaultIfEmpty()
                                       on staff.StaffID equals staffInfo.StaffID
                                       into ssi
                                       from item in ssi.DefaultIfEmpty()
                                       select new
                                       {
                                           Staff = staff,
                                           StaffInfo = item
                                       });
                    recordQuery = recordQuery.Where(r => r.Staff.Role == OperationLevel.Admin);
                    var recordList = recordQuery.ToList();
                    foreach (var record in recordList)
                    {
                        if (vendorMap.ContainsKey(record.Staff.StaffID))
                        {
                            if (record.StaffInfo != null)
                            {
                                vendorList[vendorMap[record.Staff.StaffID]].StaffInfoList.Add(record.StaffInfo);
                            }
                        }
                        else
                        {
                            vendorMap.Add(record.Staff.StaffID, vendorList.Count);
                            var vendor = new VendorModel()
                            {
                                StaffTable = record.Staff
                            };
                            if (record.StaffInfo != null)
                            {
                                vendor.StaffInfoList.Add(record.StaffInfo);
                            }
                            vendorList.Add(vendor);
                        }
                    }
                }
            }
            catch (DataException)
            {
                throw;
            }
            return PartialView("_ManageVendor", vendorList);
        }

        // TBD. TO-DO, move CompanyIdStr into StaffTable.CustomerID
        //[HttpPost]
        //[Authorize(Roles = "Staff, Admin, SysAdmin")]
        public JsonResult CheckPhoneNumberAvailable([Bind(Include = "Cellphone, StaffID")]CCS_Staff_V1 StaffTable, string CompanyIdStr)
        {
            try
            {

                if(StaffTable == null)
                {
                    return Json(false);
                }
                
                Guid customerId;
                if (!Guid.TryParse(CompanyIdStr, out customerId))
                {
                    customerId = new Guid(GetCurrentCustomerGuidString());
                }

                //var unmodifiedVendor = VendorModel.Current;
                //CCS_Staff_V1 unmodifiedVendorStaffTable = unmodifiedVendor != null ? unmodifiedVendor.StaffTable : null;
                //Guid unmodifiedVendorStaffTableGuid = unmodifiedVendorStaffTable != null ? unmodifiedVendorStaffTable.StaffID : Guid.Empty;
                string domainName = (GetCustomerInfo(customerId)).DomainName;
                string adAccount = string.Format("{0}@{1}", StaffTable.Cellphone, domainName);

                logger.Info(string.Format("Checking if {0} account is available", adAccount));

                using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                {
                    if (StaffTable.StaffID != Guid.Empty)
                    {

                        var record = (from staff in vendorEFContext.StaffTable.AsNoTracking().Where(s => (s.ADAccount == adAccount &&
                            s.StaffID != StaffTable.StaffID && s.State == CustomerAccountState.Active))
                                      select new
                                      {
                                          Staff = staff
                                      }).ToList();

                        logger.Info(string.Format("{0} account is available", adAccount));

                        return Json(record.Count == 0);
                    }
                    else
                    {

                        var record = (from staff in vendorEFContext.StaffTable.AsNoTracking().Where(s => (s.ADAccount == adAccount &&
                           s.State == CustomerAccountState.Active))
                                      select new
                                      {
                                          Staff = staff
                                      }).ToList();

                        logger.Info(string.Format("{0} account is available", adAccount));
                        return Json(record.Count == 0);
                    }
                }
            }
            catch (DataException e)
            {
                logger.Error(string.Format("Account is NOT available because exception was thrown {}", e.Message));
                return Json(false);
            }

            catch (Exception e)
            {
                logger.Error(string.Format("Account is NOT available because unknown exception was thrown {0}", e.Message));
                return Json(false);
            }

        }


        [Authorize(Roles = "Staff, Admin, Vendor")]
        public ActionResult EditVendorProfile(Guid? StaffID)
        {
            CCS_Staff_V1 staff = null;
            if (StaffID == null || StaffID == Guid.Empty)
            {
                staff = GetCurrentStaff();
            }
            else
            {
                staff = GetStaffByGuid((Guid)StaffID);
            }


            if (staff == null)
            {
                return View("Index");
            }

            VendorModel model = new VendorModel(){
                ProfilePictureUri = GetProfilePictureUri(staff.StaffID.ToString()),
                StaffTable = staff,
                StaffInfoList = GetStaffInfosByGuid(staff.StaffID)
            };

            return View(model);
        }

        [HttpPost]
        [Authorize(Roles = "Staff, Admin, Vendor")]
        public async Task<ActionResult> UploadVendorPicture(Guid? StaffID, HttpPostedFileBase uploadFile)
        {
            var nextAction = "EditVendorProfile";
            if (ModelState.IsValid)
            {
                if (uploadFile == null || uploadFile.ContentLength == 0)
                {
                    ViewBag.UploadMsg = "Nothing to upload. Please choose a file.";
                }
                else
                {
                    // Blob path format: StaffID/yyyyMMddHHmmss
                    var dateTimeFormat = "yyyyMMddHHmmss";
                    var vendor = GetCurrentStaff();
                    if (vendor == null)
                    {
                        ViewBag.Msg = string.Format("未找登录账户。请重新登录。");
                        return View("GenericErrorMsgMrct");
                    }
                    Guid staffGuid = StaffID ?? vendor.StaffID;
                    if (staffGuid == Guid.Empty)
                    {
                        ViewBag.Msg = string.Format("未找到指定账户。请重试。");
                        return View("GenericErrorMsgMrct");
                    }
                    var blobName = String.Format("{0}/{1}", staffGuid.ToString(), DateTime.UtcNow.ToString(dateTimeFormat));
                    var fileBlob = _vendorProfilePictureBlobContainer.GetBlockBlobReference(blobName);

                    // Upload local csv file to blob storage
                    using (var fileStream = uploadFile.InputStream)
                    {
                        await fileBlob.UploadFromStreamAsync(fileStream);
                    }
                    if (fileBlob == null)
                    {
                        ViewBag.UploadMsg = string.Format("图片上传成功：{0}", uploadFile.FileName);
                    }
                    else
                    {
                        return this.Redirect(Request.UrlReferrer.ToString());
                    }
                }
            }
            return RedirectToAction(nextAction, StaffID);
        }


        /*Deprecated
        [HttpPost]
        [Authorize(Roles = "Staff, Admin, SysAdmin")]
        public async Task<ActionResult> OldRegister(
            [Bind(Include = "CustomerTable, StaffTable, Password, ComparePassword")]CustomerModel customer)
        {
            // for test
            try
            {
                if (ModelState.IsValid)
                {
                    // Fill default values
                    customer.CustomerTable.LastUpdatedDate = DateTime.UtcNow;
                    // Set default customer region for now
                    customer.CustomerTable.Region = GeoRegion.Beijing;
                    customer.StaffTable.LastUpdatedDate = customer.CustomerTable.LastUpdatedDate;

                    // To do: Convert to phonetic symbol
                    customer.StaffTable.Name = customer.CustomerTable.Name;

                    var customerEFContext = (CustomerContext)CustomerContextFactory.GetInstance().GetDBEFContext();
                    customerEFContext.CustomerTable.Add(customer.CustomerTable);
                    customerEFContext.StaffTable.Add(customer.StaffTable);
                    if (string.IsNullOrWhiteSpace(customer.CustomerTable.Name) ||
                        string.IsNullOrWhiteSpace(customer.Password) ||
                        string.IsNullOrWhiteSpace(customer.ComparePassword))
                    {
                        // to do return webpage an error
                        return View(customer);
                    }
                    customer.StaffTable.geography = DbGeography.FromText(string.Format("POINT({1} {0})", customer.StaffTable.latitude, customer.StaffTable.longitude));
                    customerEFContext.SaveChanges();
                    // Customer ID is generated by DB
                    customer.StaffTable.ADAccount =
                        await CcsAdmin.Instance.CreateUserAsync(customer.StaffTable.StaffID.ToString(), customer.Password, null, OperationLevel.Vendor.ToString());

                    customer.StaffTable.ADDomain = CcsAdmin.Instance.DomainName;
                    customerEFContext.SaveChanges();
                    return View("RegisterResult", new MerchantRegisterUserModel { VendorAccountName = customer.StaffTable.ADAccount });
                }
            }
            catch (RetryLimitExceededException)
            {
                //Log the error (uncomment dex variable name and add a line here to write a log.)
                ModelState.AddModelError("", "Unable to save changes. Try again, and if the problem persists, see your system administrator.");
            }
            return View(customer);
        }*/

        //// Redirect to the action in Map controller
        //public ActionResult ShowMap(double? CenterLatitude, double? CenterLongitude, string YunziIDs, string Guids,
        //    int? StartIndex, int? Count, string ShowType, string Market, string MapStyle, string UserId,
        //    bool? FullScreen, bool? NavigateOnLoad, bool? FetchMoreDataAfterCenterChange)
        //{
        //    return RedirectToAction("ShowMap", "Map",
        //        new
        //        {
        //            CenterLatitude = CenterLatitude,
        //            CenterLongitude = CenterLongitude,
        //            YunziIDs = YunziIDs,
        //            Guids = Guids,
        //            StartIndex = StartIndex,
        //            Count = Count,
        //            ShowType = ShowType,
        //            Market = Market,
        //            MapStyle = MapStyle,
        //            UserId = UserId,
        //            FullScreen = FullScreen,
        //            NavigateOnLoad = NavigateOnLoad,
        //            FetchMoreDataAfterCenterChange = FetchMoreDataAfterCenterChange
        //        });
        //}

        [Authorize(Roles = "Staff, Vendor")]
        public async Task<ActionResult> CreatePublicMsg(string blobName)
        {
            Guid StaffGuid = Guid.Empty;
            if (IsVendorExpired(string.Empty, out StaffGuid))
            {
                return RedirectToAction("VendorExpiredPage", new { AlreadyExpired = true, VendorID = StaffGuid.ToString()});
            }
            ViewBag.StaffID = StaffGuid.ToString();

            if (string.IsNullOrEmpty(blobName))
            {
                return View();
            }
            MerchantMsgModel adsMsgModel = null;
            try
            {
                var msgBlob = _adsBlobArchiveContainer.GetBlockBlobReference(blobName);
                string blobContent = await msgBlob.DownloadTextAsync();
                adsMsgModel = await Task.Factory.StartNew(() => JsonConvert.DeserializeObject<MerchantMsgModel>(blobContent));
            }
            catch (Exception e)
            {
                logger.logException(e); 
            }
            return (adsMsgModel == null) ? View() : View(adsMsgModel);
        }

        public void SendPushNotificationToStaffSubscribers(Guid staffid, String msg)
        {
            logger.Info(string.Format("Send push notification from staff {0}. Message {1}", staffid, msg));
            try
            {
                var usersStaffFavorites = _billDB.UserStaffFavoritesTable.Where(o => o.StaffID == staffid && o.FavoriteStatus == FavoriteStatus.Favorite && o.ModifiedDate == null);
                PushNotificationsManager.PushNotifManagerInstance.SendPushNotificationToUserStaffFavorites(usersStaffFavorites, msg);
            }
            catch (Exception e)
            {
                logger.logException(e);
            }


        }

        [HttpPost]
        [ValidateInput(false)]
        [Authorize(Roles = "Staff, Vendor")]
        public async Task<ActionResult> CreatePublicMsg(
            [Bind(Include = "Title, Author, Digest, Content, BlobName")]MerchantMsgModel adsMsgModel, string Button)
        {

            Guid StaffGuid = Guid.Empty;

            if (IsVendorExpired(string.Empty, out StaffGuid))
            {
                return RedirectToAction("VendorExpiredPage", new { AlreadyExpired = true, VendorID = StaffGuid.ToString() });
            }
            
            ViewBag.StaffID = StaffGuid.ToString();

            if (ModelState.IsValid && adsMsgModel != null)
            {
                try
                {
                    var blobcontent = await Task.Factory.StartNew(() => JsonConvert.SerializeObject(adsMsgModel));
                    // Blob path format: CustomerID/yyyy/MM/dd/HHmmss
                    var dateTimeFormat = "yyyy/MM/dd/HHmmss";
                    // to do: remove customerID hardcode 
                    var customerID = GetCurrentCustomerGuidString();
                    var blobName = String.Format("{0}/{1}", customerID, DateTime.UtcNow.ToString(dateTimeFormat));
                    if (string.IsNullOrEmpty(adsMsgModel.BlobName))
                    {
                        adsMsgModel.BlobName = blobName;
                    }
                    var content = JsonConvert.SerializeObject(adsMsgModel);

                    if(string.IsNullOrEmpty(Button))
                    {
                        ;
                    }
                    else if (Button.ToLower() == "save" || Button == "保存")
                    {
                        var fileBlob = _adsBlobArchiveContainer.GetBlockBlobReference(adsMsgModel.BlobName);
                        await fileBlob.UploadTextAsync(content);
                    }
                    else if (Button.ToLower() == "publish" || Button == "发布")
                    {
                        var fileBlob = _adsBlobContainer.GetBlockBlobReference(blobName);
                        await fileBlob.UploadTextAsync(content);
                        string msgToPush = adsMsgModel.Content;
                        msgToPush = msgToPush.SplitWithNewLine();
                        msgToPush = msgToPush.Substring(0, Math.Min(Constants.PushNotifMaxLength, msgToPush.Length));
                        SendPushNotificationToStaffSubscribers(StaffGuid, msgToPush);

                    }
                }
                catch (Exception e)
                {
                    //To do: 
                    logger.logException(e);
                }
            }

            if (!string.IsNullOrEmpty(Button)
                && (Button.ToLower() == "save" || Button == "保存"))
            {
                return RedirectToAction("ManageMsg");
            }
            return View();
        }
        [Authorize(Roles = "Staff, Vendor")]
        public async Task<ActionResult> ManageMsg()
        {
            Guid StaffGuid = Guid.Empty;

            if (IsVendorExpired(string.Empty, out StaffGuid))
            {
                return RedirectToAction("VendorExpiredPage", new { AlreadyExpired = true, VendorID = StaffGuid.ToString() });
            }
            ViewBag.StaffID = StaffGuid.ToString();

            string customerID = GetCurrentCustomerGuidString();
            var msgList = new List<MerchantMsgModel>();
            string prefix = string.Format("{0}", customerID);
            foreach (IListBlobItem item in _adsBlobArchiveContainer.ListBlobs(prefix, true))
            {
                var blob = (CloudBlockBlob)item;
                string blobContent = await blob.DownloadTextAsync();
                MerchantMsgModel msgModel = JsonConvert.DeserializeObject<MerchantMsgModel>(blobContent);
                msgList.Add(msgModel);
            }

            return View(msgList);
        }

        [Authorize(Roles = "Staff")]
        public ActionResult UploadBill(string uploadMsg)
        {
            ViewBag.UploadMsg = uploadMsg;
            var customerId = GetCurrentCustomerGuidString();
            var query = new TableQuery<BillTemplateTableEntity>().Where(
                TableQuery.GenerateFilterCondition("PartitionKey",
                QueryComparisons.Equal, customerId));
            var billTypeList = new List<string>();
            var billTemplateEntityList = new List<BillTemplateTableEntity>();
            foreach (var entity in _billTemplateTable.ExecuteQuery(query))
            {
                billTypeList.Add(entity.RowKey);
                billTemplateEntityList.Add(entity);
            }
            billTypeList.Add(CCSPrototypeCommon.Constants.NewBillTypeDeclaration);
            Session["BillTemplateTableEntity"] = billTemplateEntityList;
            var list = new SelectList(billTypeList);
            ViewBag.billTypeList = list;
            return View();
        }

        [Authorize(Roles = "Admin")]
        public ActionResult SetMeterProfileTemplate()
        {
            var templateModel = new ColumnMappingTemplateModel();
            var customerID = GetCurrentCustomerGuidString();
            var result = _csvTemplateTable.Execute(TableOperation.Retrieve(customerID, Constants.MeterTemplateName));
            if (result == null || result.HttpStatusCode == 404 || result.Result == null)
            {
                result = _csvTemplateTable.Execute(TableOperation.Retrieve(customerID.ToUpperInvariant(), Constants.MeterTemplateName));
                if (result == null || result.HttpStatusCode == 404 || result.Result == null)
                {
                    return View();
                }
            }
            Type meterDataClassType = typeof(CCS_MeterData_V1);
            MemberInfo[] meterDataFieldList = meterDataClassType.GetProperties();

            var template = (DynamicTableEntity)result.Result;
            foreach (var prop in template.Properties)
            {
                templateModel.ExistMapping.Add(prop.Key, prop.Value.StringValue);
                foreach (var member in meterDataFieldList)
                {
                    if (!member.Name.Equals(prop.Key, StringComparison.InvariantCultureIgnoreCase))
                    {
                        continue;
                    }
                    var attribute = (DisplayNameAttribute)member.GetCustomAttribute(typeof(DisplayNameAttribute), false);
                    if (attribute != null)
                    {
                        templateModel.TemplateColumnList.Add(attribute.DisplayName);
                        templateModel.TemplateColumnTranslating.Add(member.Name, attribute.DisplayName);
                    }
                    else
                    {
                        templateModel.TemplateColumnList.Add(member.Name);
                        templateModel.TemplateColumnTranslating.Add(member.Name, member.Name);
                    }
                }
            }

            return View(templateModel);
        }

        [HttpPost]
        [Authorize(Roles = "Admin")]
        [ValidateAntiForgeryToken]
        public async Task<ActionResult> SetMeterProfileTemplate(FormCollection collection, string Button)
        {
            var nextView = "SetMeterProfileTemplate";
            if (Button.Equals("提交"))
            {
                var customerID = GetCurrentCustomerGuidString();
                var templateName = Constants.MeterTemplateName;
                var templateEntity = new DynamicTableEntity(customerID, templateName);
                Type meterDataClassType = typeof(CCS_MeterData_V1);
                MemberInfo[] meterDataFieldList = meterDataClassType.GetProperties();
                var validColunmHeaderList = new List<string>();
                var templateColumnTranslating = new Dictionary<string, string>();
                foreach (var member in meterDataFieldList)
                {
                    var attribute = (DisplayNameAttribute)member.GetCustomAttribute(typeof(DisplayNameAttribute), false);
                    if (attribute != null)
                    {
                        validColunmHeaderList.Add(attribute.DisplayName);
                        templateColumnTranslating.Add(attribute.DisplayName, member.Name);
                    }
                    else
                    {
                        validColunmHeaderList.Add(member.Name);
                        templateColumnTranslating.Add(member.Name, member.Name);
                    }
                }

                foreach (var key in collection.AllKeys)
                {
                    if (!validColunmHeaderList.Contains(key) || string.IsNullOrWhiteSpace(collection[key]))
                    {
                        continue;
                    }
                    templateEntity.Properties.Add(templateColumnTranslating[key], new EntityProperty(collection[key]));
                }
                var upsertOperation = TableOperation.InsertOrReplace(templateEntity);
                await _csvTemplateTable.ExecuteAsync(upsertOperation);

                var blobName = (string)Session["uploadBlobName"];
                ViewBag.uploadMsg = "设置成功";
                return RedirectToAction(nextView);
            }
            return View(nextView);
        }

        [HttpPost]
        [Authorize(Roles = "Admin")]
        [ValidateAntiForgeryToken]
        public async Task<ActionResult> UploadMeterProfileForTemplate(HttpPostedFileBase uploadFile)
        {
            string ErrorMsgView = "_ErrorMessage";

            if (ModelState.IsValid && uploadFile != null && uploadFile.ContentLength != 0)
            {
                // Blob path format: CustomerID/yyyy/MMdd/HHmmss.fileExtension
                string dateTimeFormat = "yyyy/MMdd/HHmmss";
                var customerID = GetCurrentCustomerGuidString();
                string blobName = String.Format("{0}/{1}{2}", customerID, DateTime.UtcNow.ToString(dateTimeFormat), Path.GetExtension(uploadFile.FileName));
                CloudBlockBlob fileBlob = _meterProfileBlobContainer.GetBlockBlobReference(blobName);

                // Upload local csv file to blob storage
                using (var fileStream = uploadFile.InputStream)
                {
                    await fileBlob.UploadFromStreamAsync(fileStream);
                }
                if (fileBlob == null)
                {
                    ViewBag.UploadMsg = string.Format("上传文件 [{0}] 失败 ", uploadFile.FileName);
                    return PartialView(ErrorMsgView);
                }
                else
                {
                    var templateModel = new ColumnMappingTemplateModel();
                    string[] columns = null;
                    using (Stream fs = fileBlob.OpenRead())
                    {
                        StreamReader csv = new StreamReader(fs);
                        // Get header
                        string line = csv.ReadLine();
                        columns = line.Split(',');
                    }
                    templateModel.CsvHeaderList = Utility.GetCsvTableHeader(columns);
                    templateModel.TemplateType = Constants.MeterTemplateName;
                    Type meterDataClassType = typeof(CCS_MeterData_V1);
                    MemberInfo[] meterDataFieldList = meterDataClassType.GetProperties();
                    foreach (var member in meterDataFieldList)
                    {
                        if (member.Name.Equals("MeterDataID", StringComparison.InvariantCultureIgnoreCase) ||
                            member.Name.Equals("CustomerID", StringComparison.InvariantCultureIgnoreCase))
                        {
                            continue;
                        }
                        var attribute = (DisplayNameAttribute)member.GetCustomAttribute(typeof(DisplayNameAttribute), false);
                        if (attribute != null)
                        {
                            templateModel.TemplateColumnList.Add(attribute.DisplayName);
                            templateModel.TemplateColumnTranslating.Add(attribute.DisplayName, member.Name);
                        }
                        else
                        {
                            templateModel.TemplateColumnList.Add(member.Name);
                            templateModel.TemplateColumnTranslating.Add(member.Name, member.Name);
                        }
                    }

                    var templateExist = true;
                    var result = _csvTemplateTable.Execute(TableOperation.Retrieve(customerID, Constants.MeterTemplateName));
                    if (result == null || result.HttpStatusCode == 404 || result.Result == null || columns == null)
                    {
                        result = _csvTemplateTable.Execute(TableOperation.Retrieve(customerID.ToUpperInvariant(), Constants.MeterTemplateName));
                        if (result == null || result.HttpStatusCode == 404 || result.Result == null || columns == null)
                        {
                            return PartialView("_SetColumnMappingTemplate", templateModel);
                        }
                    }
                    var template = (DynamicTableEntity)result.Result;
                    foreach (var field in meterDataFieldList)
                    {
                        if (field.Name.Equals("MeterDataID", StringComparison.InvariantCultureIgnoreCase) ||
                            field.Name.Equals("CustomerID", StringComparison.InvariantCultureIgnoreCase))
                        {
                            continue;
                        }
                        EntityProperty prop;
                        if (template.Properties.TryGetValue(field.Name, out prop))
                        {
                            try
                            {
                                var index = Utility.GetColumnByHeader(templateModel.CsvHeaderList, prop.StringValue);
                                templateModel.ExistMapping.Add(field.Name, templateModel.CsvHeaderList[index]);
                            }
                            catch (TemplateDataException)
                            {
                                continue;
                            }
                        }
                    }
                    return PartialView("_SetColumnMappingTemplate", templateModel);
                }
            }
            ViewBag.UploadMsg = "上传失败或者文件为空";
            return PartialView(ErrorMsgView);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        [Authorize(Roles = "Staff")]
        public async Task<ActionResult> UploadBill(string billType, HttpPostedFileBase uploadFile)
        {
            string nextView = "UploadBill";

            if (ModelState.IsValid)
            {
                if (uploadFile == null || uploadFile.ContentLength == 0)
                {
                    ViewBag.UploadMsg = "Nothing to upload. Please choose a file.";
                }
                else
                {
                    if (billType.Equals(CCSPrototypeCommon.Constants.NewBillTypeDeclaration))
                    {
                        billType = string.Empty;
                    }
                    // Blob path format: CustomerID/yyyy/MMdd/HHmmss.fileExtension
                    string dateTimeFormat = "yyyy/MMdd/HHmmss";
                    // to do: remove customerID hardcode 
                    var customerID = GetCurrentCustomerGuidString();
                    string blobName = String.Format("{0}/{1}{2}", customerID, DateTime.UtcNow.ToString(dateTimeFormat), Path.GetExtension(uploadFile.FileName));
                    CloudBlockBlob fileBlob = _billBlobContainer.GetBlockBlobReference(blobName);

                    // Upload local csv file to blob storage
                    using (var fileStream = uploadFile.InputStream)
                    {
                        await fileBlob.UploadFromStreamAsync(fileStream);
                    }
                    if (fileBlob == null)
                    {
                        ViewBag.UploadMsg = string.Format("Uploading file '{0}' is unsuccessful", uploadFile.FileName);
                    }
                    else
                    {
                        if (Session["BillTemplateTableEntity"] == null)
                        {
                            ViewBag.uploadMsg = "Please try again";
                            return RedirectToAction(nextView);
                        }
                        var billTemplateEntityList = (List<BillTemplateTableEntity>)Session["BillTemplateTableEntity"];
                        string[] columns = null;
                        using (Stream fs = fileBlob.OpenRead())
                        {
                            StreamReader csv = new StreamReader(fs);
                            // Header
                            string line = csv.ReadLine();
                            columns = line.Split(',');
                        }
                        if (columns == null)
                        {
                            ViewBag.UploadMsg = "This file is not supported";
                        }
                        try
                        {
                            var billTemplate = Utility.GetTemplateEntity(billType, billTemplateEntityList);
                            var billTemplateMapping = Utility.GetColumnMappingByHeader(new List<string>(columns), billTemplate.GetColumnList());
                        }
                        catch (TemplateDataException billExp)
                        {
                            var billTemplateModel = new BillTemplateModel()
                            {
                                BillType = billType,
                                BillHeaderList = Utility.GetCsvTableHeader(columns),
                                BillTemplateColumnList = BillTemplateTableEntity.GetColumnDefinationList()
                            };
                            ViewBag.UploadMsg = billExp;
                            Session["uploadBlobName"] = blobName;
                            nextView = "SetBillTemplate";
                            return View(nextView, billTemplateModel);
                        }
                        var queueMsg = new BillQueueMsg()
                        {
                            BillType = billType,
                            BlobName = blobName
                        };
                        nextView = "UploadBill";
                        var queueMsgString = await Task.Factory.StartNew(() => JsonConvert.SerializeObject(queueMsg));
                        CloudQueueMessage queueMessage = new CloudQueueMessage(queueMsgString);
                        await _billQueue.AddMessageAsync(queueMessage);
                        ViewBag.uploadMsg = "上传成功";
                        return RedirectToAction(nextView);
                    }
                }
            }
            return View(nextView);
        }

        [Authorize(Roles = "Staff")]
        public ActionResult UploadPayment(string uploadMsg)
        {
            ViewBag.UploadMsg = uploadMsg;
            return View();
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        [Authorize(Roles = "Staff")]
        public async Task<ActionResult> UploadPayment(HttpPostedFileBase uploadFile)
        {
            string nextView = "UploadPayment";

            if (ModelState.IsValid)
            {
                if (uploadFile == null || uploadFile.ContentLength == 0)
                {
                    ViewBag.UploadMsg = "Nothing to upload. Please choose a file.";
                }
                else
                {
                    // Blob path format: CustomerID/yyyy/MMdd/HHmmss.fileExtension
                    string dateTimeFormat = "yyyy/MMdd/HHmmss";
                    // to do: remove customerID hardcode 
                    var customerID = GetCurrentCustomerGuidString();
                    string blobName = String.Format("{0}/{1}{2}", customerID, DateTime.UtcNow.ToString(dateTimeFormat), Path.GetExtension(uploadFile.FileName));
                    CloudBlockBlob fileBlob = _uploadedPaymentsBlobContainer.GetBlockBlobReference(blobName);

                    // Upload local csv file to blob storage
                    using (var fileStream = uploadFile.InputStream)
                    {
                        await fileBlob.UploadFromStreamAsync(fileStream);
                    }
                    if (fileBlob == null)
                    {
                        ViewBag.UploadMsg = string.Format("Uploading file '{0}' is unsuccessful", uploadFile.FileName);
                    }
                    else
                    {

                        string[] columns = null;
                        using (Stream fs = fileBlob.OpenRead())
                        {
                            StreamReader csv = new StreamReader(fs);
                            // Header
                            string line = csv.ReadLine();
                            columns = line.Split(',');
                        }
                        if (columns == null)
                        {
                            ViewBag.UploadMsg = "This file is not supported";
                        }

                        var queueMsg = new UploadPaymentQueueMsg()
                        {
                            BlobName = blobName
                        };
                        var queueMsgString = await Task.Factory.StartNew(() => JsonConvert.SerializeObject(queueMsg));
                        CloudQueueMessage queueMessage = new CloudQueueMessage(queueMsgString);
                        await _uploadedPaymentsQueue.AddMessageAsync(queueMessage);
                        ViewBag.uploadMsg = "上传成功";
                        return RedirectToAction(nextView);
                    }
                }
            }
            return View(nextView);
        }


        [Authorize(Roles = "Staff")]
        public ActionResult UploadMeterData(string uploadMsg)
        {
            ViewBag.uploadMsg = uploadMsg;
            return View();
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        [Authorize(Roles = "Staff")]
        public async Task<ActionResult> UploadMeterData(HttpPostedFileBase uploadFile)
        {
            string nextView = "UploadMeterData";

            if (ModelState.IsValid)
            {
                if (uploadFile == null || uploadFile.ContentLength == 0)
                {
                    ViewBag.UploadMsg = "Nothing to upload. Please choose a file.";
                }
                else
                {
                    // Blob path format: CustomerID/yyyy/MMdd/HHmmss.fileExtension
                    string dateTimeFormat = "yyyy/MMdd/HHmmss";
                    // to do: remove customerID hardcode 
                    var customerID = GetCurrentCustomerGuidString();
                    const string prefix = "Raw Data";
                    string blobName = String.Format("{0}-{1}/{2}{3}", prefix, customerID, DateTime.UtcNow.ToString(dateTimeFormat), Path.GetExtension(uploadFile.FileName));
                    CloudBlockBlob fileBlob = _rawDataBlobContainer.GetBlockBlobReference(blobName);

                    // Upload local csv file to blob storage
                    using (var fileStream = uploadFile.InputStream)
                    {
                        await fileBlob.UploadFromStreamAsync(fileStream);
                    }

                    if (fileBlob == null)
                    {
                        ViewBag.UploadMsg = string.Format("Uploading file '{0}' is unsuccessful", uploadFile.FileName);
                    }
                    else
                    {
                        string[] columns = null;
                        using (Stream fs = fileBlob.OpenRead())
                        {
                            StreamReader csv = new StreamReader(fs);
                            // Get header
                            string line = csv.ReadLine();
                            columns = line.Split(',');
                        }
                        if (columns == null)
                        {
                            ViewBag.UploadMsg = "This file is not supported";
                        }
                        else
                        {
                            RawDataQueueMsg queueMsg = new RawDataQueueMsg()
                            {
                                MeasureType = null,  // Reserved for now
                                BlobName = blobName
                            };
                            var queueMsgString = await Task.Factory.StartNew(() => JsonConvert.SerializeObject(queueMsg));
                            CloudQueueMessage queueMessage = new CloudQueueMessage(queueMsgString);
                            await _rawDataQueue.AddMessageAsync(queueMessage);
                            ViewBag.uploadMsg = "上传成功";
                            return RedirectToAction(nextView);
                        }
                    }
                }
            }
            return View(nextView);
        }


        [HttpPost]
        public JsonResult UploadRawDataMobile()
        {
            string data = "NA";
            string responseMsg = "NA";
            try
            {
                //data = new System.IO.StreamReader(Request.InputStream).ReadToEnd();
                System.IO.Stream stream = Request.InputStream;
                // Blob path format: CustomerID/yyyy/MMdd/HHmmss.fileExtension
                string dateTimeFormat = "yyyy/MMdd/HHmmss";
                // to do: remove customerID hardcode 
                var customerID = "EBCTest";
                const string prefix = "Raw Data";
                string blobName = String.Format("{0}-{1}/{2}{3}", prefix, customerID, DateTime.UtcNow.ToString(dateTimeFormat), "Filename");
                CloudBlockBlob fileBlob = _rawDataBlobContainer.GetBlockBlobReference(blobName);
                responseMsg += " Uploading " + blobName;
                // Upload local csv file to blob storage

                using (var fileStream = stream)
                {
                    responseMsg += " start ";
                    fileBlob.UploadFromStream(fileStream);
                }
                responseMsg += " done ";
                if (fileBlob == null)
                {
                    responseMsg += " fileblob is null ";
                }
                else
                {
                    responseMsg += " fileblob is NOT null ";

                    string[] columns = null;
                    using (Stream fs = fileBlob.OpenRead())
                    {
                        StreamReader csv = new StreamReader(fs);
                        // Get header
                        string line = csv.ReadLine();
                        columns = line.Split(',');
                    }
                    if (columns == null)
                    {
                        responseMsg += " This file is not supported ";
                    }
                    else
                    {
                        responseMsg += " This file is  supported ";

                        RawDataQueueMsg queueMsg = new RawDataQueueMsg()
                        {
                            MeasureType = null,  // Reserved for now
                            BlobName = blobName
                        };
                        var queueMsgString = JsonConvert.SerializeObject(queueMsg);
                        CloudQueueMessage queueMessage = new CloudQueueMessage(queueMsgString);
                        _rawDataQueue.AddMessageAsync(queueMessage);
                        responseMsg += " 上传成功 ";
                    }


                }

            }
            catch (Exception ex)
            {
                return new JsonResult() { Data = new { statusCode = "Sucess", statusMsg = ex.Message + " " + responseMsg + " " + data }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
            }
            return new JsonResult() { Data = new { statusCode = "Sucess", statusMsg = responseMsg + " " + data }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
        }



        [Authorize(Roles = "Staff")]
        public async Task<ActionResult> DownloadBill()
        {
            return View();
        }

        [Authorize(Roles = "Staff")]
        public async Task<ActionResult> Download(string month, string BlobUri)
        {
            try
            {
                CloudBlockBlob blob = new CloudBlockBlob(new Uri(BlobUri));

                using (MemoryStream memStream = new MemoryStream())
                {
                    await blob.DownloadToStreamAsync(memStream);

                    return new FileContentResult(memStream.ToArray(), blob.Properties.ContentType)
                    {
                        FileDownloadName = month + BlobUri.Substring(BlobUri.LastIndexOf('-') + 1)
                    };
                }
            }
            catch (Exception)
            {
                return new HttpNotFoundResult();
            }

        }

        [Authorize(Roles = "Staff")]
        public ActionResult DownloadFileFromBlob()
        {
            string view = "HomeWaterCompanyPaymentDownload";

            DateTime time = DateTime.UtcNow;
            foreach (IListBlobItem blob in paymentsBlobContainer.ListBlobs(null, false))
            {
                Uri blobUri;
                string curMonth = time.ToString("yyyy-MM");
                if (!BlobHashMap.TryGetValue(curMonth, out blobUri))
                {
                    BlobHashMap.Add(curMonth, blob.Uri);
                }

                time = time.AddMonths(-1);
            }

            return View(view, BlobHashMap);
        }

        /*
        [HttpPost]
        [ValidateAntiForgeryToken]
        public async Task<ActionResult> UploadFileToBlob(HttpPostedFileBase uploadFile)
        {
            CloudBlockBlob fileBlob = null;
            string view = "HomeWaterCompany";

            if (ModelState.IsValid)
            {
                if (uploadFile == null || uploadFile.ContentLength == 0)
                {
                    ViewBag.UploadMsg = "Nothing to upload. Please choose a file.";
                }
                else
                {
                    // Blob path format: CustomerID/yyyy/MMdd/HHmmss.fileExtension
                    string dateTimeFormat = "yyyy/MMdd/HHmmss";
                    // to do: remove customerID hardcode 
                    string blobName = String.Format("{0}/{1}{2}", "E1581298-5894-44EE-8FE3-7017D09B206B", DateTime.UtcNow.ToString(dateTimeFormat), Path.GetExtension(uploadFile.FileName));
                    fileBlob = filesBlobContainer.GetBlockBlobReference(blobName);

                    // Upload local csv file to blob storage
                    using (var fileStream = uploadFile.InputStream)
                    {
                        await fileBlob.UploadFromStreamAsync(fileStream);
                    }

                    if (fileBlob == null)
                    {
                        ViewBag.UploadMsg = string.Format("Uploading file '{0}' is unsuccessful", uploadFile.FileName);
                    }
                    else
                    {
                        string fileURL = fileBlob.Uri.ToString();

                        // Add a message to message queue
                        CloudQueueMessage queueMessage = new CloudQueueMessage(blobName);
                        await filesQueue.AddMessageAsync(queueMessage);

                        ViewBag.UploadMsg = string.Format("File '{0}' is uploaded to blob '{1}' @ '{2}'", uploadFile.FileName, blobName, fileURL);
                        view = "HomeWaterCompanyBillDisplay";
                    }

                }
            }
            return View(view);
        }*/





        [HttpPost]
        public ActionResult AddCustomizeInfo(int? index)
        {
            ViewBag.CustomizeInfoID = Guid.NewGuid().ToString();
            return PartialView("_CustomizeInfo");
        }

        [HttpPost]
        public PartialViewResult RemoveCustomizeInfo(int? index)
        {
            return PartialView("_CustomizeInfo");
        }

        [ValidateAntiForgeryToken]
        public async Task<ActionResult> SetBillTemplate(FormCollection collection, string Button)
        {
            var nextView = "UploadBill";
            if (Session["uploadBlobName"] == null || Session["BillTemplateTableEntity"] == null)
            {
                return RedirectToAction(nextView, new { uploadMsg = "Please try again" });
            }
            if (Button.Equals("提交"))
            {
                var customerID = GetCurrentCustomerGuidString();
                var billType = collection["BillType"];
                if (string.IsNullOrWhiteSpace(billType) || billType.Equals(CCSPrototypeCommon.Constants.NewBillTypeDeclaration))
                {
                    ViewBag.UploadMsg = "Invalid bill type";
                    return RedirectToAction(nextView, new { uploadMsg = "To do: redirect to set bill template page" });
                }

                var billTemplateEntity = new BillTemplateTableEntity(customerID, billType)
                {
                    AmountDueColumn = collection["AmountDue"],
                    UserNameColumn = collection["UserName"],
                    UserAddressColumn = collection["UserAddress"],
                    CellphoneColumn = collection["Cellphone"],
                    BillPeriodColumn = collection["BillPeriod"]
                };
                if (!string.IsNullOrWhiteSpace(collection["AccountNumber"]))
                {
                    billTemplateEntity.AccountNumberColumn = collection["AccountNumber"];
                }
                if (!string.IsNullOrWhiteSpace(collection["UserNumber"]))
                {
                    billTemplateEntity.UserNumberColumn = collection["UserNumber"];
                }
                var insertOperation = TableOperation.InsertOrReplace(billTemplateEntity);
                await _billTemplateTable.ExecuteAsync(insertOperation);

                var blobName = (string)Session["uploadBlobName"];
                var queueMsg = new BillQueueMsg()
                {
                    BillType = billType,
                    BlobName = blobName
                };
                var queueMsgString = await Task.Factory.StartNew(() => JsonConvert.SerializeObject(queueMsg));
                CloudQueueMessage queueMessage = new CloudQueueMessage(queueMsgString);
                await _billQueue.AddMessageAsync(queueMessage);
                return RedirectToAction(nextView, new { uploadMsg = "上传成功" });
            }
            return RedirectToAction(nextView);
        }

        // POST: Merchant/Create
        //[HttpPost]
        public ActionResult CreateTestData(string name,
            string YunziID, string region, double? latitude, double? longitude, string address, string password, int? Count)
        {
            try
            {
                Random random = new Random(DateTime.UtcNow.DayOfYear);
                double templatitude, templongitude;
                double flag = -1;
                string YunziPrefix = "SN" + (random.NextDouble() * 100).ToString("F0");
                int DataCount = Count ?? 1;
                if (DataCount > 500)
                {
                    DataCount = 500;
                }
                //for (int i = 0; i < DataCount; i++)
                //{
                //flag = -1 * flag;
                //latitude = random.NextDouble() * 0.1 * flag * i + (CenterLatitude ?? 47.742162);
                //longitude = random.NextDouble() * 0.1 * flag * i + (CenterLongitude ?? -122.192583);

                if (latitude == null)
                {
                    templatitude = random.NextDouble() * 0.1 * flag + 47.742162;
                }
                else
                {
                    templatitude = latitude ?? 47.742162;
                }

                if (longitude == null)
                {
                    templongitude = random.NextDouble() * 0.1 * flag - 122.192583;
                }
                else
                {
                    templongitude = longitude ?? -122.192583;
                }

                CustomerModel customer = new CustomerModel();
                customer.CustomerTable = new CCS_Customer_V1();
                customer.CustomerTable.Name = name;
                customer.CustomerTable.Region = GetRegion(region);
                customer.CustomerTable.State = CustomerAccountState.Active;

                customer.StaffTable = new CCS_Staff_V1();
                customer.StaffTable.geography = DbGeography.FromText(string.Format("POINT({1} {0})", templatitude, templongitude));
                customer.StaffTable.Address = address;
                customer.StaffTable.latitude = templatitude;
                customer.StaffTable.longitude = templongitude;

                customer.Password = string.IsNullOrEmpty(password) ? "1qaz@WSX" : password;
                customer.ComparePassword = customer.Password;

                customer.StaffTable.SNyunzi = string.IsNullOrEmpty(YunziID) ? (YunziPrefix + (random.NextDouble() * 100).ToString("X4")) : YunziID;
                //OldRegister(customer);
                //}
            }
            catch
            {
                ;
            }
            return RedirectToAction("ShowMap");
        }

        public ActionResult QueryBill()
        {
            return this.RedirectToAction("QueryFiltersUI", "Merchant", new { queryType = "BillQuery" });
        }

        // All the Bill Query methods now live in class QueryableController

        // Show default page
        public ActionResult ToDo(string PageTitle, string Details)
        {
            ViewBag.Title = PageTitle;
            ViewBag.ToDoDetails = Details;
            return View();
        }



        [HttpGet]
        public ActionResult AlertSettingEdit()
        {

            CloudStorageAccount storageAccount = CloudStorageAccount.Parse(
            Microsoft.Azure.CloudConfigurationManager.GetSetting("StorageConnectionString"));

            // Create the table client.
            CloudTableClient tableClient = storageAccount.CreateCloudTableClient();

            // Create the table if it doesn't exist.
            CloudTable table = tableClient.GetTableReference("AlertSettingTable");
            table.CreateIfNotExists();

            TableQuery<AlertSettingModel> rangeQuery = new TableQuery<AlertSettingModel>().Where(
                TableQuery.CombineFilters(
                TableQuery.GenerateFilterCondition("PartitionKey", QueryComparisons.Equal, AlertSettingModel.AlertPartitionKey),
                TableOperators.And,
                TableQuery.GenerateFilterCondition("RowKey", QueryComparisons.GreaterThanOrEqual, AlertSettingModel.AlertRowKey)));

            AlertSettingModel existingModel = null;

            IEnumerable<AlertSettingModel> queryResult = table.ExecuteQuery(rangeQuery);
            if (queryResult != null)
            {
                existingModel = queryResult.OrderBy(x => x.Timestamp).LastOrDefault();
            }

            if (existingModel == null)
            {
                existingModel = new AlertSettingModel();
            }

            return View(existingModel);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult AlertSettingEdit([Bind(Include = "SumAlertUnpaidBalance,SumAlertNotifyList,SumAlertAutoNotify,PersonAlertUnpaidBalance,PersonAlertUnpaidMonths,PersonAlertAutoNotify")] AlertSettingModel model)
        {
            if (ModelState.IsValid)
            {
                CloudStorageAccount storageAccount = CloudStorageAccount.Parse(
                Microsoft.Azure.CloudConfigurationManager.GetSetting("StorageConnectionString"));

                // Create the table client.
                CloudTableClient tableClient = storageAccount.CreateCloudTableClient();

                // Create the table if it doesn't exist.
                CloudTable table = tableClient.GetTableReference("AlertSettingTable");
                table.CreateIfNotExists();

                TableOperation insertOrReplaceOperation = TableOperation.InsertOrReplace(model);

                //TableOperation updateOperation = TableOperation.Replace(model);
                table.Execute(insertOrReplaceOperation);

            }
            return View(model);
        }

        [Authorize(Roles = "Staff")]
        public ActionResult MonthlyAmountDueAnalysis()
        {
            try
            {
                var curCustomerID = new Guid(GetCurrentCustomerGuidString());
                using (var amountDueEFContext = (AmountDueContext)AmountDueContextFactory.GetInstance().GetDBEFContext())
                {
                    var amountDueResult = (from a in amountDueEFContext.MonthlyAmountDueSumTable.AsNoTracking().Where(r => (r.CustomerID == curCustomerID))
                                           select a).Take(10).ToList();
                    return View(amountDueResult);
                }
            }
            catch (Exception)
            {

            }
            return View();
        }

        [Authorize(Roles = "Staff")]
        public ActionResult BalanceStatisticAnalysis()
        {
            return View();
        }

        [HttpPost]
        [Authorize(Roles = "Staff")]
        public ActionResult BalanceStatisticAnalysis(DateTime queryDate)
        {
            var UserBalanceList = new List<UserBalanceModel>();
            try
            {
                using (var sqlConn = SQLConnFactory.GetInstance().GetSqlConnection())
                {
                    sqlConn.Open();
                    var queryUserDailyBalanceTemplate = @"
declare @Date Date
declare @CustomerID nvarchar(max)
set @Date = '{0}'
set @CustomerID = '{1}'
select * from fnQueryUserDailyBalance(@Date, @CustomerID)";
                    var queryUserDailyBalance = string.Format(queryUserDailyBalanceTemplate,
                        queryDate.ToUniversalTime().ToString(), GetCurrentCustomerGuidString());
                    var sqlCmd = new SqlCommand(queryUserDailyBalance, sqlConn);
                    var reader = sqlCmd.ExecuteReader();
                    while (reader.Read())
                    {
                        var record = new UserBalanceModel
                        {
                            CellPhone = reader.GetString(0),
                            BillDue = reader.GetDouble(1),
                            Payment = reader.GetDouble(2),
                            Balance = reader.GetDouble(3),
                        };
                        UserBalanceList.Add(record);
                    }
                }
            }
            catch (Exception)
            {

            }
            return PartialView("_BalanceStatisticAnalysis", UserBalanceList);
        }

        [Authorize(Roles = "Staff")]
        public ActionResult QueryPeriodicBlackList()
        {
            return View();
        }

        [HttpPost]
        [Authorize(Roles = "Staff")]
        public ActionResult QueryPeriodicBlackList(DateTime queryDateStart, DateTime queryDateEnd,
            double amountDueThreshold)
        {
            var UserBalanceList = new List<UserBalanceModel>();
            if (queryDateStart > queryDateEnd)
            {
                return PartialView("_QueryPeriodicBlackList", UserBalanceList);
            }
            try
            {
                using (var sqlConn = SQLConnFactory.GetInstance().GetSqlConnection())
                {
                    sqlConn.Open();
                    var queryPeriodicBlackListTemplate = @"
declare @PeriodInMonth int
declare @AsOfDate Date
declare @CustomerID nvarchar(max)
declare @Amount float

set @PeriodInMonth = {0}
set @AsOfDate = '{1}'
set @CustomerID = '{2}'
set @Amount = {3}

select * from [fnQueryBlackList] (@PeriodInMonth, @AsOfDate, @Amount, @CustomerID)";
                    var periodInMonth = 1;
                    while (queryDateStart.AddMonths(periodInMonth).Year < queryDateEnd.Year ||
                        queryDateStart.AddMonths(periodInMonth).Month < queryDateEnd.Month)
                    {
                        periodInMonth++;
                    }
                    var queryPeriodicBlackList = string.Format(queryPeriodicBlackListTemplate,
                        periodInMonth.ToString(),
                        queryDateStart.ToUniversalTime().ToString(),
                        GetCurrentCustomerGuidString(), amountDueThreshold.ToString());
                    var sqlCmd = new SqlCommand(queryPeriodicBlackList, sqlConn);
                    var reader = sqlCmd.ExecuteReader();
                    while (reader.Read())
                    {
                        var record = new UserBalanceModel
                        {
                            CellPhone = reader.GetString(0),
                            BillDue = reader.GetDouble(1),
                        };
                        UserBalanceList.Add(record);
                    }
                }
            }
            catch (Exception)
            {

            }
            return PartialView("_QueryPeriodicBlackList", UserBalanceList);
        }

        [HttpGet]
        public ActionResult AlertViewBalance()
        {

            CloudStorageAccount storageAccount = CloudStorageAccount.Parse(
            Microsoft.Azure.CloudConfigurationManager.GetSetting("StorageConnectionString"));

            // Create the table client.
            CloudTableClient tableClient = storageAccount.CreateCloudTableClient();

            // Create the table if it doesn't exist.
            CloudTable table = tableClient.GetTableReference("AlertSettingTable");
            table.CreateIfNotExists();

            TableQuery<AlertSettingModel> rangeQuery = new TableQuery<AlertSettingModel>().Where(
                TableQuery.CombineFilters(
                TableQuery.GenerateFilterCondition("PartitionKey", QueryComparisons.Equal, AlertSettingModel.AlertPartitionKey),
                TableOperators.And,
                TableQuery.GenerateFilterCondition("RowKey", QueryComparisons.GreaterThanOrEqual, AlertSettingModel.AlertRowKey)));

            AlertSettingModel existingModel = null;

            IEnumerable<AlertSettingModel> queryResult = table.ExecuteQuery(rangeQuery);
            if (queryResult != null)
            {
                existingModel = queryResult.OrderBy(x => x.Timestamp).LastOrDefault();
            }
            if (existingModel == null)
            {
                return View();
            }

            // Get data of all users with unpaided balance great than or equal to the setting balance, or longger than or equal to the setting months


            return View();
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult AlertSendNotification()
        {
            return RedirectToAction("AlertViewBalance");
        }
    }

    // Merchant - Vendor
    public partial class MerchantController
    {
        [Authorize(Roles = "Staff, Vendor, Gatherer")]
        public ActionResult VendorLogin()
        {
            List<CCS_Customer_V1> Customers = GetAllCurrentCustomers(Session, User);
            return View(Customers);
        }


        [Authorize(Roles = "Staff, Vendor, Gatherer")]
        public ActionResult VendorService(string CustomerGuid, string CustomerName)
        {
            return RedirectToAction("QueryUserBill", new { CustomerGuid = CustomerGuid, CustomerName = CustomerName });
        }

        [Authorize(Roles = "Staff, Vendor, Gatherer")]
        public ActionResult QueryUserBill(string CustomerGuid, string CustomerName)
        {
            if (!string.IsNullOrWhiteSpace(CustomerGuid))
            {
                ViewBag.CustomerGuid = CustomerGuid;
                Session[CCSPrototypeCommon.SessionConstants.CustomerID] = CustomerGuid;
            }

            if (!string.IsNullOrWhiteSpace(CustomerName))
            {
                ViewBag.CustomerName = CustomerName;
                Session[CCSPrototypeCommon.SessionConstants.CustomerName] = CustomerName;
            } 

            return View();
        }

        [Authorize(Roles = "Staff, Vendor, Gatherer")]
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult ViewUserBill(
            [Bind(Include = "UserInputAccountID")] UserModelHome userModelHome)
        {
            Guid StaffGuid = Guid.Empty;

            if (IsVendorExpired(userModelHome.UserCustomerID, out StaffGuid))
            {
                return RedirectToAction("VendorExpiredPage", new { AlreadyExpired = true, VendorID = StaffGuid.ToString() });
            }

            ViewBag.StaffID = StaffGuid.ToString();

            var userInputID = userModelHome.UserInputAccountID;

            IEnumerable<CCS_Bill_V1> billsWithThisUser = null;

            using (var billContex = (BillContext)BillContextFactory.GetInstance().GetDBEFContext())
            {
                billsWithThisUser = BillCommon.getCCSBillListWithInputID(billContex, userInputID);
            }

            if (billsWithThisUser == null || billsWithThisUser.Count() == 0)
            {

                ViewBag.Msg = string.Format("该号码未找到任何账单:{0}。请尝试其他查询号码。", userInputID);
                return View("GenericErrorMsgMrct");
            }

            return View("ShowUserBill", billsWithThisUser);

        }



        [Authorize(Roles = "Staff, Vendor, Gatherer")]
        public ActionResult QueryVendorPaymentsForUser(string CustomerGuid, string CustomerName)
        {
            if (!string.IsNullOrWhiteSpace(CustomerGuid))
            {
                ViewBag.CustomerGuid = CustomerGuid;
                Session[CCSPrototypeCommon.SessionConstants.CustomerID] = CustomerGuid;
            }

            if (!string.IsNullOrWhiteSpace(CustomerName))
            {
                ViewBag.CustomerName = CustomerName;
                Session[CCSPrototypeCommon.SessionConstants.CustomerName] = CustomerName;
            }

            return View();
        }


        [Authorize(Roles = "Staff, Vendor, Gatherer")]
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult ViewVendorPaymentsForUser(
            [Bind(Include = "UserInputAccountID")] UserModelHome userModelHome,
            DateTime? QueryStartDate, DateTime? QueryEndDate)
        {
            Guid StaffGuid = Guid.Empty;

            if (IsVendorExpired(userModelHome.UserCustomerID, out StaffGuid))
            {
                return RedirectToAction("VendorExpiredPage", new { AlreadyExpired = true, VendorID = StaffGuid.ToString() });
            }

            ViewBag.StaffID = StaffGuid.ToString();

            var userInputID = userModelHome.UserInputAccountID;
            ViewBag.UserInputID = userInputID;

            IEnumerable<CCS_Payment_V1> paymentsWithStaffIDWithUserID = null;
            List<CCS_Staff_V1> staffs = new List<CCS_Staff_V1>(); staffs.Add(new CCS_Staff_V1() { StaffID = new Guid("8051cd90-0159-e611-80c3-000d3a3130b8") });//GetCurrentStaffs();
            
            using (var paymentContext = (PaymentContext)PaymentContextFactory.GetInstance().GetDBEFContext())
            {
                using (var billContext = (BillContext)BillContextFactory.GetInstance().GetDBEFContext())
                {
                    paymentsWithStaffIDWithUserID = Commons.getCCSVendorPaymentsForUserID(billContext,
                        paymentContext, staffs, userInputID,
                        QueryStartDate, QueryEndDate);
                }
                
            }

            if (paymentsWithStaffIDWithUserID == null || paymentsWithStaffIDWithUserID.Count() == 0)
            {

                ViewBag.Msg = string.Format("No payments found for user {0} made by the vendor {1}", userInputID, StaffGuid.ToString());
                return View("GenericErrorMsgMrct");
            }

            ViewBag.QueryStartDate = QueryStartDate;
            ViewBag.QueryEndDate = QueryEndDate;
            return View("ShowVendorPaymentsForUser", paymentsWithStaffIDWithUserID);

        }

        [Authorize(Roles = "Vendor, Gatherer")]
        public ActionResult VendorViewCompany()
        {
            List<CCS_Staff_V1> staffs = GetCurrentStaffs();

            ViewBag.StaffGuid = staffs.Count > 0 ? staffs.FirstOrDefault().StaffID.ToString() : string.Empty;

            return View(staffs);
        }

        [HttpPost]
        public async Task<JsonResult> AddFeedback(string userid, string staffid, string msg)
        {
            int actionStatusCode;
            String actionStatusMsg;
            Guid UserGuid = Guid.Empty;
            Guid StaffGuid = Guid.Empty;
            try
            {
                using (var vendorEFContext = (BillContext)BillContextFactory.GetInstance().GetDBEFContext())
                {
                    if (string.IsNullOrEmpty(userid))
                    {
                        UserGuid = GetUserGuidFromSession();
                    }
                    else if (!Guid.TryParse(userid, out UserGuid))
                    {
                        return new JsonResult() { Data = new { statuscode = 0, statusmsg = "用户当前登录已过期或无效，请重新登录。" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                    }
                    if (!Guid.TryParse(staffid, out StaffGuid) || Guid.Empty == StaffGuid)
                    {
                        return new JsonResult() { Data = new { statuscode = 0, statusmsg = "所选择目标无效，请刷新页面重试。" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                    }

                    //var usersInfo = vendorEFContext.UserTable.Where(o => o.UserID == UserGuid && o.ModifiedDate == null).ToList();
                    var usersInfo = vendorEFContext.UserTable.Where(o => o.UserID == UserGuid).OrderByDescending(o => o.ModifiedDate).ToList();

                    if (usersInfo.Count() < 1)
                    {
                        return new JsonResult() { Data = new { statuscode = 0, statusmsg = "未找到当前登陆用户，无法留言。" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                    }
                    //else if (usersInfo.Count() > 1)
                    //{
                    //    return new JsonResult() { Data = new { statuscode = 0, statusmsg = "More than one record with the specified (user id) is found." }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                    //}

                    CloudStorageAccount storageAccount = CloudStorageAccount.Parse(RoleEnvironment.GetConfigurationSettingValue("StorageConnectionString"));
                    CloudBlobClient blobClient = storageAccount.CreateCloudBlobClient();
                    blobClient.DefaultRequestOptions.RetryPolicy = new LinearRetry(TimeSpan.FromSeconds(3), 3);
                    var feedbackBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.FeedbackBlobContainerName);
                    feedbackBlobContainer.CreateIfNotExists();
                    // Blob path format: VendorID/yyyy/MM/dd/HHmmssff
                    var dateTimeFormat = "yyyy/MM/dd/HHmmssff";
                    var blobName = String.Format("{0}/{1}", staffid, DateTime.UtcNow.ToString(dateTimeFormat));
                    var feedback = new UserFeedbackModel
                    {
                        UserID = usersInfo[0].UserID.ToString(),
                        Username = String.Format("{0}{1}", usersInfo[0].LastName, usersInfo[0].FirstName),
                        Content = msg,
                        CreatedTime = DateTime.Now,
                        BlobName = blobName
                    };
                    var content = JsonConvert.SerializeObject(feedback);
                    var fileBlob = feedbackBlobContainer.GetBlockBlobReference(blobName);
                    await fileBlob.UploadTextAsync(content);

                    actionStatusCode = 1;
                    actionStatusMsg = "该评论已提交。";
                }
            }
            catch (Exception e)
            {
                if (e.InnerException != null)
                {
                    actionStatusCode = 0;
                    actionStatusMsg = "InnerException: " + e.InnerException.Message;
                }
                else
                {
                    actionStatusCode = 0;
                    actionStatusMsg = "Exception: " + e.Message;
                }
            }
            CCS_FailedActions_V1 action = new CCS_FailedActions_V1();
            action.ActionType = ACTIONSTYPE.ADDFEEDBACK;
            action.Message = actionStatusMsg;
            action.Parameters = string.Format(Constants.actionTypeToParametersFormatMap[ACTIONSTYPE.ADDFEEDBACK], userid, staffid);
            action.ModifiedDate = DateTime.UtcNow;

            _actionsDB.FailedActionsTable.Add(action);
            _actionsDB.SaveChanges();

            return new JsonResult() { Data = new { statuscode = actionStatusCode, statusmsg = actionStatusMsg }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
        }

        public async Task<List<UserFeedbackModel>> GetFeedbackPerVendor(Guid staffid, int LastNumberOfDays)
        {
            var dateTimeFormat = "yyyy/MM/dd";
            var blobName = String.Format("{0}/{1}", staffid.ToString(), DateTime.UtcNow.ToString(dateTimeFormat));
            var feedbackList = new List<UserFeedbackModel>();
            CloudStorageAccount storageAccount = CloudStorageAccount.Parse(RoleEnvironment.GetConfigurationSettingValue("StorageConnectionString"));
            CloudBlobClient blobClient = storageAccount.CreateCloudBlobClient();
            blobClient.DefaultRequestOptions.RetryPolicy = new LinearRetry(TimeSpan.FromSeconds(3), 3);
            var feedbackBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.FeedbackBlobContainerName);
            feedbackBlobContainer.CreateIfNotExists();

            for (int index = 0; index < LastNumberOfDays; index++)
            {
                string prefix = string.Format("{0}/{1}", staffid.ToString(), DateTime.UtcNow.AddDays(0 - index).ToString(dateTimeFormat));
                foreach (IListBlobItem item in feedbackBlobContainer.ListBlobs(prefix, true))
                {
                    var blob = (CloudBlockBlob)item;
                    string blobContent = await blob.DownloadTextAsync();
                    UserFeedbackModel feedback = JsonConvert.DeserializeObject<UserFeedbackModel>(blobContent);
                    feedbackList.Add(feedback);
                }
            }
            return feedbackList;
        }

        [HttpPost]
        //[Authorize(Roles = "Staff, Admin, SysAdmin")]
        public async Task<ActionResult> ViewVendorFeedback(string staffid, int? LastNumberOfDays)
        {
            int QueryDays = LastNumberOfDays ?? 90;
            ViewBag.QueryDays = QueryDays;

            Guid StaffGuid = Guid.Empty;

            if (!Guid.TryParse(staffid, out StaffGuid))
            {
                return PartialView("_ViewVendorFeedback");
            }

            ViewBag.StaffGuid = StaffGuid.ToString();
            List<UserFeedbackModel> feedbacks = await GetFeedbackPerVendor(StaffGuid, QueryDays);

            return PartialView("_ViewVendorFeedback", feedbacks);
        }

        //[HttpPost]
        //[Authorize(Roles = "Vendor")]
        public async Task<ActionResult> ShowMyFeedback(string staffid, int? LastNumberOfDays)
        {
            int QueryDays = LastNumberOfDays ?? 90;
            ViewBag.QueryDays = QueryDays;

            Guid StaffGuid = Guid.Empty;

            if (string.IsNullOrWhiteSpace(staffid))
            {
                CCS_Staff_V1 vendor = GetCurrentStaff();

                if (vendor == null || vendor.StaffID == Guid.Empty)
                {
                    return View();
                }
                else
                {
                    StaffGuid = vendor.StaffID;
                }
            }
            else if (!Guid.TryParse(staffid, out StaffGuid))
            {
                return View();
            }

            ViewBag.StaffGuid = StaffGuid.ToString();

            List<UserFeedbackModel> feedbacks = await GetFeedbackPerVendor(StaffGuid, QueryDays);

            return View(feedbacks);
        }

    }
}
