﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Net;
using System.Web;
using System.Web.Mvc;
using Microsoft.Azure;
using System.Collections.Specialized;
using System.Reflection;
using CCSAdmin;
using CCSPrototypeCommon;
using CCSPrototypeCommon.Query;
using HomeWeb.Models;
using PagedList;
using System.Threading.Tasks;

using System.Security.Cryptography.X509Certificates;

namespace HomeWeb.Controllers
{
    public class PaymentsController : QueryableController
    {
        public static Logger logger
        {
            get
            {
                return new Logger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name);
            }

        }

        private PaymentContext _paymentDB = null;
        private BillContext _billDB = (BillContext)BillContextFactory.GetInstance().GetDBEFContext();
        //Utility class to store the data received when an order is queried from WeChat 
        public class QueryOrderData
        {
            public WxPayData wxPayData;
            public bool status;
        }

        public PaymentsController()
        {
            _paymentDB = (PaymentContext)PaymentContextFactory.GetInstance().GetDBEFContext();
        }

        public ActionResult QueryPayments()
        {
            return this.RedirectToAction("QueryFiltersUI", new { queryType = "PaymentQuery" });
        }

        //protected override IEnumerable<string> FillQueryBaseTypeList()
        //{
        //    List<string> QueryTypes = new List<string>();
        //    QueryTypes.Add("用户缴费历史");
        //    QueryTypes.Add("代缴点缴费历史");
        //    QueryTypes.Add("催缴员缴费历史");
        //    return QueryTypes;
        //}

        protected override IEnumerable<PropertyInfo> GetProperties(QueryType queryType)
        {
            List<PropertyInfo> result = new List<PropertyInfo>();
            result.AddRange((typeof(CCS_Payment_V1)).GetProperties().Where(x => x.Name != "UserInputAccountID" && x.Name != "PaymentCellNumber"));
            return result;
        }

        protected override string GetAssemblyQualifiedName()
        {
            return typeof(CCS_Payment_V1).AssemblyQualifiedName;
        }

        public override ActionResult QueryBillResultPaged(int? page)
        {
            IEnumerable<CCS_Payment_V1> result = (IEnumerable<CCS_Payment_V1>)Session["filterResults"];

            int pageNumber = page ?? 1;

            return PartialView("_QueryResults", result.ToPagedList(pageNumber, PageList_PageSize));
        }
        public override ActionResult QueryBillResult(FormCollection collection)
        {

            
            IEnumerable<CCS_Payment_V1> result = FilterApplied<CCS_Payment_V1>(collection);

            


            int pageNumber = 1;

            return PartialView("_QueryResults", result.ToPagedList(pageNumber, PageList_PageSize));
        }

        [HttpPost]
        public ActionResult QueryResultWithID(string UserIDForPaymentQuery)
        {
            //string UserID = collection["UserIDForPaymentQuery"] as string;

            if (string.IsNullOrEmpty(UserIDForPaymentQuery))
            {
                return PartialView("_QueryResults");
            }

            IEnumerable<CCS_Payment_V1> result = (from p in _paymentDB.PaymentTable
                                                  where p.PaymentCellNumber == UserIDForPaymentQuery
                                                  select p).ToList();
            int pageSize = 6;
            int pageNumber = 1;
            ViewBag.Title = "缴费历史记录";

            return PartialView("_QueryResults", result.ToPagedList(pageNumber, pageSize));
        }

        protected override IEnumerable<T> GetQueryResult<T>(FormCollection collection, List<AbstractFilter> searchCriteria, QueryType queryType)
        {
            IQueryable<CCS_Payment_V1> payments;
            IEnumerable<CCS_Payment_V1> result = null;

            string UserID = collection["UserIDForPaymentQuery"] as string;

            if (string.IsNullOrEmpty(UserID))
            {
                return new List<T>();
            }

            var TType = typeof(T);
            if (TType != typeof(CCS_Payment_V1))
            {
                return new List<T>();
            }

            try
            {
                payments = (from p in _paymentDB.PaymentTable
                            //where p.PaymentType == BaseType
                            where p.PaymentCellNumber == UserID
                            select p).ApplySearchCriteria(searchCriteria.AsEnumerable());

            }
            catch (Exception e)
            {
                throw new HttpException(500, e.Message);
            }
            result = payments.ToList();
            return result as IEnumerable<T>;
        }

        /*
        // GET: Payments
        public ActionResult Index()
        {
            return null;
        }
        */
        

        //// GET: Payments/Details/5
        //public ActionResult Details(long? id)
        //{
        //    if (id == null)
        //    {
        //        return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
        //    }
        //    CCS_Payment_V1 payment = _paymentDB.PaymentTable.Find(id);
        //    if (payment == null)
        //    {
        //        return HttpNotFound();
        //    }
        //    return View(payment);
        //}
        /*
        public ActionResult PayBalance(int? id)
        {
            if (id == null)
            {
                return RedirectToAction("Index", "Bills");
            }
            Bill bill = db.Bills.Find(id);

            if (bill == null)
            {
                return RedirectToAction("Index", "Bills");
            }
            Payment newPayment = new Payment();
            newPayment.PaymentBill = bill;
            newPayment.BillID = bill.BillID;
            newPayment.Amount = bill.AmountDue;

            //return View(newPayment);
            return View("Create", newPayment);
        }*/

        /// <summary>
        /// Pay balance for self or for someone else, for user app
        /// </summary>
        /// <param name="id"></param>
        /// <param name="PayById">Not supported for now.</param>
        /// <returns></returns>
        public ActionResult PayBalance(Guid id, Guid? PayById)
        {
            if (id == Guid.Empty)
            {
                
                return RedirectToAction("Index", "Bills");
            }
            CCS_Bill_V1 ccsBill = _paymentDB.BillTable.Find(id);

            if (ccsBill == null)
            {
                return RedirectToAction("Index", "Bills");
            }


            CCS_Payment_V1 newPayment = new CCS_Payment_V1()
            {
                BillID = id,
                Amount = ccsBill.AmountDue,
                PaymentMethod = PayMethod.ByUser,
                PaymentType = ccsBill.BillType,
                StaffID = null,
                PaymentStaff = null,
                PayerRole = null,
                PaymentCellNumber = Session["Cellphone"] as string,
                PaymentBill = ccsBill
            };


            //return View(newPayment);
            //return View("Create", newPayment);
            return View("Details", newPayment);
        }

        /// <summary>
        /// Pay balance for users, for Vendor app
        /// </summary>
        /// <param name="id"></param>
        /// <param name="PayById"></param>
        /// <returns></returns>
        public ActionResult PayBalanceFor(Guid id, Guid PayById)
        {
            if (id == Guid.Empty)
            {
                ViewBag.Msg = "用户账单为空，请重试。";
                return View("GenericErrorMsgMrct");
            }
            if (PayById == Guid.Empty)
            {
                ViewBag.Msg = "代缴用户为空，请重试。";
                return View("GenericErrorMsgMrct");
            }
            CCS_Staff_V1 staff = GetStaffByGuid(PayById);
            if (staff == null)
            {
                ViewBag.Msg = "代缴用户未找到，请重试。";
                return View("GenericErrorMsgMrct");
            }

            CCS_Bill_V1 ccsBill = _paymentDB.BillTable.Find(id);

            if (ccsBill == null)
            {
                ViewBag.Msg = "用户账单未找到，请重试。";
                return View("GenericErrorMsgMrct");
            }


            CCS_Payment_V1 newPayment = new CCS_Payment_V1() 
            { 
                BillID = id,
                Amount = ccsBill.AmountDue,
                PaymentMethod = PayMethod.ByVendor,
                PaymentType = ccsBill.BillType,
                StaffID = staff.StaffID,
                PaymentCellNumber = staff.Cellphone,
                PaymentStaff = staff,
                PaymentBill = ccsBill
            };


            //return View(newPayment);
            //return View("Create", newPayment);
            return View("Details_Vendor", newPayment);
        }

        public ActionResult PaySubscription([Bind(Include = "Amount,StaffID,ExtendMonth,SubscriberStaff")]CCS_SubscriptionPayment_V1 Subscription)
        {
            if (Subscription == null)
            {
                ViewBag.Msg = "续费信息为空，请重试。";
                return View("GenericErrorMsgMrct");
            }
            if (Subscription.StaffID == Guid.Empty)
            {
                ViewBag.Msg = "续费用户为空，请重试。";
                return View("GenericErrorMsgMrct");
            }

            CCS_Staff_V1 staff = null;
            using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
            {
                staff = vendorEFContext.StaffTable.Where(entry => entry.StaffID == Subscription.StaffID).FirstOrDefault();

                if (staff == null)
                {
                    ViewBag.Error = "未找到续费账户，请重试";
                    return View("GenericErrorMsgMrct");
                }
                //vendorEFContext.Configuration.ValidateOnSaveEnabled = false;
                //vendorEFContext.Entry(staff).Property(c => c.State).IsModified = true;
                //await vendorEFContext.SaveChangesAsync();
            }

            Subscription.SubscriberStaff = staff;
            return View("Details_Subscription", Subscription);
        }

        //// GET: Payments/Create
        //public ActionResult Create()
        //{
        //    return View();
        //}

        // POST: Payments/Create
        // To protect from overposting attacks, please enable the specific properties you want to bind to, for 
        // more details see http://go.microsoft.com/fwlink/?LinkId=317598.
        [HttpPost]
        [ValidateAntiForgeryToken]
        public JsonResult Create([Bind(Include = "PaymentID,PaymentType,PaymentMethod,PaymentCellNumber,Amount,BillID")] CCS_Payment_V1 payment)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    payment.PaymentID = Guid.NewGuid();
                    payment.PaySubmitTime = DateTime.UtcNow;
                    _paymentDB.PaymentTable.Add(payment);
                    _paymentDB.SaveChanges();
                }
            }
            catch (Exception)
            {

            }


            return new JsonResult() { Data = new { paymentId = payment.PaymentID, amount = payment.Amount }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
        }

        private void SetPaymentBy(CCS_Payment_V1 payment)
        {
            try
            {
                if(!Request.IsAuthenticated)
                {
                    
                    //If request is not authenticated for now we consider this payment done by end user
                    logger.Info("This payment is being done by an end user.");
                    payment.PayerRole = OperationLevel.Consumer;
                    return;
                }

                // currentStaff is going to be null if current user is defined in AD however has no associated record in CCS_Staff_V1 table   
                CCS_Staff_V1 currentStaff = GetCurrentStaff();
                if (currentStaff != null)
                {
                    payment.StaffID = currentStaff.StaffID;
                    payment.PayerRole = currentStaff.Role;

                }
                else
                {
                    if (User.IsInRole(CCSPrototypeCommon.OperationLevel.Vendor.ToString()))
                    {
                        logger.Info("This payment is being done by a vendor.");
                        payment.PayerRole = OperationLevel.Vendor;
                        return;
                    }
                    else
                    {
                        //looks like payer is authenticated however there is no staff associated with that payer in CCS_Staff_V1 table
                        if (User.IsInRole(CCSPrototypeCommon.OperationLevel.Gatherer.ToString()))
                        {
                            logger.Info("This payment is being done by a gatherer.");
                            payment.PayerRole = OperationLevel.Gatherer;
                            return;
                        }
                        else
                        {
                            if (User.IsInRole(CCSPrototypeCommon.OperationLevel.Staff.ToString()))
                            {
                                logger.Info("This payment is being done by a staff member.");
                                payment.PayerRole = OperationLevel.Staff;
                                return;
                            }
                            else
                            {
                                // A persion who payed is autheticated, but looks like the role is missing
                                logger.Info("This payment is being done by authenticated party that has no assigned role.");
                                payment.PayerRole = OperationLevel.Unkown;
                                return;
                            }
                        }
                    }

                }
            }
            catch(Exception e)
            {
                string errorMsg = e.Message;
                Exception innerExc = e.InnerException;
                errorMsg += innerExc != null ? innerExc.Message : "";
                logger.Error(errorMsg);
                throw;
            }
        }
        // POST: Payments/Create
        // To protect from overposting attacks, please enable the specific properties you want to bind to, for 
        // more details see http://go.microsoft.com/fwlink/?LinkId=317598.
        [HttpPost]
        [ValidateAntiForgeryToken]
        public JsonResult CreatePaymentAliPay([Bind(Include = "PaymentID,PaymentType,PaymentMethod,PaymentCellNumber,Amount,BillID,StaffID")] CCS_Payment_V1 payment)
        {
            String paymentOutTradeNo = null;
            try
            {
                if (ModelState.IsValid)
                {
                    payment.PaymentID = Guid.NewGuid();
                    payment.PaySubmitTime = DateTime.UtcNow;
                    _paymentDB.PaymentTable.Add(payment);
                    CCS_PaymentInfo_V1 paymentInfo = new CCS_PaymentInfo_V1();
                    paymentInfo.PaymentID = payment.PaymentID;
                    paymentInfo.PaymentOutTradeNo = getOutTradeNoAliPay();
                    _paymentDB.PaymentInfoTable.Add(paymentInfo);

                    paymentOutTradeNo = paymentInfo.PaymentOutTradeNo;

                    SetPaymentBy(payment);

                    _paymentDB.SaveChanges();
                    
                }
            }
            catch (Exception e)
            {
                logger.logException(e);
            }

            if (payment == null || paymentOutTradeNo == null)
            {
                return null;
            }


            return new JsonResult() { Data = new { paymentId = payment.PaymentID, amount = payment.Amount, outTradeNo = paymentOutTradeNo }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
        }

        // POST: Payments/Create
        // To protect from overposting attacks, please enable the specific properties you want to bind to, for 
        // more details see http://go.microsoft.com/fwlink/?LinkId=317598.
        [HttpPost]
        [ValidateAntiForgeryToken]
        public JsonResult CreatePaymentWeChat([Bind(Include = "PaymentID,PaymentType,PaymentMethod,PaymentCellNumber,Amount,BillID,StaffID")] CCS_Payment_V1 payment)
        {
            String paymentOutTradeNo = null;
            try
            {
                if (ModelState.IsValid)
                {
                    payment.PaymentID = Guid.NewGuid();
                    payment.PaySubmitTime = DateTime.UtcNow;
                    _paymentDB.PaymentTable.Add(payment);

                    CCS_PaymentInfo_V1 paymentInfo = new CCS_PaymentInfo_V1();
                    paymentInfo.PaymentID = payment.PaymentID;
                    paymentInfo.PaymentOutTradeNo = getOutTradeNoWeChat();
                    _paymentDB.PaymentInfoTable.Add(paymentInfo);

                    paymentOutTradeNo = paymentInfo.PaymentOutTradeNo;

                    SetPaymentBy(payment);

                    _paymentDB.SaveChanges();
                }
            }
            catch (Exception e)
            {
                logger.logException(e);
            }


            if (payment == null || paymentOutTradeNo == null)
            {
                return null;
            }


            return new JsonResult() { Data = new { paymentId = payment.PaymentID, amount = payment.Amount, outTradeNo = paymentOutTradeNo }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };

        }

        /**
        * get the out_trade_no for an order. 生成商户订单号，该值在商户端应保持唯一（可自定义格式规范）
        *
        */
        private String getOutTradeNoAliPay()
        {
            /*String key = DateTime.UtcNow.ToString("MMddHHmmss");
            Random rnd = new Random();
            key += rnd.Next(Int32.MinValue, Int32.MaxValue);
            key = key.Substring(0, 15);
            return key;*/
            return Guid.NewGuid().ToString().Replace("-", "");
        }

        private String getOutTradeNoWeChat()
        {
            return Guid.NewGuid().ToString().Replace("-", "");
        }


        [HttpPost]
        [ValidateAntiForgeryToken]
        public async Task<JsonResult> CreateSubscriptionPaymentAliPay([Bind(Include = "PaymentID,PayerRole,PaymentCellNumber,Amount,StaffID,ExtendMonth")] CCS_SubscriptionPayment_V1 Subscription)
        {
            String paymentOutTradeNo = null;
            try
            {
                if (ModelState.IsValid)
                {
                    Subscription.PaymentID = Guid.NewGuid();
                    Subscription.PaySubmitTime = DateTime.UtcNow;
                    _paymentDB.SubscriptionPaymentTable.Add(Subscription);
                    CCS_SubscriptionPaymentInfo_V1 SubscriptionPaymentInfo = new CCS_SubscriptionPaymentInfo_V1();
                    SubscriptionPaymentInfo.PaymentID = Subscription.PaymentID;
                    SubscriptionPaymentInfo.PaymentOutTradeNo = getOutTradeNoAliPay();
                    _paymentDB.SubscriptionPaymentInfoTable.Add(SubscriptionPaymentInfo);

                    paymentOutTradeNo = SubscriptionPaymentInfo.PaymentOutTradeNo;

                    _paymentDB.SaveChanges();

                    using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                    {
                        var staff = vendorEFContext.StaffTable.Where(entry => entry.StaffID == Subscription.StaffID).FirstOrDefault();

                        if (staff == null)
                        {
                            //TBD
                        }
                        else
                        {
                            staff.ExpirationDate = staff.ExpirationDate.AddMonths(Subscription.ExtendMonth);
                            vendorEFContext.Configuration.ValidateOnSaveEnabled = false;
                            vendorEFContext.Entry(staff).Property(c => c.State).IsModified = true;
                            await vendorEFContext.SaveChangesAsync();
                        }
                    }
                }
            }
            catch (Exception)
            {

            }

            if (Subscription == null || paymentOutTradeNo == null)
            {
                return null;
            }


            return new JsonResult() { Data = new { paymentId = Subscription.PaymentID, amount = Subscription.Amount, outTradeNo = paymentOutTradeNo }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
        }


        [HttpPost]
        [ValidateAntiForgeryToken]
        public async Task<JsonResult> CreateSubscriptionPaymentWeChat([Bind(Include = "PaymentID,PayerRole,PaymentCellNumber,Amount,StaffID,ExtendMonth")]  CCS_SubscriptionPayment_V1 Subscription)
        {
            String paymentOutTradeNo = null;
            try
            {
                if (ModelState.IsValid)
                {
                    Subscription.PaymentID = Guid.NewGuid();
                    Subscription.PaySubmitTime = DateTime.UtcNow;
                    _paymentDB.SubscriptionPaymentTable.Add(Subscription);

                    CCS_SubscriptionPaymentInfo_V1 SubscriptionPaymentInfo = new CCS_SubscriptionPaymentInfo_V1();
                    SubscriptionPaymentInfo.PaymentID = Subscription.PaymentID;
                    SubscriptionPaymentInfo.PaymentOutTradeNo = getOutTradeNoWeChat();
                    _paymentDB.SubscriptionPaymentInfoTable.Add(SubscriptionPaymentInfo);

                    paymentOutTradeNo = SubscriptionPaymentInfo.PaymentOutTradeNo;

                    _paymentDB.SaveChanges();

                    using (var vendorEFContext = (VendorContext)VendorContextFactory.GetInstance().GetDBEFContext())
                    {
                        var staff = vendorEFContext.StaffTable.Where(entry => entry.StaffID == Subscription.StaffID).FirstOrDefault();

                        if (staff == null)
                        {
                            //TBD
                        }
                        else
                        {
                            staff.ExpirationDate = staff.ExpirationDate.AddMonths(Subscription.ExtendMonth);
                            vendorEFContext.Configuration.ValidateOnSaveEnabled = false;
                            vendorEFContext.Entry(staff).Property(c => c.State).IsModified = true;
                            await vendorEFContext.SaveChangesAsync();
                        }
                    }

                }
            }
            catch (Exception)
            {

            }


            if (Subscription == null || paymentOutTradeNo == null)
            {
                return null;
            }


            return new JsonResult() { Data = new { paymentId = Subscription.PaymentID, amount = Subscription.Amount, outTradeNo = paymentOutTradeNo }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };

        }


        //// GET: Payments/Edit/5
        //public ActionResult Edit(long? id)
        //{
        //    if (id == null)
        //    {
        //        return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
        //    }
        //    Payment payment = db.Payments.Find(id);
        //    if (payment == null)
        //    {
        //        return HttpNotFound();
        //    }
        //    return View(payment);
        //}

        //// POST: Payments/Edit/5
        //// To protect from overposting attacks, please enable the specific properties you want to bind to, for 
        //// more details see http://go.microsoft.com/fwlink/?LinkId=317598.
        //[HttpPost]
        //[ValidateAntiForgeryToken]
        //public ActionResult Edit([Bind(Include = "PaymentID,PaymentType,PaymentMethod,PaymentCellNumber,PaySubmitTime")] Payment payment)
        //{
        //    if (ModelState.IsValid)
        //    {
        //        db.Entry(payment).State = EntityState.Modified;
        //        db.SaveChanges();
        //        return RedirectToAction("Index");
        //    }
        //    return View(payment);
        //}

        //// GET: Payments/Delete/5
        //public ActionResult Delete(long? id)
        //{
        //    if (id == null)
        //    {
        //        return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
        //    }
        //    Payment payment = db.Payments.Find(id);
        //    if (payment == null)
        //    {
        //        return HttpNotFound();
        //    }
        //    return View(payment);
        //}

        // POST: Payments/Delete/5
        //[HttpPost, ActionName("Delete")]
        //[ValidateAntiForgeryToken]
        //public ActionResult DeleteConfirmed(long id)
        //{
        //    Payment payment = db.Payments.Find(id);
        //    db.Payments.Remove(payment);
        //    db.SaveChanges();
        //    return RedirectToAction("Index");
        //}


        //public ActionResult PayAll(string userInputID)
        //{
        //    List<Bill> bills = BillCommon.getBillListWithInputID(db, userInputID);

        //    if (bills == null)
        //    {
        //        return RedirectToAction("Index", "Bills", new { userInputID = userInputID });
        //    }

        //    foreach (Bill oneBill in bills)
        //    {
        //        oneBill.AmountDue = 0;
        //        //account.UserInputAccountID = UserInputAccountID;
        //        db.Entry(oneBill).State = System.Data.Entity.EntityState.Modified;
        //    }

        //    db.SaveChanges();
        //    return View("Index", "Bills", bills);
        //}

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                _paymentDB.Dispose();
            }
            base.Dispose(disposing);
        }


        [HttpPost]
        public HttpResponseBase PaymentNotifyAction(string trade_status, string out_trade_no, string trade_no)
        {

            var paymentsInfo = _paymentDB.PaymentInfoTable.Where(o => o.PaymentOutTradeNo == out_trade_no).ToList();
            if (paymentsInfo.Count() == 1)
            {
                String result = trade_status + " " + trade_no;

                CCS_Payment_AliPayNotif_V1 paymentAliPayNotif = new CCS_Payment_AliPayNotif_V1();
                try
                {
                    paymentAliPayNotif.PaymentInfoID = paymentsInfo[0].PaymentInfoID;


                    SortedDictionary<string, string> sArray = new SortedDictionary<string, string>();
                    NameValueCollection coll;
                    //Load Form variables into NameValueCollection variable.
                    coll = Request.Form;
                    if (ContainsKey(coll, Constants.AliPayNotifSubject))
                    {
                        paymentAliPayNotif.PaymentSubject = coll[Constants.AliPayNotifSubject];
                    }
                    if (ContainsKey(coll, Constants.AliPayNotifTradeNo))
                    {
                        paymentAliPayNotif.PaymentTradeNo = coll[Constants.AliPayNotifTradeNo];
                    }
                    if (ContainsKey(coll, Constants.AliPayNotifBuyerEmail))
                    {
                        paymentAliPayNotif.PaymentBuyerEmail = coll[Constants.AliPayNotifBuyerEmail];
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifDiscount))
                    {
                        double num;
                        if (double.TryParse(coll[Constants.AliPayNotifDiscount], out num))
                        {
                            paymentAliPayNotif.PaymentDiscount = num;
                        }
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifType))
                    {
                        int num;
                        if (int.TryParse(coll[Constants.AliPayNotifType], out num))
                        {
                            paymentAliPayNotif.PaymentType = num;
                        }
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifNotifyType))
                    {
                        paymentAliPayNotif.PaymentNotifyType = coll[Constants.AliPayNotifNotifyType];
                    }


                    if (ContainsKey(coll, Constants.AliPayNotifQuantity))
                    {
                        int num;
                        if (int.TryParse(coll[Constants.AliPayNotifQuantity], out num))
                        {
                            paymentAliPayNotif.PaymentQuantity = num;
                        }
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifCreateTime))
                    {
                        DateTime num;
                        if (DateTime.TryParse(coll[Constants.AliPayNotifCreateTime], out num))
                        {
                            paymentAliPayNotif.PaymentCreateTime = num;
                        }
                    }


                    if (ContainsKey(coll, Constants.AliPayNotifSellerID))
                    {
                        paymentAliPayNotif.PaymentSellerID = coll[Constants.AliPayNotifSellerID];
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifNotifyTime))
                    {
                        DateTime num;
                        if (DateTime.TryParse(coll[Constants.AliPayNotifNotifyTime], out num))
                        {
                            paymentAliPayNotif.PaymentNotifyTime = num;
                        }
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifBody))
                    {
                        paymentAliPayNotif.PaymentBody = coll[Constants.AliPayNotifBody];
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifTradeStatus))
                    {
                        paymentAliPayNotif.PaymentTradeStatus = coll[Constants.AliPayNotifTradeStatus];
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifIsTotalFeeAdjust))
                    {
                        paymentAliPayNotif.PaymentIsTotalFeeAdjust = coll[Constants.AliPayNotifIsTotalFeeAdjust];
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifTotalFee))
                    {
                        double num;
                        if (double.TryParse(coll[Constants.AliPayNotifTotalFee], out num))
                        {
                            paymentAliPayNotif.PaymentTotalFee = num;
                        }
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifGMTPayment))
                    {
                        DateTime num;
                        if (DateTime.TryParse(coll[Constants.AliPayNotifGMTPayment], out num))
                        {
                            paymentAliPayNotif.PaymentGMTPayment = num;
                        }
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifSellerEmail))
                    {
                        paymentAliPayNotif.PaymentSellerEmail = coll[Constants.AliPayNotifSellerEmail];
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifPrice))
                    {
                        double num;
                        if (double.TryParse(coll[Constants.AliPayNotifPrice], out num))
                        {
                            paymentAliPayNotif.PaymentPrice = num;
                        }
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifNotifyID))
                    {
                        paymentAliPayNotif.PaymentNotifyID = coll[Constants.AliPayNotifNotifyID];
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifUseCoupon))
                    {
                        paymentAliPayNotif.PaymentUseCoupon = coll[Constants.AliPayNotifUseCoupon];
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifSignType))
                    {
                        paymentAliPayNotif.PaymentSignType = coll[Constants.AliPayNotifSignType];
                    }

                    if (ContainsKey(coll, Constants.AliPayNotifSign))
                    {
                        paymentAliPayNotif.PaymentSign = coll[Constants.AliPayNotifSign];
                    }
                    result += "_Ali pay notification got processed success fully.";
                    _paymentDB.PaymentAliPayNotif.Add(paymentAliPayNotif);
                }
                catch (Exception ex)
                {
                    result = String.Format("_{0}", ex.Message);
                }

                paymentsInfo[0].result = result;
                _paymentDB.SaveChanges(); 
                if(paymentAliPayNotif.PaymentTotalFee.HasValue)
                    SMSManager.SmsManagerInstance.SendPaymentConfirmationToPhone(Constants.DefaultPhoneNumberToSendPaymentConfirmation, paymentAliPayNotif.PaymentTotalFee.Value, logger);
            }
            Response.Write("success");
            Response.End();
            logger.Info(Response.ToString());
            return Response;


        }
        private static bool ContainsKey(NameValueCollection collection, string key)
        {
            if (collection.Get(key) == null)
            {
                return collection.AllKeys.Contains(key);
            }

            return true;
        }

        //查询订单
        private QueryOrderData QueryOrder(string transaction_id, string out_trade_no, WECHATAPPTYPE weChatAppType)
        {
            WxPayData req = new WxPayData();
            req.SetValue(Constants.WeChatTransactionId, transaction_id);
            req.SetValue(Constants.WeChatOutTradeNo, out_trade_no);
            QueryOrderData queryOrderData = new QueryOrderData();
            WxPayData res = WxPayApi.OrderQuery(req, weChatAppType);
            if (res!=null && res.GetValue(Constants.WeChatReturnCode).ToString() == "SUCCESS" &&
                res.GetValue(Constants.WeChatResultCode).ToString() == "SUCCESS")
            {
                queryOrderData.status = true;
                queryOrderData.wxPayData = res;
                return queryOrderData;
            }
            else
            {
                queryOrderData.status = false;
                queryOrderData.wxPayData = null;
                return queryOrderData;
            }
        }


        [HttpPost]
        public HttpResponseBase PaymentNotifyActionWeChatAppUser()//(string trade_status, string out_trade_no, string trade_no)
        {
            logger.Info("Received a notification from WeChat for the user app.");
            //接收从微信后台POST过来的数据
            System.IO.Stream s = Request.InputStream;
            int count = 0;
            byte[] buffer = new byte[1024];
            System.Text.StringBuilder builder = new System.Text.StringBuilder();
            while ((count = s.Read(buffer, 0, 1024)) > 0)
            {
                builder.Append(System.Text.Encoding.UTF8.GetString(buffer, 0, count));
            }
            s.Flush();
            s.Close();
            s.Dispose();
            logger.Info("Data from WeChat for the user app " + builder.ToString());
            //转换数据格式并验证签名
            WxPayData data = new WxPayData();
            try
            {
                data.FromXml(builder.ToString());
            }
            catch (Exception ex)
            {

                //若签名错误，则立即返回结果给微信支付后台
                WxPayData res = new WxPayData();
                res.SetValue(Constants.WeChatReturnCode, "FAIL");
                res.SetValue(Constants.WeChatReturnMsg, ex.Message);
                logger.Error("Failed to convert WeChat notification data to xml. Exception: " + (ex != null ?  ex.Message : "undefined"));

            }
            
            string out_trade_no = data.GetValue("out_trade_no").ToString();
            string transaction_id = data.GetValue(Constants.WeChatTransactionId).ToString();

            logger.Info(String.Format("Query WeChat order for the user app with the following parameters {0}, {1} ", out_trade_no, transaction_id));
            //查询订单，判断订单真实性
            QueryOrderData queryOrderData = QueryOrder(transaction_id, out_trade_no, WECHATAPPTYPE.USER);
            ParseWeChatNotifData(queryOrderData, out_trade_no);
            return Response;
        }

        public void ParseWeChatNotifData(QueryOrderData queryOrderData, string out_trade_no)
        {
            var paymentsInfo = _paymentDB.PaymentInfoTable.Where(o => o.PaymentOutTradeNo == out_trade_no).ToList();
            if (paymentsInfo.Count() != 1)
            {
                paymentsInfo[0].result = out_trade_no + " out_trade_no is not in our database or is in more than one row";
                _paymentDB.SaveChanges();

                return;
            }

            WxPayData data = queryOrderData.wxPayData;
            if (data == null)
            {
                paymentsInfo[0].result = "Date received from the WeChat notification is null. This may be because OrderQuery returned a failure status.";
                _paymentDB.SaveChanges();
                return;
            }

            String result = out_trade_no;
            CCS_Payment_AliPayNotif_V1 paymentAliPayNotif = new CCS_Payment_AliPayNotif_V1();
            try
            {
                paymentAliPayNotif.PaymentInfoID = paymentsInfo[0].PaymentInfoID;
                result += data.m_values.ToString();


                SortedDictionary<string, string> coll = new SortedDictionary<string, string>();
                foreach (KeyValuePair<string, object> kv in data.m_values)
                {
                    string key = kv.Key;
                    string value = kv.Value as string;
                    if (value != null)
                    {
                        coll.Add(key, value);
                    }
                }

                if (coll.ContainsKey(Constants.WeChatAppId))
                {
                    paymentAliPayNotif.WeChatAppId = coll[Constants.WeChatAppId];

                }

                if (coll.ContainsKey(Constants.WeChatBankType))
                {
                    paymentAliPayNotif.WeChatBankType = coll[Constants.WeChatBankType];

                }

                if (coll.ContainsKey(Constants.WeChatFeeType))
                {
                    paymentAliPayNotif.WeChatFeeType = coll[Constants.WeChatFeeType];

                }

                if (coll.ContainsKey(Constants.WeChatIsSubscribe))
                {
                    paymentAliPayNotif.WeChatIsSubscribe = coll[Constants.WeChatIsSubscribe];

                }

                if (coll.ContainsKey(Constants.WeChatMchId))
                {
                    paymentAliPayNotif.WeChatMchId = coll[Constants.WeChatMchId];

                }

                if (coll.ContainsKey(Constants.WeChatNonce))
                {
                    paymentAliPayNotif.WeChatNonce = coll[Constants.WeChatNonce];

                }

                if (coll.ContainsKey(Constants.WeChatResultCode))
                {
                    paymentAliPayNotif.WeChatResultCode = coll[Constants.WeChatResultCode];

                }

                if (coll.ContainsKey(Constants.WeChatReturnCode))
                {
                    paymentAliPayNotif.WeChatReturnCode = coll[Constants.WeChatReturnCode];

                }

                if (coll.ContainsKey(Constants.WeChatSign))
                {
                    paymentAliPayNotif.WeChatSign = coll[Constants.WeChatSign];

                }

                if (coll.ContainsKey(Constants.WeChatTimeEnd))
                {
                    paymentAliPayNotif.WeChatTimeEnd = coll[Constants.WeChatTimeEnd];

                }

                if (coll.ContainsKey(Constants.WeChatOpenId))
                {
                    paymentAliPayNotif.WeChatOpenId = coll[Constants.WeChatOpenId];

                }

                if (coll.ContainsKey(Constants.WeChatTradeType))
                {
                    paymentAliPayNotif.WeChatTradeType = coll[Constants.WeChatTradeType];

                }

                if (coll.ContainsKey(Constants.WeChatTransactionId))
                {
                    paymentAliPayNotif.WeChatTransactionId = coll[Constants.WeChatTransactionId];

                }

                if (coll.ContainsKey(Constants.WeChatTotalFee))
                {
                    double num;
                    if (double.TryParse(coll[Constants.WeChatTotalFee], out num))
                    {
                        paymentAliPayNotif.WeChatTotalFee = num / 100;  //FEN to CNY
                    }

                }

                if (coll.ContainsKey(Constants.WeChatCashFee))
                {
                    double num;
                    if (double.TryParse(coll[Constants.WeChatCashFee], out num))
                    {
                        paymentAliPayNotif.WeChatCashFee = num / 100;   //FEN to CNY
                    }

                }



                paymentAliPayNotif.PaymentGMTPayment = DateTime.Now;
                paymentAliPayNotif.PaymentNotifyTime = DateTime.Now;
                paymentAliPayNotif.PaymentCreateTime = DateTime.Now;
                result += "_WeChat notification got processed successfully";
                _paymentDB.PaymentAliPayNotif.Add(paymentAliPayNotif);

            }
            catch (Exception e)
            {
                result += String.Format("_{0}", e.Message);
            }

            paymentsInfo[0].result = result;
            _paymentDB.SaveChanges();

            if (!queryOrderData.status)
            {

                WxPayData res = new WxPayData();
                res.SetValue(Constants.WeChatReturnCode, "FAIL");
                res.SetValue(Constants.WeChatReturnMsg, "订单查询失败");
                logger.Error("QueryOrder status is failure.");
                Response.Write(res.ToXml());
                Response.End();
            }
            //查询订单成功
            else
            {
                WxPayData res = new WxPayData();
                res.SetValue(Constants.WeChatReturnCode, "SUCCESS");
                res.SetValue(Constants.WeChatReturnMsg, "OK");
                logger.Info("QueryOrder status is success.");
                Response.Write(res.ToXml());
                Response.End();

            }
            SMSManager.SmsManagerInstance.SendPaymentConfirmationToPhone(Constants.DefaultPhoneNumberToSendPaymentConfirmation, paymentAliPayNotif.WeChatTotalFee, logger);

        }
        [HttpPost]
        public HttpResponseBase PaymentNotifyActionWeChatAppVendor()//(string trade_status, string out_trade_no, string trade_no)
        {
            logger.Info("Received a notification from WeChat for the vendor app.");
            //接收从微信后台POST过来的数据
            System.IO.Stream s = Request.InputStream;
            int count = 0;
            byte[] buffer = new byte[1024];
            System.Text.StringBuilder builder = new System.Text.StringBuilder();
            while ((count = s.Read(buffer, 0, 1024)) > 0)
            {
                builder.Append(System.Text.Encoding.UTF8.GetString(buffer, 0, count));
            }
            s.Flush();
            s.Close();
            s.Dispose();
            logger.Info("Data from WeChat for the vendor app " + builder.ToString());
            //转换数据格式并验证签名
            WxPayData data = new WxPayData();
            try
            {
                data.FromXml(builder.ToString());
            }
            catch (Exception ex)
            {

                //若签名错误，则立即返回结果给微信支付后台
                WxPayData res = new WxPayData();
                res.SetValue(Constants.WeChatReturnCode, "FAIL");
                res.SetValue(Constants.WeChatReturnMsg, ex.Message);
                logger.Error("Failed to convert WeChat notification data to xml. Exception: " + (ex != null ? ex.Message : "undefined"));

            }

            string out_trade_no = data.GetValue("out_trade_no").ToString();
            string transaction_id = data.GetValue(Constants.WeChatTransactionId).ToString();

            logger.Info(String.Format("Query WeChat order for the vendor app with the following parameters {0}, {1} ", out_trade_no, transaction_id));
            //查询订单，判断订单真实性
            QueryOrderData queryOrderData = QueryOrder(transaction_id, out_trade_no, WECHATAPPTYPE.VENDOR);

            ParseWeChatNotifData(queryOrderData, out_trade_no);
            return Response;
        }

    }
}