﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using CCSPrototypeCommon;
using System.Net;
using System.Web.Script.Serialization;
using System.Threading.Tasks;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;
using Microsoft.WindowsAzure.Storage.Queue;
using Microsoft.WindowsAzure.Storage.RetryPolicies;
using Microsoft.WindowsAzure.Storage.Table;
using Microsoft.WindowsAzure.ServiceRuntime;
using System.IO;
using Newtonsoft.Json;
using System.Data;
using System.Data.SqlTypes;
using System.Globalization;
using System.Data.Entity.Spatial;
using PagedList;
using System.Data.Entity;
using Excel = Microsoft.Office.Interop.Excel;
using OfficeOpenXml;
namespace HomeWeb.Controllers
{
    public class RawDataController : QueryableController
    {
        public static Logger logger
        {
            get
            {
                return new Logger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name);
            }

        }

        private RawDataContext _rawDataDB = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext();
        private CustomerContext _customerDataDB = (CustomerContext)CustomerContextFactory.GetInstance().GetDBEFContext();
        private LogsContext _logsDB = (LogsContext)LogsContextFactory.GetInstance().GetDBEFContext();
        private CloudBlobContainer _meterProfileBlobContainer;
        private CloudQueue _meterProfileQueue;
        private CloudTable _csvTemplateTable;
        private CloudBlobContainer _adsBlobArchiveContainer;
        public class AccessCodeModel
        {
            public string AccessCode { get; set; }
            public string DeviceId { get; set; }

        }

        private static class FileExtensions
        {
            public const String Xlsx = ".xlsx";   //Excel file
            public const String Xls = ".xls";   //Excel file
            public const String Csv = ".csv";   //Csv file
            public const String Txt = ".txt";   //Txt file
        }

        public class AccessModel
        {
            public string AccessCode { get; set; }
            public string DeviceId { get; set; }

            public string CustomerId { get; set; }


        }

        public class DeviceIdModel
        {
            public string DeviceId { get; set; }

        }

        public class CustomerModel
        {
            public string CustomerId { get; set; }
            public string Name { get; set; }
            public CustomerModel(string CustomerId, string Name)
            {
                this.CustomerId = CustomerId;
                this.Name = Name;
            }
        }



        public class MeterProfileModel
        {
            public string MeterDataID { get; set; }
            public string MeterID { get; set; }
            public string MeterType { get; set; }
            public double? Latitude { get; set; }
            public double? Longitude { get; set; }

            public string PhoneUUID { get; set; }
            public string MeterSN { get; set; }

            public string UserName { get; set; }

            public string Address { get; set; }

            public string CustomerId { get; set; }

            public string LocationCode { get; set; }

            public MeterProfileModel(string MeterDataID, string MeterID, string MeterType, double? Latitude, double? Longitude, string PhoneUUID, string UserName, string Address, string MeterSN, string CustomerId, string LocationCode)
            {
                this.MeterDataID = MeterDataID;
                this.MeterID = MeterID;
                this.MeterType = MeterType;
                this.Latitude = Latitude;
                this.Longitude = Longitude;
                this.PhoneUUID = PhoneUUID;
                this.MeterSN = MeterSN;
                this.UserName = UserName;
                this.Address = Address;
                this.CustomerId = CustomerId;
                this.LocationCode = LocationCode;

            }
        }


        public RawDataController()
            : base()
        {
            InitializeCloudStorage();
        }

        // GET: Phone
        public ActionResult Index()
        {
            long? id = 1;
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            var phoneData = _rawDataDB.PhoneDataTable.All(x => x.PhoneUUID == "cc357fbf4895fd6e");

            return View(phoneData);


        }


        public ActionResult RawDataAnalysis()
        {
            return View();
        }

        [HttpPost]
        public ActionResult QueryCollectedDataByPeriod(string startDate, string endDate)
        {
            DateTime start, end;
            if (String.IsNullOrEmpty(startDate) || !DateTime.TryParse(startDate, out start))
            {
                start = DateTime.UtcNow;
            }

            if (String.IsNullOrEmpty(endDate) || !DateTime.TryParse(endDate, out end))
            {
                end = DateTime.UtcNow;
            }

            RawDataAnalysisModel model;
            try
            {
                using (var dbCtx = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext())
                {
                    List<String> allMeters = dbCtx.MeterDataTable.Where(m => m.ModifiedDate == null).Select(x => x.MeterID).Distinct().ToList();
                    List<String> collectedMeters = (from c in dbCtx.RawDataTable
                                                    where c.UpdateTimeStamp >= start && c.UpdateTimeStamp <= end
                                                    select c).Select(c => c.MeterDataID.ToString()).Distinct().ToList();

                    model = new RawDataAnalysisModel(start, end, allMeters, collectedMeters);
                }
            }
            catch (Exception e)
            {
                ViewBag.Msg = e.Message;
                model = null;
                throw new Exception(string.Format("QueryCollectedDataByPeriod failed with {0}", e.Message));
            }

            return PartialView("~/Views/RawData/_RawDataAnalysisPrtView.cshtml", model);
        }

        [HttpPost]
        public JsonResult GetCustomerIdsForDeviceId(DeviceIdModel deviceIdModel)
        {
            try
            {
                if (deviceIdModel == null)
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDNULL + "(access code model)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }

                string DeviceId = deviceIdModel.DeviceId;

                if (DeviceId == null)
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDNULL + "(device id)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }

                if (String.IsNullOrEmpty(DeviceId))
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.DEVICEIDISNULLOREMPTY, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDEMPTY + "(device id)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }

                if (_rawDataDB.PhoneDataTable == null)
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDNULL + "(phonedata table)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }

                var phoneData = _rawDataDB.PhoneDataTable.Where(x => x.PhoneUUID == DeviceId).ToList();

                if (phoneData == null)
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDNULL + "(phone data)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }

                HashSet<string> customerIdsNoDups = new HashSet<string>();


                if (phoneData.Count() > 0)
                {


                    foreach (var phoneDataItem in phoneData)
                    {
                        string tempCustomerID = phoneDataItem.CustomerID.ToString();
                        if (!String.IsNullOrEmpty(tempCustomerID))
                            customerIdsNoDups.Add(tempCustomerID);
                    }
                }


                var customerData = (from customer in _customerDataDB.CustomerTable
                                    where customerIdsNoDups.Contains(customer.CustomerID.ToString())
                                    select customer).ToList();
                var jsonMeterProfiles = new JavaScriptSerializer().Serialize(customerData);

                List<String> result = (from m in _logsDB.MetaTable
                                       where m.Name == Constants.AndroidRawDataBuildVersion
                                       select m.Value).ToList();
                string androidRawDataBuildVersion = "";
                if (result != null && result.Count > 0)
                {
                    androidRawDataBuildVersion = result.First();
                }

                return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.SUCCESS, statusMsg = GetMeterProfilesStatusMsg.SUCCESSREQUEST, customerIds = jsonMeterProfiles, androidRawDataBuildVersion = androidRawDataBuildVersion }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };


            }
            catch (Exception ex)
            {
                return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDEXCPETION + ex.Message }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
            }

        }

        [HttpPost]
        public JsonResult VerifyAccessCodeWithCustomerId(AccessModel accessModel)
        {
            return GetMetersOnAccessModel(accessModel, true, false);
        }

        [HttpPost]
        public JsonResult GetMeterProfiles(AccessModel accessModel)
        {
            return GetMetersOnAccessModel(accessModel, false, true);
        }

        public JsonResult GetMetersOnAccessModel(AccessModel accessModel, bool checkAccessCode, bool returnMeters)
        {
            try
            {
                if (accessModel == null)
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDNULL + "(access code model)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }

                string DeviceId = accessModel.DeviceId;

                string CustomerId = accessModel.CustomerId;

                if (DeviceId == null)
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDNULL + "(device id)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }

                if (CustomerId == null)
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDNULL + "(customerid)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }


                if (String.IsNullOrEmpty(DeviceId))
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.DEVICEIDISNULLOREMPTY, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDEMPTY + "(device id)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }


                if (String.IsNullOrEmpty(CustomerId))
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.ACCESSCODEISNULLOREMPTY, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDEMPTY + "(customer id)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }


                if (_rawDataDB.PhoneDataTable == null)
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDNULL + "(phone data table)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }

                var phoneData = _rawDataDB.PhoneDataTable.Where(x => x.PhoneUUID == DeviceId && x.CustomerID.ToString() == CustomerId).ToList();

                if (phoneData == null)
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDNULL + "(phone data)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }

                if (phoneData.Count == 0)
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.DEVICEIDNOTWHITELISTED, statusMsg = GetMeterProfilesStatusMsg.DEVICEIDNOTWHITELISTED }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }

                if (phoneData.Count > 1)
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.CONSISTENCYERROR, statusMsg = GetMeterProfilesStatusMsg.CONSISTENCYERROR }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }


                var phoneInfo = phoneData.First();

                if (phoneInfo == null)
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDNULL + "(phone info)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }

                if (phoneInfo.DisabledDate != null)
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.DEVICEDISABLED, statusMsg = GetMeterProfilesStatusMsg.DEVICEISDISABLED }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }

                if (checkAccessCode)
                {
                    string AccessCode = accessModel.AccessCode;

                    if (String.IsNullOrEmpty(AccessCode))
                    {
                        return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.ACCESSCODEISNULLOREMPTY, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDEMPTY + "(access code)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                    }

                    if (phoneInfo.PhoneAccessCode != AccessCode)
                    {
                        return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.INCORRECTACCESSCODE, statusMsg = GetMeterProfilesStatusMsg.ACCESSCODEINCORRECT, }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                    }
                }



                if (_rawDataDB.PhoneMeterRefTable == null)
                {
                    return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDNULL + "(phone meter profile table)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }

                var phoneMeters = _rawDataDB.PhoneMeterRefTable.Where(x => x.PhoneID == phoneInfo.PhoneID).ToList();
                HashSet<string> meterDataIds = new HashSet<string>();
                foreach (var phoneMeter in phoneMeters)
                {
                    meterDataIds.Add(phoneMeter.MeterDataID.ToString());
                }


                /*var meterData = (from meter in _rawDataDB.MeterDataTable
                                    where meterDataIds.Contains(meter.MeterDataID.ToString()) && CustomerId.Equals(meter.CustomerID.ToString()) 
                                    select meter).ToList();*/

                //return all meters for testing purposes

                String jsonMeterProfiles = null;
                if(returnMeters)
                {
                    var meterData = (from meter in _rawDataDB.MeterDataTable
                                     where CustomerId.Equals(meter.CustomerID.ToString()) && meter.ModifiedDate == null
                                     select meter).ToList();

                    List<MeterProfileModel> list = new List<MeterProfileModel>();

                    foreach (var m in meterData)
                    {
                        if (m != null)
                            list.Add(new MeterProfileModel(m.MeterDataID.ToString(), m.MeterID, m.MeterType, m.Latitude, m.Longitude, DeviceId, m.UserName, m.Address, m.MeterSN, m.CustomerID.ToString(), m.LocationCode));
                    }
                    jsonMeterProfiles = new JavaScriptSerializer().Serialize(list);


                }
                



                
                List<String> result = (from m in _logsDB.MetaTable
                                       where m.Name == Constants.AndroidRawDataBuildVersion
                                       select m.Value).ToList();
                string androidRawDataBuildVersion = "";
                if (result != null && result.Count > 0)
                {
                    androidRawDataBuildVersion = result.First();
                }

                //return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.SUCCESS, statusMsg = GetMeterProfilesStatusMsg.SUCCESSACCESSCODE, androidRawDataBuildVersion = androidRawDataBuildVersion }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.SUCCESS, statusMsg = GetMeterProfilesStatusMsg.SUCCESSACCESSCODE, meterData = jsonMeterProfiles, androidRawDataBuildVersion = androidRawDataBuildVersion }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };

            }
            catch (Exception ex)
            {
                return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDEXCPETION + ex.Message }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
            }
        }




        
        enum GetMeterProfilesStatusCode
        {
            FAILURE = -1,
            SUCCESS = 200,
            DEVICEDISABLED = 5001,
            INCORRECTACCESSCODE = 5002,
            DEVICEIDISNULLOREMPTY = 5003,
            ACCESSCODEISNULLOREMPTY = 5004,
            DEVICEIDNOTWHITELISTED = 5005,
            CONSISTENCYERROR = 5006,
            CUSTOMERIDISNULLOREMPTY = 5007
        };

        static class GetMeterProfilesStatusMsg
        {

            static public String DEVICEIDNOTWHITELISTED = "该设备未注册。";
            static public String ACCESSCODEINCORRECT = "访问密码错误。";
            static public String UNEXPECTEDNULL = "下载用户档案失败！";
            static public String UNEXPECTEDEMPTY = "下载用户档案失败！";
            static public String UNEXPECTEDEXCPETION = "下载用户档案失败！";
            static public String CONSISTENCYERROR = "该设备被重复注册。";
            static public String SUCCESSACCESSCODE = "下载用户档案成功！";
            static public String SUCCESSREQUEST = "下载用户档案成功！";
            static public String SUCCESSDOWNLOADMETERS = "下载用户档案成功！";
            static public String DEVICEISDISABLED = "设备已被禁用。";

        }

        [Authorize(Roles = "Staff")]
        public ActionResult DeviceWhitelist()
        {

            string currentCustomerId = GetCurrentCustomerGuidString();

            CCS_PhoneData_V1 device = new CCS_PhoneData_V1();
            IEnumerable<CCS_PhoneData_V1> devices = null;


            // Open database
            RawDataContext db = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext();
            try
            {
                devices = (from d in db.PhoneDataTable
                           where d.CustomerID.ToString() == currentCustomerId
                           orderby d.PhoneID ascending
                           select d);

            }
            catch (Exception e)
            {
                throw new HttpException(500, e.Message);
            }

            DeviceModel deviceModel = new DeviceModel(device, devices);

            return View(deviceModel);

        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public async Task<ActionResult> RegisterDevice(FormCollection form)
        {
            string phone = form["DeviceData.PhoneInfo"];
            string uuid = form["DeviceData.PhoneUUID"];
            string accessCode = form["DeviceData.PhoneAccessCode"];
            string memo = form["DeviceData.Memo"];

            if (String.IsNullOrWhiteSpace(uuid) || String.IsNullOrWhiteSpace(accessCode))
            {
                throw new InvalidOperationException("UUID or access code can not be empty");
            }

            try
            {
                string customerId = GetCurrentCustomerGuidString();
                var phoneData = _rawDataDB.PhoneDataTable.Where(x => x.PhoneUUID == uuid && x.CustomerID.ToString() == customerId);
                if (phoneData == null)
                {
                    throw new InvalidOperationException("Null/undefined reference");
                }

                if (phoneData.Count() == 0)
                {

                    CCS_PhoneData_V1 device = new CCS_PhoneData_V1();
                    device.PhoneInfo = phone;
                    device.PhoneUUID = uuid;
                    device.PhoneAccessCode = accessCode;
                    device.Memo = memo;
                    device.ModifiedDate = DateTime.UtcNow;
                    device.CustomerID = Guid.Parse(customerId);
                    //device.DisabledDate = (DateTime)SqlDateTime.MinValue;
                    _rawDataDB.PhoneDataTable.Add(device);
                    await _rawDataDB.SaveChangesAsync();

                }
                else
                {

                    CCS_PhoneData_V1 device = phoneData.First();
                    device.PhoneInfo = phone;
                    device.PhoneUUID = uuid;
                    device.PhoneAccessCode = accessCode;
                    device.Memo = memo;
                    device.ModifiedDate = DateTime.UtcNow;
                    await _rawDataDB.SaveChangesAsync();

                }




            }
            catch (DataException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new Exception(string.Format("RegisterDevice failed with {0}", e.Message));
            }

            return RedirectToAction("DeviceWhitelist");
        }

        public ActionResult EditDeviceInfo(string phoneUUID)
        {
            CCS_PhoneData_V1 device = null;

            if (phoneUUID == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }

            try
            {
                using (RawDataContext db = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext())
                {
                    string customerId = GetCurrentCustomerGuidString();
                    var phoneData = _rawDataDB.PhoneDataTable.Where(x => x.PhoneUUID == phoneUUID && x.CustomerID.ToString() == customerId).ToList();

                    if (phoneData == null)
                    {
                        return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDNULL + "(phone data)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                    }


                    if (phoneData.Count() > 1)
                    {

                        throw new Exception("Multiple devices found for this customer for this phone uuid");
                    }

                    device = phoneData.First();
                    if (device == null)
                    {
                        return HttpNotFound();
                    }
                }
            }
            catch (DataException)
            {
                throw;
            }

            return View(device);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public async Task<ActionResult> UpdateDeviceInfo(FormCollection form)
        {
            string phone = form["PhoneInfo"];
            string uuid = form["PhoneUUID"];
            string accessCode = form["PhoneAccessCode"];
            string memo = form["Memo"];

            if (String.IsNullOrWhiteSpace(uuid) || String.IsNullOrWhiteSpace(accessCode))
            {
                throw new Exception("UUID or access code can not be empty");
            }

            try
            {
                // Open database
                using (RawDataContext db = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext())
                {



                    string customerId = GetCurrentCustomerGuidString();
                    var phoneData = _rawDataDB.PhoneDataTable.Where(x => x.PhoneUUID == uuid && x.CustomerID.ToString() == customerId).ToList();

                    if (phoneData == null)
                    {
                        return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDNULL + "(phone data)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                    }


                    if (phoneData.Count() > 1)
                    {

                        throw new Exception("Multiple devices found for this customer for this phone uuid");
                    }
                    CCS_PhoneData_V1 device = phoneData.First();
                    device.PhoneInfo = phone;
                    device.PhoneAccessCode = accessCode;
                    device.Memo = memo;
                    device.ModifiedDate = DateTime.UtcNow;
                    await _rawDataDB.SaveChangesAsync();
                }
            }
            catch (DataException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new Exception(string.Format("UpdateDeviceInfo failed with {0}", e.Message));
            }

            return RedirectToAction("DeviceWhitelist");
        }

        public async Task<ActionResult> ToggleDeviceStatus(string phoneUUID)
        {
            CCS_PhoneData_V1 device = null;

            if (phoneUUID == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }

            try
            {
                using (RawDataContext db = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext())
                {

                    string customerId = GetCurrentCustomerGuidString();
                    var phoneData = _rawDataDB.PhoneDataTable.Where(x => x.PhoneUUID == phoneUUID && x.CustomerID.ToString() == customerId).ToList();

                    if (phoneData == null)
                    {
                        return new JsonResult() { Data = new { statusCode = GetMeterProfilesStatusCode.FAILURE, statusMsg = GetMeterProfilesStatusMsg.UNEXPECTEDNULL + "(phone data)" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                    }


                    if (phoneData.Count() > 1)
                    {

                        throw new Exception("Multiple devices found for this customer for this phone uuid");
                    }

                    device = phoneData.First();

                    if (device == null)
                    {
                        return HttpNotFound();
                    }

                    if (device.DisabledDate == null)
                    {
                        device.DisabledDate = DateTime.UtcNow;
                    }
                    else
                    {
                        device.DisabledDate = null;
                    }

                    device.ModifiedDate = DateTime.UtcNow;
                    await _rawDataDB.SaveChangesAsync();
                }
            }
            catch (DataException)
            {
                throw;
            }

            return RedirectToAction("DeviceWhitelist");
        }



        //[Authorize(Roles = "Staff, Admin, SysAdmin")]
        public ActionResult ViewMeterInfo(Guid? MeterDataID)
        {
            if (MeterDataID == null)
            {
                return PartialView("_ViewMeterInfo");
            }
            List<CCS_MeterData_Sub_V1> MeterSubInfos = null;
            using (var dbContext = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext())
            {
                dbContext.Configuration.LazyLoadingEnabled = false;
                var temp = dbContext.MeterDataSubTable.Where(x => x.MeterDataID == MeterDataID);

                if (temp.Count() < 1)
                {
                    CCS_MeterData_Sub_V1 EmptySubInfo = new CCS_MeterData_Sub_V1();
                    EmptySubInfo.MeterDataID = MeterDataID ?? new Guid();
                    EmptySubInfo.MeterDataTable = dbContext.MeterDataTable.Find(MeterDataID);
                    MeterSubInfos = new List<CCS_MeterData_Sub_V1>();
                    if (EmptySubInfo.MeterDataTable != null)
                    {
                        MeterSubInfos.Add(EmptySubInfo);
                    }
                }
                else
                {
                    if (dbContext.Entry(temp.First()).Reference(x => x.MeterDataTable).IsLoaded == false)
                    {
                        dbContext.Entry(temp.First()).Reference(x => x.MeterDataTable).Load();
                    }

                    MeterSubInfos = temp.ToList();
                }
            }

            //List<string> PhotoUrls = GetPhotoURLs(MeterSubInfos);

            return PartialView("_ViewMeterInfo", MeterSubInfos);

        }



        private List<string> GetPhotoURLs(List<CCS_MeterData_Sub_V1> MeterSubInfos)
        {
            List<string> PhotoUrls = new List<string>();

            foreach (CCS_MeterData_Sub_V1 info in MeterSubInfos)
            {
                PhotoUrls.Add(info.PictureBlobUri);
            }

            return PhotoUrls;
        }


        public static string ColumnsCustomSeparator = ",";//"$$$###$$$###$$$";
        public static string NewLineSeparator = "\r\n";
        string[] stringSeparators = new string[] { ColumnsCustomSeparator };

        public async Task handleCsvFile(HttpPostedFileBase file, List<String> columns, String blobName)
        {
            if ((file != null) && (file.ContentLength > 0) && !string.IsNullOrEmpty(file.FileName))
            {
                CloudBlockBlob fileBlob = _meterProfileBlobContainer.GetBlockBlobReference(blobName);

                // Upload local csv file to blob storage
                 using (var fileStream = file.InputStream)
                 {
                     await fileBlob.UploadFromStreamAsync(fileStream);
                 }

                using (Stream fs = fileBlob.OpenRead())
                {
                    StreamReader csv = new StreamReader(fs);
                    // Get header
                    string line = csv.ReadLine();
                    columns.AddRange(line.Split(','));
                }
            }
                        
        }


        public async Task  handleXlsFile(HttpPostedFileBase file, List<String> columns, String blobName)
        {
            CloudAppendBlob fileBlob = _meterProfileBlobContainer.GetAppendBlobReference(blobName);
            if ((file != null) && (file.ContentLength > 0) && !string.IsNullOrEmpty(file.FileName))   
            {
                fileBlob.CreateOrReplace();
                string fileName = file.FileName;  
                string fileContentType = file.ContentType;  
                byte[] fileBytes = new byte[file.ContentLength];  
                
                using(var package = new ExcelPackage(file.InputStream))   
                {  
                    var currentSheet = package.Workbook.Worksheets; 
                   if(currentSheet.Count == 0)
                   {
                       throw new Exception("This excel does not contains any worksheets");
                   }
                    var workSheet = currentSheet.First();  
                    var noOfCol = workSheet.Dimension.End.Column;  
                    var noOfRow = workSheet.Dimension.End.Row;  

                    try {
                        int rCnt = 0;
                        int cCnt = 0;

                        String content = String.Empty;
                        long cCells = 0;
                        for (rCnt = 1; rCnt <= noOfRow; rCnt++)
                        {
                            
                            for (cCnt = 1; cCnt <= noOfCol; cCnt++)
                            {

                                String value = String.Empty;
                                if (workSheet.Cells[rCnt, cCnt] != null && workSheet.Cells[rCnt, cCnt].Value != null)
                                {
                                    value = DbFunctions.AsUnicode(workSheet.Cells[rCnt, cCnt].Value.ToString());
                                }
                                if(rCnt == 1)
                                    columns.Add(value);

                                content += value;
                                if (cCnt < noOfCol)
                                {
                                    content += ColumnsCustomSeparator;
                                }
                            }
                            content += NewLineSeparator;

                            //this is needed to avoid memory overload
                            cCells++;
                            if (cCells % 100 == 0)
                            {
                                await fileBlob.AppendTextAsync(content);
                                content = String.Empty;
                            }
                        }
                        await fileBlob.AppendTextAsync(content);

                        
                    }
                    catch(Exception ex)
                    {
                        throw;
                    }
                    finally
                    {

                    }

                }  
            }  
        }
        
        [HttpPost]
        [Authorize(Roles = "Staff")]
        [ValidateAntiForgeryToken]
        //[Authorize(Roles = "Staff")]
        public async Task<ActionResult> UploadMeterProfile(HttpPostedFileBase uploadFile)
        {
            string nextView = "UploadMeterProfile";

            if (ModelState.IsValid && uploadFile != null && uploadFile.ContentLength != 0)
            {
                // Blob path format: CustomerID/yyyy/MMdd/HHmmss.fileExtension
                string dateTimeFormat = "yyyy/MMdd/HHmmss";
                var customerID = GetCurrentCustomerGuidString();
                string blobName = String.Format("{0}/{1}{2}", customerID, DateTime.UtcNow.ToString(dateTimeFormat), Path.GetExtension(uploadFile.FileName));
                
                var result = _csvTemplateTable.Execute(TableOperation.Retrieve(customerID, Constants.MeterTemplateName));
                if (result == null || result.HttpStatusCode == 404 || result.Result == null)
                {
                    result = _csvTemplateTable.Execute(TableOperation.Retrieve(customerID.ToUpperInvariant(), Constants.MeterTemplateName));
                    if (result == null || result.HttpStatusCode == 404 || result.Result == null)
                    {
                        ViewBag.UploadMsg = "Failed to retrieve CSV file mapping template";
                        return View(nextView);
                    }
                }

                string fileExtension = Path.GetExtension(uploadFile.FileName).ToLower();
                List<String> columns = new List<string>();
                switch(fileExtension)
                {
                    case FileExtensions.Xls:
                    case FileExtensions.Xlsx:
                        await handleXlsFile(uploadFile, columns, blobName);
                        break;
                    case FileExtensions.Csv:
                        await handleCsvFile(uploadFile, columns, blobName);
                        break;
                    default:
                        await handleCsvFile(uploadFile, columns, blobName);
                        break;
                }

                var headerList = Utility.GetCsvTableHeader(columns.ToArray());
                var template = (DynamicTableEntity)result.Result;
                EntityProperty mandatoryProp1;
                EntityProperty mandatoryProp2;
                if (!template.Properties.TryGetValue("AccountNumber", out mandatoryProp1))
                {
                    throw new ArgumentNullException("Invalid entity, AccountNumber property not found!");
                }
                if (!template.Properties.TryGetValue("UserNumber", out mandatoryProp2))
                {
                    throw new ArgumentNullException("Invalid entity, UserNumber property not found!");
                }

                string mandatoryProp1Unicode = DbFunctions.AsUnicode(mandatoryProp1.StringValue);
                string mandatoryProp2Unicode = DbFunctions.AsUnicode(mandatoryProp2.StringValue);
                try
                {
                    Utility.GetColumnByHeader(headerList, DbFunctions.AsUnicode(mandatoryProp1Unicode));
                    Utility.GetColumnByHeader(headerList, DbFunctions.AsUnicode(mandatoryProp2Unicode));
                }
                catch (TemplateDataException)
                {
                    ViewBag.UploadMsg = string.Format("This file is missing necessary column {0}, {1}",
                        mandatoryProp1Unicode, mandatoryProp2Unicode
                        );
                    return View(nextView);
                }

                MeterProfileQueueMsg queueMsg = new MeterProfileQueueMsg()
                {
                    CustomerID = null,  // Reserved for now
                    BlobName = blobName
                };
                var queueMsgString = await Task.Factory.StartNew(() => JsonConvert.SerializeObject(queueMsg));
                CloudQueueMessage queueMessage = new CloudQueueMessage(queueMsgString);
                await _meterProfileQueue.AddMessageAsync(queueMessage);
                return RedirectToAction(nextView, new { uploadMsg = "上传成功" });
            }
            ViewBag.UploadMsg = "上传失败或者文件为空";
            return View(nextView);
        }

        public ActionResult UploadMeterProfile(string uploadMsg)
        {
            ViewBag.uploadMsg = uploadMsg;
            return View();
        }

        protected override void InitializeCloudStorage()
        {
            // Open storage account
            CloudStorageAccount storageAccount = CloudStorageAccount.Parse(ConfigurationMgr.GetSetting("StorageConnectionString"));
            CloudBlobClient blobClient = storageAccount.CreateCloudBlobClient();
            blobClient.DefaultRequestOptions.RetryPolicy = new LinearRetry(TimeSpan.FromSeconds(3), 3);

            _adsBlobArchiveContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.AdsBlobArchiveContainerName);
            //_adsBlobArchiveContainer.CreateIfNotExists();

            _meterProfileBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.MeterProfileStorageName);
            //_meterProfileBlobContainer.CreateIfNotExists();

            // Get queue context object
            CloudQueueClient queueClient = storageAccount.CreateCloudQueueClient();
            queueClient.DefaultRequestOptions.RetryPolicy = new LinearRetry(TimeSpan.FromSeconds(3), 3);
            _meterProfileQueue = queueClient.GetQueueReference(CCSPrototypeCommon.Constants.MeterProfileStorageName);
            //_meterProfileQueue.CreateIfNotExists();

            CloudTableClient tableClient = storageAccount.CreateCloudTableClient();
            tableClient.DefaultRequestOptions.RetryPolicy = new LinearRetry(TimeSpan.FromSeconds(3), 3);
            _csvTemplateTable = tableClient.GetTableReference(CCSPrototypeCommon.Constants.CsvTemplateTableName);
            //_csvTemplateTable.CreateIfNotExists();

        }

        public class MeterUploadDataModel
        {
            public List<String> MeterIDList { set; get; }
            public String MeterLocationCode { set; get; }
            public Double Latitude { set; get; }
            public Double Longitude { set; get; }
            public List<String> Images { set; get; }

        };


        [HttpPost]
        public async Task<JsonResult> UploadMeterInfoMobile(MeterUploadDataModel data)
        {
            string responseMsg = String.Empty;
            if (data == null)
            {
                responseMsg += " Meter data was null ";
            }
            else
            {
                //responseMsg = String.Format("LocationCode: {0}, Lat: {1}, Log: {2}",
                //                            data.MeterLocationCode,
                //                            data.Latitude,
                //                            data.Longitude);

                //responseMsg += "MeterDateID: ";

                try
                {
                    using (RawDataContext db = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext())
                    {
                        int i = 0;
                        foreach (String id in data.MeterIDList)
                        {
                            CCS_MeterData_V1 meter = await db.MeterDataTable.FindAsync(new Guid(id));
                            if (meter != null)
                            {
                                meter.LocationCode = data.MeterLocationCode;
                                meter.Latitude = data.Latitude;
                                meter.Longitude = data.Longitude;
                                meter.geography = DbGeography.FromText(string.Format("POINT({1} {0})", meter.Latitude, meter.Longitude));
                                meter.CreationDate = DateTime.UtcNow;
                                await db.SaveChangesAsync();
                                ++i;
                            }
                        }
                        responseMsg += String.Format("{0} meter records have been successfully updated", i);
                    }
                }
                catch (Exception e)
                {
                    logger.Error("Exception in UploadMeterInfoMobile" + e.ToString());
                }

            }
            return new JsonResult() { Data = new { statusCode = "Success", statusMsg = responseMsg }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
        }

        [Authorize(Roles = "Staff, Admin")]
        public ActionResult QueryMeterData()
        {
            return View("QueryMeterByID");
        }

        [Authorize(Roles = "Staff, Admin")]
        public ActionResult EditMeterData(string UserInputMeterID)
        {
            if (string.IsNullOrEmpty(UserInputMeterID))
            {
                return PartialView("_ViewMeterInfo");
            }
            Guid MetaDataIDGuid = Guid.Empty;
            if (false == Guid.TryParse(UserInputMeterID, out MetaDataIDGuid))
            {
                MetaDataIDGuid = Guid.Empty;
            }

            List<CCS_MeterData_V1> MeterDataList = new List<CCS_MeterData_V1>();
            using (var dbContext = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext())
            {
                dbContext.Configuration.LazyLoadingEnabled = false;
                var temp = MetaDataIDGuid == Guid.Empty ?
                    (from x in dbContext.MeterDataTable  // Multiple columns search
                     where
                        x.CreationDate != null &&
                        x.ModifiedDate == null &&
                        (x.AccountNumber == UserInputMeterID //收费号
                            || x.UserName == UserInputMeterID     //用户姓名
                            || x.MeterID == UserInputMeterID      //表ID
                            || x.MeterSN == UserInputMeterID      //表SN 
                            || x.MeterNumber == UserInputMeterID)   //表号
                     orderby x.CreationDate descending
                     group x by new { x.AccountNumber, x.MeterID } into g
                     select g.FirstOrDefault())
                    :
                    (from x in dbContext.MeterDataTable  //GUID match
                     where
                        x.MeterDataID == MetaDataIDGuid
                     select x);

                MeterDataList = temp.ToList();
            }

            if (MeterDataList.Count() <= 1)
            {
                return PartialView("_EditMeterInfo", MeterDataList.FirstOrDefault());
            }
            else
            {
                int pageNumber = 1;
                return PartialView("_QueryResults", MeterDataList.ToPagedList(pageNumber, PageList_PageSize));
            }
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult UpdateMeterData([Bind(Include = "MeterDataID,MeterType,UserName,AccountNumber,UserNumber,MeterSN,MeterID,MeterNumber,Address,ApartmentNumber,UserType,Office,PhoneNumber,Memo1,CutoffDate,CollectType,TerminalNumber,TerminalAddress,TerminalEquipmentID,Memo2,LocationCode,Latitude,Longitude,geography,CreationDate")]CCS_MeterData_V1 model)
        {
            CCS_MeterData_V1 CurrentData = null;

            if (ModelState.IsValid && model != null)
            {
                if (model.MeterDataID == Guid.Empty)
                {
                    return PartialView("_ViewMeterInfo");
                }

                DateTime CurrentTime = DateTime.UtcNow;
                try
                {
                    using (var dbContext = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext())
                    {
                        CurrentData = dbContext.MeterDataTable.Find(model.MeterDataID);

                        if (HasMeterDataChanged(CurrentData, model))
                        {
                            CurrentData.ModifiedDate = CurrentTime;
                            dbContext.Entry(CurrentData).State = System.Data.Entity.EntityState.Modified;

                            model.CreationDate = CurrentTime;
                            model.MeterDataID = Guid.Empty;
                            model.ModifiedDate = null;
                            if (model.Latitude != null && model.Longitude != null)
                            {
                                model.geography = DbGeography.FromText(string.Format("POINT({1} {0})", model.Latitude, model.Longitude));
                            }
                            else
                            {
                                model.geography = null;
                            }

                            dbContext.MeterDataTable.Add(model);
                            dbContext.Entry(model).State = System.Data.Entity.EntityState.Added;

                            dbContext.SaveChanges();
                        }

                    }
                }
                catch (Exception e)
                {
                    ViewBag.ErrorMessage = e.Message;
                }

            }
            else
            {
                foreach (var val in ModelState.Values)
                {
                    if (val.Errors.Count() > 0)
                    {
                        ViewBag.ErrorMessage += string.Empty + val.Errors[0].ErrorMessage + val.Errors[0].Exception + Environment.NewLine;
                    }
                }
            }

            if (CurrentData != null && model != null && CurrentData.MeterDataID != model.MeterDataID)
            {
                return RedirectToAction("EditMeterData", new { UserInputMeterID = model.MeterDataID });
            }
            else
            {
                return PartialView("_EditMeterInfo", model);
            }

        }


        private bool HasMeterDataChanged(CCS_MeterData_V1 a, CCS_MeterData_V1 b)
        {
            return a != null && b != null
                && a.MeterDataID == b.MeterDataID
                && a.AccountNumber == b.AccountNumber
                && a.UserNumber == b.UserNumber
                && (a.Address != b.Address || a.ApartmentNumber != b.ApartmentNumber
                    || a.CollectType != b.CollectType || a.CutoffDate != b.CutoffDate
                    || a.Latitude != b.Latitude || a.LocationCode != b.LocationCode
                    || a.Longitude != b.Longitude || a.Memo1 != b.Memo1
                    || a.Memo2 != b.Memo2 || a.MeterID != b.MeterID
                    || a.MeterNumber != b.MeterNumber || a.MeterSN != b.MeterSN
                    || a.MeterType != b.MeterType || a.Office != b.Office
                    || a.PhoneNumber != b.PhoneNumber || a.TerminalAddress != b.TerminalAddress
                    || a.TerminalEquipmentID != b.TerminalEquipmentID || a.TerminalNumber != b.TerminalNumber
                    || a.UserName != b.UserName || a.UserType != b.UserType
                    );
        }


        [HttpPost]
        public ActionResult MeterHistory(string userNumber, string accountNumber)
        {

            List<CCS_MeterData_V1> meterHistoryList = new List<CCS_MeterData_V1>();
            // Open database
            RawDataContext db = (RawDataContext)RawDataContextFactory.GetInstance().GetDBEFContext();
            var curCustomerID = new Guid(GetCurrentCustomerGuidString());

            var meterData = db.MeterDataTable.Where(x => x.UserNumber == userNumber && x.AccountNumber == accountNumber &&
                x.CustomerID == curCustomerID).OrderByDescending(x => x.CreationDate).ToList();

            if (meterData != null && meterData.Count != 0)
            {

                foreach (CCS_MeterData_V1 m in meterData)
                {

                    if (m.ModifiedDate != null)
                    {
                        meterHistoryList.Add(m);
                    }
                    else
                    {
                        meterHistoryList.Insert(0, m);
                    }
                }
            }



            return PartialView("_MeterHistory", meterHistoryList);


        }

        [HttpGet]
        public async Task<ActionResult> ShowWhatIsNew()
        {


            String blobName = Utils.Constants.MeterAppNoticeBlobName;
            if (string.IsNullOrEmpty(blobName))
            {
                return View();
            }
            HomeWeb.Models.MerchantMsgModel adsMsgModel = null;
            try
            {
                var msgBlob = _adsBlobArchiveContainer.GetBlockBlobReference(blobName);
                string blobContent = await msgBlob.DownloadTextAsync();
                adsMsgModel = await Task.Factory.StartNew(() => JsonConvert.DeserializeObject<HomeWeb.Models.MerchantMsgModel>(blobContent));


            }
            catch (Exception)
            {
                //To do: 
            }

            return View(adsMsgModel);
        }


    }
}