﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using CCSAdmin;
using CCSPrototypeCommon;
using Microsoft.Azure;
using System.Threading.Tasks;

using System.Security.Cryptography.X509Certificates;
namespace HomeWeb.Models
{
    public class UserModelHomeController : Controller
    {
        public static Logger logger
        {
            get
            {
                return new Logger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name);
            }

        }

        private static BillContext _billDB = null;
        private static PaymentContext _paymentDB = null;

        public UserModelHomeController()
        {
            _billDB = (BillContext)BillContextFactory.GetInstance().GetDBEFContext();
            _paymentDB = (PaymentContext)PaymentContextFactory.GetInstance().GetDBEFContext();
        }

        public ActionResult Index(bool? ShowBackBtn)
        {
            ViewBag.ShowBackBtn = ShowBackBtn ?? false;

            if (UserPasswordContext.GetInstance().isUserAuthenticated(Session, Request))
            {
                var mobilePhone = Session[SessionConstants.UserLoginMobileNumber] as string;
                if (string.IsNullOrWhiteSpace(mobilePhone))
                {
                    //log out to avoid infinite loop
                    UserPasswordContext.GetInstance().InvalidateCookieAndLogOut(Session, Response);
                    return RedirectToAction("GeneralError", "Error", new
                    {
                        message = string.Format("当前登录链接无效。"),
                        linkText = "请重新登录。",
                        act = "Index",
                        ctrl = "UserModelHome"
                    });

                }
                string userInputId = mobilePhone;
                CCS_User_V1 user = null;
                if (userInputId.Length < 15)
                {
                    user = (from u in _billDB.UserTable
                                 where u.Cellphone == userInputId
                             select u).FirstOrDefault();
                }
                else
                {
                    user = (from u in _billDB.UserTable
                             where u.IDNumber == userInputId
                             select u).FirstOrDefault();
                }
                if(user == null)
                {
                    logger.Warning(string.Format("No user found with id {0}", userInputId));
                }
                else
                {
                    logger.Info(string.Format("Userid  {0} found for user input id {1}", user.UserID, userInputId));
                    ViewBag.userId = user.UserID;
                }
                return View("ShowUserBill", BillCommon.getCCSBillListWithInputID((BillContext)BillContextFactory.GetInstance().GetDBEFContext(), mobilePhone));
            }
            return View();
        }


        // POST: UserModeHome/Login
        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Login(
            [Bind(Include = "UserInputAccountID")] UserModelHome userModelHome)
        {

            if (userModelHome == null)
            {
                ViewBag.Msg = "未输入登录信息。";
                return View();
            }

            var userInputID = userModelHome.UserInputAccountID;
            //var usersWithThisInputID = BillCommon.getCCSUserListWithInputID((BillContext)BillContextFactory.GetInstance().GetDBEFContext(), userInputID);
            /*if (usersWithThisInputID == null || usersWithThisInputID.Count() == 0)
            {
                return RedirectToAction("GeneralError", "Error", new
                {
                    message = string.Format("Did not find any user with this id:{0}", userInputID),
                    linkText = "try another user",
                    act = "Index",
                    ctrl = "UserModelHome"
                });
            }*/

            try
            {
                //register user in AAD
                if (IsMobileNumber(userInputID))
                {
                    string randPassword = UserPasswordContext.CreateRandomVerificationCode();

                    var errorMsg = UserPasswordContext.GetInstance().SetTempVerificationCodeAndSendToPhone(userInputID, randPassword);


                    if (errorMsg == "测试阶段，显示验证码")
                    {
                        errorMsg = ""; //clear this error message for testing purpose. 
                    }
                    else
                    {
                        randPassword = "";
                    }
                    //send user one-time password
                    return View("SecretDisplayView", new LogInModel { UserName = userInputID, VerificationCode = randPassword, Mobile = userInputID, ErrorMsg = errorMsg });
                }
                else
                {
                    return RedirectToAction("GeneralError", "Error", new
                    {
                        message = string.Format("输入号码无效。"),
                        linkText = "请重新登录。",
                        act = "Index",
                        ctrl = "UserModelHome"
                    });

                    //ViewBag.Msg = "Only mobile phone users could be able to use this feature";
                    //return View("本功能只适用于手机端。");
                }
            }
            catch (System.Data.Entity.Validation.DbEntityValidationException e)
            {
                String msg = "";
                foreach (var eve in e.EntityValidationErrors)
                {
                    msg += string.Format("Entity of type \"{0}\" in state \"{1}\" has the following validation errors:",
                        eve.Entry.Entity.GetType().Name, eve.Entry.State);
                    foreach (var ve in eve.ValidationErrors)
                    {
                        msg += string.Format("- Property: \"{0}\", Error: \"{1}\"",
                            ve.PropertyName, ve.ErrorMessage);
                    }
                }
                logger.Error(msg);
                return RedirectToAction("GeneralError", "Error", new
                {
                    message = string.Format("查询账单时发生错误。此错误信息已经被记录，我们会尽快展开调查。"),
                    linkText = "请重新登录。",
                    act = "Index",
                    ctrl = "UserModelHome"
                });
                //ViewBag.Msg = "查询账单时发生错误。此错误信息已经被记录，我们会尽快展开调查。";
                //return View(e);
            }
            catch (Exception e)
            {

                //ViewBag.Msg = e.Message + " " + (e.InnerException != null ? e.InnerException.Message : "");
                //System.Diagnostics.Trace.WriteLine(e);
                string msg = e.Message + " " + e.InnerException;
                logger.Error(msg);
                return RedirectToAction("GeneralError", "Error", new
                {
                    message = string.Format("查询账单时发生错误。此错误信息已经被记录，我们会尽快展开调查。"),
                    linkText = "请重新登录。",
                    act = "Index",
                    ctrl = "UserModelHome"
                });
                //ViewBag.Msg = "查询账单时发生错误。此错误信息已经被记录，我们会尽快展开调查。";
                //return View(e);
            }


        }


        public ActionResult ViewPaymentHistory(string UserAccountID, DateTime? QueryStart, int? Count, bool? PartialOnly)
        {
            Guid UserGuid;
            string mobilePhone = null;

            if (UserPasswordContext.GetInstance().isUserAuthenticated(Session, Request))
            {
                if (string.IsNullOrWhiteSpace(UserAccountID)
                    || !Guid.TryParse(UserAccountID, out UserGuid))
                {
                    mobilePhone = Session[SessionConstants.UserLoginMobileNumber] as string;
                    if (string.IsNullOrWhiteSpace(mobilePhone))
                    {
                        //log out to avoid infinite loop
                        UserPasswordContext.GetInstance().InvalidateCookieAndLogOut(Session, Response);
                        return RedirectToAction("GeneralError", "Error", new
                        {
                            message = string.Format("当前登录链接无效。"),
                            linkText = "请重新登录。",
                            act = "Index",
                            ctrl = "UserModelHome"
                        });
                    }
                }

                IEnumerable<CCS_Payment_V1> payments = BillCommon.getCCSPaymentsWithUsers(_paymentDB, 
                    BillCommon.getCCSUserListWithInputID(_billDB, mobilePhone),
                    QueryStart, Count);

                if (PartialOnly == null || PartialOnly == false)
                {
                    return View("ViewMyActivities", payments);
                }
                else
                {
                    return PartialView("_ViewMorePaymentHistory", payments);
                }
            }
            else
            {
                return RedirectToAction("Index");
            }

        }


        public ActionResult ViewBillHistory(string UserAccountID, string UserBillType)
        {
            Guid UserGuid;

            if (string.IsNullOrWhiteSpace(UserAccountID)
                || string.IsNullOrWhiteSpace(UserBillType)
                || !Guid.TryParse(UserAccountID, out UserGuid))
            {
                return RedirectToAction("GeneralError", "Error", new
                {
                    message = "查询历史账单失败，输入用户GUID：'"
                        + UserAccountID + "', 查询账单类型 = '" + UserBillType + "'",
                    linkText = "显示我的账单。",
                    act = "Index",
                    ctrl = "UserModelHome"
                });
            }
            if (UserPasswordContext.GetInstance().isUserAuthenticated(Session, Request))
            {
                var billsWithThisUser = BillCommon.getCCSUserBillHistory(
                    (BillContext)BillContextFactory.GetInstance().GetDBEFContext(), UserGuid, UserBillType);
                return View(billsWithThisUser);
            }
            else
            {
                return RedirectToAction("Index");
            }

        }
        public ActionResult Logout(string Redirect)
        {
            UserPasswordContext.GetInstance().InvalidateCookieAndLogOut(Session, Response);
            // redirect back to user mode home so user can obtain verification code.   

            string RedirectActionName = "Index";
            string RedirectControllerName = "UserModelHome";

            if (!string.IsNullOrEmpty(Redirect) && "shake".Equals(Redirect, StringComparison.OrdinalIgnoreCase))
            {
                RedirectActionName = "ReadyForShake";
                RedirectControllerName = "Map";
            }

            return RedirectToAction(RedirectActionName, RedirectControllerName);
        }

        [HttpPost]
        public JsonResult AddToken(string userid, string deviceToken)
        {
            var usersInfo = _billDB.UserTable.Where(o => o.UserID.ToString() == userid).ToList();
            if (usersInfo.Count() != 1)
            {


                return new JsonResult() { Data = new { statuscode = 0, statusmsg = "A record with the specified id cannot be found in our database or more than one is found." }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
            }
            try
            {
                if (!String.IsNullOrEmpty(deviceToken) && deviceToken != usersInfo[0].iOSDeviceToken)
                {
                    usersInfo[0].iOSDeviceToken = deviceToken;
                    usersInfo[0].ModifiedDate = DateTime.UtcNow;
                    _billDB.SaveChanges();
                    return new JsonResult() { Data = new { statuscode = 1, statusmsg = "It was successfully saved." }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
                }



            }
            catch (Exception e)
            {
                return new JsonResult() { Data = new { statuscode = 0, statusmsg = "Exception: " + e.InnerException.Message }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
            }
            return new JsonResult() { Data = new { statuscode = 1, statusmsg = "Sucess" }, JsonRequestBehavior = JsonRequestBehavior.AllowGet };
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult GetAuthenticatedBillsByVerificationCode(LogInModel loginInfo)
        {
            ViewBag.userid = null;
            CCSPrototypeCommon.UserPasswordContext.VerificationFailure failure = UserPasswordContext.GetInstance().IsUserAuthenticatedByVerificationCode(loginInfo.Mobile, loginInfo.VerificationCode);
            if (failure == UserPasswordContext.VerificationFailure.Passed)
            {
                //after verification is passed create a new user in Users table
                BillContext dbBill = (BillContext)BillContextFactory.GetInstance().GetDBEFContext();
                var record = dbBill.UserTable.Where(item => item.Cellphone == loginInfo.Mobile).FirstOrDefault();
                if (record == null)
                {
                    var userRecord = new CCS_User_V1();
                    userRecord.Cellphone = loginInfo.Mobile;
                    userRecord.ModifiedDate = DateTime.UtcNow;
                    dbBill.UserTable.Add(userRecord);
                    dbBill.SaveChanges();

                    Session[SessionConstants.UserLoginGuid] = userRecord.UserID;
                }
                else
                {
                    Session[SessionConstants.UserLoginGuid] = record.UserID;
                }

                Session[SessionConstants.UserIsAuthenticatedByVerificationCode] = true;
                Session[SessionConstants.UserLoginMobileNumber] = loginInfo.Mobile;


                if (loginInfo.SaveCookie)
                {
                    UserPasswordContext.GetInstance().CreatePermantPasswordAndWriteIntoCookie(Response, loginInfo.Mobile);
                }
                ViewBag.userid = loginInfo.Mobile;
                //var data = BillCommon.getCCSBillListWithInputID((BillContext)BillContextFactory.GetInstance().GetDBEFContext(), loginInfo.Mobile);
                //return View("ShowUserBill", data);
                return RedirectToAction("Index");
            }
            else
            {
                return RedirectToAction("GeneralError", "Error", new
                {
                    message = failure == UserPasswordContext.VerificationFailure.CodeExpired ?
                        "验证码已经过期。" : "输入验证码错误。",
                    linkText = "请重新登录。",
                    act = "Index",
                    ctrl = "UserModelHome"
                });
            }
        }


        private bool IsMobileNumber(string userInputId)
        {
            //Todo: Add more intelegience here. 
            if (userInputId.Length >= 15)
                return false;
            return true;
        }


        public ActionResult AboutUs()
        {
            return View();
        }

    }
}
