﻿using System.Collections;
using System.Data.Entity;
using System.Reflection;
using System.Web;
using CCSPrototypeCommon;
using CCSPrototypeCommon.Query;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;
using Microsoft.WindowsAzure.Storage.Queue;
using Microsoft.WindowsAzure.Storage.RetryPolicies;
using Microsoft.WindowsAzure.Storage.Table;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using PagedList;

namespace HomeWeb.Controllers
{
    public partial class VerifiedBillController : QueryableController
    {
        public VerifiedBillController()
            : base()
        {
        }

        // Show default page
        public ActionResult Index()
        {
            return View();
        }

        public ActionResult QueryBill()
        {
            return this.RedirectToAction("QueryFiltersUI", "VerifiedBill", new { queryType = "VerifiedBillQuery" });
        }

        public override ActionResult QueryBillResultPagedWithType(int? page, string billTypeValue)
        {
            query = SubVerifiedBillQuery.GetQueryType(billTypeValue);

            int pageNumber;
            switch (query)
            {
                case QueryType.CCSOnlyWeChatPayment:

                    var result1 = (IEnumerable<Payment_Wechat_Statement_Comparison_Fail_Transaction_On_CCS_Only_V1>)Session["filterResults"];
                    pageNumber = page ?? 1;
                    return PartialView("_QueryResultsWeChatCCSOnly", result1.ToPagedList(pageNumber, PageList_PageSize));

                case QueryType.WeChatOnlyPayment:
                    var result2 = (IEnumerable<Payment_Wechat_Statement_Comparison_Fail_Transaction_On_Wechat_Only_V1>)Session["filterResults"];
                    pageNumber = page ?? 1;
                    return PartialView("_QueryResultsWeChatServerOnly", result2.ToPagedList(pageNumber, PageList_PageSize));

                case QueryType.MisMatchWeChatPayment:

                    var result3 = (IEnumerable<Payment_Wechat_Statement_Comparison_Fail_Transaction_Match_Amount_Not_Match_V1>)Session["filterResults"];
                    pageNumber = page ?? 1;
                    return PartialView("_QueryResultsWeChatMismatch", result3.ToPagedList(pageNumber, PageList_PageSize));

                case QueryType.SuccessfulWeChatPayment:

                    var result4 = (IEnumerable<Payment_Wechat_Statement_Comparison_SUCCESS_V1>)Session["filterResults"];
                    pageNumber = page ?? 1;
                    return PartialView("_QueryResultsWeChatSucceed", result4.ToPagedList(pageNumber, PageList_PageSize));

                default:
                    throw new Exception(string.Format("Unexpected QueryType:{0} in VerifiedBillController", query));
            }

        }

        

        //
        // Query Bill methods
        //
        [HttpPost]
        public override ActionResult QueryBillResult(FormCollection collection)
        {
            int pageNumber;
            string billTypeValue = collection["billType"];
            query = SubVerifiedBillQuery.GetQueryType(billTypeValue);
 
            switch (query)
            {
                case QueryType.CCSOnlyWeChatPayment:
                    IEnumerable<Payment_Wechat_Statement_Comparison_Fail_Transaction_On_CCS_Only_V1> result1 = FilterApplied<Payment_Wechat_Statement_Comparison_Fail_Transaction_On_CCS_Only_V1>(collection);
                    pageNumber = 1;
                    return PartialView("_QueryResultsWeChatCCSOnly", result1.ToPagedList(pageNumber, PageList_PageSize));

                case QueryType.WeChatOnlyPayment:
                    IEnumerable<Payment_Wechat_Statement_Comparison_Fail_Transaction_On_Wechat_Only_V1> result2 = FilterApplied<Payment_Wechat_Statement_Comparison_Fail_Transaction_On_Wechat_Only_V1>(collection);
                    pageNumber = 1;
                    return PartialView("_QueryResultsWeChatServerOnly", result2.ToPagedList(pageNumber, PageList_PageSize));

                case QueryType.MisMatchWeChatPayment:
                    IEnumerable<Payment_Wechat_Statement_Comparison_Fail_Transaction_Match_Amount_Not_Match_V1> result3 = FilterApplied<Payment_Wechat_Statement_Comparison_Fail_Transaction_Match_Amount_Not_Match_V1>(collection);
                    pageNumber = 1;
                    return PartialView("_QueryResultsWeChatMismatch", result3.ToPagedList(pageNumber, PageList_PageSize));

                case QueryType.SuccessfulWeChatPayment:
                    IEnumerable<Payment_Wechat_Statement_Comparison_SUCCESS_V1> result4 = FilterApplied<Payment_Wechat_Statement_Comparison_SUCCESS_V1>(collection);
                    pageNumber = 1;
                    return PartialView("_QueryResultsWeChatSucceed", result4.ToPagedList(pageNumber, PageList_PageSize));
                default:
                    throw new Exception(string.Format("Unexpected QueryType:{0} in VerifiedBillController", query));
            }
        }

        protected override string GetAssemblyQualifiedName()
        {
            switch (query) //query is assigned in base class.
            {
                case QueryType.CCSOnlyWeChatPayment:
                    return typeof(Payment_Wechat_Statement_Comparison_Fail_Transaction_On_CCS_Only_V1).AssemblyQualifiedName;

                case QueryType.WeChatOnlyPayment:
                    return typeof (Payment_Wechat_Statement_Comparison_Fail_Transaction_On_Wechat_Only_V1).AssemblyQualifiedName;

                case QueryType.MisMatchWeChatPayment:
                    return typeof (Payment_Wechat_Statement_Comparison_Fail_Transaction_Match_Amount_Not_Match_V1).AssemblyQualifiedName;
                
                case QueryType.SuccessfulWeChatPayment:
                    return typeof (Payment_Wechat_Statement_Comparison_SUCCESS_V1).AssemblyQualifiedName;

                default:
                    throw new Exception(string.Format("Unexpected QueryType:{0} in VerifiedBillController: GetAssemblyQualifiedName", query));
            }
        }

        protected override IEnumerable<string> FillQueryBaseTypeList(QueryType queryType)
        {
            List<string> result = new List<string>();
            switch (queryType)
            {
                case QueryType.VerifiedBillQuery:
                    result.Add(SubVerifiedBillQuery.SuccessfulWeChatPayment);
                    result.Add(SubVerifiedBillQuery.MisMatchWeChatPayment);
                    result.Add(SubVerifiedBillQuery.CcsOnlyWeChatPayment);
                    result.Add(SubVerifiedBillQuery.WeChatOnlyPayment);
                    break;
                default:
                    throw new Exception("Unsupported query type: " + queryType);
            }

            return result;
        }

        protected override IEnumerable<PropertyInfo> GetProperties(QueryType queryType)
        {
            List<PropertyInfo> result = new List<PropertyInfo>();
            query = queryType;

            switch (queryType)
            {
               case QueryType.SuccessfulWeChatPayment:
                    var props = typeof (Payment_Wechat_Statement_Comparison_SUCCESS_V1).GetProperties();
                    var subPros = props.Where(p => p.Name == "TradeTime");
                    result.AddRange(subPros);
                    break;
                case QueryType.MisMatchWeChatPayment:
                    var props2 = typeof(Payment_Wechat_Statement_Comparison_Fail_Transaction_Match_Amount_Not_Match_V1).GetProperties();
                    var subPros2 = props2.Where(p => p.Name == "TradeTime");
                    result.AddRange(subPros2);
                    break;
                case QueryType.CCSOnlyWeChatPayment:
                    var props3 = typeof(Payment_Wechat_Statement_Comparison_Fail_Transaction_On_CCS_Only_V1).GetProperties();
                    var subPros3 = props3.Where(p => p.Name == "TradeTime");
                    result.AddRange(subPros3);
                    break;
                case QueryType.WeChatOnlyPayment:
                    var props4 = typeof(Payment_Wechat_Statement_Comparison_Fail_Transaction_On_Wechat_Only_V1).GetProperties();
                    var subPros4 = props4.Where(p => p.Name == "TradeTime");
                    result.AddRange(subPros4);
                    break;
                default:
                    throw new Exception(string.Format("unknown query type {0}", queryType));
            }
            return result;
        }

        protected override IEnumerable<T> GetQueryResult<T>(FormCollection collection, List<AbstractFilter> searchCriteria, QueryType queryType)
        {
            // Open database
            PaymentContext db = (PaymentContext)PaymentContextFactory.GetInstance().GetDBEFContext();

            switch (queryType)
            {
                case QueryType.SuccessfulWeChatPayment:

                    IEnumerable<Payment_Wechat_Statement_Comparison_SUCCESS_V1> result1 = 
                        QueryResultByType<Payment_Wechat_Statement_Comparison_SUCCESS_V1>(searchCriteria, db.Payment_Wechat_Statement_Comparison_SUCCESS_V1);
                    return result1 as IEnumerable<T>;
                case QueryType.MisMatchWeChatPayment:
                    IEnumerable<Payment_Wechat_Statement_Comparison_Fail_Transaction_Match_Amount_Not_Match_V1> result2 =
                        QueryResultByType<Payment_Wechat_Statement_Comparison_Fail_Transaction_Match_Amount_Not_Match_V1>(searchCriteria, db.Payment_Wechat_Statement_Comparison_Fail_Transaction_Match_Amount_Not_Match_V1);
                    return result2 as IEnumerable<T>;

                case QueryType.CCSOnlyWeChatPayment:
                    IEnumerable<Payment_Wechat_Statement_Comparison_Fail_Transaction_On_CCS_Only_V1> result3 =
                        QueryResultByType<Payment_Wechat_Statement_Comparison_Fail_Transaction_On_CCS_Only_V1>(searchCriteria, db.Payment_Wechat_Statement_Comparison_Fail_Transaction_On_CCS_Only_V1);
                    return result3 as IEnumerable<T>;

                case QueryType.WeChatOnlyPayment:
                    IEnumerable<Payment_Wechat_Statement_Comparison_Fail_Transaction_On_Wechat_Only_V1> result4 =
                        QueryResultByType<Payment_Wechat_Statement_Comparison_Fail_Transaction_On_Wechat_Only_V1>(searchCriteria, db.Payment_Wechat_Statement_Comparison_Fail_Transaction_On_Wechat_Only_V1);
                    return result4 as IEnumerable<T>;

                default:
                    throw new Exception(string.Format("unknown query type {0}", queryType));
            }
        }

        private static IEnumerable<T> QueryResultByType<T>(List<AbstractFilter> searchCriteria, DbSet<T> dbSet ) 
            where T:class
        {
            IQueryable<T> meters;
            IEnumerable<T> result = null;
            try
            {
                meters = (from m in dbSet select m).AsQueryable().ApplySearchCriteria(searchCriteria.AsEnumerable());
            }
            catch (Exception e)
            {
                throw new HttpException(500, e.Message);
            }

            if (meters == null)
            {
                result = new List<T>();
            }
            else
            {
                result = meters.ToList();
            }
            return result;
        }

    }
}
