﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.Storage;
using CCSPrototypeCommon;
using Microsoft.WindowsAzure.Storage.Blob;
using System.Text;
using System.IO;
using ThreadedWorkerRole;
using Microsoft.Azure;
using CCSPayments.Model;
using CCSPayments;
using Alipay = AlipayStatement;
namespace HomeWorker
{
    
    class WxConfigurationOverride : WxConfiguration
    {
        string _appID;
        string _merchantID;
        string _appSecret;
        string _paymentKey;

        public WxConfigurationOverride(string appIdKey, string merchantIdKey, string appSecretKey, string paymentKey)
        {
            _appID = Utility.ReadSetting(appIdKey);
            _merchantID = Utility.ReadSetting(merchantIdKey);
            _appSecret = Utility.ReadSetting(appSecretKey);
            _paymentKey = Utility.ReadSetting(paymentKey);
        }

        public override string AppID
        {
            get
            {
                return _appID;
            }
        }

        public override string AppSecret
        {
            get
            {
                return _appSecret;
            }
        }

        public override string MerchantID
        {
            get
            {
                return _merchantID;
            }
        }
        public override string PaymentSecretKey
        {
            get
            {
                return _paymentKey;
            }
        }
    }

    abstract class StatementBase
    {
        private LogsContext _logsDB = (LogsContext)LogsContextFactory.GetInstance().GetDBEFContext();
        static DateTime defaultLastDownloadedDate = new DateTime(2015, 11, 1);
        DateTime GetLastDownloadedStatementTimeStampInGMT()
        {
            string value = Utility.ReadSetting(_lastDownloadedDateTag);
            
            DateTime lastDownloadedDate = defaultLastDownloadedDate;
            if (!DateTime.TryParse(value, out lastDownloadedDate))
            {
                lastDownloadedDate = defaultLastDownloadedDate;
            }

            return lastDownloadedDate;
        }
        void SetLastDownloadedStatementTimeStampInGMT(DateTime date)
        {
            Utility.WriteSetting(_lastDownloadedDateTag, date.ToShortDateString());           
        }
      
        string _lastDownloadedDateTag;
        public StatementBase(string lastDownloadedDateTag)
        {
            _lastDownloadedDateTag = lastDownloadedDateTag;
        }

        public bool Download() 
        {
            DateTime beginTime = GetLastDownloadedStatementTimeStampInGMT().AddDays(1);
            
            DateTime endTime = DateTime.Now.Date;

            if (beginTime >= endTime) return false; // all dates are checked, should wait for 1 more day.

            bool hasMoreWork = DownloadStatement(beginTime);
            if (hasMoreWork)
            {
                SetLastDownloadedStatementTimeStampInGMT(beginTime);
            }

            return hasMoreWork; // not done yet, we need more loops
        }

        public abstract bool DownloadStatement(DateTime beginTime);
       
    }

    class AlipayStatement : StatementBase
    {
       public AlipayStatement() 
           : base(Constants.StatementLastDownloadedDateAlipay)
        {
        }

       public override bool DownloadStatement(DateTime beginTimeInGMT)
       {
           Alipay.AlipayStatementDownloader downloader = new Alipay.AlipayStatementDownloader();
           List<Alipay.AlipayStatementQuery> results = new List<Alipay.AlipayStatementQuery>();
           string errorCode = "";
           try
           {
               bool isSucceeded = downloader.Download(beginTimeInGMT, results, out errorCode);
               if (isSucceeded)
               {
                   var db = (PaymentContext)PaymentContextFactory.GetInstance().GetDBEFContext();


                   foreach (var result in results)
                   {
                       CCS_Payment_Statement_Alipay_V1 record = new CCS_Payment_Statement_Alipay_V1();
                       record.balance = result.balance;
                       record.buyer_account = result.buyer_account;
                       record.currency = result.currency;
                       record.deposit_bank_no = result.deposit_bank_no;
                       record.goods_title = result.goods_title;
                       record.income = result.income;
                       record.iw_account_log_id = result.iw_account_log_id;
                       record.memo = result.memo;
                       record.merchant_out_order_no = result.merchant_out_order_no;
                       record.outcome = result.outcome;
                       record.partner_id = result.partner_id;
                       record.rate = result.rate;
                       record.seller_account = result.seller_account;
                       record.seller_fullname = result.seller_fullname;
                       record.service_fee = result.service_fee;
                       record.service_fee_ratio = result.service_fee_ratio;
                       record.sign_product_name = result.sign_product_name;
                       record.sub_trans_code_msg = result.sub_trans_code_msg;
                       record.total_fee = result.total_fee;
                       record.trade_no = result.trade_no;
                       record.trade_refund_amount = result.trade_refund_amount;
                       record.trans_code_msg = result.trans_code_msg;
                       record.trans_date = result.trans_date;

                       db.PaymentStatementAlipayTable.Add(record);
                   }

                   try
                   {
                       db.SaveChanges();
                       return true;
                   }
                   catch (Exception e)
                   {
                       Trace.TraceError("Exception in DownloadAlipayStatement: '{0}'", e.Message);
                       throw;
                   }
               }
               else
               {
                   // download failed, check the error code;
                   throw new Exception(errorCode);
               }
           }
           catch (Exception e)
           {
               Trace.TraceError("Exception in DownloadAlipayStatement: '{0}'", e.Message);
               throw;
           }
           throw new Exception("not possible to here");
       }
    }

    class WechatStatement : StatementBase
    {
        WxConfiguration _config;
        public WechatStatement(WxConfiguration config, string lastDownloadedDateTag)
            : base(lastDownloadedDateTag)
        {
            _config = config;
        }

        public override bool DownloadStatement(DateTime beginTimeInGMT)
        {
            DateTime billDate = beginTimeInGMT;
            BillType billType = BillType.ALL;

            WxPayment wxPayApi = new WxPayment();
            IPaymentData resp = wxPayApi.DownloadBill(billDate, billType, _config);
            WxDownloadBillResponse wxDownloadBillResponse = (WxDownloadBillResponse)resp;

            if (wxDownloadBillResponse == null)
                throw new Exception(String.Format("Error occured when download wechat statement: no response on date {0}", billDate));
            else if (!wxDownloadBillResponse.RequestStatus)
            {
                if (string.Compare(wxDownloadBillResponse.ErrorCode, "FAIL", true) != 0)
                    throw new Exception(String.Format("Error occured when download wechat statement: unkown errorcode on date {0} : {1}", billDate, wxDownloadBillResponse.ErrorCode));
                if (string.Compare(wxDownloadBillResponse.ErrorMessage, "No Bill Exist", true) == 0)
                {
                    // good case, ignore this
                    return true; // continue query next day
                }
                else if (string.Compare(wxDownloadBillResponse.ErrorMessage, "invalid bill_date", true) == 0)
                {
                    // good case, we should stop now
                    return false; // stop download
                }
                else
                    throw new Exception(String.Format("Error occured when download wechat statement: unkown ErrorMessage on date {0} : {1}", billDate, wxDownloadBillResponse.ErrorMessage));
            }
            else
            {
                if (wxDownloadBillResponse.ResponseDetails == null)
                    throw new Exception(String.Format("Error occured when download wechat statement: no responseDetails on date {0}", billDate));
                var db = (PaymentContext)PaymentContextFactory.GetInstance().GetDBEFContext();

                foreach (WxDownloadBillResponseDetailsBillTypeAll detail in wxDownloadBillResponse.ResponseDetails)
                {
                    CCS_Payment_Statement_Wechat_V1 record = new CCS_Payment_Statement_Wechat_V1();
                    //record.StatementID = Guid.NewGuid();
                    record.TradeTime = detail.TradeTime;
                    record.WxAppId = detail.WxAppId;
                    record.MerchantId = detail.MerchantId;
                    record.SubMerchantId = detail.SubMerchantId;
                    record.DeviceId = detail.DeviceId;
                    record.WxOrderId = detail.WxOrderId;
                    record.MerchantOrderId = detail.MerchantOrderId;
                    record.UserId = detail.UserId;
                    record.TransactionType = detail.TransactionType;
                    record.TransactionStatus = detail.TransactionStatus;
                    record.PayerBank = detail.PayerBank;
                    record.CurrencyType = detail.CurrencyType;
                    record.TotalPaymentAmount = detail.TotalPaymentAmount;
                    record.TotalCouponPaymentAmount = detail.TotalCouponPaymentAmount;
                    record.ProductName = detail.ProductName;
                    record.ProductDescription = detail.ProductDescription;
                    record.ProcessingFee = detail.ProcessingFee;
                    record.ProcessingFeeRate = detail.ProcessingFeeRate;
                    record.WxRefundId = detail.WxRefundId;
                    record.MerchantRefundId = detail.MerchantRefundId;
                    record.RefundAmount = detail.RefundAmount;
                    record.RefundCouponAmount = detail.RefundCouponAmount;
                    record.RefundType = detail.RefundType;
                    record.RefundStatus = detail.RefundStatus;
                    db.PaymentStatementWechatTable.Add(record);
                }
                try
                {
                    db.SaveChanges();
                    return true;
                }
                catch (Exception e)
                {
                    Trace.TraceError("Exception in DownloadWechatStatement: '{0}'", e.Message);
                    throw;
                }
            }

            throw new Exception("not possible to here");
        }
    }

    class WechatVenderStatement : WechatStatement
    {
        public WechatVenderStatement()
            : base(new WxConfigurationOverride(Constants.StatementWechatVenderAppId, Constants.StatementWechatVenderMerchantID, Constants.StatementWechatVenderSecretKey, Constants.StatementWechatVenderPaymentKey), Constants.StatementLastDownloadedDateWechatVender)
        { }
    }

    class WechatUserStatement : WechatStatement
    {
        public WechatUserStatement() : base(new WxConfigurationOverride(Constants.StatementWechatUserAppId, Constants.StatementWechatUserMerchantID, Constants.StatementWechatUserSecretKey, Constants.StatementWechatUserPaymentKey), Constants.StatementLastDownloadedDateWechatUser)
        {}
    }

    class DownloadStatementWorker : WorkerEntryPoint
    {
        public static Logger logger
        {
            get
            {
                return new Logger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name);
            }

        }

        private readonly CancellationTokenSource cancellationTokenSource = new CancellationTokenSource();
        private readonly ManualResetEvent runCompleteEvent = new ManualResetEvent(false);

        public override void Run()
        {

            Trace.TraceInformation("DownloadStatementWorker is running");

            int exceptionCount = 0;

            while (true)
            {
                try
                {
                    runCompleteEvent.Reset();

                    List<StatementBase> statements = new List<StatementBase>();
                    statements.Add(new AlipayStatement());
                    statements.Add(new WechatUserStatement());
                    statements.Add(new WechatVenderStatement());
                    bool done = true;
                    foreach (var statement in statements)
                    {
                        try
                        {
                            bool hasMoreWork = statement.Download(); ;
                            done &= !hasMoreWork;
                            // success, reset exceptionCount to 0
                            exceptionCount = 0;
                        }
                        catch (Exception e)
                        {
                            exceptionCount++;
                            Trace.TraceError("Exception in {0}: '{1}'", statement.ToString(), e.Message);
                            logger.logException(e);
                        }
                    }

                    runCompleteEvent.Set();

                    int delay = 0;
                    if (done && exceptionCount == 0) // completed, without exception
                    {
                        // sleep to next 10 am
                        var now = DateTime.Now;
                        var next = now.AddDays(1);
                        next = next.AddHours(10 - next.Hour);
                        next = next.AddMinutes(-next.Minute);
                        delay = (int)((next - now).TotalMilliseconds);
                        logger.Info(string.Format("SqlTaskWorker is done at {0}. sleep for 1 day.", now));
                    }
                    else if (exceptionCount > 0) // hit expection in consecutive loops
                    {
                        // sleep 1 minute per hit, max 10 minutes. to avoid flush wechat/alipay
                        delay = 60000 * Math.Max(exceptionCount, 10);
                        logger.Info(string.Format("SqlTaskWorker hit exception at {0}, retry in {1} seconds", DateTime.Now, delay));
                    }
                    else if (!done)
                    {
                        // we have more data to download, wait for 5 seconds.
                        delay = 1000;
                        logger.Info(string.Format("SqlTaskWorker is done for {0}, wait for 5 seconds to download next day's statement", DateTime.Now));
                    }
                    Thread.Sleep(delay);
                }
                catch(Exception e)
                {
                    logger.logException(e);
                }
            }
        }

        public override bool OnStart()
        {
            // TODO, what is this? do we need 12 here?

            // Set the maximum number of concurrent connections
            ServicePointManager.DefaultConnectionLimit = 12;

            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.

            bool result = base.OnStart();

            Trace.TraceInformation("DownloadStatementWorker has been started");

            return result;
        }

        public override void OnStop()
        {
            Trace.TraceInformation("DownloadStatementWorker is stopping");

            this.cancellationTokenSource.Cancel();
            this.runCompleteEvent.WaitOne();

            base.OnStop();

            Trace.TraceInformation("DownloadStatementWorker has stopped");
        }

        private async Task RunAsync(CancellationToken cancellationToken)
        {
            // TODO: Replace the following with your own logic.
            while (!cancellationToken.IsCancellationRequested)
            {
                Trace.TraceInformation("Working");
                await Task.Delay(1000);
            }
        }
    }
}

