﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.Storage;
using CCSPrototypeCommon;
using Microsoft.WindowsAzure.Storage.Blob;
using System.Text;
using System.IO;
using ThreadedWorkerRole;
using Microsoft.Azure;
using CCSPayments.Model;
using CCSPayments;
namespace HomeWorker
{

    class SqlTaskWorker : WorkerEntryPoint
    {
        public static Logger logger
        {
            get
            {
                return new Logger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name);
            }

        }
        private readonly CancellationTokenSource cancellationTokenSource = new CancellationTokenSource();
        private readonly ManualResetEvent runCompleteEvent = new ManualResetEvent(false);

        public override void Run()
        {
            Trace.TraceInformation("SqlTaskWorker is running");


            while (true)
            {
                runCompleteEvent.Reset();

                //
                try
                {
                    var db = (PaymentContext)PaymentContextFactory.GetInstance().GetDBEFContext();

                    db.Database.ExecuteSqlCommand("exec spRunDailyWork");
                }
                catch (Exception e)
                {

                    Trace.TraceError("Exception in SqlTaskWorker: '{0}'", e.Message);
                    logger.logException(e);
                }

                runCompleteEvent.Set();

               

                // sleep to next 11 am
                var now = DateTime.Now;
                var next = now.AddDays(1);
                next = next.AddHours(11 - next.Hour);
                next = next.AddMinutes(-next.Minute);
                int delay = (int)((next - now).TotalMilliseconds);

                logger.Info(string.Format("SqlTaskWorker is done at {0}. sleep for 1 day", now));

                Thread.Sleep(delay);
            }
        }

        public override bool OnStart()
        {
            // TODO, what is this? do we need 12 here?

            // Set the maximum number of concurrent connections
            ServicePointManager.DefaultConnectionLimit = 12;

            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.

            bool result = base.OnStart();

            Trace.TraceInformation("SqlTaskWorker has been started");

            return result;
        }

        public override void OnStop()
        {
            Trace.TraceInformation("SqlTaskWorker is stopping");

            this.cancellationTokenSource.Cancel();
            this.runCompleteEvent.WaitOne();

            base.OnStop();

            Trace.TraceInformation("SqlTaskWorker has stopped");
        }

        private async Task RunAsync(CancellationToken cancellationToken)
        {
            // TODO: Replace the following with your own logic.
            while (!cancellationToken.IsCancellationRequested)
            {
                Trace.TraceInformation("Working");
                await Task.Delay(1000);
            }
        }
    }
}

