﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Diagnostics;
using System.Configuration;
using System.Linq;
using System.Net;
using System.Threading;
using System.Threading.Tasks;
using System.Data.Entity;
using System.Data;
using System.Web;
using CCSPrototypeCommon;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;
using Microsoft.WindowsAzure.Storage.Queue;
using Microsoft.WindowsAzure.Storage.Table;
using Microsoft.WindowsAzure.Storage.Table.DataServices;
using Microsoft.WindowsAzure.Storage.RetryPolicies;
using ThreadedWorkerRole;
using Microsoft.Azure;
using Newtonsoft.Json;
using System.Globalization;
using System.Text;
using System.Data.SqlClient;


namespace HomeWorker
{
    public class TransferMeterProfileToDBWorker : WorkerEntryPoint
    {
        public static Logger logger
        {
            get
            {
                return new Logger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name);
            }

        }
        private CloudQueue filesQueue;
        private CloudBlobContainer filesBlobContainer;
        private CloudTable _csvTemplateTable;

        public override void Run()
        {
            Trace.TraceInformation("TransferMeterProfileToDBWorker is running");

            CloudQueueMessage msg = null;

            while (true)
            {
                try
                {
                    msg = this.filesQueue.GetMessage();

                    if (msg != null)
                    {
                        filesQueue.DeleteMessage(msg);
                        ProcessQueueMessage(msg);
                    }
                    else
                    {
                        System.Threading.Thread.Sleep(1000);
                    }
                }
                catch (StorageException e)
                {
                    Trace.TraceError("StorageException in TransferMeterProfileToDBWorker: '{0}'", e.Message);
                    logger.logException(e);
                    if (msg != null && msg.DequeueCount > 5)
                    {
                        this.filesQueue.DeleteMessage(msg);
                    }

                    System.Threading.Thread.Sleep(5000);
                }
                catch (Exception e)
                {
                    Trace.TraceError("Exception in TransferMeterProfileToDBWorker: '{0}'", e.Message);
                    logger.logException(e);
                }

            }
        }


        private async void ProcessQueueMessage(CloudQueueMessage msg)
        {
            Trace.TraceInformation("Processing queue message '{0}'", msg);

            MeterProfileQueueMsg meterProfileMsg = await Task.Factory.StartNew(() => JsonConvert.DeserializeObject<MeterProfileQueueMsg>(msg.AsString));
            string blobName = meterProfileMsg.BlobName;
            if (string.IsNullOrWhiteSpace(blobName))
            {
                throw new Exception("Null blob name");
            }
            string customerID = Utility.GetCustomerIDFromBlobName(blobName);
            // Insert records in DB
            try
            {
                logger.Info(string.Format("Got a blob with name {0}", blobName));
                CloudBlob fileBlob = this.filesBlobContainer.GetBlobReference(blobName);
                using (Stream fs = fileBlob.OpenRead())
                {
                    StreamReader csv = new StreamReader(fs);
                    // Consume header
                    var line = csv.ReadLine();
                    if (line == null)
                    {
                        throw new Exception(string.Format("Failed to read content from blob {0}", blobName));
                    }
                    string[] columnList = line.Split(',');
                    var result = _csvTemplateTable.Execute(TableOperation.Retrieve(customerID, Constants.MeterTemplateName));
                    if (result == null || result.HttpStatusCode == 404 || result.Result == null || columnList == null)
                    {
                        result = _csvTemplateTable.Execute(TableOperation.Retrieve(customerID.ToUpperInvariant(), Constants.MeterTemplateName));
                        if (result == null || result.HttpStatusCode == 404 || result.Result == null || columnList == null)
                        {
                            throw new TemplateDataException(string.Format("No {0} csv template", Constants.MeterTemplateName));
                        }
                    }
                    
                    var headerList = Utility.GetCsvTableHeader(columnList);
                    var template = (DynamicTableEntity)result.Result;
                    var headerMapping = new Dictionary<string, int>();
                    var tmpTableCreation = new StringBuilder("CREATE TABLE " + Constants.MeterDataTempTable + " (");
                    var table = new DataTable();
                    var newcomingList = new List<string>();
                    var containsOpBit = false;
                    var inheritColumnList = new List<string>();
                    foreach (var prop in template.Properties)
                    {
                        try
                        {
                            tmpTableCreation.Append(prop.Key.ToString() + " NVARCHAR(255), ");
                            table.Columns.Add(new DataColumn(prop.Key.ToString(), typeof(string)));
                            headerMapping.Add(prop.Key.ToString(), Utility.GetColumnByHeader(headerList, System.Data.Entity.DbFunctions.AsUnicode(prop.Value.StringValue)));
                            newcomingList.Add(prop.Key.ToString());
                            if (prop.Key.ToString().Equals("OperationBit", StringComparison.OrdinalIgnoreCase))
                            {
                                containsOpBit = true;
                            }
                        }
                        catch (TemplateDataException ex)
                        {
                            inheritColumnList.Add(prop.Key.ToString());
                            continue;
                        }
                    }
                    tmpTableCreation.Append("[Latitude] FLOAT(53) NULL, [Longitude] FLOAT (53) NULL, [geography] [sys].[geography] NULL)");

                    while ((line = csv.ReadLine()) != null)
                    {
                        Trace.TraceInformation("line {0}", line);
                        columnList = line.Split(',');
                        var row = table.NewRow();
                        foreach (var prop in newcomingList)
                        {
                            row[prop] = columnList[headerMapping[prop]];
                        }
                        table.Rows.Add(row);
                    }
                    using (var sqlConn = SQLConnFactory.GetInstance().GetSqlConnection())
                    {
                        sqlConn.Open();
                        using (var trans = sqlConn.BeginTransaction())
                        {
                            var cmd = new SqlCommand(tmpTableCreation.ToString(), sqlConn, trans);
                            await cmd.ExecuteNonQueryAsync();
                            
                            using (var bulk = new SqlBulkCopy(sqlConn, SqlBulkCopyOptions.Default, trans))
                            {
                                bulk.DestinationTableName = Constants.MeterDataTempTable;
                                await bulk.WriteToServerAsync(table);
                            }
                            // 1. Merge backward
                            var mergeBackward = new StringBuilder("UPDATE t SET t.[Latitude] = m.[Latitude], t.[Longitude] = m.[Longitude], t.[geography] = m.[geography]");
                            foreach (var prop in inheritColumnList)
                            {
                                mergeBackward.Append(", t.[" + prop + "] = m.[" + prop + "]");
                            }
                            mergeBackward.Append(" FROM " + Constants.MeterDataTempTable + " t INNER JOIN " + Constants.MeterDataTable +
                                " m ON t.AccountNumber = m.AccountNumber AND t.UserNumber = m.UserNumber AND m.ModifiedDate IS NULL AND m.CustomerID = '" +
                                customerID + "' ");
                            logger.Info(mergeBackward.ToString());
                            cmd.CommandText = mergeBackward.ToString();
                            await cmd.ExecuteNonQueryAsync();

                            // 2. Delete duplicate
                            var deleteDup = new StringBuilder("DELETE t FROM " + Constants.MeterDataTempTable + " t INNER JOIN " +
                                Constants.MeterDataTable + " m ON t.AccountNumber = m.AccountNumber AND t.UserNumber = m.UserNumber AND m.ModifiedDate IS NULL AND m.CustomerID = '" +
                                customerID + "' WHERE ");
                            foreach (var prop in newcomingList)
                            {
                                if (prop.Equals("OperationBit", StringComparison.OrdinalIgnoreCase))
                                {
                                    continue;
                                }
                                deleteDup.Append("t." + prop + " = m." + prop + " AND ");
                            }
                            if (containsOpBit)
                            {
                                deleteDup.Append(" t.OperationBit <> 'R'");
                            }
                            else
                            {
                                deleteDup.Remove(deleteDup.Length - 4, 4);
                            }
                            logger.Info(deleteDup.ToString());
                            cmd.CommandText = deleteDup.ToString();
                            await cmd.ExecuteNonQueryAsync();

                            // 3. Inactive the old record
                            var inactiveOld = new StringBuilder("UPDATE m SET m.ModifiedDate = GETUTCDATE() FROM " +
                                Constants.MeterDataTable + " m INNER JOIN " + Constants.MeterDataTempTable +
                                " t ON t.AccountNumber = m.AccountNumber AND t.UserNumber = m.UserNumber AND m.ModifiedDate IS NULL AND m.CustomerID = '" +
                                customerID + "' ");
                            logger.Info(inactiveOld.ToString());
                            cmd.CommandText = inactiveOld.ToString();
                            await cmd.ExecuteNonQueryAsync();

                            // 4. Merge forward
                            var mergeForward = new StringBuilder("INSERT INTO " + Constants.MeterDataTable + " (");
                            foreach (var prop in newcomingList)
                            {
                                if (prop.Equals("OperationBit", StringComparison.OrdinalIgnoreCase))
                                {
                                    continue;
                                }
                                mergeForward.Append(prop + ", ");
                            }
                            foreach (var prop in inheritColumnList)
                            {
                                if (prop.Equals("OperationBit", StringComparison.OrdinalIgnoreCase))
                                {
                                    continue;
                                }
                                mergeForward.Append(prop + ", ");
                            }
                            mergeForward.Append("Latitude, Longitude, geography, CreationDate, CustomerID) SELECT ");
                            foreach (var prop in newcomingList)
                            {
                                if (prop.Equals("OperationBit", StringComparison.OrdinalIgnoreCase))
                                {
                                    continue;
                                }
                                mergeForward.Append(prop + ", ");
                            }
                            foreach (var prop in inheritColumnList)
                            {
                                if (prop.Equals("OperationBit", StringComparison.OrdinalIgnoreCase))
                                {
                                    continue;
                                }
                                mergeForward.Append(prop + ", ");
                            }
                            mergeForward.Append("Latitude, Longitude, geography, GETUTCDATE(), '" + customerID + "' FROM " +
                                Constants.MeterDataTempTable);
                            if (containsOpBit)
                            {
                                mergeForward.Append(" WHERE OperationBit <> 'R'");
                            }
                            logger.Info(mergeForward.ToString());
                            cmd.CommandText = mergeForward.ToString();
                            await cmd.ExecuteNonQueryAsync();
                            trans.Commit();
                        }
                    }
                }
            }
            catch (Exception e)
            {
                Trace.TraceError(String.Format("Exception in ProcessQueueMessage: '{0}'", e.Message));
                logger.logException(e);
            }

        }


        private void InsertRecord(RawDataContext db, string[] columnList)
        {
            CCS_MeterData_V1 meterProfileRecord = new CCS_MeterData_V1();
            meterProfileRecord.MeterType = columnList[0];

            meterProfileRecord.UserName = columnList[1];
            meterProfileRecord.AccountNumber = columnList[2];
            meterProfileRecord.UserNumber = columnList[3];
            meterProfileRecord.MeterSN = columnList[4];
            meterProfileRecord.MeterID = columnList[5];
            meterProfileRecord.MeterNumber = columnList[6];
            meterProfileRecord.Address = columnList[7];
            meterProfileRecord.ApartmentNumber = columnList[8];
            meterProfileRecord.UserType = columnList[9];
            meterProfileRecord.Office = columnList[10];
            meterProfileRecord.PhoneNumber = columnList[11];
            meterProfileRecord.Memo1 = columnList[12];
            meterProfileRecord.Memo2 = columnList[13];
            CultureInfo provider = CultureInfo.InvariantCulture;
            meterProfileRecord.CutoffDate = DateTime.ParseExact(columnList[14], "yyyy-MM-dd HH:mm:ss", provider);
            meterProfileRecord.CollectType = columnList[15];
            meterProfileRecord.TerminalNumber = columnList[16];
            meterProfileRecord.TerminalEquipmentID = columnList[17];
            meterProfileRecord.TerminalAddress = columnList[18];
            meterProfileRecord.Latitude = Double.Parse(columnList[19]);
            meterProfileRecord.Longitude = Double.Parse(columnList[20]);
            meterProfileRecord.CreationDate = DateTime.UtcNow;

            db.MeterDataTable.Add(meterProfileRecord);

        }


        public override bool OnStart()
        {
            // Set the maximum number of concurrent connections
            ServicePointManager.DefaultConnectionLimit = 12;

            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.


            // Open storage account
            CloudStorageAccount storageAccount = CloudStorageAccount.Parse(ConfigurationMgr.GetSetting("StorageConnectionString"));

            // Create blob container
            CloudBlobClient blobClient = storageAccount.CreateCloudBlobClient();
            filesBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.MeterProfileStorageName);
            if (filesBlobContainer.CreateIfNotExists())
            {
                filesBlobContainer.SetPermissions(
                    new BlobContainerPermissions
                    {
                        PublicAccess = BlobContainerPublicAccessType.Blob
                    });
            }

            // Create quere storage
            CloudQueueClient queueClient = storageAccount.CreateCloudQueueClient();
            filesQueue = queueClient.GetQueueReference(CCSPrototypeCommon.Constants.MeterProfileStorageName);
            filesQueue.CreateIfNotExists();

            CloudTableClient tableClient = storageAccount.CreateCloudTableClient();
            tableClient.DefaultRequestOptions.RetryPolicy = new LinearRetry(TimeSpan.FromSeconds(3), 3);
            _csvTemplateTable = tableClient.GetTableReference(CCSPrototypeCommon.Constants.CsvTemplateTableName);
            _csvTemplateTable.CreateIfNotExists();

            Trace.TraceInformation("Storage initialized");

            return base.OnStart();
        }

    }
}
