﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Diagnostics;
using System.Configuration;
using System.Linq;
using System.Net;
using System.Threading;
using System.Threading.Tasks;
using System.Data.Entity;
using System.Data;
using System.Web;
using CCSPrototypeCommon;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;
using Microsoft.WindowsAzure.Storage.Queue;
using Microsoft.WindowsAzure.Storage.Table;
using Microsoft.WindowsAzure.Storage.Table.DataServices;
using Microsoft.WindowsAzure.Storage.RetryPolicies;
using ThreadedWorkerRole;
using Microsoft.Azure;
using Newtonsoft.Json;
using System.Globalization;
using System.Text;
using System.Data.SqlClient;

namespace HomeWorker
{
    public class UploadPaymentBlobToDBWorker : WorkerEntryPoint
    {
        public static Logger logger
        {
            get
            {
                return new Logger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name);
            }

        }

        private CloudQueue filesQueue;
        private CloudBlobContainer filesBlobContainer;

        private CloudTable _csvTemplateTable;
        public override void Run()
        {
            Trace.TraceInformation("UploadPaymentBlobToDBWorker is running");

            CloudQueueMessage msg = null;
            while (true)
            {
                try
                {
                    
                    msg = this.filesQueue.GetMessage();

                    if (msg != null)
                    {
                        filesQueue.DeleteMessage(msg);
                        ProcessQueueMessage(msg);
                    }
                    else
                    {
                        System.Threading.Thread.Sleep(1000);

                    }
                }
                catch (StorageException e)
                {
                    Trace.TraceError("StorageException in TransferRawDataToDBWorker: '{0}'", e.Message);
                    logger.logException(e);
                    if (msg != null && msg.DequeueCount > 5)
                    {
                        this.filesQueue.DeleteMessage(msg);
                    }

                    System.Threading.Thread.Sleep(5000);
                }
                catch (Exception e)
                {
                    Trace.TraceError("Exception in TransferRawDataToDBWorker: '{0}'", e.Message);

                    logger.logException(e);

                }

            }
        }


        private async void ProcessQueueMessage(CloudQueueMessage msg)
        {
            Trace.TraceInformation("Processing queue message '{0}'", msg);

            var db = (PaymentContext)PaymentContextFactory.GetInstance().GetDBEFContext();
            var dbCustomer = (CustomerContext)CustomerContextFactory.GetInstance().GetDBEFContext();
            var dbBill = (BillContext)BillContextFactory.GetInstance().GetDBEFContext();
            UploadPaymentQueueMsg uploadPaymentMsg = await Task.Factory.StartNew(() => JsonConvert.DeserializeObject<UploadPaymentQueueMsg>(msg.AsString));
            if (uploadPaymentMsg == null)
            {
                throw new Exception("Queue message is NULL");

            }
            string blobName = uploadPaymentMsg.BlobName;
            //string blobName = msg.AsString;
            if (string.IsNullOrWhiteSpace(blobName))
            {
                throw new Exception("Null blob name");
            }




            string customerID = Utility.GetCustomerIDFromBlobName(blobName);
            var customers = dbCustomer.CustomerTable.Where(o => o.CustomerID.ToString() == customerID).ToList();
            if(customers.Count == 0 )
            {
                logger.Error(string.Format("Customer ID not found {0}", customerID));
                return;
            }

            var customer  = customers[0];
            // Insert records in DB
            try
            {
                //logInfo("start reading");
                CloudBlockBlob fileBlob = this.filesBlobContainer.GetBlockBlobReference(blobName);
                using (Stream fs = fileBlob.OpenRead())
                {
                    StreamReader csv = new StreamReader(fs);
                    // Consume header
                    var line = csv.ReadLine();
                    string[] columnList = line.Split(',');

                    var result = _csvTemplateTable.Execute(TableOperation.Retrieve(customerID, Constants.MeterTemplateName));
                    if (result == null || result.HttpStatusCode == 404 || result.Result == null || columnList == null)
                    {
                        result = _csvTemplateTable.Execute(TableOperation.Retrieve(customerID.ToUpperInvariant(), Constants.MeterTemplateName));
                        if (result == null || result.HttpStatusCode == 404 || result.Result == null || columnList == null)
                        {
                            throw new TemplateDataException(string.Format("No {0} csv template", Constants.UploadPaymentTemplateName));
                        }
                    }

                    
                    var headerList = Utility.GetCsvTableHeader(columnList);
                    var template = (DynamicTableEntity)result.Result;
                    var headerMapping = new Dictionary<string, int>();
                    var tmpTableCreation = new StringBuilder("CREATE TABLE " + Constants.UploadPaymentDataTempTable + " (");
                    var table = new DataTable();
                    var currentList = new List<string>();
                    var containsOpBit = false;
                    foreach (var prop in template.Properties)
                    {
                        //logInfo("Prop detected " + prop);
                        try
                        {
                            // To do: verify property key is valid field in CCS_MeterData_V1
                            headerMapping.Add(prop.Key.ToString(), Utility.GetColumnByHeader(headerList, prop.Key.ToString()));
                             
                                switch (prop.Value.PropertyType)
                                {
                                    case EdmType.String:
                                        //logInfo("Add string column " + prop.Key.ToString());
                                        tmpTableCreation.Append(prop.Key.ToString() + " NVARCHAR(MAX) NULL, ");
                                        table.Columns.Add(new DataColumn(prop.Key.ToString(), typeof(String)));
                                        break;
                                    case EdmType.Double:
                                        //logInfo("Add double column " + prop.Key.ToString());
                                        tmpTableCreation.Append(prop.Key.ToString() + " FLOAT (53) NOT NULL, ");
                                        table.Columns.Add(new DataColumn(prop.Key.ToString(), typeof(Double)));
                                        break;
                                    case EdmType.DateTime:
                                        //logInfo("Add datetime column " + prop.Key.ToString());
                                        tmpTableCreation.Append(prop.Key.ToString() + " DATETIME NULL,");
                                        table.Columns.Add(new DataColumn(prop.Key.ToString(), typeof(DateTime)));
                                        break;
                                }






                            //logInfo("currentlist " + String.Join(";", currentList));
                            currentList.Add(prop.Key.ToString());
                                
                            if (prop.Key.ToString().Equals("OperationBit", StringComparison.OrdinalIgnoreCase))
                            {
                                containsOpBit = true;
                            }
                        }
                        catch (TemplateDataException)
                        {
                            continue;
                        }
                    }


                    tmpTableCreation.Append("[ModifiedDate]  DATETIME   NULL, [UserID]   UNIQUEIDENTIFIER  NULL, [CustomerID] UNIQUEIDENTIFIER NULL)");

                    //logInfo("start reading actual data");
                    while ((line = csv.ReadLine()) != null)
                    {
                        //logInfo("line " + line);
                        Trace.TraceInformation("line {0}", line);
                        columnList = line.Split(',');
                        var row = table.NewRow();
                        foreach (var prop in currentList)
                        {
                                
                                if (table.Columns[prop].DataType == typeof(System.String))
                                {
                                    //logInfo("look for string prop " + prop);
                                    String propValue = columnList[headerMapping[prop]];
                                    row[prop] = propValue;
                                    if (prop == "PaymentCellNumber")
                                    {
                                        var users = dbBill.UserTable.Where(o => o.Cellphone == propValue).ToList();
                                        if(users.Count == 0)
                                        {
                                            var userRecord = new CCS_User_V1();
                                            userRecord.Cellphone = propValue;
                                            userRecord.ModifiedDate = DateTime.UtcNow;
                                            dbBill.UserTable.Add(userRecord);
                                                
                                        }
                                    }

                                }
                                if (table.Columns[prop].DataType == typeof(System.Double))
                                {
                                    //logInfo("look for double prop " + prop);
                                    row[prop] = Double.Parse(columnList[headerMapping[prop]]);
                                }
                                if (table.Columns[prop].DataType == typeof(System.DateTime))
                                {
                                    //logInfo("look for datetime prop " + prop);
                                    row[prop] = DateTime.Parse(columnList[headerMapping[prop]]);
                                }

                        }
                        //logInfo("store in the table " + row.ToString());

                        table.Rows.Add(row);
                    }
                    dbBill.SaveChanges();

                    using (var sqlConn = SQLConnFactory.GetInstance().GetSqlConnection())
                    {
                        //logInfo("CREATE TEMP TABLE " + tmpTableCreation);
                        sqlConn.Open();
                        using (var trans = sqlConn.BeginTransaction())
                        {
                            var cmd = new SqlCommand(tmpTableCreation.ToString(), sqlConn, trans);
                            await cmd.ExecuteNonQueryAsync();
                            using (var bulk = new SqlBulkCopy(sqlConn, SqlBulkCopyOptions.Default, trans))
                            {
                                bulk.DestinationTableName = Constants.UploadPaymentDataTempTable;
                                await bulk.WriteToServerAsync(table);
                            }

                            //logInfo("DONE TEMP TABLE CREATION");

                            //logInfo("//1");
                            //1
                            var getUserIDs = new StringBuilder("UPDATE t SET t.UserID = u.UserID FROM " +
                                Constants.UploadPaymentDataTempTable + " t INNER JOIN " + Constants.UserDataTable +
                                " u ON  t.PaymentCellNumber = u.Cellphone");
                            cmd.CommandText = getUserIDs.ToString();
                            await cmd.ExecuteNonQueryAsync();

                            //logInfo("//2");
                            // 2. Delete duplicate
                            var deleteDup = new StringBuilder("DELETE t FROM " + Constants.UploadPaymentDataTempTable + " t INNER JOIN " + Constants.UploadedPaymentDataTable +
                                " p ON p.UserID = t.UserID and t.PaySubmitTime = p.PaySubmitTime and p.ModifiedDate IS NULL AND p.CustomerID = '" +
                                customerID + "' WHERE ");
                            foreach (var prop in currentList)
                            {
                                if (prop.Equals("OperationBit", StringComparison.OrdinalIgnoreCase))
                                {
                                    continue;
                                }
                                if (prop.Equals("PaymentCellNumber", StringComparison.OrdinalIgnoreCase))
                                {
                                    continue;
                                }

                                deleteDup.Append("t." + prop + " = p." + prop + " AND ");
                            }
                            if (containsOpBit)
                            {
                                deleteDup.Append(" t.OperationBit <> 'R'");
                            }
                            else
                            {
                                deleteDup.Remove(deleteDup.Length - 4, 4);
                            }
                            cmd.CommandText = deleteDup.ToString();
                            await cmd.ExecuteNonQueryAsync();

                            //logInfo("//3");
                            // 3. Inactive the old record
                            var inactiveOld = new StringBuilder("UPDATE p SET p.ModifiedDate = GETUTCDATE() FROM " +
                                Constants.UploadedPaymentDataTable + " p INNER JOIN " + Constants.UploadPaymentDataTempTable +
                                " t ON t.UserID = p.UserID  AND p.CustomerID = '" +
                                customerID + "' ");
                            cmd.CommandText = inactiveOld.ToString();
                            await cmd.ExecuteNonQueryAsync();

                            //logInfo("//4");
                            // 4. Merge forward
                            var mergeForward = new StringBuilder("INSERT INTO " + Constants.UploadedPaymentDataTable + " (");
                            foreach (var prop in currentList)
                            {
                                if (prop.Equals("OperationBit", StringComparison.OrdinalIgnoreCase))
                                {
                                    continue;
                                }

                                if (prop.Equals("PaymentCellNumber", StringComparison.OrdinalIgnoreCase))
                                {
                                    continue;
                                }
                                mergeForward.Append(prop + ", ");
                            }
                            mergeForward.Append("UserID, CustomerID) SELECT ");
                            foreach (var prop in currentList)
                            {
                                if (prop.Equals("OperationBit", StringComparison.OrdinalIgnoreCase))
                                {
                                    continue;
                                }
                                if (prop.Equals("PaymentCellNumber", StringComparison.OrdinalIgnoreCase))
                                {
                                    continue;
                                }
                                mergeForward.Append(prop + ", ");
                            }
                            mergeForward.Append("UserID, '" + customerID + "' FROM " +
                                Constants.UploadPaymentDataTempTable);
                            if (containsOpBit)
                            {
                                mergeForward.Append(" WHERE OperationBit <> 'R'");
                            }
                            cmd.CommandText = mergeForward.ToString();
                            await cmd.ExecuteNonQueryAsync();

                            //logInfo("Done");
                        }
                    }

                    await db.SaveChangesAsync();
                }
            }
            catch (Exception e)
            {
                Trace.TraceError(String.Format("Exception in ProcessQueueMessage: '{0}'", e.Message));
                logger.logException(e);
            }


        }

        public string removeFirstLastDoubleQuotes(string str)
        {
            if (string.IsNullOrEmpty(str))
                return str;
            if (str.StartsWith("\""))
                str = str.Substring(1);
            if (str.EndsWith("\""))
                str = str.Substring(0, str.Length - 1);

            return str;
        }
        public override bool OnStart()
        {
            // Set the maximum number of concurrent connections
            ServicePointManager.DefaultConnectionLimit = 12;

            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.


            // Open storage account
            CloudStorageAccount storageAccount = CloudStorageAccount.Parse(ConfigurationMgr.GetSetting("StorageConnectionString"));

            // Create blob container
            CloudBlobClient blobClient = storageAccount.CreateCloudBlobClient();
            filesBlobContainer = blobClient.GetContainerReference(CCSPrototypeCommon.Constants.UploadedPaymentsStorageName);
            if (filesBlobContainer.CreateIfNotExists())
            {
                filesBlobContainer.SetPermissions(
                    new BlobContainerPermissions
                    {
                        PublicAccess = BlobContainerPublicAccessType.Blob
                    });
            }

            // Create quere storage
            CloudQueueClient queueClient = storageAccount.CreateCloudQueueClient();
            filesQueue = queueClient.GetQueueReference(CCSPrototypeCommon.Constants.UploadedPaymentsStorageName);
            filesQueue.CreateIfNotExists();

            CloudTableClient tableClient = storageAccount.CreateCloudTableClient();
            tableClient.DefaultRequestOptions.RetryPolicy = new LinearRetry(TimeSpan.FromSeconds(3), 3);
            _csvTemplateTable = tableClient.GetTableReference(CCSPrototypeCommon.Constants.CsvTemplateTableName);
            _csvTemplateTable.CreateIfNotExists();

            Trace.TraceInformation("Storage initialized");

            return base.OnStart();
        }

    }
}
