package my.ccs.projectrawdata;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.RadioButton;
import android.widget.TextView;
import android.widget.Toast;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import my.ccs.projectrawdata.model.MeterProfileModel;
import my.ccs.projectrawdata.model.RawData;
import my.ccs.projectrawdata.sqlite.MySQLiteHelper;

/**
 * Created by tafeng on 6/4/2016.
 */
public class MeterCollectDialogActivity extends Activity {

    private RadioButton meterTypeWifiRdBtn;
    private RadioButton meterTypeBusRdBtn;
    private RadioButton meterTypeElectronicRdBtn;
    private RadioButton meterTypeManualRdBtn;

    private EditText meterSNEditText;
    private EditText meterAddrEditText;
    private EditText meterBuildingNumEditText;
    private EditText meterCustomerIDText;
    private EditText meterCustomerNameText;
    private EditText meterReadingText;

    private ImageButton meterImgPreview1;
    private ImageButton meterImgPreview2;
    private ImageButton currentImageButton;
    private List<ImageButton> imageButtons = new LinkedList<>();

    private Uri fileUri;
    private Button collectCancelled;
    private Button collectConfirmed;

    Bitmap bitmapIconFrame;
    Bitmap bitmapIconUpload;

    // Camera activity request codes
    public static final int CAMERA_CAPTURE_IMAGE_REQUEST_CODE = 100;
    public static final int CAMERA_CAPTURE_VIDEO_REQUEST_CODE = 200;

    public static final int MEDIA_TYPE_IMAGE = 1;
    public static final int MEDIA_TYPE_VIDEO = 2;

    MeterProfileModel.MeterTypes queryMeterTypes = null;
    String meterDataID = null;
    String meterSN = null;
    String meterAddr = null;
    String meterBuildingNum = null;
    String meterCustomerID = null;
    String meterCustomerName = null;
    String lastMeterReading = "";


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_metercollect_dialog);

        meterSNEditText = (EditText) findViewById(R.id.meterSNText);
        meterAddrEditText = (EditText) findViewById(R.id.meterAddressText);
        meterBuildingNumEditText = (EditText) findViewById(R.id.meterBuildingNumberText);
//      meterCustomerIDText = (EditText) findViewById(R.id.meterCustomerIDText);
        meterCustomerNameText = (EditText) findViewById(R.id.meterCustomerNameText);
        meterReadingText = (EditText) findViewById(R.id.meterReadingText);

        meterImgPreview1 = (ImageButton) findViewById(R.id.meterImgPreview1);

        imageButtons.add(meterImgPreview1);

        collectCancelled = (Button) findViewById(R.id.meterQueryCancelled);
        collectConfirmed = (Button) findViewById(R.id.meterQueryConfirmed);

        // Populate editboxes
        Bundle b = getIntent().getExtras();
        meterDataID = b.getString(Constants.MeterDataIDKey);
        meterSN = b.getString(Constants.MeterSNKey);
        meterAddr = b.getString(Constants.MeterAddressKey);
        meterBuildingNum = b.getString(Constants.MeterBuildingNumKey);
        meterCustomerID = b.getString(Constants.MeterCustomerIDKey);
        meterCustomerName = b.getString(Constants.MeterCustomerNameKey);

        meterAddrEditText.setText(meterAddr);
        meterSNEditText.setText(meterSN);
        meterBuildingNumEditText.setText(meterBuildingNum);
//      meterCustomerIDText.setText(meterCustomerID);
        meterCustomerNameText.setText(meterCustomerName);

        MySQLiteHelper db = new MySQLiteHelper(MeterCollectDialogActivity.this);

        //db.addRawData(new RawData(meterDataID, "MeasureType1", "rateschedule", "readtype", 200, 1.4, "status", new Date()));

        List<RawData> oldData = db.getRawData(meterDataID, 1);

        try {
            if (oldData != null) {
                lastMeterReading = String.valueOf(oldData.get(0).Read);
            }
        }
        catch (NumberFormatException ex)
        {
            //pass
        }
        catch (Exception ex)
        {
            //pass
            Log.d(Constants.MyTag, ex.getMessage());
        }

        meterReadingText.setText(lastMeterReading);

        collectCancelled.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                onCancelClicked();
            }
        });

        collectConfirmed.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                onCollectClicked();
            }

        });


        // Image
        // Checking camera availability
        if (!isDeviceSupportCamera()) {
            Utilities.showToastMsg(MeterCollectDialogActivity.this, Constants.CameraNotSupported);
            // will close the app if the device does't have camera
            finish();
        }

        DisplayMetrics metrics = new DisplayMetrics();
        getWindow().getWindowManager().getDefaultDisplay().getMetrics(metrics);
        int displayWidth = metrics.widthPixels;

        bitmapIconFrame = BitmapFactory.decodeResource(getResources(), R.drawable.icon_frame);
        bitmapIconFrame = Bitmap.createScaledBitmap(bitmapIconFrame, displayWidth/5, displayWidth/5, true);

        meterImgPreview1.setImageBitmap(bitmapIconFrame);

        for(final ImageButton imageButton : imageButtons)
        {
            imageButton.setOnClickListener(new View.OnClickListener() {
                public void onClick(View view) {

                    // custom dialog
                    final Dialog dialog = new Dialog(MeterCollectDialogActivity.this);
                    Window window = dialog.getWindow();
                    window.requestFeature(Window.FEATURE_NO_TITLE);
                    dialog.setContentView(R.layout.choose_picture_action);
                    window.setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
                    dialog.setCancelable(true);

                    ImageButton btnCapturePicture;
                    ImageButton btnRemovePicture;

                    TextView btnCapturePictureTxt;
                    TextView btnRemovePictureTxt;

                    btnCapturePicture = (ImageButton) dialog.findViewById(R.id.btnCapturePicture);
                    btnRemovePicture = (ImageButton) dialog.findViewById(R.id.btnRemovePicture);
                    btnCapturePictureTxt = (TextView) dialog.findViewById(R.id.btnCapturePictureTxt);
                    btnRemovePictureTxt = (TextView) dialog.findViewById(R.id.btnRemovePictureTxt);
                    /**
                     * Capture image button click event
                     */
                    btnCapturePicture.setOnClickListener(new View.OnClickListener() {

                        @Override
                        public void onClick(View v) {
                            // capture picture
                            if(dialog != null)
                            {
                                dialog.dismiss();
                            }
                            currentImageButton = imageButton;
                            captureImage();

                        }
                    });
                    btnCapturePictureTxt.setOnClickListener(new View.OnClickListener() {
                        public void onClick(View view) {
                            // capture picture
                            if(dialog != null)
                            {
                                dialog.dismiss();
                            }
                            currentImageButton = imageButton;
                            captureImage();

                        }
                    });


                    /**
                     * Capture image button click event
                     */
                    btnRemovePicture.setOnClickListener(new View.OnClickListener() {

                        @Override
                        public void onClick(View v) {

                            if(dialog != null)
                            {
                                dialog.dismiss();
                            }
                            imageButton.setImageBitmap(bitmapIconFrame);

                        }
                    });
                    btnRemovePictureTxt.setOnClickListener(new View.OnClickListener() {
                        public void onClick(View view) {
                            if(dialog != null)
                            {
                                dialog.dismiss();
                            }
                            imageButton.setImageBitmap(bitmapIconFrame);

                        }
                    });

                    dialog.show();

                }
            });


        }


    }

    protected void onCancelClicked() {
        Toast.makeText(MeterCollectDialogActivity.this, Constants.OperationCancelled, Toast.LENGTH_LONG).show();
        finish();
    }

    protected void onCollectClicked() {
        Map<String, MeterProfileModel> allMeters = MeterProfilesDataHolder.getInstance(MeterCollectDialogActivity.this).getListMeters();
        MeterProfileModel meter = null;
        for (MeterProfileModel m : allMeters.values()) {
            if (m.MeterDataID.equalsIgnoreCase(meterDataID)) {
                meter = m;
                break;
            }
        }

        if (meter == null) {
            return;
        }

        String read = meterReadingText.getText().toString();

        if (read.equals("")) {
            Toast.makeText(MeterCollectDialogActivity.this, Constants.RequiredDataMissing, Toast.LENGTH_LONG).show();
            return;
        }

        Double meterReading = Double.parseDouble(read);
        MySQLiteHelper db = new MySQLiteHelper(MeterCollectDialogActivity.this);

        db.addRawData(new RawData(meterDataID, Constants.Manual, "rateschedule", "readtype", meterReading, meterReading, "status", new Date()));
        meter.LastRead = String.valueOf(meterReading);
        meter.LastCollected = new Date();
        meter.isCollected = true;
        db.updateMeterData(meter);

        Utilities.showToastMsg(MeterCollectDialogActivity.this, "数据采集成功");
        finish();
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        // If we've received a touch notification that the user has touched
        // outside the app, finish the activity.
        if (MotionEvent.ACTION_OUTSIDE == event.getAction()) {
            finish();
            return true;
        }

        // Delegate everything else to Activity.
        return super.onTouchEvent(event);
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }


    @Override
    public void onBackPressed() {
        Toast.makeText(MeterCollectDialogActivity.this, Constants.OperationCancelled, Toast.LENGTH_LONG).show();
        finish();
    }

    @Override
    public void onStart() {

        DisplayMetrics metrics = new DisplayMetrics();
        getWindow().getWindowManager().getDefaultDisplay().getMetrics(metrics);
        int displayWidth = metrics.widthPixels;
        int displayHeight = metrics.heightPixels;
        getWindow().setLayout((int) (displayWidth * 0.9), (int) (displayHeight * 0.85));
        super.onStart();

    }


    /**
     * Creating file uri to store image/video
     */
    public Uri getOutputMediaFileUri(int type) {
        return Uri.fromFile(getOutputMediaFile(type));
    }




    /**
     * Here we store the file url as it will be null after returning from camera
     * app
     */
    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);

        // save file url in bundle as it will be null on screen orientation
        // changes
        outState.putParcelable("file_uri", fileUri);
    }

    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);

        // get the file url
        fileUri = savedInstanceState.getParcelable("file_uri");
    }

    /**
     * Receiving activity result method will be called after closing the camera
     * */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        // if the result is capturing Image
        if (requestCode == CAMERA_CAPTURE_IMAGE_REQUEST_CODE) {
            if (resultCode == RESULT_OK) {

                // successfully captured the image
                // launching upload activity
                //launchUploadActivity(true);
                String filePath = fileUri.getPath();
                if (filePath != null) {
                    /*Toast.makeText(getApplicationContext(),
                            "Filepath " + filePath, Toast.LENGTH_LONG).show();*/

                    // Displaying the image or video on the screen
                    previewMedia(true);
                } else {
                    Utilities.showToastMsg(MeterCollectDialogActivity.this, "Sorry, file path is missing!");

                }

            } else if (resultCode == RESULT_CANCELED) {
                // user cancelled Image capture
                Utilities.showToastMsg(MeterCollectDialogActivity.this, "User cancelled image capture");

            } else {
                // failed to capture image
                Utilities.showToastMsg(MeterCollectDialogActivity.this, "Sorry! Failed to capture image");
            }
        }
    }



    /**
     * returning image / video
     */
    private static File getOutputMediaFile(int type) {

        // External sdcard location
        File mediaStorageDir = new File(
                Environment
                        .getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES),
                Constants.IMAGE_DIRECTORY_NAME);

        // Create the storage directory if it does not exist
        if (!mediaStorageDir.exists()) {
            if (!mediaStorageDir.mkdirs()) {
                return null;
            }
        }

        // Create a media file name
        String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss",
                Locale.getDefault()).format(new Date());
        File mediaFile;
        if (type == MEDIA_TYPE_IMAGE) {
            mediaFile = new File(mediaStorageDir.getPath() + File.separator
                    + "IMG_" + timeStamp + ".jpg");
        } else if (type == MEDIA_TYPE_VIDEO) {
            mediaFile = new File(mediaStorageDir.getPath() + File.separator
                    + "VID_" + timeStamp + ".mp4");
        } else {
            return null;
        }

        return mediaFile;
    }


    /**
     * Displaying captured image/video on the screen
     * */
    private void previewMedia(boolean isImage) {
        // Checking whether captured media is image or video
        if (isImage) {
            if(currentImageButton != null)
            {
                currentImageButton.setVisibility(View.VISIBLE);
                // bimatp factory

                DisplayMetrics metrics = new DisplayMetrics();
                getWindow().getWindowManager().getDefaultDisplay().getMetrics(metrics);
                //getDialog().getWindow().setBackgroundDrawable(new ColorDrawable(0));
                int width = metrics.widthPixels/5;
                int height = width;

                Bitmap bitmap = BitmapFactory.decodeFile(fileUri.getPath());
                bitmap=Bitmap.createScaledBitmap(bitmap, width,height, true);
                currentImageButton.setImageBitmap(bitmap);
            }
        }
    }

    /**
     * Checking device has camera hardware or not
     * */
    private boolean isDeviceSupportCamera() {
        return getApplicationContext().getPackageManager().hasSystemFeature(PackageManager.FEATURE_CAMERA);
    }


    /**
     * Launching camera app to capture image
     */
    private void captureImage() {
        Intent intent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);

        fileUri = getOutputMediaFileUri(MEDIA_TYPE_IMAGE);

        intent.putExtra(MediaStore.EXTRA_OUTPUT, fileUri);

        // start the image capture Intent
        startActivityForResult(intent, CAMERA_CAPTURE_IMAGE_REQUEST_CODE);
    }

}
