package my.ccs.projectrawdata;

import android.app.Activity;
import android.content.Context;
import android.graphics.Typeface;
import android.util.DisplayMetrics;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.AbsListView;
import android.widget.ArrayAdapter;
import android.widget.BaseExpandableListAdapter;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.ImageButton;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import my.ccs.projectrawdata.model.MeterListItem;
import my.ccs.projectrawdata.model.MeterProfileModel;
import my.ccs.projectrawdata.sqlite.MySQLiteHelper;


public class MeterProfilesDataHolder {
    private Context context = null;
    Map<String, MeterProfileModel> listMeters = new LinkedHashMap<>();

    List<MeterProfileModel> filteredMeters = new LinkedList<MeterProfileModel>();

    public Map<String, MeterProfileModel> getListMeters() {
        if (listMeters != null && listMeters.size() > 0)
            return listMeters;
        else {
            listMeters = new LinkedHashMap<>();
            MySQLiteHelper dbHelper = new MySQLiteHelper(context);
            List<MeterProfileModel>  meters = dbHelper.getAllMeterData();

            int numberOfCollected = 0;
            for (MeterProfileModel m :
                 meters) {

                if (m.LastCollected != null)
                {
                    numberOfCollected++;
                    m.isCollected = true;
                }
                else
                {
                    m.isCollected = false;
                }
                listMeters.put(m.MeterDataID, m);
            }
            Utilities.showToastMsg((Activity)context, String.format("共找到%d块水表, 其中%d块水表已采集数据", listMeters.size(), numberOfCollected));
            return  listMeters;
        }
    }

    public void setFilteredMeters(List<MeterProfileModel> source) {
        if (source != null) {
            //filteredMeters.clear();
            //filteredMeters.addAll(source);
            filteredMeters = source;
        }
    }

    public List<MeterProfileModel> getFilteredMeters() {
        return filteredMeters;
    }

    public void setListMeters(Map<String, MeterProfileModel> data) {this.listMeters = data;}
    public void nullifyListMeters() {this.listMeters = null;}
    private static final MeterProfilesDataHolder holder = new MeterProfilesDataHolder();
    public static MeterProfilesDataHolder getInstance(Context  context) {holder.context = context; return holder;}
}




class MetersListAdapter extends ArrayAdapter<MeterProfileModel> {

    public MetersListAdapter(Context context, int textViewResourceId) {
        super(context, textViewResourceId);
    }

    public MetersListAdapter(Context context, int resource, List<MeterProfileModel> items) {

        super(context, resource, items);

    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {

        View itemView = convertView;

        if (itemView == null) {
            LayoutInflater vi;
            vi = LayoutInflater.from(getContext());
            itemView = vi.inflate(R.layout.meter_listview, parent, false);
            DisplayMetrics metrics = new DisplayMetrics();
            ((WindowManager)getContext().getSystemService(Context.WINDOW_SERVICE)).getDefaultDisplay().getMetrics(metrics);
            int screenHeight = metrics.heightPixels;

            itemView.setLayoutParams(new AbsListView.LayoutParams(AbsListView.LayoutParams.MATCH_PARENT, screenHeight/15));
        }

        MeterProfileModel meter = getItem(position);

        if (meter != null) {

            // Set icon
            ImageView imageView = (ImageView) itemView.findViewById(R.id.item_icon);
            imageView.setImageResource(meter.isCollected ? R.drawable.icon_ball_green : R.drawable.icon_ball_red);

            // Set meter SN
            TextView meterSN = (TextView) itemView.findViewById(R.id.meterSN);

            if (meterSN != null && meter.MeterSN != null) {
                meterSN.setText(meter.MeterSN);
            }

            // Set meter address
            TextView meterAddr = (TextView) itemView.findViewById(R.id.meterAddress);

            if (meterAddr != null && meter.Address != null) {

                meterAddr.setText(meter.Address);
            }




        }

        return itemView;
    }

}

class MetersInventoryListAdapter extends ArrayAdapter<MeterProfileModel> {

    public MetersInventoryListAdapter(Context context, int textViewResourceId) {
        super(context, textViewResourceId);
    }

    public MetersInventoryListAdapter(Context context, int resource, List<MeterProfileModel> items) {

        super(context, resource, items);

    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {

        View itemView = convertView;

        if (itemView == null) {
            LayoutInflater vi;
            vi = LayoutInflater.from(getContext());
            itemView = vi.inflate(R.layout.meterinv_listview, parent, false);
            DisplayMetrics metrics = new DisplayMetrics();
            ((WindowManager)getContext().getSystemService(Context.WINDOW_SERVICE)).getDefaultDisplay().getMetrics(metrics);
            int screenHeight = metrics.heightPixels;

            itemView.setLayoutParams(new AbsListView.LayoutParams(AbsListView.LayoutParams.MATCH_PARENT, screenHeight/15));
        }

        MeterProfileModel meter = getItem(position);

        if (meter != null) {

            // Set icon
            ImageView imageView = (ImageView) itemView.findViewById(R.id.item_icon_inv);
            imageView.setImageResource(meter.isCollected ? R.drawable.icon_ball_green : R.drawable.icon_ball_red);

            // Set meter SN
            TextView meterSN = (TextView) itemView.findViewById(R.id.meterSN_inv);

            if (meterSN != null && meter.MeterSN != null ) {
                meterSN.setText(meter.MeterSN);
            }

            // Set meter address
            TextView meterAddr = (TextView) itemView.findViewById(R.id.meterAddress_inv);

            if (meterAddr != null && meter.Address != null) {

                meterAddr.setText(meter.Address);
            }

            // Set meter reading
            TextView meterReading = (TextView) itemView.findViewById(R.id.meterReading_inv);

            if (meterReading != null && meter.LastRead != null) {
                meterReading.setText(meter.LastRead);
            }

            // Set meter last collected
            TextView meterLastCollected = (TextView) itemView.findViewById(R.id.meterLastCollected_inv);

            if (meterLastCollected != null && meter.LastCollected != null) {
                meterLastCollected.setText(Utilities.dateFormatDateAndTime.format(meter.LastCollected));
            }

        }

        return itemView;
    }

}



class ExpandableMetersListAdapter extends BaseExpandableListAdapter {

    private Context _context;

    // Our getChildView & getGroupView use the viewholder patter
    // Here are the viewholders defined, the inner classes are
    // at the bottom
    private ChildViewHolder childViewHolder;
    private GroupViewHolder groupViewHolder;

    // child data in format of header title, child title
    private HashMap<String, List<MeterProfileModel>> _listDataChild;
    private List<MeterListItem> _listDataHeader; // header titles

    public ExpandableMetersListAdapter(Context context,
                                       List<MeterListItem> listGroupData,
                                       HashMap<String, List<MeterProfileModel>> listChildData) {

        if(listChildData == null)
            listChildData = new LinkedHashMap<>();
        this._context = context;
        this._listDataHeader = listGroupData;
        this._listDataChild = listChildData;

    }

    @Override
    public Object getChild(int groupPosition, int childPosititon) {
        return this._listDataChild.get(this._listDataHeader.get(groupPosition).getLocationCode())
                .get(childPosititon);
    }

    @Override
    public long getChildId(int groupPosition, int childPosition) {
        return childPosition;
    }

    @Override
    public View getChildView(int groupPosition, final int childPosition,
                             boolean isLastChild, View convertView, ViewGroup parent) {

        final int mGroupPosition = groupPosition;
        final int mChildPosition = childPosition;
        final MeterProfileModel meter = (MeterProfileModel) getChild(groupPosition, childPosition);

        if (convertView == null) {
            LayoutInflater infalInflater = (LayoutInflater) this._context
                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            convertView = infalInflater.inflate(R.layout.meters_grouped_listitem, null);

            childViewHolder = new ChildViewHolder();

            childViewHolder.meterAddress = (TextView) convertView
                    .findViewById(R.id.meterAddressTxtView);

            childViewHolder.meterSN = (TextView) convertView
                    .findViewById(R.id.meterSNTxtView);

            childViewHolder.checkBox = (CheckBox) convertView
                    .findViewById(R.id.meterCheckboxListItem);
            childViewHolder.itemIcon = (ImageView) convertView.findViewById(R.id.item_icon);

            convertView.setTag(R.layout.meters_grouped_listitem, childViewHolder);

        }
        else
        {
            childViewHolder = (ChildViewHolder) convertView
                    .getTag(R.layout.meters_grouped_listitem);
        }
        childViewHolder.itemIcon.setImageResource(meter.isCollected ? R.drawable.icon_ball_green : R.drawable.icon_ball_red);
        childViewHolder.meterSN.setText(meter.MeterSN);
        childViewHolder.meterAddress.setText(meter.Address);
        childViewHolder.checkBox.setOnCheckedChangeListener(null);
        childViewHolder.checkBox.setChecked(meter.isChecked);
        childViewHolder.checkBox.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {

            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {


                MeterListItem group = (MeterListItem) getGroup(mGroupPosition);
                MeterProfileModel meter = (MeterProfileModel) getChild(mGroupPosition, mChildPosition);
                meter.isChecked = isChecked;
                if (isChecked) {

                    //to be implemented
                    boolean isGroupChecked = true;
                    List<MeterProfileModel> itemsForGroup = _listDataChild.get(group.getLocationCode());
                    for (MeterProfileModel m : itemsForGroup)
                    {
                        if(!m.isChecked)
                        {
                            isGroupChecked = false;
                            break;
                        }
                    }
                    group.setIsChecked(itemsForGroup.size() != 0 && isGroupChecked);


                } else {
                    group.setIsChecked(false);
                }

                notifyDataSetChanged();
            }
        });



        return convertView;
    }

    @Override
    public int getChildrenCount(int groupPosition) {
        return this._listDataChild.get(this._listDataHeader.get(groupPosition).getLocationCode())
                .size();
    }

    @Override
    public Object getGroup(int groupPosition) {
        return this._listDataHeader.get(groupPosition);
    }

    @Override
    public int getGroupCount() {
        return this._listDataHeader.size();
    }

    @Override
    public long getGroupId(int groupPosition) {
        return groupPosition;
    }

    @Override
    public View getGroupView(int groupPosition, boolean isExpanded,
                             View convertView, ViewGroup parent) {
        final int mGroupPosition = groupPosition;
        MeterListItem  group = (MeterListItem) getGroup(groupPosition);
        if (convertView == null) {
            LayoutInflater infalInflater = (LayoutInflater) this._context
                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            convertView = infalInflater.inflate(R.layout.meters_grouped_listview, null);

            // Initialize the GroupViewHolder defined at the bottom of this document
            groupViewHolder = new GroupViewHolder();

            groupViewHolder.locCodeTxtView = (TextView) convertView.findViewById(R.id.locCodeTxtView);
            groupViewHolder.checkBox = (CheckBox)  convertView.findViewById(R.id.meterCheckbox);

            convertView.setTag(groupViewHolder);
        }
        else
        {
            groupViewHolder = (GroupViewHolder) convertView.getTag();
        }

        groupViewHolder.locCodeTxtView.setTypeface(null, Typeface.BOLD);
        groupViewHolder.locCodeTxtView.setText(group.getLocationCode());
        //set listener to null before checking the state so that clistener is not triggered
        groupViewHolder.checkBox.setOnCheckedChangeListener(null);
        groupViewHolder.checkBox.setChecked(group.getIsChecked());
        groupViewHolder.checkBox.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {

            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {


                MeterListItem  group = (MeterListItem) getGroup(mGroupPosition);
                List<MeterProfileModel> itemsForGroup = _listDataChild.get(group.getLocationCode());
                group.setIsChecked(isChecked);
                if (isChecked) {

                    for (MeterProfileModel meterProfileModel : itemsForGroup)
                    {
                        meterProfileModel.isChecked = true;
                    }

                } else {
                    for (MeterProfileModel meterProfileModel : itemsForGroup)
                    {
                        meterProfileModel.isChecked = false;
                        notifyDataSetChanged();
                    }
                }

                notifyDataSetChanged();
            }
        });

        return convertView;
    }

    @Override
    public boolean hasStableIds() {
        return false;
    }

    @Override
    public boolean isChildSelectable(int groupPosition, int childPosition) {
        return true;
    }


    public final class GroupViewHolder {

        TextView locCodeTxtView;
        CheckBox checkBox;
    }

    public final class ChildViewHolder {

        TextView meterSN;
        TextView meterAddress;
        ImageView itemIcon;
        CheckBox checkBox;
    }
}

