package my.ccs.projectrawdata;

import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.view.MotionEvent;
import android.view.View;
import android.view.Window;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.Toast;

import com.baidu.location.LocationClient;
import com.baidu.location.LocationClientOption;
import com.baidu.mapapi.SDKInitializer;
import com.baidu.mapapi.map.MapStatusUpdate;
import com.baidu.mapapi.map.MapStatusUpdateFactory;
import com.baidu.mapapi.map.MapView;
import com.baidu.mapapi.map.MyLocationConfiguration;

import java.util.Map;

import my.ccs.projectrawdata.model.MeterProfileModel;
import my.ccs.projectrawdata.model.MeterQueryModel;

/**
 * Created by tafeng on 5/28/2016.
 */
public class MeterSearchDialogActivity extends Activity {

    private RadioButton meterTypeWifiRdBtn;
    private RadioButton meterTypeBusRdBtn;
    private RadioButton meterTypeElectronicRdBtn;
    private RadioButton meterTypeManualRdBtn;

    private EditText meterSNEditText;
    private EditText meterAddrEditText;
    private EditText meterBuildingNumEditText;

    private CheckBox meterPositionedChkBox;

    private Button queryCancelled;
    private Button queryConfirmed;

    MeterProfileModel.MeterTypes queryMeterTypes = null;
    String queryMeterSN = null;
    String queryMeterAddr = null;
    String queryMeterBuildingNum = null;

    public static final String SharedPreferenceName = "CachedQueryConditions";
    public static final String MeterTypeKeyName = "MeterTypeKey";
    public static final String MeterAddressKeyName = "MeterAddressKey";
    public static final String MeterSNKeyName = "MeterSNKey";
    public static final String MeterBuildingNumKeyName = "MeterBuildingNumKey";
    public static final String MeterPositionedKeyName = "MeterPositionedKey";

    public SharedPreferences sharedPreferences = null;
    public String cachedMeterType;
    public String cachedMeterSN;
    public String cachedMeterAddress;
    public String cachedMeterBuildingNum;
    public Boolean cachedMeterPositioned;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_metersearch_dialog);

        meterTypeWifiRdBtn = (RadioButton) findViewById(R.id.meterTypeWiFiRdBtn);
        meterTypeBusRdBtn = (RadioButton) findViewById(R.id.meterTypeBusRdBtn);
        meterTypeElectronicRdBtn = (RadioButton) findViewById(R.id.meterTypeElectronicRdBtn);
        meterTypeManualRdBtn = (RadioButton) findViewById(R.id.meterTypeManualRdBtn);

        meterSNEditText = (EditText) findViewById(R.id.meterSNText);
        meterAddrEditText = (EditText) findViewById(R.id.meterAddressText);
        meterBuildingNumEditText = (EditText) findViewById(R.id.meterBuildingNumberText);

        meterPositionedChkBox = (CheckBox) findViewById(R.id.meterPositionedChkBox);

        queryCancelled = (Button) findViewById(R.id.meterQueryCancelled);
        queryConfirmed = (Button) findViewById(R.id.meterQueryConfirmed);

        // populate views with saved conditions
        sharedPreferences = getSharedPreferences(SharedPreferenceName, Context.MODE_PRIVATE);
        cachedMeterType = sharedPreferences.getString(MeterTypeKeyName, Constants.WiFi);
        cachedMeterSN = sharedPreferences.getString(MeterSNKeyName, Constants.EMPTYSTRING);
        cachedMeterAddress = sharedPreferences.getString(MeterAddressKeyName, Constants.EMPTYSTRING);
        cachedMeterBuildingNum = sharedPreferences.getString(MeterBuildingNumKeyName, Constants.EMPTYSTRING);
        cachedMeterPositioned = sharedPreferences.getBoolean(MeterPositionedKeyName, false);

        switch (cachedMeterType) {
            case Constants.WiFi:
                meterTypeWifiRdBtn.setChecked(true);
                break;
            case Constants.Bus:
                meterTypeBusRdBtn.setChecked(true);
                break;
            case Constants.Electronic:
                meterTypeElectronicRdBtn.setChecked(true);
                break;
            case Constants.Manual:
                meterTypeManualRdBtn.setChecked(true);
                break;
        }

        meterAddrEditText.setText(cachedMeterAddress);
        meterSNEditText.setText(cachedMeterSN);
        meterBuildingNumEditText.setText(cachedMeterBuildingNum);

        meterPositionedChkBox.setChecked(cachedMeterPositioned);

        queryCancelled.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                onCancelClicked();
            }
        });

        queryConfirmed.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                onQueryClicked();
            }

        });

    }

    protected void onCancelClicked() {
        //saveQueryConditions();

        Toast.makeText(MeterSearchDialogActivity.this, "查询操作已被取消", Toast.LENGTH_LONG).show();

        finish();
    }

    protected void onQueryClicked() {
        saveQueryConditions();

        if (meterTypeWifiRdBtn.isChecked()) {
            queryMeterTypes = MeterProfileModel.MeterTypes.WiFi;
        }
        else if (meterTypeBusRdBtn.isChecked()) {
            queryMeterTypes = MeterProfileModel.MeterTypes.Bus;
        }
        else if (meterTypeElectronicRdBtn.isChecked()) {
            queryMeterTypes = MeterProfileModel.MeterTypes.Electronic;
        }
        else {
            queryMeterTypes = MeterProfileModel.MeterTypes.Manual;
        }

        queryMeterSN = meterSNEditText.getText().toString();
        queryMeterAddr = meterAddrEditText.getText().toString();
        queryMeterBuildingNum = meterBuildingNumEditText.getText().toString();

        MeterQueryModel query = new MeterQueryModel(queryMeterTypes, queryMeterSN, queryMeterAddr, queryMeterBuildingNum, meterPositionedChkBox.isChecked());
        Map<String, MeterProfileModel> allMeters = MeterProfilesDataHolder.getInstance(MeterSearchDialogActivity.this).getListMeters();
        MeterProfilesDataHolder.getInstance(MeterSearchDialogActivity.this).setFilteredMeters(query.queryMeters(allMeters));

        finish();
    }

    private void saveQueryConditions() {
        String selectedMeterType;

        SharedPreferences.Editor editor = sharedPreferences.edit();

        if (meterTypeWifiRdBtn.isChecked()) {
            selectedMeterType = Constants.WiFi;
        }
        else if (meterTypeBusRdBtn.isChecked()) {
            selectedMeterType = Constants.Bus;
        }
        else if (meterTypeElectronicRdBtn.isChecked()) {
            selectedMeterType = Constants.Electronic;
        }
        else {
            selectedMeterType = Constants.Manual;
        }

        editor.putString(MeterTypeKeyName, selectedMeterType);
        editor.putString(MeterSNKeyName, meterSNEditText.getText().toString());
        editor.putString(MeterAddressKeyName, meterAddrEditText.getText().toString());
        editor.putString(MeterBuildingNumKeyName, meterBuildingNumEditText.getText().toString());
        editor.putBoolean(MeterPositionedKeyName, meterPositionedChkBox.isChecked());
        editor.commit();
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        // If we've received a touch notification that the user has touched
        // outside the app, finish the activity.
        if (MotionEvent.ACTION_OUTSIDE == event.getAction()) {
            finish();
            return true;
        }

        // Delegate everything else to Activity.
        return super.onTouchEvent(event);
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }


    @Override
    public void onBackPressed() {
        Toast.makeText(MeterSearchDialogActivity.this, "查询操作已被取消", Toast.LENGTH_LONG).show();
        finish();
    }

    @Override
    public void onStart() {

        DisplayMetrics metrics = new DisplayMetrics();
        getWindow().getWindowManager().getDefaultDisplay().getMetrics(metrics);
        int displayWidth = metrics.widthPixels;
        int displayHeight = metrics.heightPixels;
        getWindow().setLayout((int)(displayWidth * 0.9), (int) (displayHeight * 0.75));
        super.onStart();

    }

}