package my.ccs.projectrawdata;

import android.content.Context;
import android.content.SharedPreferences;
import android.util.Log;

import java.util.Date;

/**
 * Created by Hayk on 12/24/2015.
 */
public class MySharedPref {

    class PrefConstants
    {

        public static final String MY_PREFS_NAME = "mysharedpref";
        public static final String ACCESSCODE_EXP_DATE_TAG = "accesscodeexpdate";
        public static final String SELECTEDCUSTOMERID_TAG = "selectedcustomerid";
        public static final String CUSTOMERIDS_TAG = "customerids";
    }

    private static MySharedPref sInstance;
    public final SharedPreferences prefs;

    private MySharedPref(Context context)
    {
        prefs = context.getSharedPreferences(PrefConstants.MY_PREFS_NAME, context.MODE_PRIVATE);
    }

    public static synchronized void initializeInstance(Context context) {
        if (sInstance == null) {
            sInstance = new MySharedPref(context);
        }
    }

    public static synchronized MySharedPref getInstance() {
        if (sInstance == null) {
            throw new IllegalStateException(MySharedPref.class.getSimpleName() +
                    " is not initialized, call initializeInstance(..) method first.");
        }
        return sInstance;
    }


    public void setAccessCodeExpDate()
    {


        Date now = new Date();
        Date validUntil = new Date(now.getTime() + Constants.ACCESSCODE_EXPIRATION_PERIOD);
        Log.i(Constants.MyTag, "Today is " + now);
        Log.i(Constants.MyTag, "Access code valid until " + validUntil.getTime());

        SharedPreferences.Editor editor = prefs.edit();
        editor.putLong(PrefConstants.ACCESSCODE_EXP_DATE_TAG, validUntil.getTime());
        editor.commit();


    }

    public void setAccessCodeExpDateToExpired()
    {
        SharedPreferences.Editor editor = prefs.edit();
        editor.putLong(PrefConstants.ACCESSCODE_EXP_DATE_TAG, -1);
        editor.commit();
    }

    public Date getAccessCodeExpDate()
    {
        long l = prefs.getLong(PrefConstants.ACCESSCODE_EXP_DATE_TAG, -1);
        if(l != -1)
        {
            return  new Date(l);
        }

        //if null or -1 the expiration date is considered expired
        return  null;
    }

    public void setCurrentSelectedCustomerID(String customerId)
    {

        SharedPreferences.Editor editor = prefs.edit();
        editor.putString(PrefConstants.SELECTEDCUSTOMERID_TAG, customerId);
        editor.commit();


    }

    public String getCurrentSelectedCustomerID()
    {
        return prefs.getString(PrefConstants.SELECTEDCUSTOMERID_TAG, null);

    }

    public void setCurrentCustomerIDs(String customerIdJson)
    {

        SharedPreferences.Editor editor = prefs.edit();
        editor.putString(PrefConstants.CUSTOMERIDS_TAG, customerIdJson);
        editor.commit();


    }

    public String getCurrentCustomerIDs()
    {
        return prefs.getString(PrefConstants.CUSTOMERIDS_TAG, null);

    }


    public boolean clear() {
        return prefs.edit()
                .clear()
                .commit();
    }

}
