package my.ccs.projectrawdata;

import android.app.Activity;
import android.app.DialogFragment;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.ApplicationInfo;
import android.graphics.LinearGradient;
import android.graphics.Point;
import android.os.Build;
import android.provider.Settings;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Display;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.Toast;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.os.Bundle;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.lang.Exception;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.TimeZone;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import javax.net.ssl.HttpsURLConnection;

import my.ccs.projectrawdata.model.CustomerModel;
import my.ccs.projectrawdata.model.MeterProfileModel;

/**
 * Created by Hayk on 12/12/2015.
 */
public class Utilities {


    public static class JsonHelper
    {
        public static String getStringValueOrNull(JSONObject jsonItem, String name) throws JSONException
        {
            if(jsonItem.isNull(name))
            {
                return null;
            }

            return  jsonItem.getString(name);
        }
    }
    static  public  String versionName = null;

    static SimpleDateFormat dateFormatDateAndTime = new SimpleDateFormat("MM/dd/yyyy HH:mm:ss");

    public static void showToastMsg(Activity activity, final String message) {
        final Context context = activity;
        activity.runOnUiThread(new Runnable() {
            public void run() {
                Toast.makeText(context, message, Toast.LENGTH_LONG).show();
            }
        });

    }

    public static void showAlertDialog(String title, String message, Activity activity, Window window, boolean disableUserInteraction)
    {
        final Activity activityFnl = activity;
        final String messageFnl = message;
        final boolean disableUserInteractionFnl = disableUserInteraction;
        final Window windowFnl = window;
        activity.runOnUiThread(new Runnable() {
            public void run() {
                if (disableUserInteractionFnl && windowFnl != null) {
                    windowFnl.setFlags(WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE, WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE);

                }

                AlertDialog.Builder builder = new AlertDialog.Builder(activityFnl);
                builder.setMessage(messageFnl)
                        .setPositiveButton(android.R.string.ok, null);
                AlertDialog dialog = builder.create();
                dialog.getWindow().requestFeature(Window.FEATURE_NO_TITLE);
                dialog.show();

            }
        });


    }



    public static int getHeight(Activity window) {
        DisplayMetrics metrics = new DisplayMetrics();
        window.getWindowManager().getDefaultDisplay().getMetrics(metrics);
        int height = metrics.heightPixels;
        return  height;
    }

    public static int getWidth(Activity window) {
        DisplayMetrics metrics = new DisplayMetrics();
        window.getWindowManager().getDefaultDisplay().getMetrics(metrics);
        int width = metrics.widthPixels;
        return  width;
    }


    static public boolean isAccessCodeExpired()
    {
        Calendar c = Calendar.getInstance();
        SimpleDateFormat sdf = new SimpleDateFormat("dd:MMMM:yyyy HH:mm:ss a");
        String strDate = sdf.format(new Date());

        Date validUntil = MySharedPref.getInstance().getAccessCodeExpDate();
        if(validUntil == null)
        {
            return  true;
        }
        if(new Date().before(validUntil))
        {
            return  false;
        }
        return true;
    }

    static  public MeterProfileModel parseJsonToMeterProfileModel(JSONObject jsonItem) throws JSONException
    {
        String meterDataId = JsonHelper.getStringValueOrNull(jsonItem, Constants.Json_Meter_Tags.JSONRespMeterDataIDTag);
        String meterId = JsonHelper.getStringValueOrNull(jsonItem, Constants.Json_Meter_Tags.JSONRespMeterIDTag);
        String phoneUUId = JsonHelper.getStringValueOrNull(jsonItem, Constants.Json_Meter_Tags.JSONRespPhoneUUIDTag);
        String meterSN = JsonHelper.getStringValueOrNull(jsonItem, Constants.Json_Meter_Tags.JSONRespMeterSNTag);

        Double latitude = null;
        Double longitude = null;
        try
        {
            latitude =  jsonItem.getDouble(Constants.Json_Meter_Tags.JSONRespMeterLatitudeTag);
        }
        catch (JSONException e)
        {
            //Log.e(Constants.MyTag, e.getMessage());
        }

        try
        {
            longitude = jsonItem.getDouble(Constants.Json_Meter_Tags.JSONRespMeterLongitudeTag);
        }
        catch (JSONException e)
        {
            //Log.e(Constants.MyTag, e.getMessage());
        }


        String username = JsonHelper.getStringValueOrNull(jsonItem, Constants.Json_Meter_Tags.JSONRespUserNameTag);
        String meterType = JsonHelper.getStringValueOrNull(jsonItem, Constants.Json_Meter_Tags.JSONRespMeterTypeTag);
        String meterAddress = JsonHelper.getStringValueOrNull(jsonItem, Constants.Json_Meter_Tags.JSONRespMeterAddressTag);
        String customerId = JsonHelper.getStringValueOrNull(jsonItem, Constants.Json_Meter_Tags.JSONRespCustomerIdTag);
        String locationCode = JsonHelper.getStringValueOrNull(jsonItem, Constants.Json_Meter_Tags.JSONRespLocationCodeTag);
        if(locationCode == null || locationCode.equals("null"))
        {
            Log.i(Constants.MyTag, "null");
        }

        return new MeterProfileModel(meterDataId, meterId, latitude, longitude, phoneUUId, meterType, username, meterAddress, meterSN, customerId, locationCode);
    }

    static  public CustomerModel parseJsonToCustomerModel(JSONObject jsonItem) throws JSONException
    {
        String customerId = jsonItem.getString(Constants.Json_Customer_Tags.JSONRespCustomerIDTag);
        String customerName = jsonItem.getString(Constants.Json_Customer_Tags.JSONRespCustomerNameTag);


        return new CustomerModel(customerId, customerName);
    }

    static public Map<String, MeterProfileModel> filterMeters(Map<String, MeterProfileModel> listMeters, String meterIdPattern) {


        Map<String, MeterProfileModel> filteredListMeters = new LinkedHashMap<>();
        if(listMeters != null) {
            for(MeterProfileModel m : listMeters.values())
            {
                String locCode = m.LocationCode;
                if(locCode == null || locCode.isEmpty())
                {
                    locCode = "Unknown location code.";
                }
                if (locCode != null && locCode.toLowerCase().contains(meterIdPattern.toLowerCase())) {
                    filteredListMeters.put(m.MeterSN, m);
                }
            }

        }
        return filteredListMeters;
    }

    public static final HashMap<String, String> Provinces = new HashMap<String, String>() {{
        put("北京市", "11");
        put("天津市", "12");
        put("上海市", "13");
        put("重庆市", "14");
        put("湖北省", "42");
        put("广东省", "44");
        put("湖北省", "42");
        put("贵州省", "52");

    }};

    public static final HashMap<String, String> Cities = new HashMap<String, String>() {{
        put("北京市", "00");
        put("天津市", "00");
        put("上海市", "00");
        put("重庆市", "00");
        put("武汉市", "01");
        put("佛山市", "06");
        put("遵义市", "06");
    }};

    public static final HashMap<String, String> Districts = new HashMap<String, String>() {{
        put("东城区", "01");
        put("西城区", "02");
        put("朝阳区", "05");
        put("丰台区", "06");
        put("石景山区", "07");
        put("海淀区", "08");
        put("门头沟区", "09");
        put("房山区", "11");
        put("通州区", "12");
        put("顺义区", "13");
        put("昌平区", "14");
        put("大兴区", "15");
        put("怀柔区", "16");
        put("平谷区", "17");
        put("密云县", "28");
        put("延庆县", "29");
        put("市辖区", "01");
        put("江岸区", "02");
        put("江汉区", "03");
        put("硚口区", "04");
        put("汉阳区", "05");
        put("武昌区", "06");
        put("青山区", "07");
        put("洪山区", "11");
        put("东西湖区", "12");
        put("汉南区", "13");
        put("蔡甸区", "14");
        put("江夏区", "15");
        put("黄陂区", "16");
        put("新洲区", "17");
        put("禅城区", "04");
        put("南海区", "05");
        put("禅城区", "04");
        put("顺德区", "06");
        put("三水区", "07");
        put("高明区", "08");

        put("红花岗区", "02");
        put("汇川区", "03");
        put("遵义县", "21");
        put("桐梓县", "22");
        put("绥阳县", "23");
        put("正安县", "24");
        put("道真仡佬族苗族自治县", "25");
        put("务川仡佬族苗族自治县", "26");
        put("凤冈县", "27");
        put("湄潭县", "28");
        put("余庆县", "29");
        put("习水县", "30");
        put("赤水市", "81");
        put("仁怀市", "82");


    }};


    public static final Double[][] Locations = {
            {39.912563, 116.223257},
            {39.913818, 116.363625},
            {39.916105, 116.402631},
            {39.8120212, 116.468746},
            {39.989556, 116.527387},
            {39.875362, 116.381934},
            {40.015643, 116.302595},
            {40.060278, 116.635471},
            {39.753432, 116.64237},
            {40.451767, 115.972306}
    };
	
	
	
}


