package my.ccs.projectrawdata.model.tasks;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.AsyncTask;
import android.util.Log;
import android.widget.EditText;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import javax.net.ssl.HttpsURLConnection;

import my.ccs.projectrawdata.CShowProgress;
import my.ccs.projectrawdata.Constants;
import my.ccs.projectrawdata.MeterProfilesDataHolder;
import my.ccs.projectrawdata.MySharedPref;
import my.ccs.projectrawdata.R;
import my.ccs.projectrawdata.Utilities;
import my.ccs.projectrawdata.model.MeterProfileModel;
import my.ccs.projectrawdata.sqlite.MySQLiteHelper;

/**
 * Created by Hayk on 6/3/2016.
 */
public class UploadMeterInfoTask extends BaseTask {

    private List<MeterProfileModel> meters = null;
    private static double curLatitude = 0.0;
    private static double curLongitude = 0.0;

    private Activity activity = null;
    public UploadMeterInfoTask(Activity activity, List<MeterProfileModel> meters, double curLatitude, double curLongitude)
    {
        this.meters = meters;
        this.activity = activity;
        this.curLatitude = curLatitude;
        this.curLongitude = curLongitude;
    }
    public void run()
    {
        new SendPostUploadTask().execute();

    }

    private String uploadInfoForMeters( ) {

        OutputStream output = null;
        HttpURLConnection conn = null;
        try {
            if(meters == null || meters.size() == 0)
            {
                taskStatusCode = TaskStatusCode.SucceededNoJSON;
                taskStatusMsg = "There are no meters to upload";
                return  taskResponse;
            }
            URL url = new URL(Constants.ServerURLs.uploadMeterDataUrl);
            conn = (HttpURLConnection) url.openConnection();
            conn.setRequestMethod("POST");
            conn.setRequestProperty("User-Agent", "USER_AGENT");
            conn.setRequestProperty("Content-Type", "application/json");

            EditText provET = (EditText) activity.findViewById(R.id.provinceID);
            EditText cityET = (EditText) activity.findViewById(R.id.cityID);
            EditText districtET = (EditText) activity.findViewById(R.id.districtID);
            EditText latET = (EditText) activity.findViewById(R.id.locLatitude);
            EditText lgET = (EditText) activity.findViewById(R.id.locLongitude);
            EditText seqET = (EditText) activity.findViewById(R.id.sequenceID);

            String lat = latET.getText().toString();
            String lg = lgET.getText().toString();
            String locationCode = provET.getText().toString() +
                    cityET.getText().toString() +
                    districtET.getText().toString() +
                    String.format("%05d", (int) Math.ceil(Double.parseDouble(lat) * 100)) +
                    String.format("%05d", (int) Math.ceil(Double.parseDouble(lg) * 100)) +
                    seqET.getText().toString();

            ArrayList<String> images = new ArrayList<>();

            JSONObject meterData = new JSONObject();
            JSONArray meterList = new JSONArray();

            for (int i = 0; i< meters.size(); i++ ) {
                meterList.put(meters.get(i).MeterDataID);
            }

            MySQLiteHelper dbhelper = new MySQLiteHelper(activity);
            dbhelper.updateMeterLocations(meters, curLatitude, curLongitude, locationCode);

            meterData.put(Constants.Json_Meter_Tags.JSONRespMeterIDListTag, meterList);
            meterData.put(Constants.Json_Meter_Tags.JSONRespMeterLocationCodeTag, locationCode);
            meterData.put(Constants.Json_Meter_Tags.JSONRespMeterLatitudeTag, Double.toString(curLatitude));
            meterData.put(Constants.Json_Meter_Tags.JSONRespMeterLongitudeTag, Double.toString(curLongitude));
            meterData.put(Constants.JSONRespImagesTag, images);

            output = conn.getOutputStream();
            BufferedWriter writer = new BufferedWriter(
                    new OutputStreamWriter(output, "UTF-8"));
            //writer.write(getQuery(params));
            writer.write(meterData.toString());
            writer.flush();
            writer.close();
            output.close();

            int responseCode=conn.getResponseCode();

            //remove this code later

            if (responseCode == HttpsURLConnection.HTTP_OK) {

                taskResponse = "";
                String line;
                BufferedReader br=new BufferedReader(new InputStreamReader(conn.getInputStream()));
                while ((line=br.readLine()) != null) {
                    taskResponse+=line;
                }
                taskStatusCode = TaskStatusCode.SucceededJSON;
            }
            else {
                taskStatusMsg="Failed with response code: " + responseCode;
                taskStatusCode = TaskStatusCode.Failed;

            }

            if(taskStatusCode == TaskStatusCode.SucceededJSON) {
                JSONObject json = null;
                json = new JSONObject(taskResponse);
                final String statusCode = json.getString(Constants.JSONRespStatusCodeTag);
                final String statusMsg = json.getString(Constants.JSONRespStatusMsgTag);
                taskStatusMsg  = statusMsg;
            }

        }
        catch (Exception e)
        {
            taskStatusCode = TaskStatusCode.Failed;
            taskStatusMsg = e.getMessage();
        }
        finally {

            if (conn != null)
                conn.disconnect();
        }
        return taskResponse;
    }


    private class SendPostUploadTask extends AsyncTask<Void, Void, String> {

        //ProgressDialog pleaseWaitDialog;

        @Override
        protected void onPreExecute() {
            /*pleaseWaitDialog = ProgressDialog.show(UploadMeterInfoActivity.this, Constants.DIALOGTITLE, "Uploading meter info...", true, true);
            pleaseWaitDialog.setOnCancelListener(new DialogInterface.OnCancelListener() {

                public void onCancel(DialogInterface dialog) {
                    Log.e(Constants.MyTag, "onCancel");
                    SendPostUploadTask.this.cancel(true);
                }
            });*/

            CShowProgress.getInstance().showProgress(activity);

        }


        @Override
        protected void onPostExecute(final String result) {

            Log.i(Constants.MyTag, "onPostExecute");
            //pleaseWaitDialog.dismiss();
            CShowProgress.getInstance().hideProgress();
            Utilities.showToastMsg(activity, taskStatusMsg);

        }
        @Override
        protected void onCancelled() {
            super.onCancelled();
            //Utilities.hideProgressBar(progressBarDownloadMeters, getWindow(), true);
            Log.e(Constants.MyTag, "onCanceled");
            //pleaseWaitDialog.dismiss();
            CShowProgress.getInstance().hideProgress();

        }



        /*int dataCount = 0;
        String response = "error";

        Bitmap ShrinkBitmap(String file, int width, int height)
        {

            BitmapFactory.Options bmpFactoryOptions = new BitmapFactory.Options();
            bmpFactoryOptions.inJustDecodeBounds = true;
            Bitmap bitmap = BitmapFactory.decodeFile(file, bmpFactoryOptions);

            int heightRatio = (int)Math.ceil(bmpFactoryOptions.outHeight/(float)height);
            int widthRatio = (int)Math.ceil(bmpFactoryOptions.outWidth/(float)width);

            if (heightRatio > 1 || widthRatio > 1)
            {
                if (heightRatio > widthRatio)
                {
                    bmpFactoryOptions.inSampleSize = heightRatio;
                } else {
                    bmpFactoryOptions.inSampleSize = widthRatio;
                }
            }

            bmpFactoryOptions.inJustDecodeBounds = false;
            bitmap = BitmapFactory.decodeFile(file, bmpFactoryOptions);
            return bitmap;
        }*/

        @Override
        protected String  doInBackground(Void... params) {
            //for debugging
            /*List<MeterProfileModel> x = new LinkedList<>();
            for (int i = 0; i< 10; i++ ) {
                x.add(meters.get(i));
            }*/


            return  uploadInfoForMeters();
        }
    }
}
