package my.ccs.project.utils.sensoro;

import android.app.Activity;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.widget.Toast;

import com.sensoro.beacon.kit.Beacon;
import com.sensoro.beacon.kit.BeaconManagerListener;
import com.sensoro.cloud.SensoroManager;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;

import my.ccs.project.activities.CCSReadyForShakeActivity;
import my.ccs.project.constants.CCSBasicConstants;
import my.ccs.project.utils.logcatreader.CCSFileLogger;

/**
 * Created by Hayk on 6/19/2016.
 */
public class CCSSensoroListenerManager implements BeaconManagerListener {

    public static CCSSensoroListenerManager sharedSensoroListenerManager = new CCSSensoroListenerManager();
    HashMap<String, BeaconExt> serialNumberToBeacon = new HashMap<String, BeaconExt>();
    static Object lockObj = new Object();
    private static CCSFileLogger fileLogger = null;
    public  HashMap<String, BeaconExt> getSerialNumberToBeacon()
    {
        return serialNumberToBeacon;
    }
    private CCSSensoroListenerManager()
    {
    }

    public void updateFileLogger(CCSFileLogger fileLoggerLatest)
    {
        fileLogger = fileLoggerLatest;
    }

    private final Handler sensoroHandler = new Handler(){
        @Override
        public void handleMessage(Message msg) {
            if (msg != null) {
                switch (msg.what) {
                    case 1:   //update sensoros
                    {
                        try
                        {
                            ArrayList<Beacon> listOfBeacons = (ArrayList<Beacon>) msg.obj;
                            if(serialNumberToBeacon != null) {
                                for (Beacon beacon : listOfBeacons) {
                                    if (beacon != null && !serialNumberToBeacon.containsKey(beacon.getSerialNumber())) {
                                        serialNumberToBeacon.put(beacon.getSerialNumber(), new BeaconExt(beacon, false, Calendar.getInstance().getTime()));
                                    } else {
                                        BeaconExt b = serialNumberToBeacon.get(beacon.getSerialNumber());
                                        if (b != null) {
                                            b.isMarkedToRemove = false;
                                            b.markedToRemoveDate = Calendar.getInstance().getTime();
                                        }
                                    }

                                }
                            }
                        }

                        catch (Exception e)
                        {

                        }
                        break;
                    }
                    case 2: //new sensoro found
                    {
                        try {

                            Beacon beacon = (Beacon) msg.obj;
                            String sn = beacon.getSerialNumber();


                            if (!serialNumberToBeacon.containsKey(sn)) {
                                serialNumberToBeacon.put(sn, new BeaconExt(beacon, false, Calendar.getInstance().getTime()));
                            } else {
                                BeaconExt b = serialNumberToBeacon.get(beacon.getSerialNumber());

                                b.isMarkedToRemove = false;
                                b.markedToRemoveDate = Calendar.getInstance().getTime();

                            }
                        }

                        catch (Exception e)
                        {

                        }

                        break;
                    }
                    case 3: //some sensoro gone out of range
                    {
                        Beacon beacon = (Beacon) msg.obj;
                        try{
                            String sn = beacon.getSerialNumber();
                            if (serialNumberToBeacon.containsKey(sn)) {

                                BeaconExt b = serialNumberToBeacon.get(sn);
                                b.isMarkedToRemove = true;
                                b.markedToRemoveDate = Calendar.getInstance().getTime();
                                Log.d(CCSBasicConstants.MyTag, String.format("Beacon %s marked to be removed.", b.b.getSerialNumber()));
                                //if(fileLogger != null)
                                //    fileLogger.writeToFile(String.format("Beacon %s marked to be removed.", b.b.getSerialNumber()));
                            }


                        }
                        catch (Exception e)
                        {

                        }
                        break;
                    }
                }
            }
        }
    };


   /* private void initSensoroListener() {
        if(beaconManagerListener == null) {
            beaconManagerListener = new BeaconManagerListener() {*/

    @Override
    public void onUpdateBeacon(ArrayList<Beacon> beacons) {
        // Refresh sensor info

        Message msg = Message.obtain(); // Creates an new Message instance
        if (msg != null) {
            msg.what = 1;
            msg.obj = beacons; // Put the string into Message, into "obj" field.
            if (sensoroHandler != null) {
                msg.setTarget(sensoroHandler);
                msg.sendToTarget();
            }
        }
    }

    @Override
    public void onNewBeacon(Beacon beacon) {
        // New sensor found
        Log.d(CCSBasicConstants.MyTag, "Found a new beacon.");
        //if(fileLogger != null)
        //    fileLogger.writeToFile("Found a new beacon.");
        Message msg = Message.obtain(); // Creates an new Message instance
        if (msg != null) {
            msg.what = 2;
            msg.obj = beacon; // Put the string into Message, into "obj" field.
            if (sensoroHandler != null) {
                msg.setTarget(sensoroHandler);
                msg.sendToTarget();
            }
        }
    }

    @Override
    public void onGoneBeacon(Beacon beacon) {
        // A sensor disappears from the range
        Log.d(CCSBasicConstants.MyTag, "A new beacon to be gone.");
        //if(fileLogger != null)
        //    fileLogger.writeToFile("A new beacon to be gone.");
        Message msg = Message.obtain(); // Creates an new Message instance
        if (msg != null) {
            msg.what = 3;
            msg.obj = beacon; // Put the string into Message, into "obj" field.
            if (sensoroHandler != null) {
                msg.setTarget(sensoroHandler);
                msg.sendToTarget();
            }

        }

    }
           // };
    //    }

   // }

    public class BeaconExt
    {
        public Beacon b;
        boolean isMarkedToRemove;
        Date markedToRemoveDate;
        private int removeInterval = 90000;
        public BeaconExt(Beacon b, boolean isMarkedToRemove, Date markedToRemoveDate) {
            this.b = b;
            this.isMarkedToRemove = isMarkedToRemove;
            this.markedToRemoveDate = markedToRemoveDate;
        }

        public boolean checkRemove()
        {

            Log.d(CCSBasicConstants.MyTag, String.format("Beacon %s, marked to remove %b, interval %d", b.getSerialNumber(), this.isMarkedToRemove, Calendar.getInstance().getTime().getTime() - this.markedToRemoveDate.getTime()));
            //if(fileLogger != null)
            //    fileLogger.writeToFile(String.format("Beacon %s, marked to remove %b, interval %d", b.getSerialNumber(), this.isMarkedToRemove, Calendar.getInstance().getTime().getTime() - this.markedToRemoveDate.getTime()));
            if(isMarkedToRemove && (Calendar.getInstance().getTime().getTime() - this.markedToRemoveDate.getTime() > removeInterval))
            {
                return true;
            }

            return  false;
        }
    }

    public LinkedList<BeaconExt> getCurrentActiveSensorosOrdered()
    {
        LinkedList<BeaconExt> orderedList = new LinkedList<BeaconExt>();
        if(serialNumberToBeacon != null) {
            for (String key : serialNumberToBeacon.keySet()) {
                int index = 0;
                BeaconExt value = (BeaconExt) serialNumberToBeacon.get(key);
                if (value != null) {
                    for (int i = 0; i < orderedList.size(); i++) {

                        BeaconExt value2 = orderedList.get(i);
                        if (value.b.getAccuracy() < value2.b.getAccuracy())
                            break;
                        index++;
                    }

                }
                orderedList.add(index, value);

            }
        }

        return  orderedList;
    }

    public String getCurrentActiveSensorosAsString()
    {
        String sensoroListStr = null;
        LinkedList<BeaconExt> orderedList = getCurrentActiveSensorosOrdered();

        if(orderedList.size() != 0) {
            sensoroListStr = CCSBasicConstants.EMPTYSTRING;
            for (int i = 0; i < orderedList.size(); i++) {
                sensoroListStr += orderedList.get(i).b.getSerialNumber() + ",";
            }

            sensoroListStr = sensoroListStr.replaceAll(",$", "");
        }
        return sensoroListStr;
    }

}
