//
//  ViewController.swift
//  CCS China Portal
//
//  Created by CCS China Portal on 10/10/15.
//  Copyright © 2015 CCS China Portal. All rights reserved.
//

import UIKit

import CoreLocation

   
@objc class CCSReadyToShakeViewController: UIViewController, CBCentralManagerDelegate, UIWebViewDelegate, WXApiDelegate, BMKMapViewDelegate, BMKLocationServiceDelegate,UIScrollViewDelegate {
   
    // outlets
    @IBOutlet var versionLbl: UILabel!
    @IBOutlet var easyJsWebView: EasyJSWebView!
    @IBOutlet var detectedBeaconsLbl: UILabel!
    @IBOutlet var loadingView: UIView!
    @IBOutlet var activityIndicator : UIActivityIndicatorView!
    
    var locationService: BMKLocationService!
    var centralManager :  CBCentralManager? = nil
    var orderedList = [SBKBeacon]()
    var lat : Double? = nil
    var lng : Double? = nil
    var userId : String? = nil;
    let webViewManager : CCSWebViewManager = CCSWebViewManager()
    let activityIndicatorManager : CCSActivityIndicatorManager = CCSActivityIndicatorManager()
    let sensoroManager :  CCSSensoroManager = CCSSensoroManager.sharedInstance
  
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder);
        UIView.setAnimationsEnabled(false);
     
    }
    
    override init(nibName nibNameOrNil: String!, bundle nibBundleOrNil: NSBundle!) {
       
        super.init(nibName:nil, bundle:nil);
        UIView.setAnimationsEnabled(false);
    }
    
    func initLoadingView()
    {
        loadingView.frame = CGRectMake(0, 0, 80, 80)
        loadingView.center = self.view.center
        loadingView.backgroundColor = UIColor.blackColor()
        loadingView.alpha = 0.7
        loadingView.clipsToBounds = true
        loadingView.layer.cornerRadius = 10
        loadingView.hidden = true
        activityIndicator.transform = CGAffineTransformMakeScale(2.0, 2.0);
        activityIndicator.hidesWhenStopped = true
    }
    
    func initWebView()
    {
        let userAgent = UIWebView().stringByEvaluatingJavaScriptFromString("navigator.userAgent")! + " " + CCSBasicConstants.userAgent
        NSUserDefaults.standardUserDefaults().registerDefaults(["UserAgent" : userAgent])
        easyJsWebView.proxyDelegate.realDelegate = self;
        let interface =  MyJSInterface.init(viewController: self);
        easyJsWebView.addJavascriptInterfaces(interface, withName: CCSBasicConstants.easyJsInterfaceName);
        easyJsWebView.backgroundColor  = UIColor.clearColor()
        easyJsWebView.scrollView.delegate = self;
        
    }
    
    func initLocService()
    {
        locationService = BMKLocationService()
        locationService.allowsBackgroundLocationUpdates = true
        startLocation();
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        WXApi.registerApp(WECHAT_APP_ID, withDescription: CCSWeChatConstants.appDescription);
        if(SBKBeaconManager.sharedInstance() != nil)
        {
            SBKBeaconManager.sharedInstance().delegate  = CCSSensoroManager.sharedInstance;
        }
        initLoadingView()
        initLocService()
        initWebView()
        webViewManager.loadUrlInWebView(easyJsWebView, url: CCSUrlConstants.readyToShakeUrl)
    }
 
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        locationService.delegate = self
    }
    
    override func viewWillDisappear(animated: Bool) {
        super.viewWillDisappear(animated)
        locationService.delegate = self
    }

    func webViewDidFinishLoad(webView: UIWebView) {
        activityIndicatorManager.stopActivityIndicator(activityIndicator, loadingView: loadingView)
        let urlLoaded: String? = webView.request!.URL?.absoluteString;
        print("finish")
        print(urlLoaded)
    }
    
    func webViewDidStartLoad(webView: UIWebView) {
        activityIndicatorManager.startActivityIndicator(activityIndicator, loadingView: loadingView)
        let urlLoaded: String? = webView.request!.URL?.absoluteString;
        print(urlLoaded)
    }
    
    func webView(webView: UIWebView, shouldStartLoadWithRequest request: NSURLRequest, navigationType: UIWebViewNavigationType) -> Bool{
        let urlLoaded: String? = request.URL?.absoluteString;
        if (urlLoaded != nil)
        {
            let urlLoadedUnwrap :String = urlLoaded!;
            if(urlLoadedUnwrap.characters.startsWith(CCSBasicConstants.userIdStartCharacters.characters) == true)
            {
                let useridArr : [String] = urlLoadedUnwrap.componentsSeparatedByString("=");
                userId = useridArr[1];
                let settings = UIUserNotificationSettings(forTypes: [.Alert, .Badge, .Sound], categories: nil);
                UIApplication.sharedApplication().registerUserNotificationSettings(settings);
                
            }
        }
        return true;
    }
    
    func webView(webView: UIWebView, didFailLoadWithError error: NSError?)
    {
        activityIndicatorManager.stopActivityIndicator(activityIndicator, loadingView: loadingView)
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func checkBluetoothServices() -> Bool
    {
        if (self.centralManager == nil) {
            self.centralManager =  CBCentralManager(delegate: self, queue: nil);
        }
        if (self.centralManager!.state == CBCentralManagerState.PoweredOff) {
            return false;
        }
        else if(self.centralManager!.state == CBCentralManagerState.PoweredOn){
            return true;
        }
        return true;
    }
    
    func centralManagerDidUpdateState(central: CBCentralManager) {
        
    }
    

    // MARK: - BMKMapViewDelegate
    // MARK: - BMKLocationServiceDelegate
    
    /**
    *在地图View将要启动定位时，会调用此函数
    *@param mapView 地图View
    */
    func willStartLocatingUser() {
    }
    
    /**
     *用户方向更新后，会调用此函数
     *@param userLocation 新的用户位置
     */
    func didUpdateUserHeading(userLocation: BMKUserLocation!) {
    }
    
    /**
     *用户位置更新后，会调用此函数
     *@param userLocation 新的用户位置
     */
    func didUpdateBMKUserLocation(userLocation: BMKUserLocation!) {
        
        lat = userLocation.location.coordinate.latitude;
        lng = userLocation.location.coordinate.longitude;
        //let detectedLat : Double  = userLocation.location.coordinate.latitude;
        //let detectedLng : Double = userLocation.location.coordinate.longitude;
        //let latLngModel : CCSUtilities.LatLngModel =  CCSUtilities.LatLngModel(lat: detectedLat, lng: detectedLng);//CCSUtilities.bd2gcj(detectedLat, bdLng: detectedLng)
        //lat = latLngModel.lat
        //lng = latLngModel.lng
        
    }
    
    /**
     *在地图View停止定位后，会调用此函数
     *@param mapView 地图View
     */
    func didStopLocatingUser() {
    }
    
    func startLocation() {
        locationService.startUserLocationService()

    }

    func stopLocation() {
        locationService.stopUserLocationService()
    }
    
    override func canBecomeFirstResponder() -> Bool {
        return true;
    }
    
    // MARK: Motion detection
    override func motionEnded(motion: UIEventSubtype, withEvent event: UIEvent?) {
        if(motion == UIEventSubtype.MotionShake){
        let urlLoaded : String? = easyJsWebView.request?.URL?.absoluteString;
        print(urlLoaded)
        if(urlLoaded != nil && urlLoaded!.lowercaseString.containsString(CCSUrlConstants.readyToShakeUrl))
        {
            activityIndicatorManager.startActivityIndicator(activityIndicator, loadingView: loadingView)
            let time = dispatch_time(dispatch_time_t(DISPATCH_TIME_NOW), CCSBasicConstants.delayWhenShaked * Int64(NSEC_PER_SEC))
            dispatch_after(time, dispatch_get_main_queue()) {
                //put your code which should be executed with a delay here
                self.goToVendorList();
            }
        }
        }
    }
    
    func showAlert() {
        let alertController = UIAlertController(title: "iPhone shaked", message:
            "To be implemented!", preferredStyle: UIAlertControllerStyle.Alert)
        alertController.addAction(UIAlertAction(title: "Dismiss", style: UIAlertActionStyle.Default,handler: nil))
        
        self.presentViewController(alertController, animated: true, completion: nil)
        
       
        
    }
    
    func shakeImgTap()
    {
        activityIndicatorManager.startActivityIndicator(activityIndicator, loadingView: loadingView)
        let time = dispatch_time(dispatch_time_t(DISPATCH_TIME_NOW), CCSBasicConstants.delayWhenShaked * Int64(NSEC_PER_SEC))
        dispatch_after(time, dispatch_get_main_queue()) {
            //put your code which should be executed with a delay here
            self.goToVendorList();
        }
        
    }
    /*
    func goToVendorListWithLocation()
    {
        var str : String = "";
        orderedList = [SBKBeacon]()
        for (_, beacon) in sensoroManager.serialNumberToBeacon{
            var index : Int = 0
            for beacon2 in orderedList
            {
                if(beacon.proximity.rawValue < beacon2.proximity.rawValue)
                {
                    break;
                }
                index = index + 1;
            }
            orderedList.insert(beacon, atIndex: index);
        }
        
        for beacon in orderedList{
            str += beacon.serialNumber+",";
        }
        if(str.characters.count != 0){
            str.removeAtIndex(str.endIndex.predecessor());
        }
        var latSend : String = ""
        var lngSend : String = ""
        var urlStr : String = ""
        if(lat != nil && lng != nil)
        {
            latSend = lat!
            lngSend = lng!
            urlStr = CCSUrlConstants.showMapUrl + "?" + "CenterLatitude=" + latSend + "&CenterLongitude=" + lngSend + "&YunziIDs=" + str + "&ShowType=list";
            
        }
        else
        {
            urlStr = CCSUrlConstants.showMapUrl + "?" + "YunziIDs=" + str + "&ShowType=list";
        }
        
        if(userId != nil)
        {
            urlStr += "&UserId=" + userId!;
        }
        
        let requestURL = NSURL(string:urlStr)
        let request = NSURLRequest(URL: requestURL!)
        easyJsWebView.loadRequest(request);
    }*/
    
    func goToVendorListNoSensoros(showType: String)
    {
        do{
            print("goToVendorListNoSensoros")
            
            var urlStr : String = ""
            
            // Convert to bing coordinates to store in the db
            //let latLng : CCSUtilities.LatLngModel = CCSUtilities.bd2gcj(lat, bdLng: lng);
            
            if(self.lat != nil && self.lng != nil)
            {
                urlStr = CCSUrlConstants.showMapUrl + "?" + "CenterLatitude=" + String(self.lat!) + "&CenterLongitude=" + String(self.lng!) + "&ShowType=" + showType;
            }
            else
            {
                urlStr = CCSUrlConstants.showMapUrl + "?" + "ShowType=" + showType;
            }
            
            if(self.userId != nil)
            {
                urlStr += "&UserId=" + self.userId!;
            }
            print(urlStr)
            let requestURL = NSURL(string:urlStr)
            if(requestURL != nil)
            {
                let request = NSURLRequest(URL: requestURL!)
                self.easyJsWebView.loadRequest(request);
            }
        }
        catch
        {
            print("error")
        }
        
        
    }
    
    func goToVendorList()
    {
        print(sensoroManager.serialNumberToBeacon.count)
        var str : String = "";
        orderedList = [SBKBeacon]()
        for (_, beacon) in sensoroManager.serialNumberToBeacon{
            var index : Int = 0
            for beacon2 in orderedList
            {
                if(beacon.proximity.rawValue < beacon2.proximity.rawValue)
                {
                    break;
                }
                index = index + 1;
            }
            orderedList.insert(beacon, atIndex: index);
        }
        
        for beacon in orderedList{
            str += beacon.serialNumber+",";
        }
        
        if(str.characters.count != 0){
            str.removeAtIndex(str.endIndex.predecessor());
        }
        if(str.characters.count == 0)
        {
            str = "-1";
        }
        
        var urlStr : String = CCSUrlConstants.showMapWhenShakeUrl + "?" + "YunziIDs=" + str + "&ShowType=list";
        if(userId != nil)
        {
            urlStr += "&UserId=" + userId!;
        }
        print(urlStr)
        let requestURL = NSURL(string:urlStr)
        let request = NSURLRequest(URL: requestURL!)
        easyJsWebView.loadRequest(request);
    }
    
    func transitionToViewControllerWithId(viewControllerID: String, customUrlToLoad: String) -> Void
    {
        let mainStoryboard = UIStoryboard(name: CCSBasicConstants.mainStoryboardName, bundle: NSBundle.mainBundle())
        let vc : CCSCustomUrlViewController = mainStoryboard.instantiateViewControllerWithIdentifier(viewControllerID) as! CCSCustomUrlViewController
        vc.urlToLoad = customUrlToLoad;
        let transition: CATransition = CATransition()
        let timeFunc : CAMediaTimingFunction = CAMediaTimingFunction(name: kCAMediaTimingFunctionEaseInEaseOut)
        transition.duration = 0.25
        transition.timingFunction = timeFunc
        transition.type = kCATransitionPush
        transition.subtype = kCATransitionFromRight
        self.navigationController?.view.layer.addAnimation(transition, forKey: kCATransition)
        self.navigationController?.pushViewController(vc, animated: false)
        
    }
}

