//
//  CCSSensoroManager.swift
//  CCSChinaPortalVendors
//
//  Created by CCS China Portal on 6/17/16.
//  Copyright © 2016 CCS China Portal. All rights reserved.
//

import Foundation

class CCSSensoroManager : NSObject, SBKBeaconManagerDelegate
{
    static let sharedInstance = CCSSensoroManager()
    var serialNumberToBeacon: [String:SBKBeacon] = [String:SBKBeacon]()
    var serialNumberToLastSignalDate: [String:NSDate] = [String:NSDate]()
    func beaconManager(beaconManager: SBKBeaconManager,  didRangeNewBeacon beacon: SBKBeacon) -> Void
    {
        print(String(serialNumberToBeacon.count))
        // ignore if for any reason serial number is null
        if(beacon.serialNumber == nil)
        {
            return;
        }
        
        // serialNumberToBeacon dictionary -> key is the serial number, value is the actual beacon object
        // serialNumberToLastSignalDate dictionary -> key is the serial number, value is the last timestamp when a signal received from this beacon
        if(serialNumberToBeacon[beacon.serialNumber] == nil)
        {
            serialNumberToBeacon[beacon.serialNumber] = beacon;
            
        }
        
        // update last signal timestamp
        serialNumberToLastSignalDate[beacon.serialNumber] = NSDate()
    }
    
    func beaconManager(beaconManager: SBKBeaconManager,  beaconDidGone beacon: SBKBeacon) -> Void
    {
        print(String(serialNumberToBeacon.count))
        // check if the sensoro should be removed and if so remove it
        if(beacon.serialNumber != nil && self.shouldRemoveBeacon(beacon))
        {
            serialNumberToBeacon.removeValueForKey(beacon.serialNumber);
            serialNumberToLastSignalDate.removeValueForKey(beacon.serialNumber)
        }
    }
    
    /* called each second with list of sensoros in the range */
    /*func beaconManager(beaconManager: SBKBeaconManager!, didRangeBeacons beacons: [AnyObject]!, inRegion region: SBKBeaconID!) {
    print("didRangeBeacons running " + String(beacons.count))
    // loop through the list of beacon and update dictionaries if anything is missing
    for beaconObj in beacons
    {
    let beacon : SBKBeacon = beaconObj as! CLBeacon
    
    
    print("beacon " + beacon.serialNumber)
    // ignore if for any reason serial number is null
    if(beacon.serialNumber == nil)
    {
    return;
    }
    print("Range beacon " + beacon.serialNumber)
    
    // serialNumberToBeacon dictionary -> key is the serial number, value is the actual beacon object
    // serialNumberToLastSignalDate dictionary -> key is the serial number, value is the last timestamp when a signal received from this beacon
    if(serialNumberToBeacon[beacon.serialNumber] == nil)
    {
    serialNumberToBeacon[beacon.serialNumber] = beacon
    
    }
    
    //update last signal timestamp
    serialNumberToLastSignalDate[beacon.serialNumber] = NSDate()
    
    }
    
    // scan to remove any sensoros from which there was no signal more than a minute
    self.scanAvailableSensoroForRemoval()
    }*/
    
    func beaconManager(beaconManager: SBKBeaconManager,  scanDidFinishWithBeacons beacons: Array<AnyObject>) -> Void
    {
        print(String(serialNumberToBeacon.count))
        //print("scanDidFinishWithBeacons")
        // loop through the list of beacon and update dictionaries if anything is missing
        for beaconObj in beacons
        {
            let beacon : SBKBeacon = beaconObj as! SBKBeacon
            
            // ignore if for any reason serial number is null
            if((beacon.serialNumber == nil))
            {
                return;
            }
            
            // serialNumberToBeacon dictionary -> key is the serial number, value is the actual beacon object
            // serialNumberToLastSignalDate dictionary -> key is the serial number, value is the last timestamp when a signal received from this beacon
            if(serialNumberToBeacon[beacon.serialNumber] == nil)
            {
                serialNumberToBeacon[beacon.serialNumber] = beacon
                
            }
            //update last signal timestamp
            serialNumberToLastSignalDate[beacon.serialNumber] = NSDate()
            
        }
        
        // scan to remove any sensoros from which there was no signal more than a minute
        self.scanAvailableSensoroForRemoval()
    }
    
    // this is called once each second and checks if there are any beacons that should be removed
    func scanAvailableSensoroForRemoval()
    {
        let copyBeaconsMap = serialNumberToBeacon
        for(_, beacon) in copyBeaconsMap
        {
            if(beacon.serialNumber != nil && self.shouldRemoveBeacon(beacon))
            {
                serialNumberToBeacon.removeValueForKey(beacon.serialNumber);
            }
        }
    }
    
    // make a decision if the beacon should be removed or not
    // return true -> remove
    // return false -> do NOT remove
    func shouldRemoveBeacon(beacon: SBKBeacon) -> Bool
    {
        
        // if the key does not exist this will be false. In that case do nothing
        if(serialNumberToBeacon[beacon.serialNumber] != nil)
        {
            // check the last signal timestamp before removing
            // if last signal date is not available do NOT remove (to be discussed)
            if(serialNumberToLastSignalDate.indexForKey(beacon.serialNumber) != nil)
            {
                /* check if current timestamp is one minute older than beacon last signal timestamp*/
                let lastSignalDate : NSDate = serialNumberToLastSignalDate[beacon.serialNumber]!
                let secondsToRemove: NSTimeInterval = Double(CCSBasicConstants.sensoroLastSignalToRemoveInterval)
                let lastSignatlDateWithDaysAdded: NSDate = lastSignalDate.dateByAddingTimeInterval(secondsToRemove)
                let currentDate = NSDate()
                if(lastSignatlDateWithDaysAdded.compare(currentDate) == NSComparisonResult.OrderedAscending)
                {
                    // should be removed
                    return true
                }
                
            }
            
        }
        return false
        
    }
    
    
}
